/***************************************************************************
				                          SoundAlsa.cpp
                             -------------------                                         
    begin                : Wed Dec 6 2000                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		 : Anton Safonov (Maddy)                                         
    author email         : as@bestlinux.net                                          
    url			 : http://www.bestlinux.net                          
    email		 : linux@bestlinux.net                              

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <sys/errno.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <linux/soundcard.h>
#include <unistd.h>
#include <string.h>

#include <qstring.h>
#include <qstrlist.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qaccel.h>
#include <qmessagebox.h>
#include <qwindowdefs.h>
#include <qlayout.h>

#include "linlinehelp.h"
#include "liconloader.h"
#include "config.h"
#include "SoundAlsa.h"
#include "lsound.h"


#define _(String) gettext((String))
#define N_(String) (String)
#define IRQ_BANK 				1
#define DMA_BANK 				1<<1
#define PORT_BANK 			1<<2
#define SIZE_BANK				1<<3
#define ENABLE_BANK			1<<4

/* Typing all this out is a pain... */
#define STANDART_IRQ    {IRQ_BANK,5,7,9,10,-1}
#define LOW_DMA         {DMA_BANK,0,1,3,-1}
#define HIGH_DMA				{DMA_BANK,5,6,7,-1}
#define SB16_IO         {PORT_BANK,0x220,0x240,0x260,0x280,-1}
#define WSS_IO          {PORT_BANK,0x530,0x534,0x604,0xE80,0xF40,-1}
#define MPU401_IO       {PORT_BANK,0x330,0x300,-1}

#define ESS_SND_PORT		{PORT_BANK,0x220,0x240,0x260,-1}
#define SND_PORTS_ALL		{PORT_BANK,0x210,0x220,0x230,0x240,0x250,0x260,0x270,0x530,0xe80,0xf40,-1}
#define IRQS_ALL				{IRQ_BANK,3,5,7,9,10,11,12,15,-1}
#define DMA8    				{DMA_BANK,0,1,3,-1}
#define SIZE_64					{SIZE_BANK,4, 8, 16, 32, 64,-1}
#define SIZE_128  			{SIZE_BANK,4, 8, 16, 32, 64, 128, -1}
#define SIZE_256				{SIZE_BANK,4, 8, 16, 32, 64, 128, 256, -1}
#define SND_ENABLE			{ENABLE_BANK,1, 0, -1}
#define DMA            	{DMA_BANK,0,1,3,5,6,7,-1}
#define MPU_PORTS       {PORT_BANK,0x300,0x310,0x320,0x330,0x332,0x334,-1}
#define FM_PORTS        {PORT_BANK,0x388,0x398,0x3a0,0x3a8,-1}
#define FALSE						false
#define GESBUF					{SIZE_BANK,1,128,256,512,1024,2048,4096,-1}
#define CHANNELS				{SIZE_BANK,1,2,3,4,5,6,7,8,-1}
#define	JOYSTICK_DAC		{SIZE_BANK,0,4,8,12,16,20,24,28,31,-1}
#define SND_VOICES			{SIZE_BANK,14,16,18,20,22,24,26,28,30,32,-1}


#define SND_MINOR_OSS_MIXER	0	/* /dev/mixer - OSS 3.XX compatible */
#define SND_MINOR_OSS_SEQUENCER	1	/* /dev/sequencer - OSS 3.XX compatible */
#define	SND_MINOR_OSS_MIDI	2	/* /dev/midi - native midi interface - OSS 3.XX compatible - UART */
#define SND_MINOR_OSS_PCM	3	/* alias */
#define SND_MINOR_OSS_PCM_8	3	/* /dev/dsp - 8bit PCM - OSS 3.XX compatible */
#define SND_MINOR_OSS_AUDIO	4	/* /dev/audio - SunSparc compatible */
#define SND_MINOR_OSS_PCM_16	5	/* /dev/dsp16 - 16bit PCM - OSS 3.XX compatible */
#define SND_MINOR_OSS_SNDSTAT	6	/* /dev/sndstat - for compatibility with OSS */
#define SND_MINOR_OSS_RESERVED7	7	/* reserved for future use */
#define SND_MINOR_OSS_MUSIC	8	/* /dev/music - OSS 3.XX compatible */
#define SND_MINOR_OSS_DMMIDI	9	/* /dev/dmmidi0 - this device can have another minor # with OSS */
#define SND_MINOR_OSS_DMFM	10	/* /dev/dmfm0 - this device can have another minor # with OSS */
#define SND_MINOR_OSS_MIXER1	11	/* alternate mixer */
#define SND_MINOR_OSS_PCM1	12	/* alternate PCM (GF-A-1) */
#define SND_MINOR_OSS_MIDI1	13	/* alternate midi - SYNTH */
#define SND_MINOR_OSS_DMMIDI1	14	/* alternate dmmidi - SYNTH */
#define SND_MINOR_OSS_RESERVED15 15	/* reserved for future use */

#define MNOGO					2000
#define DEFAULT_SPACING 5

#define Snd_port 							"snd_port"
#define Snd_mpu_port 					"snd_mpu_port"
#define Snd_irq 							"snd_irq"
#define Snd_mpu_irq 					"snd_mpu_irq"
#define Snd_dma8 							"snd_dma8"
#define Snd_dma8_size 				"snd_dma8_size"
#define Snd_dma16 						"snd_dma16"
#define Snd_dma16_size				"snd_dma16_size"
#define Snd_dma1 							"snd_dma1"
#define Snd_dma1_size 				"snd_dma1_size"
#define Snd_dma2 							"snd_dma2"
#define Snd_dma2_size 				"snd_dma2_size"
#define Snd_fm_port 					"snd_fm_port"
#define Snd_dac_frame_size 		"snd_dac_frame_size"
#define Snd_adc_frame_size 		"snd_adc_frame_size"
#define Snd_enable_midi 			"snd_enable_midi"
#define Snd_enable_fm 				"snd_enable_fm"
#define Snd_isapnp 						"snd_isapnp"
#define Snd_wssport 					"snd_wssport"
#define Snd_wssirq 						"snd_wssirq"
#define Snd_wssdma 						"snd_wssdma"
#define Snd_wssdma_size 			"snd_wssdma_size"
#define Snd_sbport 						"snd_sbport"
#define Snd_sbirq 						"snd_sbirq"
#define Snd_sbdma8 						"snd_sbdma8"
#define Snd_sbdma8_size 			"snd_sbdma8_size"
#define Snd_sbdma16 					"snd_sbdma16"
#define Snd_sbdma16_size 			"snd_sbdma16_size"
#define Snd_conp_frame_size		"snd_conp_frame_size"
#define Snd_prop_frame_size 	"snd_prop_frame_size"
#define Snd_conc_frame_size 	"snd_conc_frame_size"
#define Snd_proc_frame_size 	"snd_proc_frame_size"
#define Snd_pbk_frame_size 		"snd_pbk_frame_size"
#define Snd_cap_frame_size 		"snd_cap_frame_size"
#define Snd_mic_frame_size 		"snd_mic_frame_size"
#define Snd_gesbuf						"snd_gesbuf"
#define Snd_pcm_channels_p		"snd_pcm_channels_p"
#define Snd_pcm_channels_c 		"snd_pcm_channels_c"
#define Snd_ac97_frame_size 	"snd_ac97_frame_size"
#define Snd_joystick_dac			"snd_joystick_dac"
#define Snd_midi							"snd_midi"
#define Snd_pcm_voices				"snd_pcm_voices"
#define Snd_effect						"snd_effect"
#define Snd_wavetable_size		"snd_wavetable_size"
#define Snd_dmaa_size					"snd_dmaa_size"
#define Snd_dmac_size					"snd_dmac_size"
#define Snd_reverb						"snd_reverb"
#define	Snd_mge								"snd_mge"
#define Snd_awe_port					"snd_awe_port"
#define	Snd_dac1_frame_size		"snd_dac1_frame_size"
#define	Snd_dac2_frame_size		"snd_dac2_frame_size"
#define Snd_midi_enable				"snd_midi_enable"
#define Snd_pcm_channels			"snd_pcm_channels"
#define Snd_voices						"snd_voices"
#define Snd_gf1_port					"snd_gf1_port"
#define Snd_irq1							"snd_irq1"
#define Snd_sb_port						"snd_sb_port"
#define Snd_wss_port					"snd_wss_port"
#define Snd_midi_port					"snd_midi_port"
#define Snd_port_tc						"snd_port_tc"
#define Snd_joystick					"snd_joystick"


/*struct CardValues {
		char	cValue[10];
};*/

struct CardModules {
		char 					cName[30];
		int				  	iValues[13];
};

struct SoundCardsTable {
    char 					modulename[32];
    char 					descr[256];
		CardModules 	Amodules[20];

};

struct SoundCardsTable
    ACardsArray[] = {
    {"", N_("No Sound Card"),{ {"",{-1} } } },
//
    {"adatpcr", N_("Alesis ADAT/pcr"),{ {"",{-1} } }},
//
    {"ad1816a", N_("Analog Devices AD1816"),
			{{Snd_dma1_size, SIZE_128}, {Snd_dma2_size, SIZE_128}, {"", {-1} }}},
//
    {"ad1848", N_("Analog Devices AD1847/AD1848"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, LOW_DMA}, {Snd_dma1_size, SIZE_64}, {"", {-1} }}},
//
    {"cmipci", N_("ASOUND ASONIC-8738 CMI8738"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_enable_midi, SND_ENABLE},
			 {Snd_enable_fm, SND_ENABLE}, {Snd_mpu_port, MPU_PORTS}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},
//
    {"als100", N_("Avance Logic ALS100/ALS110/ALS120"),
			{{Snd_dma8_size, SIZE_64}, {Snd_dma16_size, SIZE_128}, {"", {-1} }}},
//
    {"cs4235", N_("Aopen AW37 CS4235, Aopen AW37 Pro CS4235/CS9236"),{ {"",{-1} } }},
//
    {"es1938", N_("Aopen AW180 ES1938"),{ {"",{-1} } }},
//
    {"cs461x", N_("Aopen AW230(CS4280)/AW320(CS4614)"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},
//
    {"es1968", N_("Aopen AW300 maestro-2"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_gesbuf, GESBUF},
			 {Snd_midi_enable, SND_ENABLE}, {Snd_pcm_channels_p, CHANNELS}, {Snd_pcm_channels_c, CHANNELS}, {"", {-1} }}},

    {"ymfpci", N_("Aopen AW724, Aopen AW744/AW744 Pro"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"interwave", N_("AMD Interwave"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, DMA},
			 {Snd_dma1_size, SIZE_128}, {Snd_dma2, DMA}, {Snd_dma2_size, SIZE_128},
			 {Snd_joystick_dac, JOYSTICK_DAC}, {Snd_midi, SND_ENABLE}, {Snd_pcm_voices, CHANNELS},
			 {Snd_effect, SND_ENABLE}, {"", {-1} }}},

    {"azt2320", N_("AzTech AZT2320"),
			{{Snd_dma1_size, SIZE_64}, {Snd_dma2_size, SIZE_64}, {"", {-1} }}},

    {"trident", N_("AzTech PCI 64-Q3D/288-Q3DII"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_pcm_channels, CHANNELS},
			 {Snd_wavetable_size, GESBUF}, {"", {-1} }}},

    {"sgalaxy", N_("AzTech Sound Galaxy AD1848"),{ {"",{-1} } }},

    {"ymfpci", N_("A-trend 3DS724A YMF724"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"fm801", N_("A-trend 3DS801A FM801"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"cmipci", N_("AudioExcel AV 510/MD Mate"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_enable_midi, SND_ENABLE},
			 {Snd_enable_fm, SND_ENABLE}, {Snd_mpu_port, MPU_PORTS}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"trident", N_("Best Union Miss Melody 4dwave PCI 4DWave DX"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_pcm_channels, CHANNELS},
			 {Snd_wavetable_size, GESBUF}, {"", {-1} }}},

    {"sonicvibes", N_("Best Union Miss Melody S3 Sound SonicVibes PCI"),
			{{Snd_dmaa_size, SIZE_128}, {Snd_dmac_size, SIZE_128}, {Snd_reverb, SND_ENABLE},
			 {Snd_mge, SND_ENABLE}, {"", {-1} }}},

    {"es1968", N_("Best Union Miss Melody Maestro Sound maestro-1"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_gesbuf, GESBUF},
			 {Snd_midi_enable, SND_ENABLE}, {Snd_pcm_channels_p, CHANNELS}, {Snd_pcm_channels_c, CHANNELS}, {"", {-1} }}},

    {"ymfpci", N_("Best Union Miss Melody Samba Tornado YMF724"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"es1938", N_("Best Union Miss Melody Solo-1 Sound ES1938 (Solo-1)"),{ {"",{-1} } }},

    {"cmi8330", N_("C-Media CMI8330"),
			{{Snd_wssport, SND_PORTS_ALL}, {Snd_wssirq, IRQS_ALL}, {Snd_wssdma, DMA},
			 {Snd_wssdma_size, SIZE_128}, {Snd_sbport, SND_PORTS_ALL}, {Snd_sbirq, IRQS_ALL},
			 {Snd_sbdma8, DMA8}, {Snd_sbdma8_size, SIZE_128}, {Snd_sbdma16, DMA},
			 {Snd_sbdma16_size, SIZE_128}, {"", {-1} }}},

    {"cmipci", N_("C-Media CMI8338/CMI8738"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_enable_midi, SND_ENABLE},
			 {Snd_enable_fm, SND_ENABLE}, {Snd_mpu_port, MPU_PORTS}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"trident", N_("Chic Technology True Sound 4d wave-DX/NX"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_pcm_channels, CHANNELS},
			 {Snd_wavetable_size, GESBUF}, {"", {-1} }}},

    {"ymfpci", N_("Chic Technology 724 YMF724"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"cs4231", N_("Cirrus Logic CS4231"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_mpu_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL},
			 {Snd_mpu_irq, IRQS_ALL}, {Snd_dma1, DMA}, {Snd_dma1_size, SIZE_128},
			 {Snd_dma2, DMA}, {Snd_dma2_size, SIZE_128}, {"", {-1} }}},

    {"cs4232", N_("Cirrus Logic CS4232/CS4232A"),
			{{Snd_irq, IRQS_ALL}, {Snd_mpu_irq, IRQS_ALL}, {Snd_dma1, LOW_DMA},
			 {Snd_dma1_size, SIZE_64}, {Snd_dma2, SIZE_64}, {Snd_dma2_size, SIZE_64},
			 {Snd_isapnp, SND_ENABLE}, {"", {-1} }}},

    {"cs4236", N_("Cirrus Logic CS4235/CS4236/CS4236B/CS4237B/CS4238B/CS4239"),
			{{Snd_irq, IRQS_ALL}, {Snd_mpu_irq, IRQS_ALL}, {Snd_dma1, LOW_DMA},
			 {Snd_dma1_size, SIZE_64}, {Snd_dma2, SIZE_64}, {Snd_dma2_size, SIZE_64},
			 {Snd_isapnp, SND_ENABLE}, {"", {-1} }}},
//
    {"ad1848", N_("Cirrus Logic CS4248"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, LOW_DMA}, {Snd_dma1_size, SIZE_64}, {"", {-1} }}},

    {"cs4281", N_("Cirrus Logic CS4281"),{ {"",{-1} } }},
//
    {"cs461x", N_("Cirrus Logic CS4610/CS4612/CS4614/CS4615/CS4280"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"cmipci", N_("Compustar WinFast CMI-8738/PCI C3DX Sound"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_enable_midi, SND_ENABLE},
			 {Snd_enable_fm, SND_ENABLE}, {Snd_mpu_port, MPU_PORTS}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"sb8", N_("Creative Labs Soundblaster 1.0/1.5/2.x/Pro"),
			{{Snd_port, ESS_SND_PORT}, {Snd_irq, STANDART_IRQ}, {Snd_dma8, DMA8},
			 {Snd_dma8_size, SIZE_64}, {"", {-1} }}},

    {"sb16", N_("Creative Labs Soundblaster 16/16 ASP/16 PnP"),
			{{Snd_port, ESS_SND_PORT}, {Snd_mpu_port, MPU401_IO}, {Snd_irq, STANDART_IRQ},
			 {Snd_dma8, DMA8}, {Snd_dma8_size, SIZE_64}, {Snd_dma16, HIGH_DMA},
			 {Snd_dma16_size, SIZE_128}, {"", {-1} }}},

    {"ens1371", N_("Creative Labs Soundblaster 16 PCI/Ensoniq AudioPCI"),
			{{Snd_dac1_frame_size, SIZE_128}, {Snd_dac2_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"sb16", N_("Creative Labs Soundblaster Vibra16C/16CL/16S/16X"),
			{{Snd_port, ESS_SND_PORT}, {Snd_mpu_port, MPU401_IO}, {Snd_irq, STANDART_IRQ},
			 {Snd_dma8, DMA8}, {Snd_dma8_size, SIZE_64}, {Snd_dma16, HIGH_DMA},
			 {Snd_dma16_size, SIZE_128}, {"", {-1} }}},

    {"sbawe", N_("Creative Labs Soundblaster 32/AWE/AWE PnP"),
			{{Snd_port, ESS_SND_PORT}, {Snd_mpu_port, MPU401_IO}, {Snd_irq, STANDART_IRQ},
			 {Snd_dma8, DMA8}, {Snd_dma8_size, SIZE_64}, {Snd_dma16, HIGH_DMA},
			 {Snd_dma16_size, SIZE_128}, {Snd_awe_port, SND_PORTS_ALL}, {"", {-1} }}},

    {"sbawe", N_("Creative Labs Soundblaster Awe64 Gold/Value"),
			{{Snd_port, ESS_SND_PORT}, {Snd_mpu_port, MPU401_IO}, {Snd_irq, STANDART_IRQ},
			 {Snd_dma8, DMA8}, {Snd_dma8_size, SIZE_64}, {Snd_dma16, HIGH_DMA},
			 {Snd_dma16_size, SIZE_128}, {Snd_awe_port, SND_PORTS_ALL}, {"", {-1} }}},

    {"ens1371", N_("Creative Labs Soundblaster PCI 64/PCI 128"),
			{{Snd_dac1_frame_size, SIZE_128}, {Snd_dac2_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"emu10k1", N_("Creative Labs Soundblaster PCI 512"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"emu10k1", N_("Creative Labs Soundblaster Live/Live Platinum/Live Value"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"es1968", N_("Diamond Multimedia MonsterSound mx400"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_gesbuf, GESBUF},
			 {Snd_midi_enable, SND_ENABLE}, {Snd_pcm_channels_p, CHANNELS}, {Snd_pcm_channels_c, CHANNELS}, {"", {-1} }}},

    {"es1968", N_("Diamond Multimedia Sonic Impact s100"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_gesbuf, GESBUF},
			 {Snd_midi_enable, SND_ENABLE}, {Snd_pcm_channels_p, CHANNELS}, {Snd_pcm_channels_c, CHANNELS}, {"", {-1} }}},

    {"maestro3", N_("Diamond Multimedia Sonic Impact s100"),{ {"",{-1} } }},

    {"interwave", N_("Dynasonic 3-d Interwave"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, DMA},
			 {Snd_dma1_size, SIZE_128}, {Snd_dma2, DMA}, {Snd_dma2_size, SIZE_128},
			 {Snd_joystick_dac, JOYSTICK_DAC}, {Snd_midi, SND_ENABLE}, {Snd_pcm_voices, CHANNELS},
			 {Snd_effect, SND_ENABLE}, {"", {-1} }}},

    {"ens1370", N_("Ensoniq ES1370"),
			{{Snd_dac1_frame_size, SIZE_128}, {Snd_dac2_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"ens1371", N_("Ensoniq ES1371"),
			{{Snd_dac1_frame_size, SIZE_128}, {Snd_dac2_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"es1968", N_("ESS Technology Canyon3d"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_gesbuf, GESBUF},
			 {Snd_midi_enable, SND_ENABLE}, {Snd_pcm_channels_p, CHANNELS}, {Snd_pcm_channels_c, CHANNELS}, {"", {-1} }}},

    {"es1688", N_("ESS Technology ES688/ES1688"),
			{{Snd_port, SB16_IO}, {Snd_mpu_port, MPU_PORTS}, {Snd_irq, STANDART_IRQ},
			 {Snd_mpu_irq, STANDART_IRQ}, {Snd_dma8, DMA8}, {Snd_dma8_size, SIZE_64}, {"", {-1} }}},

    {"es968", N_("ESS Technology ES968"),{{Snd_dma1_size, SIZE_64}, {"", {-1} }}},

    {"es18xx", N_("ESS Technology ES18xx"),
			{{Snd_port, SB16_IO}, {Snd_mpu_port, MPU_PORTS}, {Snd_irq, STANDART_IRQ},
			 {Snd_dma1, DMA8}, {Snd_dma1_size, SIZE_64}, {Snd_dma2, DMA8},
			 {Snd_dma2_size, SIZE_64}, {Snd_isapnp, SND_ENABLE}, {"", {-1} }}},
//
    {"es1938", N_("ESS Technology ES1938 (Solo-1)/(Solo-1e)"),{ {"",{-1} } }},

    {"es1968", N_("ESS Technology maestro-1/2/2e/2em"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_gesbuf, GESBUF},
			 {Snd_midi_enable, SND_ENABLE}, {Snd_pcm_channels_p, CHANNELS}, {Snd_pcm_channels_c, CHANNELS}, {"", {-1} }}},

    {"maestro3", N_("ESS Technology ES1988/ES1989 (Maestro-3/ Allegro)"),{ {"",{-1} } }},

    {"fm801", N_("Fortemedia FM801"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"gusclassic", N_("Gravis Ultrasound Classic"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, DMA},
			 {Snd_dma1_size, SIZE_128}, {Snd_dma2, DMA}, {Snd_dma2_size, SIZE_128},
			 {Snd_joystick_dac, JOYSTICK_DAC}, {Snd_voices, SND_VOICES}, {"", {-1} }}},

    {"gusextreme", N_("Gravis Ultrasound Extreme"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_gf1_port, SND_PORTS_ALL}, {Snd_mpu_port, MPU_PORTS},
			 {Snd_irq, IRQS_ALL}, {Snd_mpu_irq, IRQS_ALL}, {Snd_irq1, IRQS_ALL},
			 {Snd_dma8, LOW_DMA}, {Snd_dma8_size, SIZE_64}, {Snd_dma1, DMA},	
			 {Snd_dma1_size, SIZE_128}, {Snd_joystick_dac, JOYSTICK_DAC}, {Snd_voices, SND_VOICES}, {"", {-1} }}},

    {"gusmax", N_("Gravis Ultrasound Max"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, DMA},
			 {Snd_dma1_size, SIZE_128}, {Snd_dma2, DMA}, {Snd_dma2_size, SIZE_128},
			 {Snd_joystick_dac, JOYSTICK_DAC}, {Snd_voices, SND_VOICES}, {"", {-1} }}},

    {"interwave", N_("Gravis Ultrasound PnP"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, DMA},
			 {Snd_dma1_size, SIZE_128}, {Snd_dma2, DMA}, {Snd_dma2_size, SIZE_128},
			 {Snd_joystick_dac, JOYSTICK_DAC}, {Snd_midi, SND_ENABLE}, {Snd_pcm_voices, CHANNELS},
			 {Snd_effect, SND_ENABLE}, {"", {-1} }}},

    {"ymfpci", N_("Guillemot MaxiSound Fortissimo"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"trident", N_("HIS 4dwave PCI"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_pcm_channels, CHANNELS},
			 {Snd_wavetable_size, GESBUF}, {"", {-1} }}},

    {"trident", N_("Hoontech Soundtrack Digital-NX"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_pcm_channels, CHANNELS},
			 {Snd_wavetable_size, GESBUF}, {"", {-1} }}},

    {"ymfpci", N_("Hoontech Soundtrack Digital-XG"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"ice1712", N_("IC Ensemble Envy24"),
			{{Snd_conp_frame_size, SIZE_128}, {Snd_prop_frame_size, SIZE_128}, {Snd_conc_frame_size, SIZE_256},
			 {Snd_proc_frame_size, SIZE_256}, {"", {-1} }}},

    {"intel8x0", N_("Intel 440MX/i810/i810E/i820"),
			{{Snd_pbk_frame_size, SIZE_128}, {Snd_cap_frame_size, SIZE_128}, {Snd_mic_frame_size, SIZE_128}, {"", {-1} }}},

    {"trident", N_("Jaton Sonicwave 4d"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_pcm_channels, CHANNELS},
			 {Snd_wavetable_size, GESBUF}, {"", {-1} }}},

    {"ymfpci", N_("Labway D1X"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"maestro3", N_("Labway E20"),{ {"",{-1} } }},

    {"opl3sa2", N_("Labway Xwave128"),
			{{Snd_port, {0x370}}, {Snd_sb_port, {0x220, 0x240}}, {Snd_wss_port, WSS_IO},
			 {Snd_midi_port, MPU401_IO}, {Snd_fm_port, FM_PORTS}, {Snd_irq, IRQS_ALL},
			 {Snd_dma1, LOW_DMA}, {Snd_dma1_size, SIZE_64}, {Snd_dma2, LOW_DMA},
			 {Snd_dma2_size, SIZE_64}, {"", {-1} }}},

    {"ymfpci", N_("Labway Xwave192/320/576/G5X"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"fm801", N_("Labway XwaveG7X QS3000A"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"ymfpci", N_("Labway XwaveJ8X/5000/5000Pro/6000/6000Pro"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"mtpav", N_("Mark of the Unicorn Micro Express/Midi Time/Midi Express XT"),{ {"",{-1} } }},

    {"ice1712", N_("Midiman Delta 44/66/1010"),
			{{Snd_conp_frame_size, SIZE_128}, {Snd_prop_frame_size, SIZE_128}, {Snd_conc_frame_size, SIZE_256},
			 {Snd_proc_frame_size, SIZE_256}, {"", {-1} }}},

    {"cmipci", N_("Midiman Delta DiO 2448"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_enable_midi, SND_ENABLE},
			 {Snd_enable_fm, SND_ENABLE}, {Snd_mpu_port, MPU_PORTS}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"ice1712", N_("Midiman Delta DiO 2496"),
			{{Snd_conp_frame_size, SIZE_128}, {Snd_prop_frame_size, SIZE_128}, {Snd_conc_frame_size, SIZE_256},
			 {Snd_proc_frame_size, SIZE_256}, {"", {-1} }}},

    {"ymfpci", N_("NEC PK-UG-X013"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"mozart", N_("Oak Technology Mozart"),
			{{Snd_port, WSS_IO}, {Snd_irq, IRQS_ALL}, {Snd_dma1, LOW_DMA},
			 {Snd_dma1_size, SIZE_64}, {"", {-1} }}},

    {"opti92x-ad1848", N_("Opti 82C924/82C925/82C929 based on Analog Devices chips"),
			{{Snd_port, WSS_IO}, {Snd_mpu_port, MPU_PORTS}, {Snd_fm_port, FM_PORTS},
			 {Snd_irq, IRQS_ALL}, {Snd_mpu_irq, IRQS_ALL}, {Snd_dma1, LOW_DMA},
			 {Snd_dma1_size, SIZE_64}, {"", {-1} }}},

    {"opti92x-ad1848", N_("Opti 82C924/82C925/82C929 based on Crystal chips"),
			{{Snd_port, WSS_IO}, {Snd_mpu_port, MPU_PORTS}, {Snd_fm_port, FM_PORTS},
			 {Snd_irq, IRQS_ALL}, {Snd_mpu_irq, IRQS_ALL}, {Snd_dma1, LOW_DMA},
			 {Snd_dma1_size, SIZE_64}, {Snd_dma2, LOW_DMA}, {Snd_dma2_size, SIZE_64}, {"", {-1} }}},

    {"opti93x", N_("Opti 82C931"),
			{{Snd_port, WSS_IO}, {Snd_mpu_port, MPU_PORTS}, {Snd_fm_port, FM_PORTS},
			 {Snd_irq, IRQS_ALL}, {Snd_mpu_irq, IRQS_ALL}, {Snd_dma1, LOW_DMA},
			 {Snd_dma1_size, SIZE_64}, {Snd_dma2, LOW_DMA}, {Snd_dma2_size, SIZE_64}, {"", {-1} }}},

    {"interwave", N_("Philips PCA761AW"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, DMA},
			 {Snd_dma1_size, SIZE_128}, {Snd_dma2, DMA}, {Snd_dma2_size, SIZE_128},
			 {Snd_joystick_dac, JOYSTICK_DAC}, {Snd_midi, SND_ENABLE}, {Snd_pcm_voices, CHANNELS},
			 {Snd_effect, SND_ENABLE}, {"", {-1} }}},

    {"rme9652", N_("RME Hammerfall/Hammerfall Light"),{ {"",{-1} } }},

    {"rme96", N_("RME Digi96/8/8PRO/8PST/8PAD"),{ {"",{-1} } }},

    {"pdplus", N_("SEK'D Prodif Plus"),{ {"",{-1} } }},

    {"sonicvibes", N_("PINE Schubert 32 PCI"),
			{{Snd_dmaa_size, SIZE_128}, {Snd_dmac_size, SIZE_128}, {Snd_reverb, SND_ENABLE},
			 {Snd_mge, SND_ENABLE}, {"", {-1} }}},

    {"ymfpci", N_("Sony VAIO N505VX/PCG-F480"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"sonicvibes", N_("S3 SonicVibes PCI"),
			{{Snd_dmaa_size, SIZE_128}, {Snd_dmac_size, SIZE_128}, {Snd_reverb, SND_ENABLE},
			 {Snd_mge, SND_ENABLE}, {"", {-1} }}},

    {"opti9xx", N_("Shuttle HOT-241/245"),{ {"",{-1} } }},

    {"ens1370", N_("Shuttle HOT-255"),
			{{Snd_dac1_frame_size, SIZE_128}, {Snd_dac2_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {"", {-1} }}},

    {"interwave", N_("STB Soundrage 32"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, DMA},
			 {Snd_dma1_size, SIZE_128}, {Snd_dma2, DMA}, {Snd_dma2_size, SIZE_128},
			 {Snd_joystick_dac, JOYSTICK_DAC}, {Snd_midi, SND_ENABLE}, {Snd_pcm_voices, CHANNELS},
			 {Snd_effect, SND_ENABLE}, {"", {-1} }}},

    {"interwave-stb", N_("STB Ultrasound 32-Pro"),
			{{Snd_port, SND_PORTS_ALL}, {Snd_irq, IRQS_ALL}, {Snd_dma1, DMA},
			 {Snd_dma1_size, SIZE_128}, {Snd_dma2, DMA}, {Snd_dma2_size, SIZE_128},
			 {Snd_joystick_dac, JOYSTICK_DAC}, {Snd_midi, SND_ENABLE}, {Snd_pcm_voices, CHANNELS},
			 {Snd_effect, SND_ENABLE}, {Snd_port_tc, {0x350,0x360,0x370,0x380}}, {"", {-1} }}},

    {"ymfpci", N_("Toshiba Satellite Pro 4200"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"opl3sa2", N_("Toshiba Tecra 8000"),
			{{Snd_port, {0x370}}, {Snd_sb_port, {0x220, 0x240}}, {Snd_wss_port, WSS_IO},
			 {Snd_midi_port, MPU401_IO}, {Snd_fm_port, FM_PORTS}, {Snd_irq, IRQS_ALL},
			 {Snd_dma1, LOW_DMA}, {Snd_dma1_size, SIZE_64}, {Snd_dma2, LOW_DMA},
			 {Snd_dma2_size, SIZE_64}, {"", {-1} }}},

    {"trident", N_("Trident 4dwave DX/NX"),
			{{Snd_dac_frame_size, SIZE_64}, {Snd_adc_frame_size, SIZE_64}, {Snd_pcm_channels, CHANNELS},
			 {Snd_wavetable_size, GESBUF}, {"", {-1} }}},

    {"sonicvibes", N_("Turtle BeachDaytona"),
			{{Snd_dmaa_size, SIZE_128}, {Snd_dmac_size, SIZE_128}, {Snd_reverb, SND_ENABLE},
			 {Snd_mge, SND_ENABLE}, {"", {-1} }}},

    {"cs4236", N_("Turtle BeachMalibu"),
			{{Snd_irq, IRQS_ALL}, {Snd_mpu_irq, IRQS_ALL}, {Snd_dma1, LOW_DMA},
			 {Snd_dma1_size, SIZE_64}, {Snd_dma2, LOW_DMA}, {Snd_dma2_size, SIZE_64},
			 {Snd_isapnp, SND_ENABLE}, {"", {-1} }}},

    {"wavefront", N_("Turtle BeachTropez"),{ {"",{-1} } }},

    {"wavefront", N_("Turtle BeachTropez+"),{ {"",{-1} } }},

    {"via686a", N_("VIA via82c686"),
			{{Snd_mpu_port, MPU_PORTS}, {Snd_joystick, SND_ENABLE}, {Snd_pbk_frame_size, SIZE_128},
			 {Snd_cap_frame_size, SIZE_128}, {"", {-1} }}},

    {"opl3sa2", N_("Yamaha OPL3-SA2/OPL3-SA3/YMF701/711/715/718/719"),
			{{Snd_port, {0x370}}, {Snd_sb_port, {0x220, 0x240}}, {Snd_wss_port, WSS_IO},
			 {Snd_midi_port, MPU401_IO}, {Snd_fm_port, FM_PORTS}, {Snd_irq, IRQS_ALL},
			 {Snd_dma1, LOW_DMA}, {Snd_dma1_size, SIZE_64}, {Snd_dma2, LOW_DMA},
			 {Snd_dma2_size, SIZE_64}, {"", {-1} }}},

    {"ymfpci", N_("Yamaha Waveforce 192XG/Digital"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"ymfpci", N_("Yamaha YMF724/YMF740/YMF744/YMF754"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_ac97_frame_size, SIZE_128},
			 {Snd_mpu_port, MPU_PORTS}, {Snd_mpu_irq, IRQS_ALL}, {Snd_fm_port, FM_PORTS}, {"", {-1} }}},

    {"cmipci", N_("Zoltrix Nightingale"),
			{{Snd_dac_frame_size, SIZE_128}, {Snd_adc_frame_size, SIZE_128}, {Snd_enable_midi, SND_ENABLE},
			 {Snd_enable_fm, SND_ENABLE}, {Snd_mpu_port, MPU_PORTS}, {Snd_fm_port, FM_PORTS}, {"", {-1}}}}};



QArray < struct SoundCardsTable > ACardsArrayShow ( sizeof( ACardsArray )/sizeof( struct SoundCardsTable ) ) ;

int ACardsArrayShow_size;

/***************************************************************************/

void SoundAlsa::InitializeValues(){
/*
 //VarsPubNull();
 int inum = 0;

 int cardsArray_size = sizeof( cardsArray )/sizeof( struct SoundCardsTable );

 for( int i = 0; i < cardsArray_size; i++ ){
		//if (cardsArray[i].type != UNSPEC){
			cardsArrayShow[inum] = cardsArray[i];			
      inum++;
    //}
  }
 cardsArrayShow_size = inum;
 Sort( cardsArrayShow, cardsArrayShow_size );	
*/
}

/***************************************************************************/

void SoundAlsa::AListBoxInit(){
/*
  QStringList list;

  for( int i = 0; i < cardsArrayShow_size; i++ ){
    if (cardsArrayShow[i].type != UNSPEC) {
    list.append( QString( cardsArrayShow[i].descr ) );
    }
  }

	for ( QStringList::Iterator it = list.begin(); it != list.end(); it++ )						
  listType->insertItem( *it  );*/
}
/***************************************************************************/

SoundAlsa::SoundAlsa(QWidget *parent, const char *name ) : QWidget(parent,name)
{

//   InitializeValues();
//It' s neccessary to change the path of sndconfig	 	

#ifdef SOUND_BY_SNDCONFIG
   QString s_sndconfig = "/usr/sbin/sndconfig";
   #ifdef GBLI_DEVEL
      s_sndconfig = "sndconfig"; 	
   #endif

	 s_sndconfig = s_sndconfig + " --quiet";	
//	 printf("Sndconfig: %s \n", s_sndconfig.ascii() );	
   int snd_result = system( s_sndconfig );
//   printf ( "SND_RESULT: %d\n", snd_result );
#else
#ifdef noooooooooooooo
   #ifndef GBLI_DEVEL
      system( QString( "rm -f /etc/sysconfig/hwconf "+ QString( CONF_MODULES ) +" /etc/conf.modules /etc/sysconfig/soundcard" ).ascii() );
      system( "kudzu -q -s -t10" );
   #endif
#endif
#endif


	SoundCardID =0;

  QVBoxLayout *LLeft = new QVBoxLayout ( 3 );

	LVLeftBox = new QVBoxLayout ( 3 );
	LVRightBox = new QVBoxLayout ( 3 );

	LVLeftLbl = new QVBoxLayout ( 3 );
	LVRightLbl = new QVBoxLayout ( 3 );

	LHField = new QHBoxLayout ( 3 );

  QPushButton *testButton;

	testButton = new QPushButton ( tr("TEST CONF"), this );

//  QHBoxLayout *LRight = new QHBoxLayout ( 3 );
  QVBoxLayout *AMain = new QVBoxLayout( this, 20  );
	
//	QGridLayout *ParamGrid = new QGridLayout( this, 7, 2, 2, 5);

//	GUIStyle		GStyles;


	QComboBox *AListBox = new QComboBox ( FALSE, this, "read-only combo" );
	
//	AListBox->setMinimumWidth( AListBox->sizeHint().width() + 10 );

//	InitializeValues();

	int cardsArray_size = sizeof( ACardsArray )/sizeof( struct SoundCardsTable );

	for( int i = 0; i < cardsArray_size; i++ )
		AListBox->insertItem( QString( ACardsArray[i].descr ));

	

	for ( int i = 0; i<6; i++ )
	{
		ParamBoxes[i] = new QComboBox ( FALSE, this, "read-only combo" );
		ParamLabels[i] = new QLabel ( tr("test"), this );
		LVLeftBox->addWidget ( ParamBoxes[i] );
		LVLeftBox->addSpacing ( DEFAULT_SPACING );
		LVLeftLbl->addWidget ( ParamLabels[i] );
		LVLeftLbl->addSpacing ( DEFAULT_SPACING );
	};

	for ( int i = 6; i<12; i++ )
	{
		ParamBoxes[i] = new QComboBox ( FALSE, this, "read-only combo" );
		ParamLabels[i] = new QLabel ( tr("test"), this );
		LVRightBox->addWidget ( ParamBoxes[i] );
		LVRightBox->addSpacing ( DEFAULT_SPACING );
		LVRightLbl->addWidget ( ParamLabels[i] );
		LVRightLbl->addSpacing ( DEFAULT_SPACING );
	}

/*		LVLeftBox->addStretch ( MNOGO );
		LVLeftLbl->addStretch ( MNOGO );
		LVRightBox->addStretch ( MNOGO );
		LVRightLbl->addStretch ( MNOGO );*/

	for ( int i = 0; i<12; i++)
	{
   	ParamBoxes[i]->hide();
		ParamLabels[i]->hide();
		ParamLabels[i]->setAutoMask ( true );
		ParamBoxes[i]->setAutoMask ( true );
	}

	//LRight->addStretch();

//	LHLeft->addWidget ( ParamBoxes[0] );
//	LHRight->addWidget ( ParamBoxes[1] );


//  LHField->addWidget ( LParams ,1);
//	LHField->addStretch(MNOGO);

	LLeft->addWidget( AListBox ,1);

	AMain->addLayout ( LLeft, 1 );
	AMain->addLayout ( LHField, MNOGO );
	AMain->addStretch ( MNOGO );
	LHField->addStretch ( MNOGO );
	LHField->addLayout ( LVLeftLbl, 3 );
	LHField->addLayout ( LVLeftBox, 1 );
	LHField->addLayout ( LVRightLbl, 3 );
	LHField->addLayout ( LVRightBox, 1 );
	LHField->addStretch ( MNOGO );
	LHField->addWidget ( testButton );

	connect( testButton, SIGNAL( clicked () ), this, SLOT( testButtonClicked() ) );
	connect( AListBox, SIGNAL( highlighted ( int )), this, SLOT( ShowSelectedItem( int )) );
	
}
/***************************************************************************/
SoundAlsa::~SoundAlsa()
{
}
/***************************************************************************/
/**  */
void SoundAlsa::finish()
{
	if (SoundCardID != 0) WriteAlsaConfModules();
	
}

/***************************************************************************/

void SoundAlsa::ShowSelectedItem( int index )
{
	SoundCardID = index;

	QString				str;

	for ( int i = 0; i<14; i++)
	{
   	ParamBoxes[i]->hide();
		ParamBoxes[i]->clear();
    ParamLabels[i]->hide();
	}

	for ( unsigned int i=0; ACardsArray[index].Amodules[i].iValues[0]!=-1; i++ )
	{
		ParamLabels[i]->setText( tr(ACardsArray[index].Amodules[i].cName) );
		ParamBoxes[i]->show();
		ParamLabels[i]->show();

   	switch (ACardsArray[index].Amodules[i].iValues[0])
		{
		 	case ENABLE_BANK 	:	
					ParamBoxes[i]->insertItem( tr ("Enable") );
          ParamBoxes[i]->insertItem( tr ("Disable") );
					break;
			case PORT_BANK		:	
					for ( int j=1; ACardsArray[index].Amodules[i].iValues[j] != -1; j++ ){
     				str = "";
						str = "0x" + str.setNum ( ACardsArray[index].Amodules[i].iValues[j], 16 );
     				ParamBoxes[i]->insertItem( str );
					}
					break;
			default						:	
					for ( int j=1; ACardsArray[index].Amodules[i].iValues[j] != -1; j++ ){
     				str = "";
						str = str.setNum ( ACardsArray[index].Amodules[i].iValues[j], 10 );
     				ParamBoxes[i]->insertItem( str );
			   	}
					break;
		}

	}
//	WriteAlsaConfModules();
}

/**  */
void SoundAlsa::WriteAlsaConfModules()
{

	strMod = NEWROOT + CONF_MODULES;

//	strMod = CONF_MODULES;

//	printf ("My test!!! %s \n" , strMod.ascii() );
//	 printf("Sndconfig: %s \n", s_sndconfig.ascii() );	

	QFile modconf( strMod );
	if( modconf.open( IO_WriteOnly | IO_Append ) )
	{
  	QTextStream mc( &modconf );

		mc << "#----BEGIN: Generated by GBLI, do not edit. ----" << endl;
		mc << "alias char-major-116 snd" << endl;
		mc << "alias snd-card-0 snd-card-" << ACardsArray[SoundCardID].modulename << endl;
		mc << "alias char-major-14 soundcore" << endl;
		mc << "alias sound-slot-0 sound-card-0" << endl;
		mc << "alias sound-service-0-" << SND_MINOR_OSS_MIXER << " " << "snd-mixer-oss" << endl;
		mc << "alias sound-service-0-" << SND_MINOR_OSS_SEQUENCER << " " << "snd-seq-oss" << endl;
		mc << "alias sound-service-0-" << SND_MINOR_OSS_PCM << " " << "snd-pcm-oss" << endl;
		mc << "alias sound-service-0-" << SND_MINOR_OSS_MUSIC << " " << "snd-seq-oss" << endl;
		mc << "alias sound-service-0-" << SND_MINOR_OSS_PCM1 << " " << "snd-pcm-oss" << endl;
		mc << "options snd snd_major=116 snd_cards_limit=1 snd_device_mode=0666 snd_device_gid=0 snd_device_uid=0" << endl;

		mc << "options snd-card-" << ACardsArray[SoundCardID].modulename << " ";

		for ( unsigned int i=0; ACardsArray[SoundCardID].Amodules[i].iValues[0]!=-1; i++ )
		{

			if ( ACardsArray[SoundCardID].Amodules[i].iValues[0] == ENABLE_BANK )
			{
       	if ( ParamBoxes[i]->currentText() == "Enable" )
					mc << ACardsArray[SoundCardID].Amodules[i].cName << "=1 ";
				else
					mc << ACardsArray[SoundCardID].Amodules[i].cName << "=0 ";
			}
      else
				mc << ACardsArray[SoundCardID].Amodules[i].cName << "=" << ParamBoxes[i]->currentText() << " ";
		}

		mc << "snd_index=0 snd_id=CARD_0" << endl;
		mc << "#----END: Generated by GBLI, do not edit. ----" << endl;

		modconf.close();
	}

}
/**  */
void SoundAlsa::testButtonClicked(){
   finish();
}

QString SoundAlsa::ConvertToLable(QString lblstr)
{
/* 	switch (lblstr)
	{
   	case Snd_port 				: return "port # for chip"; break;
		case Snd_mpu_port     :	return "port # for MPU-401 chip"; break;
		case Snd_irq     			:	return "IRQ # for chip"; break;
		case Snd_irq     			:	return "IRQ # for chip"; break;
    case  Snd_mpu_irq 		: return "IRQ # for MPU-401 port"; break;
    case  Snd_dma8 				: return "8-bit DMA # for chip"; break;
    case  Snd_dma8_size 	: return "max 8-bit DMA size in kB"; break;
    case  Snd_dma16 			: return "16-bit DMA # for chip"; break
    case  Snd_dma16_size	: return "max 16-bit DMA size in kB"; break;
    case  Snd_dma1 				: return "first DMA #"; break;
    case  Snd_dma1_size 	: return "max first DMA size in kB"; break;
    case  Snd_dma2 				: return "second DMA #"; break;
    case  Snd_dma2_size 	: return "max first DMA size in kB"; break;
    case  Snd_fm_port 		: return "port # for OPL3 device"; break;
    case  Snd_dac_frame_size 		"snd_dac_frame_size"
    case  Snd_adc_frame_size 		"snd_adc_frame_size"
    case  Snd_enable_midi 			"snd_enable_midi"
    case  Snd_enable_fm 				"snd_enable_fm"
    case  Snd_isapnp 						"snd_isapnp"
    case  Snd_wssport 					"snd_wssport"
    case  Snd_wssirq 						"snd_wssirq"
    case  Snd_wssdma 						"snd_wssdma"
    case  Snd_wssdma_size 			"snd_wssdma_size"
    case  Snd_sbport 						"snd_sbport"
    case  Snd_sbirq 						"snd_sbirq"
    case  Snd_sbdma8 						"snd_sbdma8"
    case  Snd_sbdma8_size 			"snd_sbdma8_size"
    case  Snd_sbdma16 					"snd_sbdma16"
    case  Snd_sbdma16_size 			"snd_sbdma16_size"
    case  Snd_conp_frame_size		"snd_conp_frame_size"
    case  Snd_prop_frame_size 	"snd_prop_frame_size"
    case  Snd_conc_frame_size 	"snd_conc_frame_size"
    case  Snd_proc_frame_size 	"snd_proc_frame_size"
    case  Snd_pbk_frame_size 		"snd_pbk_frame_size"
    case  Snd_cap_frame_size 		"snd_cap_frame_size"
    case  Snd_mic_frame_size 		"snd_mic_frame_size"
    case  Snd_gesbuf						"snd_gesbuf"
    case  Snd_pcm_channels_p		"snd_pcm_channels_p"
    case  Snd_pcm_channels_c 		"snd_pcm_channels_c"
    case  Snd_ac97_frame_size 	"snd_ac97_frame_size"
    case  Snd_joystick_dac			"snd_joystick_dac"
    case  Snd_midi							"snd_midi"
    case  Snd_pcm_voices				"snd_pcm_voices"
    case  Snd_effect						"snd_effect"
    case  Snd_wavetable_size		"snd_wavetable_size"
    case  Snd_dmaa_size					"snd_dmaa_size"
    case  Snd_dmac_size					"snd_dmac_size"
    case  Snd_reverb						"snd_reverb"
    case 	Snd_mge								"snd_mge"
    case  Snd_awe_port					"snd_awe_port"
    case 	Snd_dac1_frame_size		"snd_dac1_frame_size"
    case 	Snd_dac2_frame_size		"snd_dac2_frame_size"
    case  Snd_midi_enable				"snd_midi_enable"
    case  Snd_pcm_channels			"snd_pcm_channels"
    case  Snd_voices						"snd_voices"
    case  Snd_gf1_port					"snd_gf1_port"
    case  Snd_irq1							"snd_irq1"
    case  Snd_sb_port						"snd_sb_port"
    case  Snd_wss_port					"snd_wss_port"
    case  Snd_midi_port					"snd_midi_port"
    case  Snd_port_tc						"snd_port_tc"
    case  Snd_joystick					"snd_joystick"
		default   					: return ""; breaj;
  }                                           */
}

           		