/***************************************************************************
                              addmodulespage.cpp
                             -------------------                                         
    begin                : Tue Jan 4 2000                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "addmodulespage.h"

#include "config.h"
#include "sysconfigfile.h"
#include "liconloader.h"
#include "log.h"
#include "fdisk.h"

#include <stdlib.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qtextview.h>
#include <qcheckbox.h>
#include <qobjectlist.h>

static QString SubModTable[][10] = {
	 {"Multimedia", "sound", "media", "drm", "agp", ""},
	 {"Network", "net", "tokenring", "wan", "isdn", "ipv4", "ipv6", "ipx", ""},
	 {"Disks", "cdrom", "scsi","ide", "paride", "md", "block", ""},
	 {"Pcmcia", "pcmcia", ""},
	 {"Devices", "parport", "pnp", "usb", "char", ""},
	 {"Input", "mouse", "input", "joystick", ""},
	 {"Misc", "crypto", "arch", "fs", "ieee1394", "irda", "watchdog", "i2c", "tel", ""},
	 {""}};

ModuleOptionsHelpDialog::ModuleOptionsHelpDialog(QWidget* parent, module *curModule, const char *name=0 )
	: QDialog( parent, name, true ){

	this->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );


	param	*curParam = curModule->parameters;
	Params = curParam;

	QVBoxLayout	*MainBox = new QVBoxLayout( this, 0, 6 );

	QHBoxLayout	*h1Box = new QHBoxLayout( 10 );
	QHBoxLayout	*hBox = new QHBoxLayout( 10 );

	paramCombo = new QComboBox( " ", this);
	paramCombo->setEditable(false);
	connect(paramCombo, SIGNAL( highlighted( int ) ), this, SLOT( paramChange( int ) ) );

	while (curParam)
	{
		QString	strLabel = curParam->name;
		strLabel = strLabel.replace( QRegExp("_"), " ");
		paramCombo->insertItem(strLabel);
		curParam = curParam->next;
	}
	
	QLabel* label = new QLabel( tr("Module paramters"), this );
	label->setAutoMask( true );
	
	hBox->addSpacing( 10 );
	hBox->addWidget(label);
	hBox->addWidget(paramCombo);
	hBox->addSpacing( 10 );
	

	paramDescr = new QMultiLineEdit( this );
	paramDescr->setMinimumWidth(400);
	paramDescr->setReadOnly( true );
	paramDescr->setText(Params->description);
	paramDescr->setWordWrap( QMultiLineEdit::WidgetWidth );	
	
	h1Box->addSpacing( 10 );
	h1Box->addWidget(paramDescr);
	h1Box->addSpacing( 10 );

	QPushButton	*btn = new QPushButton( tr ("OK"), this );
	btn->setMaximumWidth(100);

	MainBox->addSpacing( 10 );
	MainBox->addLayout(hBox);
	MainBox->addLayout(h1Box);
	MainBox->addWidget(btn);
	MainBox->addSpacing( 10 );

	connect( btn, SIGNAL( clicked() ), this, SLOT( accept() ) );		
}

void ModuleOptionsHelpDialog::paramChange(int index)
{
	int i=0;
	param	*curParam = Params;
	while (curParam)
	{
  	if (i==index)
	    {
				paramDescr->setText(curParam->description);
				break;	
			}
		curParam = curParam->next;
		i++;
	}
	
}

/*********************************************************************/

//ModuleOptionsDialog::ModuleOptionsDialog( QWidget* parent, imodule *mInstalled, module *curModule, QString &module, QString &strOptions, const char *name=0 )
ModuleOptionsDialog::ModuleOptionsDialog( QWidget* parent, imodule *mInstalled, module *curModule, QString &module, const char *name=0 )
	: QDialog( parent, name, true ){

		strModule = module;
		Module = curModule;
		bool Exists = FALSE;
		imodule	*curiModule = mInstalled;
		iparam	*curiParam = NULL;;

		while (curiModule)
		{
			if (curiModule->name == curModule->name)
			{
				Exists = TRUE;
        curiParam = curiModule->params;
				break;
			}
			curiModule = curiModule->next;
		}

		param	*curParam = curModule->parameters;

		pCount=0;
		while(curParam)
		{
      vals	*curValue = curParam->values;
      while (curValue)
			{
      	curValue = curValue->next;
				pCount++;
			}
    	curParam = curParam->next;
		}

		Params = (QComboBox**)::malloc(sizeof(QComboBox*)*pCount);

		this->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

    QHBoxLayout * lh = new QHBoxLayout( this, 0, 6 );

    QVBoxLayout * l = new QVBoxLayout( 2 );
		lh->addSpacing( 10 );
		lh->addLayout( l );
		lh->addSpacing( 10 );

		l->addSpacing( 12 );

		QHBoxLayout	*lhb;

		curParam = curModule->parameters;
		pCount = 0;

  	while (curParam)
		{
			lhb = new QHBoxLayout( 6 );		
			l->addLayout( lhb );
      QString	strLabel = curParam->name;
      strLabel = strLabel.replace( QRegExp("_"), " ");
			QLabel *label = new QLabel( strLabel, this );
			label->setMinimumWidth(75);
//			QLabel *label = new QLabel( tr( "Enter additional options for" ) +" \""+ strModule +"\":", this );
			lhb->addWidget( label );
			label->setAutoMask( true );

			vals	*curValue = curParam->values;
      while (curValue) {
//				Params[pCount] = new QComboBox( strOptions, this );
//				lastTag=pCount;
//				Params[pCount] = new QHckComboBox( this," ", &lastTag );
				Params[pCount] = new QComboBox(" ", this);
				Params[pCount]->setAutoMask(true);
				Params[pCount]->setMinimumWidth(30);
					if (curParam->values) {
		
    				if (Exists) {
//              printf("TEST PUT : %d, %s\n",pCount, curiParam->param.ascii());
							Params[pCount]->insertItem(curiParam->param);
							curiParam = curiParam->next;
            }

    	     	Params[pCount]->insertItem(curParam->def);
      	    	
						if (curParam->values->values != "")
						{
          		QString	tmp;
							int b=0,c=0;
							while (b>=0) {
    	        	b = curParam->values->values.find(",",c);
								tmp = curParam->values->values.mid(c,b-c);
								c = b+1;
          	    if (tmp!=curParam->def && tmp!=Params[pCount]->text(0)) Params[pCount]->insertItem(tmp);
            	}
						}
					}
				curValue = curValue->next;
//				if (curParam->hasDef)
				lhb->addWidget( Params[pCount], 1 );
				pCount++;
			}
			curParam = curParam->next;
		}

		l->addSpacing( 10 );

		lhb = new QHBoxLayout( 6 );		
		l->addLayout( lhb );
		btnOK = new QPushButton( tr( "OK" ), this );
		lhb->addWidget( btnOK );
    connect( btnOK, SIGNAL( clicked() ), this, SLOT( changeOptionsSlot() ) );

		btnCancel = new QPushButton( tr( "Cancel" ), this );
		lhb->addWidget( btnCancel );
    connect( btnCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );

		btnHelp = new QPushButton( tr("Help"), this);
		lhb->addWidget( btnHelp );
    connect(btnHelp, SIGNAL(clicked()), this, SLOT(showHelp()));

		l->addSpacing( 5 );
		lh->addSpacing( 6 );

		int tmpIt=0;//Added by dk
		for (tmpIt=0;tmpIt<pCount;tmpIt++)
		{

		}
}

void ModuleOptionsDialog::showHelp() {

	QMessageBox::warning(this,"Test",QString::number(lastTag));
	ModuleOptionsHelpDialog	*mod = new ModuleOptionsHelpDialog( this, Module, "module_options" );
  mod->setCaption( tr( "Help" ) );
//	mod->paramChange(lastTag);
	mod->exec();
	mod->disconnect( this );
}

void ModuleOptionsDialog::changeOptionsSlot(){
	accept();
}

/********************************************************************/

int AddModulesPage::registerTypes(QString type)
{
	types	*curType = modTypes;
	types	*tmpType;
	QString	tmp;
	int a=0;
	if (!curType) {
		modTypes = new types;
    curType = modTypes;
		curType->next = NULL;
    curType->name = type;
//		printf("%s\n",type.ascii());
    curType->index = a;
		return a;
	}

	while(curType) {
//    printf ("\n%p, %s\n", curType, type.ascii());
//		printf ("\n%d, %s\n", curType->index, curType->name.ascii());
		if(curType->name==type)
			return  curType->index;
    tmpType = curType;
		curType = curType->next;
		a++;
	}

	curType = new types;
	curType->name = type;
//	printf("%s\n",type.ascii());
	curType->index = a;
	curType->next = NULL;
	tmpType->next = curType;
	return a;

}

/********************************************************************/

void AddModulesPage::loadModulesData()
{
	QString	sReadMod, sParamMod, tmp;
	QFile	fModules( INSTALL_SOURCE +"/Install/live/boot/module-info" );
	int paramCount=1374;

  param *curParam = NULL;
	param	*nextParam = NULL;
	vals	*curValue = NULL;
//	vals	*nextValue;
	module	*curModule = NULL;
	module	*nextModule = NULL;
	
	fModules.open( IO_ReadOnly );
	QTextStream	t(&fModules);
	while ( ! t.eof() )
	{	
		sReadMod = t.readLine();
		sReadMod.truncate(sReadMod.find("#"));
/* 		Removing comments */

		if ((sReadMod.ascii()[0] != 9) && (sReadMod.ascii()[0] != 32) && (sReadMod.length()!=0))
		{
//			printf ( "'%s'\n", sReadMod.ascii() );

      //Fill in name of module
      if (curModule==NULL) {
      	curModule = new module;
        Modules = curModule;
      }
      else {
      	nextModule = new module;
        curModule->next = nextModule;
        curModule = nextModule;
      }
        curModule->name = "";
        curModule->type = 0;
				curModule->description = "";
        curModule->parameters = NULL;
        curModule->next = NULL;
        curModule->hasParam = FALSE;
				curModule->name = sReadMod;
				paramCount = 0;

		}
		else
		{
			sReadMod = sReadMod.stripWhiteSpace();
      if ( paramCount == 0 ) {
//				printf( "	Type        : %s\n",sReadMod.ascii() );

        //Fill in type of module

				QString	strType = sReadMod.simplifyWhiteSpace();
	      QString alias = "";
        int	as = strType.find(" ");

				if (as > 0) {
          alias = strType.mid(as+1,strType.length());
					strType.truncate(as);
        }

				curModule->type = registerTypes(strType);
				curModule->alias = alias;

				sReadMod = t.readLine();
				tmp = sReadMod.mid( sReadMod.find("\"")+1, sReadMod.find("\"", sReadMod.find("\"")+1)-sReadMod.find("\"")-1);
				
//        printf( "	Description : %s\n", tmp.ascii() );
				
        //Fill in description of module;

        curModule->description = tmp;
        paramCount ++;
	    }
			else {
        sReadMod = sReadMod.stripWhiteSpace();
        if (sReadMod.length() != 0) {
					int	dStart, dEnd, mStart, mEnd = 0;
					dStart = sReadMod.find("\""); dEnd = sReadMod.find("\"",dStart+1);
        	tmp = sReadMod.mid ( 0, dStart-1);
					tmp = tmp.stripWhiteSpace();
//          printf( "		Param%d      : %s\n",paramCount, tmp.ascii());

          //Fill in Parameters
          if (!curModule->parameters){
            curParam = new param;
            curParam->values = NULL;
            curParam->next = NULL;
            curModule->parameters = curParam;
		      }	
          else {
            nextParam = new param;
            curParam->next = nextParam;
            curParam = nextParam;
            curParam->values = NULL;
            curParam->next = NULL;
          }
					curModule->hasParam = TRUE;
          curParam->name = tmp;	

					tmp = sReadMod.mid ( dStart+1, dEnd-dStart-1);

					while ((dEnd<0) && (!t.eof())){
						sReadMod = t.readLine();
            sReadMod = sReadMod.stripWhiteSpace();
            dEnd=sReadMod.find("\"");
            tmp=tmp+" "+sReadMod.mid(0,dEnd);
          }
//					printf( "		Description : %s\n", tmp.ascii() );

          //Fill in parameters descriptioin
          curParam->description = tmp;


					mStart = sReadMod.find("\"", dEnd+1);
          int minS=sReadMod.find("min="), maxS=sReadMod.find("max=",minS);

          if ((minS>dEnd)&&(minS<mStart)&&(maxS>dEnd)&&(maxS<mStart)){
          	tmp = sReadMod.mid(minS+4,maxS-minS-4-1);
						tmp=tmp.stripWhiteSpace();
						int min = tmp.toInt();
						tmp = sReadMod.mid(maxS+4,mStart-maxS-4-1);
            tmp=tmp.stripWhiteSpace();
            int max = tmp.toInt();
//            printf( "		MinMax      : %d , %d\n", min, max);

            //Fill in parameters min and max
            curParam->hasMinMax = TRUE;
            curParam->min = min;
            curParam->max = max;
          }
          else curParam->hasMinMax = FALSE;

//          tmp = "";
//					int t=0;
					if (mStart<0) {

						curValue = new vals;
  	        curParam->values = curValue;
    	      curValue->next = NULL;
      	    curValue->mask = "";
        	  curValue->values = "";
					}

            while (mStart>0){
              QString	val;
              int	a,b;
//							t++;
//			        if (t>50) { printf("ERROR HERE");break;}
              mEnd = sReadMod.find("\"", mStart+1);
							tmp=sReadMod.mid(mStart+1, mEnd-mStart-1);
              a = tmp.find("{");

              if (curParam->values==NULL) {
              	curValue = new vals;
                curParam->values = curValue;
                curValue->next = NULL;
              }
              else {
              	vals *nextValue = new vals;
								curValue->next = nextValue;
                curValue = nextValue;
                curValue->next = NULL;
              }
							if (a>0){
              	b = tmp.find("}",a+1);
				        val = tmp.mid(a+1, b-a-1);
                tmp.truncate(a);
//								printf ( "		Mask        : %s\n", tmp.ascii() );
//  	            printf("			Values : %s\n", val.ascii());
                curValue->mask = tmp;
                curValue->values = val;
              }
//              else printf ( "		Mask        : %s\n", tmp.ascii() );
              else {
	              curValue->mask = tmp;
  	            curValue->values = "";
              }
              mStart=sReadMod.find("\"",mEnd+1);
            }

          int	defS=sReadMod.find("d=",mEnd+1);
					if (defS>mEnd){
          	tmp=sReadMod.mid(defS+2, sReadMod.length());
//						printf("		Default     : %s\n", tmp.ascii());
            curParam->hasDef = TRUE;
            curParam->def = tmp;
          }
          else
          	curParam->hasDef = FALSE;
//          printf("\n");
					paramCount ++;
        }
			}
		}
	}
	fModules.close();
	
}

AddModulesPage::AddModulesPage( QWidget * parent, const char * name )
    : GBLIPage( parent, name ), initialized( false )
{
		InstModules=NULL;
    modTypes = NULL;
		loadModulesData();
    QHBoxLayout * lh = new QHBoxLayout( this, 0, 6 );

//    lh->addStretch( 1 );
    lh->addSpacing( 3 );

    QVBoxLayout * l = new QVBoxLayout( 0 );
		lh->addLayout( l, 500 );

    QLabel* label = new QLabel( tr("Module types:"), this );
//		label->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );
		label->setAutoMask( true );
    listSubType = new QComboBox( " ", this );
    listSubType->setEditable(false);
    listType = new QComboBox( " ", this );
		listType->setEditable(false);
		connect( listType, SIGNAL( highlighted( int ) ), this, SLOT( typeChanged( int ) ) );
		connect( listSubType, SIGNAL( highlighted( int ) ), this, SLOT( typeSubChanged( int ) ) );

    QHBoxLayout * lhb = new QHBoxLayout( 6 );
		l->addStretch( 1 );
		l->addSpacing( 2 );
		l->addLayout( lhb, 1 );
    lhb->addWidget( label );
    lhb->addWidget( listSubType, 0, AlignLeft );
    lhb->addWidget( listType, 0, AlignLeft );
    lhb->addStretch( 5 );

		l->addSpacing( 4 );

    listModel = new QListView( this );
		listModel->setShowSortIndicator ( true );
		listModel->setHScrollBarMode( QScrollView::AlwaysOff );
		int col = listModel->addColumn( tr( "Module" ) );
    listModel->setColumnWidthMode( col, QListView::Maximum );
		col = listModel->addColumn( tr( "Driver name" ) );
    listModel->setColumnWidthMode( col, QListView::Maximum );
		listModel->setMinimumHeight( 40 );
    l->addWidget( listModel, 5 );

		chkProbe = new QCheckBox(tr("&Probe module"), this);
		btnAdd = new QPushButton( tr( "&Install module" ), this );
    chkProbe->setAutoMask( true );
    chkProbe->setChecked( true );
    lhb = new QHBoxLayout( 0 );
		l->addLayout( lhb, 0 );
    lhb->addWidget(chkProbe, 10);
    lhb->addWidget( btnAdd, 10 );
		connect( btnAdd, SIGNAL( clicked() ), this, SLOT( installModule() ) );

		l->addSpacing( 2 );
		l->addStretch( 1 );

    label = new QLabel( tr("Installed modules:"), this );
		label->setAutoMask( true );
    l->addWidget( label );
		label->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );
    listInstalled = new QListView( this );
		listInstalled->setShowSortIndicator ( true );
		listInstalled->setHScrollBarMode( QScrollView::AlwaysOff );
		col = listInstalled->addColumn( tr( "Module" ) );
    listInstalled->setColumnWidthMode( col, QListView::Maximum );
		col = listInstalled->addColumn( tr( "Driver name" ) );
    listInstalled->setColumnWidthMode( col, QListView::Maximum );
		listInstalled->setMinimumHeight( 40 );
		l->addWidget( listInstalled, 5 );
		listInstalled->setSorting( -1 );
		connect( listInstalled, SIGNAL( selectionChanged() ), this, SLOT( selectionChanged() ) );

		btnRemove = new QPushButton( tr( "&Remove module" ), this );
		btnEdit = new QPushButton( tr( "&Options" ), this );
    lhb = new QHBoxLayout( 6 );
		l->addLayout( lhb, 0 );
    lhb->addWidget( btnRemove,5 );
    lhb->addWidget( btnEdit, 5 );
		connect( btnRemove, SIGNAL( clicked() ), this, SLOT( removeModule() ) );
		connect( btnEdit, SIGNAL( clicked() ), this, SLOT( callEditOptions() ) );
		btnEdit->setEnabled( false );

		l->addStretch( 1 );

//    lh->addStretch( 1 );
    lh->addSpacing( 3 );
}

void AddModulesPage::showEvent( QShowEvent * ){
	if( !initialized ){
		QString str( "/boot/module-info" );
		loadData( str );
		initialized = true;
	}
}

void AddModulesPage::initSubTypes(){

	int	i=0;

	SubModTable[0][0] = tr("Multimedia");
	SubModTable[1][0] = tr("Network");
	SubModTable[2][0] = tr("Disks");
	SubModTable[3][0] = tr("Pcmcia");
	SubModTable[4][0] = tr("Devices");
	SubModTable[5][0] = tr("Input");
	SubModTable[6][0] = tr("Miscellaneous");

	while (SubModTable[i][0]!="")
	{
		listSubType->insertItem (SubModTable[i][0]);  	
		i++;
	};

	typeSubChanged(0);
}


bool AddModulesPage::loadData( QString &path ){

	  qApp->setOverrideCursor( Qt::waitCursor );
		// autoprobing to get modules loaded ;)
		Fdisk::availableDisks();

		initSubTypes();

		typeChanged( 0 );

	if( QFile( "/var/lib/pcmcia/stab" ).exists() ){
		if( !system( "grep \"Nokia C110\" /var/lib/pcmcia/stab >/dev/null 2>&1" ) ){
			system( "insmod -f nokia_c110" );
		}
	}		

	QFile inf( "/proc/modules" );
	if( inf.open( IO_ReadOnly ) ){
  	QTextStream ints( &inf );
		QString temp;
		QListViewItem *item;
		bool found = false;
		while( (temp = ints.readLine()).length() ){
			temp = temp.left( temp.find( " " ) );
//      printf("TESTING!!! : %s",temp.ascii());
				module	*AllModules = Modules;
				while (AllModules) {
          if ( temp == AllModules->name ) {
//						printf(" == %s", AllModules->name.ascii());
          	imodule	*newModule = getParameters(AllModules);	
	          regInstalledModules(FALSE, AllModules, newModule);
						item = new QListViewItem( listInstalled, AllModules->name,
							AllModules->description );
						found = true;
            break;
					}
        	AllModules = AllModules->next;
        }
//      printf("\n");
		}
		btnEdit->setEnabled( found );
		inf.close();
	}

	selectionChanged();

	qApp->restoreOverrideCursor();	

	return true;
}

void AddModulesPage::selectionChanged(){
        QListViewItem* item = listInstalled->selectedItem();
        if( !item ) {
                btnEdit->setEnabled( false );
                btnRemove->setEnabled( false );
                return;
        }
        btnRemove->setEnabled( true );
        QString module( item->text(0) );
        imodule *exiModule = InstModules;
        while (exiModule)
        {
                if (exiModule->name == module)
                        if (!exiModule->params) {
                                btnEdit->setEnabled( false );
                                break;
                        }
                        else {
                                btnEdit->setEnabled( true );
                                break;
                        }
        exiModule = exiModule->next;
        }
}

void AddModulesPage::typeSubChanged( int a_i ){
	QString	subType = listSubType->text(a_i);

	int	i=0,j=1;
	
	listType->clear();

	while(SubModTable[i][0]!="")
	{
    if(SubModTable[i][0] == subType)
    	while(SubModTable[i][j]!="")
			{
        listType->insertItem(SubModTable[i][j]);
      	j++;
			}			
  	i++;
	}
	typeChanged(0);
}

void AddModulesPage::typeChanged( int a_i ){
	
	QListViewItem *item;
  module	*curModule = Modules;
	int iType = 0;

	QString	strType = listType->text(a_i);
	types	*tmpTypes = modTypes;
	while (tmpTypes)
	{
    if (tmpTypes->name == strType) { iType = tmpTypes->index; break; }
  	tmpTypes = tmpTypes->next;
	}

	listModel->setSorting( -1 );
	listModel->clear();

//	printf ("\n %p , %d, %s\n",curModule, curModule->type, curModule->name.ascii());

	while (curModule)
	{
  	if (curModule->type == iType)
			item = new QListViewItem(listModel, curModule->name, curModule->description);
		curModule = curModule->next;
	}	

	listModel->setSorting( 1 );
	listModel->sort();

}

void AddModulesPage::installModule(){
	QListViewItem* item = listModel->selectedItem();
	if( !item ) return;
	QString module( item->text(0) );

	QListViewItem* newitem = listInstalled->firstChild();
	if( newitem )
   	do{
   		if( newitem->text( 0 ) == module ) return;
   	}while( (newitem = newitem->nextSibling() ) );

	if( editOptions( module, !chkProbe->isChecked() ) == QDialog::Rejected ) return;

#ifndef GBLI_DEVEL
	if (chkProbe->isChecked()) {
	  qApp->setOverrideCursor( Qt::waitCursor );

		QString str = " " + getParamString( module );
		QString call = "modprobe -qs "+ module;
		if( str ) call += str;
		int result = system( call.ascii() );
		Log::logMain( 33, "AddModules - modprobe result: %d\n", result );
		qApp->restoreOverrideCursor();	
		if( result ){
			QMessageBox::information( this, QObject::tr( "Error!" ),
  	    QObject::tr( "Failed to load module." ),
			  QMessageBox::Ok );
			remove_iModule( module );
	//if( module != "i82365" ) // test
			return;
		}
	}
#endif

	item = new QListViewItem( listInstalled,
		item->text( 0 ), item->text( 1 ) );

//	item->setSelected( true );
	selectionChanged();

	listInstalled->sort();
}

void AddModulesPage::remove_iModule( QString &rModule ){

	imodule	*curiModule = InstModules;
	imodule	*prev = NULL;
	while (curiModule)
	{
		if (curiModule->name == rModule)
		{
    	if (!prev)
				InstModules = curiModule->next;
			else
				prev->next = curiModule->next;
			iparam	*curParam = curiModule->params;
      iparam	*tmp;
			while (curParam)
			{
      	tmp = curParam;
				curParam = curParam->next;
				delete tmp;
			}
			delete curiModule;
			break;
			
		}
		prev = curiModule;
		curiModule = curiModule->next;
	}

}


void AddModulesPage::removeModule(){
	QListViewItem* item = listInstalled->selectedItem();
	if( !item ) return;
//	QListViewItem* item_above = item->itemAbove();
	QString module( item->text(0) );

#ifndef GBLI_DEVEL
	int result = system( "rmmod -s "+ module );
	if( result ){
		QMessageBox::information( this, QObject::tr( "Error!" ),
      QObject::tr( "Failed to unload module. It is probably in use." ),
		  QMessageBox::Ok );
		return;
	}
#endif

	listInstalled->takeItem( item );

	remove_iModule( module );

	selectionChanged();	
}

void AddModulesPage::callEditOptions(){
	QListViewItem* item = listInstalled->selectedItem();
	if( !item ) return;
	QString module( item->text(0) );
	imodule	*exiModule = InstModules;
	while (exiModule)
	{

		if (exiModule->name == module)
			if (!exiModule->params)
				return;
			else
				break;
     	exiModule = exiModule->next;
	}
	
	editOptions( module, !chkProbe->isChecked());
}

int AddModulesPage::editOptions( QString &sModule, bool no_probing ){

	imodule	*oldModule = NULL;
	imodule	*newModule = NULL;
	imodule	*inst_test = NULL;
	module	*curModule = Modules;
	if (!curModule) return QDialog::Rejected;
	else while(curModule)
	{
		if (curModule->name == sModule) break;
		curModule	= curModule->next;
	}

	int dlg_res;
	ModuleOptionsDialog	*mod = NULL;

	if (!curModule->hasParam) {

  	newModule = getParameters(curModule);
	
	if (!installedModule(curModule))
			newModule->name = curModule->name;
			newModule->alias = curModule->alias;
			newModule->type = curModule->type;
			
	//what about {} ? /
		dlg_res = QDialog::Accepted;

	}
	else {


		mod = new ModuleOptionsDialog( this, InstModules, curModule, sModule, "module_options" );
		connect( mod, SIGNAL( changeOptions( QString &, QString & ) ), this, SLOT( changeOptions( QString &, QString & ) ) );
		mod->setCaption( tr( "Options" ) );
		mod->exec();
		mod->disconnect( this );

		dlg_res = mod->result();

		if (dlg_res) {

//			printf("testing dlg_res : TRUE\n");
			newModule = getParameters(curModule);

			if (installedModule(curModule)) {
			    oldModule = copyModules(sModule);			
    			}else
			    regInstalledModules(FALSE, curModule, newModule, mod);
//CPQARRAY BUG FIX!

/*			if (!installedModule(curModule)) {
				regInstalledModules(FALSE, curModule, newModule, mod);
      }
			else {
				oldModule = copyModules(sModule);
  	  	regInstalledModules(TRUE, curModule, newModule, mod);
    	}*/
		}
  }

			

#ifndef GBLI_DEVEL
	if( !no_probing && dlg_res && installedModule(curModule)){
	  qApp->setOverrideCursor( Qt::waitCursor );
		QListViewItem* newitem = listInstalled->firstChild();
 		int result = 0;

		if( newitem )
	   	do{
   			if( newitem->text( 0 ) == sModule ){
					result = system( "rmmod -s "+ sModule );

        	if( result ){

        		QMessageBox::information( this, QObject::tr( "Error!" ),
                QObject::tr( "Failed to unload module. It is probably in use." ),
        		  QMessageBox::Ok );
                regInstalledModules(TRUE, curModule, oldModule, mod);
//        		changeOptions( sModule, *opt );
        	}
					break;
				}
	   	}while( (newitem = newitem->nextSibling() ) );
		if( !result ){


//			printf("Mew Module exist\n");

		QString	newParams;
		if (newModule!=NULL) 
		    newParams = getParamString(newModule->name);
		else
		    printf("NB! newModule==NULL");
	
		result = system( "modprobe -s "+ sModule +" "+ newParams );
  		if( result ){

			inst_test = installedModule(curModule);

			if (inst_test->installed) {
//				printf("Module is installed\n");
  			QMessageBox::information( this, QObject::tr( "Error!" ),
      		QObject::tr( "Failed to set new options. Re-check your options." ),
  		  	QMessageBox::Ok );
		regInstalledModules(TRUE, curModule, oldModule, mod);
				QString oldParams = getParamString(oldModule->name);
//	  		changeOptions( sModule, *opt );
  			result = system( "modprobe -s "+ sModule +" "+ oldParams );
			}

			}
  	}
		qApp->restoreOverrideCursor();
	}
#endif

	return dlg_res;
}

imodule* AddModulesPage::copyModules( QString &modName) {
	imodule	*result = new imodule;
	imodule	*tmp = InstModules;

//	printf("TEST : %s\n",modName.ascii());

	while (tmp)
	{
  	if (tmp->name == modName)
		{
    	iparam	*fromParam = tmp->params;
			iparam	*toParam = new iparam;
			result->name = tmp->name;
//      printf("Test : %s\n",tmp->name.ascii());
			result->params = toParam;
			while (fromParam)
			{
        toParam->param = fromParam->param;
				if (fromParam->next)
        {
        	toParam->next = new iparam;
					toParam = toParam->next;
        }
        else
					toParam->next = NULL;
      	fromParam = fromParam->next;
			}			
			break;

		}
		tmp = tmp->next;
	}
	return result;
}

//QString AddModulesPage::getParamString(imodule *oldModule) {
QString AddModulesPage::getParamString(QString	&modName) {

//	imodule	*tmpModule = InstModules;
	imodule	*oldModule = NULL;
	imodule	*iModules = InstModules;
	QString	result = "";
	module *tmpModule = Modules;

	while (iModules)
	{
  	if (iModules->name == modName) {
			oldModule = iModules;
			break;
		}

		iModules = iModules->next;
	}

	if (!oldModule) return result;

	while (tmpModule)
	{
  	if (oldModule->name == tmpModule->name)
		{
//			printf("modName = %s\n", modName.ascii());
//      printf("oldModule->name = %s\n",oldModule->name.ascii());
//      printf("tmpModule->name = %s\n",tmpModule->name.ascii());
//			printf("Testinnnnnng5.7.4\n");

/*			if (tmpModule->parameters)
				printf("Parameters\n");
			else
				printf("No Parameters\n");*/

			iparam	*oldParam = oldModule->params;
      param	*orParam = tmpModule->parameters;
			while (orParam)
			{
//			printf("Testinnnnnng5.7.5\n");

				vals	*orValues = orParam->values;
				QString	tmp = "";				

				if (orValues) {
//			printf("Testinnnnnng5.7.6\n");

					if (oldParam->param.length() > 0) tmp = " "+orParam->name+"="+oldParam->param;
				  orValues = orValues->next;
          oldParam = oldParam->next;
        }
				while (orValues)
				{
//			printf("Testinnnnnng5.7.7\n");

					if ((tmp.length()>0) && (oldParam->param.length()>0)) tmp = tmp+","+oldParam->param;
        	orValues = orValues->next;
          oldParam = oldParam->next;
        }
      	orParam = orParam->next;
	      result += tmp;
      }
     	break;
		}
		tmpModule = tmpModule->next;
	}

	return result;
}

imodule *AddModulesPage::installedModule(module *Module) {

  imodule	*iModule = InstModules;
	imodule	*exiModule = NULL;

	while (iModule)
	{
		if (iModule->name == Module->name) {
			exiModule = iModule;
      break;
		}
		iModule = iModule->next;
	}
	
	return exiModule;
}

imodule *AddModulesPage::getParameters(module 	*Module) {

	imodule *iModule = InstModules;
	imodule	*prev = NULL;
	imodule	*exiModule = installedModule(Module);

	if (!exiModule)
	{
		if (iModule)
		{

			while(iModule)
			{
				prev = iModule;
				iModule = iModule->next;
			}			
			iModule = new imodule;
			iModule->installed = false;
			prev->next = iModule;
		}
		else
		{
	  		iModule = new imodule;
				InstModules = iModule;
		}
		iModule->next = NULL;
		iModule->params = NULL;
	}
	else iModule = exiModule;

//	regInstalledModules(Exist, Module, iModule, mod);

	return iModule;

}

QString AddModulesPage::getStrType( int a_type ){
	types	*tmpTypes = modTypes;

	while (tmpTypes) {
    if (tmpTypes->index == a_type)
			return tmpTypes->name;
  	tmpTypes = tmpTypes->next;
	}

	return "";
}

void AddModulesPage::regInstalledModules(bool Exist, module *from, imodule *to, ModuleOptionsDialog	*mod){

	to->name = from->name;
	to->alias = from->alias;
	to->type = from->type;
	to->installed = true;
	int i=0;
	iparam	*curiParam = to->params;
	param	*curParam = from->parameters;
	while (curParam){
		vals	*curValue = curParam->values;
		while (curValue){
    	if (!curiParam)	{
				curiParam = new iparam;
				to->params = curiParam;
				curiParam->next = NULL;
			}
			else	{
        if (!Exist) {
	      	curiParam->next = new iparam;
					curiParam = curiParam->next;
					curiParam->next = NULL;
        }
				else if (i!=0) curiParam = curiParam->next;
			}

      if (mod)
				curiParam->param = mod->Params[i++]->currentText();
      else
        curiParam->param = "";

			if (curValue->next)
				curiParam->subParam = TRUE;
			else
				curiParam->subParam = FALSE;
			curValue = curValue->next;
		}
	curParam = curParam->next;
	}
}


void AddModulesPage::changeOptions( QString &module, QString &new_options ){
	DOptions.replace( module, new QString( new_options ) );
}

void AddModulesPage::finish(){

	QListViewItem* item = listInstalled->firstChild();
	if( !item ) return;
	QFile outf( NEWROOT + CONF_MODULES );

	if( outf.open( IO_WriteOnly | IO_Append ) ){
    QTextStream outt( &outf );
		outt << "alias parport_lowlevel parport_pc" << endl;
		outt << "pre-install pcmcia_core /etc/rc.d/init.d/pcmcia start" << endl;

		bool hasPCMCIA = false;

		imodule	*fModules = InstModules;

		QDict<int> DAliases;
		
		while (fModules) {
			QString	strParam = getParamString(fModules->name);
			QString	strAlias = fModules->alias;
			if (strAlias.find("[") > 0)
				strAlias.truncate(strAlias.find("["));
			QString	tmpAlias = strAlias;
		
			strParam=strParam.stripWhiteSpace();

			if (!fModules->alias.isEmpty())//{ //tmp {
		
			if (fModules->alias.find("sound")<0) //skip sound modules
			{
				if (((tmpAlias[tmpAlias.length()-1] == '0') || (tmpAlias == "scsi_hostadapter")) && (tmpAlias.find("char-major") < 0)){
	  			int	*alNumber = DAliases.find(strAlias);
					if ( alNumber )	{
		     		if (tmpAlias != "scsi_hostadapter")
							tmpAlias.truncate(tmpAlias.length()-1);
						tmpAlias=tmpAlias+QString::number(*alNumber+1);
						*alNumber= *alNumber+1;
						DAliases.insert(strAlias, alNumber);
					}
					else {
    				alNumber = new int;
						*alNumber = 0;
						DAliases.insert(strAlias, alNumber);
					}	
				}	
//		   	printf("alias %s %s\n",tmpAlias.ascii(), fModules->name.ascii());
        outt << "alias " << tmpAlias << " " << fModules->name << endl;
				if (!strParam.isEmpty()) {
//					printf("options %s %s\n",tmpAlias.ascii(), strParam.ascii());
          outt << "options " << fModules->name << " " << strParam << endl;
        }
			}
			else {
				if (!strParam.isEmpty()) {
//					printf("options %s %s\n",fModules->name.ascii(), strParam.ascii());
          outt << "options " << fModules->name << " " << strParam << endl;
				}
			}
//}//tmp
      if (getStrType(fModules->type) == "pcmcia") {
				if( fModules->name == "i82365" || fModules->name == "tcic" || fModules->name == "yenta_socket" ){
					hasPCMCIA = true;
			    SysConfigFile sysconfig( NEWROOT.ascii(), "/etc/sysconfig/pcmcia" );
			    sysconfig.load();
			    sysconfig.set( "PCMCIA", "yes", false );
			    sysconfig.set( "PCIC", fModules->name, false );
			    if( fModules->name == "yenta_socket" )
          sysconfig.set( "PCIC_OPTS", "", false );
					else
			    sysconfig.set( "PCIC_OPTS", "do_scan=0", false );
			    sysconfig.set( "CORE_OPTS", "", false );
			    sysconfig.save();
				}else if( !hasPCMCIA && fModules->name == "pcmcia_core" )
					hasPCMCIA = true;
      }
			fModules = fModules->next;
		}
		outf.close();


		if( QFile( NEWROOT +"/etc/init.d/pcmcia" ).exists() ){
			char cmd[256];
		  	sprintf( cmd, "chroot %s /sbin/chkconfig --%s pcmcia", NEWROOT.ascii(), (hasPCMCIA?"add":"del") );
				system( cmd );
			if (hasPCMCIA){
			  sprintf( cmd, "chroot %s /sbin/chkconfig --level 235 pcmcia on", NEWROOT.ascii());
				system( cmd );

			}

		}
	}else
		Log::logMain( 32, "Failed to write modules configuration!" );
}

/** Stores selected option for Module Options Help Dialog */
void ModuleOptionsDialog::valClicked(){
	QComboBox *tmpPoint;
	int i=0;//debug only
	//for (tmpPoint=(QComboBox*)(((QObjectList*)(this->children()))->first());tmpPoint;tmpPoint=(QComboBox*)(((QObjectList*)(this->children()))->next()))
  for (i=0;i<pCount;i++)
	{
	  printf("%d",i);//Here should be normal code, just determining which checkbox was selected.
    if (Params[i]->hasFocus())
		{
		 lastTag=i;
		 break;
		}
		i++;
	}
}


