/***************************************************************************
                          adslpage.cpp  -  description
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Oleg Golovachov (Demiurg)
    author email         : og@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "adslpage.h"
#include "configfile.h"
#include "sysconfigfile.h"
#include "liconloader.h"
#include "log.h"

#include <qlineedit.h>
#include <qvalidator.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsocket.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>

#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>

#include "linlinehelp.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#define NET_DEBUG
#ifdef NET_DEBUG
//	# define D(args...)	printf(##args)
	# define D(args...)	Log::logMain( 33, ##args )
#else
	# define D(args...)	do { } while (0)
#endif


ADSLPage::ADSLPage( Netinfo *net, QWidget * parent, const char * name )
    : QWidget( parent, name ), info( net )
{

		this->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

    bg = new QButtonGroup( 0 );
    QRadioButton *d , *f;

    f = new QRadioButton( tr( "No ADSL" ), this );
		f->setAutoMask(true);
    bg->insert( f, 0); // 0 index for No ADSL
    //d = new QRadioButton( tr( "ADSL configured using PPPoE" ), this ); - commented by dk
    d = new QRadioButton( tr( "ADSL present" ), this );
		d->setAutoMask(true);
    bg->insert( d ,1);// 1 - for ADSL present ,see setMode

    connect( bg, SIGNAL(clicked(int)), this, SLOT(setMode(int)) );

		devCombo = new QComboBox(0,this);
		for (int iAdapter = 0; iAdapter < ADAPTERS_MAX; iAdapter++)
		{
			QString strAdapter = "eth" + QString::number(iAdapter);
			if (info->getMode( strAdapter ) != -1)
				devCombo->insertItem(strAdapter.ascii());
		}
		if (devCombo->count() == 0)
			devCombo->insertItem("eth0");

		
		userEdit = new extLineEdit(this,0,0);   //remove limitations old 0,3
		passwdEdit = new extLineEdit(this,0,0); //old 0,3
		passwdEdit->setEchoMode(QLineEdit::Password);

		QVBoxLayout *loVert = new QVBoxLayout( this, 5 );
		QHBoxLayout *loHorz = new QHBoxLayout();
		loVert->addLayout( loHorz, 1 );
		loVert->addStretch( 1 );

		QLabel *l2 = new QLabel(tr("Device:"), this);
    l2->setAutoMask(true);
//		QLabel *l1 = new QLabel(tr("IP Address:"), this);
//    l1->setAutoMask(true);
//		QLabel *l3 = new QLabel(tr("Netmask:"), this);
//    l3->setAutoMask(true);
		QLabel *l4 = new QLabel(tr("Username:"), this);
    l4->setAutoMask(true);
		QLabel *l5 = new QLabel(tr("Password:"), this);
    l5->setAutoMask(true);
		QGridLayout *grid = new QGridLayout(4,4,5);
		grid->addMultiCellWidget(f,0,0,0,1);
		grid->addMultiCellWidget(d,1,1,0,1);
//		grid->addMultiCellWidget(r,2,2,0,1);
		grid->addWidget(l2,0,2,AlignLeft);
		grid->addWidget(devCombo,0,3,AlignLeft);
		grid->addWidget(l4,1,2,AlignLeft);
		grid->addWidget(userEdit,1,3,AlignLeft);//passwdEdit
		grid->addWidget(l5,2,2,AlignLeft);
		grid->addWidget(passwdEdit,2,3,AlignLeft);
//		grid->addWidget(l1,1,2,AlignLeft);
//		grid->addWidget(ipEdit,1,3,AlignLeft);
//		grid->addWidget(l3,2,2,AlignLeft);
//		grid->addWidget(netmaskEdit,2,3,AlignLeft);

//    grid->setColStretch(0,1);
    grid->setColStretch(1,1);
//    grid->setColStretch(2,1);
		grid->setColStretch(3,1);

		loHorz->addLayout(grid);

//    addInputWidget( ipEdit );
//    addInputWidget( netmaskEdit );

    connect( info, SIGNAL(newConfig()), this, SLOT(updateWidgets()) );

    setMode(0);

		updateWidgets();
}

ADSLPage::~ADSLPage()
{
}

//////////////////////////////////////////////////////////////////
// Do return/focus handling
//////////////////////////////////////////////////////////////////
void
ADSLPage::addInputWidget(QWidget *edit)
{
    connect( edit, SIGNAL(returnPressed()),
	     this, SLOT(updateInfo()) );
    edit->installEventFilter(this);
}

bool
ADSLPage::eventFilter(QObject *, QEvent *e)
{
    if (e->type() == QEvent::FocusIn || e->type() == QEvent::FocusOut)
    	updateInfo();
    return false;

}

//////////////////////////////////////////////////////////////////
// Debug function
//////////////////////////////////////////////////////////////////
#ifdef NET_DEBUG
#define DUMP_WIDGET(w)	\
	printf("%-15s: \"%s\"%s\n", \
		#w, w->text().ascii(), w->edited()? " (edited)" : "")
#define DUMP_ALL_WIDGETS	\
	do {	\
		DUMP_WIDGET(routerEdit); \
		DUMP_WIDGET(ns1Edit); \
		DUMP_WIDGET(ns2Edit); \
		DUMP_WIDGET(nameEdit); \
		DUMP_WIDGET(nisEdit); \
	} while (0)
#endif



//////////////////////////////////////////////////////////////////
// Update displayed values from probe data
//////////////////////////////////////////////////////////////////
void ADSLPage::updateWidgets()
{
//    D("ADSLPage::update widgets\n");
		D("ADSLPage::update widgets\n");
    if (!userEdit)
			return;
//    DUMP_ALL_WIDGETS;

//    if (!ipEdit->edited())
//        ipEdit->setAddress(info->getADSLAddress());

//    if (!netmaskEdit->edited())
//        netmaskEdit->setAddress(info->getADSLNetmask());
		
		if (info->getADSLCard())
			for (int i = 0; i < devCombo->count(); i++)
				if (!strcmp(info->getADSLCard(), devCombo->text(i).ascii())) devCombo->setCurrentItem(i);

		setMode(info->getADSLMode());
//#ifdef GBLI_DEVEL
//		printf("After setMode in updateWidgets!\n");
//#endif

		userEdit->setText(info->getADSLUser());
		passwdEdit->setText(info->getADSLPass());
}

//////////////////////////////////////////////////////////////////
// Update network config from user input
//////////////////////////////////////////////////////////////////
void ADSLPage::updateInfo()
{
    bool	changed = false;


//    if (netmaskEdit->edited()
//    	&& info->setADSLNetmask(netmaskEdit->text(), Netinfo::User))
//				changed = true;
//    if (ipEdit->edited() && info->setADSLAddress(ipEdit->text(), Netinfo::User))
//			changed = true;
    if (info->setADSLCard(devCombo->currentText(), Netinfo::User))
			changed = true;
		if (userEdit->edited() && info->setADSLUser(userEdit->text(), Netinfo::User))
			changed = true;
		if (passwdEdit->edited() && info->setADSLPass(passwdEdit->text(), Netinfo::User))
			changed = true;

    if( changed )
			info->probe( true );

		// lets try make it this way, not perfect, especially eth0 ;) (TODO)
//		system( ("ifconfig eth0 "+ ipEdit->text()).ascii() );
			
#ifdef NET_DEBUG_DUMP
    info->save("/tmp/netdebug");
#endif
}

void ADSLPage::setMode( int nm )
{
    bool e = ( nm != 0 );
		int iNm;
		QRadioButton *pRadioButton = NULL;

		devCombo->setEnabled( e );
		userEdit->setEnabled( e );
		passwdEdit->setEnabled( e );

		//Adding selection redraw

		if (nm==Netinfo::NoConfig) iNm = 0;
		else iNm = 1;
	
		if ((pRadioButton = ((QRadioButton*)bg->find(iNm))) !=NULL ){
				bg->setButton( iNm );
				pRadioButton->setFocus();
//		#ifdef GBLI_DEVEL
//				if (!pRadioButton->hasFocus())
//				perror("ADSL Focus not set!!!!");
//		#endif
				// it looks, it could be re-set by itself call :)
				// that can be needed ,if reading configuration
		}

//#ifdef GBLI_DEVEL
//		printf("ADSLPage::setMode %d\n",iNm);
//#endif
    info->setADSLMode( (Netinfo::Setup)nm );
}

