/***************************************************************************
                          changepartitiondialog.cpp
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "config.h"
#include "liconloader.h"
#include "changepartitiondialog.h"
#include "diskitem.h"
#include "fdisk.h"
#include "mountpoint.h"

#include <unistd.h>
#include <stdio.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qfont.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qsize.h>

/*  The change partition dialog is ....well.... used to change/edit a
  partition. It's a "private" auxiliary class of the partition view. */
ChangePartitionDialog::ChangePartitionDialog(QWidget* parent, const char *name)
    : QDialog( parent, name, TRUE )
{
		setFontPropagation( QWidget::AllChildren );
		setFont( QFont::defaultFont() );
//		setMinimumHeight( 5 );
//		setMinimumWidth( 5 );

		this->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

    QVBoxLayout* vbox = new QVBoxLayout( this, 6, 0 );
    QHBoxLayout* hbox;
    QLabel* label;

    // size settings

    groupSize = new QButtonGroup( tr("Partition Boundaries"), this );
		groupSize->setAutoMask( true );
		groupSize->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );
//    groupSize->setMinimumHeight( 5 );
    vbox->addWidget( groupSize );

    QVBoxLayout* gbvbox = new QVBoxLayout( groupSize, 10, 6 );//#### fixed rotten QLayout with QGroupBox

    hbox = new QHBoxLayout();
    QHBoxLayout *h1 = new QHBoxLayout();
    QHBoxLayout *h2 = new QHBoxLayout();

    h1->addSpacing( 10 );
    label = new QLabel( tr("Possible disk areas"), groupSize );
		label->setAutoMask( true );
    h1->addWidget( label );

    freeSpaces = new QComboBox( FALSE, groupSize );
    connect( freeSpaces, SIGNAL( activated( int ) ),
	     this, SLOT( freeSpaceComboActivated( int ) ) );
    h1->addWidget( freeSpaces );

#ifdef ADVANCED_SIZE
    freeSpaces->insertItem( "999999 - 999999" );
#else
    freeSpaces->insertItem( "999999" );
    QSize sf = freeSpaces->sizeHint();
    sf.setWidth( sf.width() /*/ 2 -5 */ );
		freeSpaces->setFixedSize( sf );
    label = new QLabel( " "+ tr("MB"), groupSize );
		label->setAutoMask( true );
/*    QFont fnt = label->font();
    fnt.setBold( TRUE );
    label->setFont(fnt);*/
    h1->addWidget( label );
#endif

#ifdef abcd
    gbvbox->addSpacing( 10 );

    hbox->addStretch( 10 );

    hbox = new QHBoxLayout;
    gbvbox->addLayout( hbox );
#endif

    //  hbox->addStretch();

#ifdef ADVANCED_SIZE
    label = new QLabel( tr("Start"), groupSize );
		label->setAutoMask( true );
    hbox->addWidget( label );
		hbox->addSpacing( 3 );
    startEdit = new QLineEdit( groupSize );
    startEdit->setValidator( new QIntValidator( groupSize ) );
    connect( startEdit, SIGNAL( textChanged(const QString&) ),
	     this, SLOT (lineEditChanged( const QString& ) ) );
    hbox->addWidget( startEdit );
    hbox->addSpacing( 12 );
    label = new QLabel( tr("End"), groupSize );
		label->setAutoMask( true );
    hbox->addWidget( label );
		hbox->addSpacing( 3 );
    endEdit = new QLineEdit( groupSize );
    endEdit->setValidator( new QIntValidator(groupSize ) );
    connect( endEdit, SIGNAL( textChanged(const QString&) ),
	     this, SLOT (lineEditChanged( const QString& ) ) );
    hbox->addWidget( endEdit );
    hbox->addSpacing( 12 );
    sizeInfo = new QLabel( "10000TB", groupSize );
		sizeInfo->setAutoMask( true );
    QFont f = sizeInfo->font();
    f.setBold( TRUE );
    sizeInfo->setFont(f);
		sizeInfo->setFixedSize( sizeInfo->sizeHint() );
    hbox->addWidget( sizeInfo );
#else

    h2->addSpacing( 10 );
    label = new QLabel( tr("Size") +" ", groupSize );
		label->setAutoMask( true );
    h2->addWidget( label );

    sizeEdit = new QLineEdit( "999999", groupSize );
    QSize sfh = sizeEdit->sizeHint();
    sfh.setWidth( sfh.width() / 2 -5  );
		sizeEdit->setFixedSize( sfh );
    sizeEdit->setAlignment( Qt::AlignRight );
    sizeEdit->setMaxLength( 6 );
    sizeEdit->setValidator( new QIntValidator( 1, 999999, sizeEdit ) );
    h2->addWidget( sizeEdit );
    connect( sizeEdit, SIGNAL( textChanged(const QString&) ),
	     this, SLOT (lineEditChanged( const QString& ) ) );
    label = new QLabel( " "+ tr("MB"), groupSize );
		label->setAutoMask( true );
    QFont f = label->font();
    f.setBold( TRUE );
    h2->addWidget( label );

    label = new QLabel( "    -    ", groupSize );
		label->setAutoMask( true );
    h2->addWidget( label );

    sizeInfo = new QLabel( "OK", groupSize );
		sizeInfo->setAutoMask( true );
    sizeInfo->setFont(f);
    int sh = (int)(sizeInfo->sizeHint().width());
		sizeInfo->setMinimumWidth( sh + 40 );
    h2->addWidget( sizeInfo );

    h2->addStretch();
#endif

    gbvbox->addSpacing( 15 );
    gbvbox->addLayout( h1 );
    gbvbox->addSpacing( 5 );
    gbvbox->addLayout( h2 );
    gbvbox->addSpacing( 15 );

    // other settings
    groupSystem = new QButtonGroup( tr("System type"), this );
		groupSystem->setAutoMask( true );
		groupSystem->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );
//    groupSystem->setMinimumHeight( 5 );
    vbox->addWidget( groupSystem );

//groupSystem->hide();

    QVBoxLayout* vbox1 = new QVBoxLayout( groupSystem, 10, 0 );

    QVBoxLayout* vbox2 = new QVBoxLayout( );
    QVBoxLayout* vbox3 = new QVBoxLayout( );

    QHBoxLayout* hbox1 = new QHBoxLayout( 6 );
    QHBoxLayout* hMain = new QHBoxLayout( 6 );


    vbox2->addSpacing( 15 );
    systemCombo = new QComboBox( FALSE, groupSystem );
    systemCombo->insertItem("DOS 16-bit FAT");
    connect( systemCombo, SIGNAL( activated( const QString& ) ),
	     this, SLOT( systemComboActivated( const QString& ) ) );
    vbox2->addWidget( systemCombo );

	  label = new QLabel( tr("Mount Point"), groupSystem );
		label->setAutoMask( true );
    hbox1->addWidget( label );

    mountPointEdit = new QComboBox( TRUE, groupSystem );
    mountPointEdit->insertItem( tr( "Unused" ) );
    mountPointEdit->insertItem( "/" );
    mountPointEdit->insertItem( "/usr" );
    mountPointEdit->insertItem( "/boot" );
    mountPointEdit->insertItem( "/home" );
    mountPointEdit->insertItem( "/var" );
    mountPointEdit->insertItem( "/usr/local" );
    mountPointEdit->insertItem( "/tmp" );
    mountPointEdit->insertItem( "/opt" );
    mountPointEdit->insertItem( tr("<other>") );
    connect( mountPointEdit, SIGNAL( textChanged( const QString& ) ),
	     this, SLOT( mountPointComboActivated( const QString& ) ) );
    hbox1->addWidget( mountPointEdit );
		
    bootableCheck = new QCheckBox( tr("Bootable")/* Partitions"*/, groupSystem );
		bootableCheck->setAutoMask( true );
//		hbox2->addSpacing( 10 );
    vbox2->addSpacing( 5 );
    vbox2->addWidget( bootableCheck );
    vbox2->addSpacing( 15 );

    vbox3->addSpacing( 15 );
    vbox3->addLayout( hbox1 );
		formatCheck = new QCheckBox( /*tr("Partitions To */tr("Format"), groupSystem );
		formatCheck->setAutoMask( true );
    vbox3->addSpacing( 5 );		
    vbox3->addWidget( formatCheck );
    vbox3->addSpacing( 15 );
		
    hMain->addSpacing( 15 );
    hMain->addLayout( vbox2 );
	  hMain->addSpacing( 15 );
    hMain->addLayout( vbox3 );
    hMain->addSpacing( 15 );


    // the buttons
//   vbox1->addSpacing( 15 );
   vbox1->addLayout( hMain );
//	 vbox1->addSpacing( 15 );


    QHBoxLayout *hbox3 = new QHBoxLayout( 7 );

    vbox->addSpacing( 5 );
    vbox->addLayout( hbox3 );

    hbox3->addStretch();

    ok = new QPushButton( tr("Ok"), this );
    connect( ok, SIGNAL( clicked() ), this, SLOT( userAgreed() ) );
    ok->setDefault( TRUE );
    hbox3->addWidget( ok );

    cancel = new QPushButton( tr("Cancel"), this );
    connect( cancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    hbox3->addWidget( cancel );

    hbox3->addSpacing( 7 );

		setFont( QFont::defaultFont() );
}

/*
  This function is supposed to be called after the caller of the
  dialog set all communication parameters. It will update the
  graphical user interface and inform the dialog about the current
  disk.
 */
void ChangePartitionDialog::doSettings(DiskItem* disk_)
{
    disk = disk_;
    origId = id;

    systemCombo->clear();
    systemCombo->insertItem( Fdisk::sysname(id) );
    if( id != 0x19 )
			systemCombo->insertItem( Fdisk::sysname(0x19));
    if( id != 0x83 )
			systemCombo->insertItem( Fdisk::sysname(0x83));
    if( id != 0x82 )
			systemCombo->insertItem( Fdisk::sysname(0x82));
    if( id != 0x5 && !disk->hasExtendedPartition() ) // only one extended partition possible
			systemCombo->insertItem( Fdisk::sysname(0x5));

    if( id != 0x0b )
			systemCombo->insertItem( Fdisk::sysname(0x0b) );
    if( id != 0x6 )
			systemCombo->insertItem( Fdisk::sysname(0x6) );

    bootableCheck->setChecked( bootable );

    if( id == 0x82 || id == 0x05 ){ // linux swap || extended
			bootableCheck->setChecked( FALSE );
			bootableCheck->setEnabled( FALSE );
			mountPointEdit->setCurrentItem( 0 );
			mountPointEdit->setEnabled( FALSE );
    }else{
			bootableCheck->setEnabled( TRUE );
			mountPointEdit->setEnabled( TRUE );
			QString m = MountPoints::mountPoint( name ).stripWhiteSpace();
//printf( "mpnt: \"%s\"\n", m.ascii() );
			if( !m.isEmpty() )
	  	  mountPointEdit->setEditText( m );
			else
	  	  mountPointEdit->setCurrentItem( 0 ); // indicate unused
    }

    curStart = start;
    curEnd = start+size-1;
    curSize = size;
    disk->chooseCurrentItem( freeSpaces, curStart, curEnd );
    if( isNewPartition ){
			freeSpaceComboActivated( 0 );
			willBeFormated = true;
	    updateFormatCheck();
    }else{
//printf( "1 curStart %d, curEnd %d, curSize %d \n", (int)curStart, (int)curEnd, (int)(curSize/2) );
#ifdef ADVANCED_SIZE
      startEdit->setText( QString::number( disk->cylinderOfSector( curStart ) ) );
      endEdit->setText( QString::number( disk->cylinderOfSector( curEnd ) ) );
#else
      sizeEdit->setText( QString::number( disk->megaByteOfSectors( curSize ) ) );
#endif
//      updateSizeInfo();
  	}
}

// update the state and availability of "format" chackbox according
// to mountpoint etc.
void ChangePartitionDialog::updateFormatCheck()
{
    bool mustBeFormated = Fdisk::isSwapType(id) | mountPoint() == "/" | mountPoint() == "/usr" | mountPoint() == "/boot"
      | curStart != start | curSize != size;

    if( mountPoint() == "/" || mountPoint() == "/usr" || mountPoint() == "/boot" || Fdisk::isSwapType(id) ){
			formatCheck->setEnabled( false );
			formatCheck->setChecked( true );
    }else if( Fdisk::isExtendedType(id) ){
			formatCheck->setEnabled( false );
			formatCheck->setChecked( false );
    } else if ( isNew || mustBeFormated ) {
			formatCheck->setEnabled( true );
			formatCheck->setChecked( true );
    } else {
			formatCheck->setEnabled( true );
			formatCheck->setChecked( willBeFormated );
    }
    willBeFormated = formatCheck->isChecked();
}

/*void ChangePartitionDialog::setupCurrent()
{
    int iMax = freeSpaces->count();
    signed long lStart, lEnd;
    for( int i = 0; i < iMax; i++ ){
#ifdef ADVANCED_SIZE
      getLimits( i, &lStart, &lEnd );
      lStart = disk->sectorOfCylinder( lStart );
      lEnd = disk->sectorOfCylinder( lEnd );
printf( "curEnd %d, lEnd %d\n", (int)curEnd, (int)lEnd );
printf( "curStart %d, curEnd %d, curSize %d \n", (int)curStart, (int)curEnd, (int)(curSize/2) );
      if( curStart >= lStart &&
          curEnd <= lEnd ){
#else
      if( maxLbl->getText().toInt() <= freeSpaces->text(i).toInt() )
#endif
        if( freeSpaces->currentItem() != i )
          freeSpaces->setCurrentItem( i );
        break;
      }
    }
}

void ChangePartitionDialog::getLimits( const int a_index, signed long *a_start, signed long *a_end )
{
    QString s = freeSpaces->text( a_index );
    int minus = s.find( '-' );
    *a_start = disk->sectorOfCylinder( (signed long)s.left( minus - 1 ).simplifyWhiteSpace().toULong() );
    *a_end = disk->sectorOfCylinder( (signed long)s.right( s.length() - minus - 2 ).simplifyWhiteSpace().toULong() );
} */

/*
  Recalculate the size of the partition and do the validation. If the
  partition is invalid, indicate this to the user and disable the "OK"
  button.
 */
bool ChangePartitionDialog::updateSizeInfo()
{
//printf( "updateSizeInfo\n" );
//printf( "3 curStart %d, curEnd %d, curSize %d \n", (int)curStart, (int)curEnd, (int)(curSize/2) );
  bool result = false;
#ifdef ADVANCED_SIZE
  curStart = disk->sectorOfCylinder( startEdit->text().toInt() );
  curEnd = disk->sectorOfCylinder( endEdit->text().toInt() );
#else
  curEnd = disk->sectorsOfMegaByte( sizeEdit->text().toInt() ) + curStart;
  curEnd = disk->sectorOfCylinder( disk->cylinderOfSector( curEnd ) );
#endif

//printf( "4 curStart %d, curEnd %d, curSize %d \n", (int)curStart, (int)curEnd, (int)(curSize/2) );

  if( number > disk->nrPrimary || curStart == 0 )
    curStart += disk->sectors; // DOS mode and some space for the boot manager

  if ( disk->cylinderOfSector( start ) == disk->cylinderOfSector( curStart ) )
    curStart = start; // fall back to the previous value

//printf( "5 curStart %d, curEnd %d, curSize %d \n", (int)curStart, (int)curEnd, (int)(curSize/2) );

  curEnd += disk->heads * disk->sectors - 1; // align at exact _end_ of a cylinder

//printf( "5.5 curStart %d, curEnd %d, curSize %d \n", (int)curStart, (int)curEnd, (int)(curSize/2) );

  if ( disk->cylinderOfSector( start + size - 1 ) == disk->cylinderOfSector( curEnd ) )
    curEnd = start + size - 1;

  curSize = curEnd - curStart + 1;

//printf( "6 curStart %d, curEnd %d, curSize %d \n", (int)curStart, (int)curEnd, (int)(curSize/2) );

  bool valid = (number > disk->nrPrimary) ?
    disk->verifyFreeSpacesLogical( curStart, curEnd, number )
    :
    disk->verifyFreeSpacesPrimary( curStart, curEnd, number );

  QString n;
  if( valid ) {
#ifdef ADVANCED_SIZE
    n.setNum( disk->megaByteOfSectors(curSize) );
    n += tr("MB");
#else
    n = tr( "OK" );
#endif
    result = true;
  }else{
    n = tr("invalid");
  }
  sizeInfo->setText( n );

  ok->setEnabled( valid );
  updateFormatCheck();
  return result;
}

// equals "finish" for the dialog, checks the data and stores it to variables
void ChangePartitionDialog::userAgreed()
{
  	bool accepting = true;
  	bootable = bootableCheck->isChecked();
  	isNew |= (start != curStart) | (size != curSize );
  	if( Fdisk::isExtendedType(id) ){
  	  willBeFormated = false;
  	}else{
  	  willBeFormated = /*isNew | */formatCheck->isChecked();
  	}
  	if( !isNew && willBeFormated && origId != 0x82 ){ //user is trying to destroy old data
			this->hide();
  		accepting = !QMessageBox::warning( this, tr("Warning"),
  			tr( "You are about to lose data! Are you sure you want to do this?" ),
  			tr("Yes"), tr("No"), "", 1 );
  	}
  	if( accepting ){
    	start = curStart;
    	size = curSize;
//printf( "**New partition: start=%d, size=%d\n", (signed int)start, (signed int)size );
			accept();
  	}else{
			QString mp = mountPoint();
			ASSERT( mp.length() );
			if( mp != "/" && mp != "/usr" && mp != "/boot" && !Fdisk::isSwapType(id) ){
  				formatCheck->setChecked( false );
  				willBeFormated = false;
  		}
			this->show();
		}
}

void ChangePartitionDialog::lineEditChanged( const QString& a_str )
{
    if( Fdisk::isExtendedType( id ) )
			warnAboutSubpartitions();
    updateSizeInfo();
}

// changes the region on the disk,
// e.g. then you have hda1 and hda3, but hda2 was deleted, then you
// have free space in the centre and at the end
void ChangePartitionDialog::freeSpaceComboActivated( int a_index )
{
    disk->getCurrentFreeRegion( freeSpaces, &curStart, &curEnd );
    curSize = curEnd - curStart;
//printf( "1.1 curStart %d, curEnd %d, curSize %d \n", (int)curStart, (int)curEnd, (int)(curSize/2) );

#ifdef ADVANCED_SIZE
    startEdit->setText( QString::number( disk->cylinderOfSector( curStart ) ) );
    endEdit->setText( QString::number( disk->cylinderOfSector( curEnd ) ) );
#else
    sizeEdit->setText( freeSpaces->currentText() );
#endif
}

// type of the systems changed, make nice things like swap size
// and bootable flags
void ChangePartitionDialog::systemComboActivated( const QString& s)
{
    int oldId = id;
    id = Fdisk::systype( s );

    if (Fdisk::isExtendedType( oldId ) && !Fdisk::isExtendedType(id) ){
			warnAboutSubpartitions();
    }

    if (id == 0x82) { // Linux swap
    	bootableCheck->setChecked( FALSE );
    	bootableCheck->setEnabled( FALSE );
    	mountPointEdit->setCurrentItem( 0 );
    	mountPointEdit->setEnabled( FALSE );
    	
    	// added for ease of use for newbye users
    	sizeEdit->setText( "128" );
    	updateSizeInfo();
    }else if (id == 0x5) { // extended
      bootableCheck->setChecked(FALSE);
      bootableCheck->setEnabled(FALSE);
      mountPointEdit->setCurrentItem( 0 );
      mountPointEdit->setEnabled(FALSE);
    } else {
      bootableCheck->setEnabled( TRUE );
      mountPointEdit->setEnabled( TRUE );
    }

    updateFormatCheck();
}

void ChangePartitionDialog::mountPointComboActivated( const QString& mp)
{
  if( mp == tr("<other>") ){
      mountPointEdit->setEditText("");
      mountPointEdit->setFocus();
  }

  ok->setEnabled( !(mp.simplifyWhiteSpace().left(4) == "/etc" && mountPointEdit->count()) );

  updateFormatCheck();
	updateBootCheck();
}

void ChangePartitionDialog::warnAboutSubpartitions()
{
    if ( hasSubpartitions && isVisible() ) {
			QMessageBox::information( this, tr("Logical partitions"),
	      tr("You are changing an extended partition. If you confirm the changes, \nall its subpartitions will be invalid and therefore removed.") );
			hasSubpartitions = FALSE;
    }
}

// enable/dosable boot check box
void ChangePartitionDialog::updateBootCheck()
{
	QString mp = mountPointEdit->currentText();
	if( mp != "/" && mp != "/boot" ) return;
		
	int n = 1;
	PartitionItem* p = disk->partition( n );
	bool hasBootable = false;
	while( p ){
		if( p->bootable && (p->id == 0x83 || p->id == 0x19) ){// has bootable Linux partition
			hasBootable = true;
			break;
		}
		n++;
	  p = disk->partition( n );
	}
	if( !hasBootable )
		bootableCheck->setChecked( true );
}
