/***************************************************************************
                                diskitem.cpp
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	        Best Linux - Linux from Finland.
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "diskitem.h"
#include "diskorpartitionitem.h"
#include "changepartitiondialog.h"
#include "fdisk.h"
#include "mountpoint.h"
#include "partitionview.h"
#include "partitionitem.h"
#include "liconloader.h"
#include "swapper.h"

#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <qwidget.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qmessagebox.h>
#include <qapplication.h>

DiskItem::DiskItem( const QString& disk_, QListView* parent, ChangePartitionDialog* dialog, QListViewItem *after )
    : QObject(0, 0), DiskOrPartitionItem( parent, after )
{

    partitionDialog = dialog;

    disk = disk_;
    valid = TRUE;
    changed = FALSE;
    original = TRUE;

    valid = Fdisk::getDiskGeometry( disk, heads, sectors, cylinders );

//printf( "**geometry: %s - H/S/C %d/%d/%d\n", disk.ascii(), heads, sectors, cylinders );
    signed long size = heads*sectors*cylinders;

    int col = 0;
    QString c;
    setText(col, " "+ disk );
//    setPixmap(col, LIconLoader::loadIcon("hds_full.gif"));
		col++;

#ifdef ADVANCED_SIZE_VIEW
    setText(col++, "1" );
    c.setNum( cylinderOfSector( size-1 ) );
    setText(col++, c);
#endif

		if( disk.left(7) == "/dev/hd" ){
			QString drive = disk.mid( 5, 8 );
			QFile f( "/proc/ide/"+ drive +"/model" );
			if( f.open( IO_ReadOnly ) ){
	      QTextStream is(&f);
  	    QString line = is.readLine();
				if( line.length() )
			    setText(col++, line );
				f.close();
			}
		}else
	    setText(col++, "" );

    c.setNum( megaByteOfSectors(size) );
    setText(3, c +" "+ tr("MB") + "  ");
		col++;

    nrPrimary = 4;
    if (disk.left(8) == "/dev/rd/")
      nrPrimary=7;
    if (disk.left(9) == "/dev/ida/")
      nrPrimary=15;

//    if (disk.left(9) == "/dev/ida/")
//      nrPrimary=15;

    partitions = Fdisk::getPartitionTable( disk );
    initPartitionTable();
}


DiskItem::~DiskItem()
{
}


void DiskItem::initPartitionTable()
{
    QTextIStream t( &partitions );	    // use a text stream
    QString s;

    while( valid && !t.eof() ) {
    	s = t.readLine();	    // line of text excluding '\n'
    	if( !s.isEmpty() && s[0] == '/' )
	      insertPartition( s );
    }

    // look carefully for partition overlaps
    // Note: Mylex controllers support 7 primary partitions
    int i, j;
    for ( i = 1; valid && i <= nrPrimary; i++ ) {
    	PartitionItem * ip = primaryPartition( i );

    	if ( !ip ) {
  	    // RAID device sometimes have just 4 primary partitions, but
  	    // 'n' maximum... they are just unused. Having just 4 is
  	    // still valid.
  	    if (i<=4)
  				valid = FALSE;
  	    break;
    	}

    	for( j=1; valid && j <= nrPrimary; j++ ) {
  	    if ( i != j ) {
      		PartitionItem * jp = primaryPartition( j );
      		if ( jp == ip ) {
      		    valid = FALSE; // internal error, really
      		    break;
      		}
      		// RAID device sometimes have just 4 primary partitions, but
      		// 'n' maximum... they are just unused. Having just 4 is
      		// still valid.
      		if( !jp ){
     		    if (j<=4)
	      			valid = FALSE;
						break;
      		}

      		if ( ip->start < jp->start ) {
    		    // if it's before, it has to be entirely before
    		    if ( ip->start + ip->size > jp->start )
	      			valid = FALSE;
      		} else if ( ip->start > jp->start ) {
    		    // after -> entirely after
    		    if ( ip->start < jp->start + jp->size )
	      			valid = FALSE;
      		} else if ( ip->id || jp->id ) {
    		    valid = FALSE;
      		}
   	    }
    	}
    }

    if ( valid ){
   		sortChildItems( 0, false );
			enforceSortOrder();
			return;
		}

    // we have an invalid partition table.
    // Present the user a clean view instead.
    valid = TRUE;
    for ( i = 1; i <= nrPrimary; i++ )
	  	if (primaryPartition(i))
		    removeOrClear( primaryPartition( i ) );
    for ( i = 1; i <= nrPrimary; i++ )
	  	if ( !primaryPartition( i ) )
		    (void) new PartitionItem( this, this, false /*TRUE:Maddy 20000130*/, i, FALSE, 0, 0, 0 );
		
		sortChildItems( 0, false );
		enforceSortOrder();
}

/*
  capacity of the disk in MegaByte
 */
double DiskItem::capacity()
{
    return megaByteOfSectors(heads * sectors * cylinders);
}



signed long DiskItem::cylinderOfSector( signed long s )
{
    return ( heads * sectors ) ? (s / (heads * sectors )  + 1) : 0;
}

signed long DiskItem::sectorOfCylinder( signed long c )
{
    return ( c - 1 ) * heads * sectors;
}


double DiskItem::megaByteOfSectors( signed long s )
{
  double result = double(s) / (2.0 * 1024);
//  if( result > 5 )
    result = (signed long)result;
//  else // rounding upto decimal place
//    result = ( (signed long)(result * 10) ) / 10.0;

  return result;
}

signed long DiskItem::sectorsOfMegaByte( double s )
{
  signed long result = (signed long)( double(s) * 2048 );
  return result;
}

/*Insert a single partition based on the passed line in sfdisk dump format:

    <partition_name> : start=  <start>, size= <size>, Id=<size [, bootable]

  for example:
    /dev/sda1 : start=       63, size=   96327, Id=83, bootable       */
void DiskItem::insertPartition( const QString& line )
{
    //  /dev/sda1 : start=       63, size=   96327, Id=83, bootable

    if ( line.find( disk ) != 0 )
			return; // wrong line

    int colon = line.find(':', disk.length());
    if ( colon < 0 )
			return;

    int number;
    if (line.mid(disk.length(),1) == "p") // skip the 'p'
			number = line.mid( disk.length()+1, colon - disk.length()-1 ).toInt();
    else
			number = line.mid( disk.length(), colon - disk.length() ).toInt();

    if ( number < 0 || number > 63 )
			return;

    signed long start = 0;
    signed long size = 0;
    int id = 0;
    bool bootable = FALSE;
	
    int len;
    int pos;

    QRegExp startR( "start= *[0-9]+" );
    pos = startR.match(line, 0, &len);
    if ( len > 6 && pos > colon )
			start = line.mid(pos+6, len-6).toInt();
    QRegExp sizeR( "size= *[0-9]+" );
    pos = sizeR.match(line, 0, &len);
    if ( len > 5 && pos > colon )
			size = line.mid(pos+5, len-5).toInt();
    QRegExp idR("Id= *[0-9a-fx]+");
    if (!line.contains(idR))	// MM 990705: we need an Id, or we get problems
    	return;			// with BSD disklabels...
    pos = idR.match(line, 0, &len);
    if ( len > 3 && pos > colon ){
    	QString ids = line.mid(pos+3, len-3);
     	QTextIStream is( &ids );
     	is.flags( QTextStream::hex );
     	is >> id;
    }

    bootable = line.contains("bootable");

    PartitionItem* p = findPartition( start );
    PartitionItem* np = 0;
    if ( p && p->isExtended() ) {
			np = new PartitionItem( p, this, FALSE, number, bootable,
				 start, size, id );
	
	// check whether the partition is really inside
			valid &= (start > p->start && start + size <= p->start + p->size );
    } else {
			// a primary partition must not overlap another
  		if ( p )
  	    valid = FALSE;
  		if ( number > nrPrimary || number < 1 )
  	    valid = FALSE;
  		if ( valid ){
  	    np = new PartitionItem( this, this, FALSE, number, bootable,
					start, size, id );
			}
    }

    // check overall boundary and whether empty partitions are really marked as empty
    valid &= ( start >= 0 &&
      start < heads*sectors*cylinders &&
      size >= 0 && (size > 0 || id == 0) );
		
//		sortChildItems( 0, false );
//		enforceSortOrder();
}

/*
  Write the (modified) partition table back to the disk.
 */
bool DiskItem::writePartitionTable(){
    if( !changed )
			return true;
    QString table;
    QTextOStream t( &table );

    t << "\n# partition table of " << disk << ",\tunit: sectors\n";

    int n = 1;
    PartitionItem* p = partition( n++ );
    while( p ) {
			t << p->dump();
			p = partition( n++ );
    }

    bool result = Fdisk::writePartitionTable( disk, table );

    changed = FALSE;
    original = FALSE;
		return result;
}


void DiskItem::defaultPartitionTable()
{
    if (original)
			return;

    while (firstChild() ) {
			delete ((PartitionItem*)firstChild());
    }
    initPartitionTable();

    changed = TRUE;
    original = TRUE;
}



/* Find the primary or extended partition that contains the given sector. Return 0 otherwise. */
PartitionItem* DiskItem::findPartition( signed long sector )
{
    QListViewItem* i = firstChild();
    while (i) {
     	PartitionItem* pi = (PartitionItem*)i;
     	if ( sector >= pi->start && sector <= pi->end() ) {
   	    return pi;
     	}
     	i = i->nextSibling();
    }

    return 0;
}

/*
  Is there an extended partition available?
 */
bool DiskItem::hasExtendedPartition()
{
    return extendedPartition() != 0;
}


/*
  Return the extended partition, 0 if no extended partition exists in
  the partition table.
 */
PartitionItem* DiskItem::extendedPartition()
{
    QListViewItem* i = firstChild();
    while (i) {
	PartitionItem* pi = (PartitionItem*)i;
	if ( pi->isExtended() )
	    return pi;
	i = i->nextSibling();
    }
    return 0;
}

/*
  Clear the current partition if it is a primary partition. If it's a
  logical partition, remove it.  Also remove all subpartitions in case
  the extended partition is cleared.
 */
void DiskItem::removeOrClear( PartitionItem * current)
{
    if ( !current )
    	return;

    if ( current->isLogical() ) {
    	delete current;
    	current = 0;
    	renumberLogicalPartitions();
    } else {
     	while (current->firstChild() )
     	    delete current->firstChild();
     	MountPoints::clearMountPoint(current->name());
     	current->start = 0;
     	current->size = 0;
     	current->id = 0;
     	current->bootable = FALSE;
     	current->isNew = TRUE;
     	current->willBeFormated = FALSE;
     	Fdisk::markAsNew( current->name() );
    }

    if (current)
      current->updateView();

    changed = TRUE;
    original = FALSE;
}


/*Edit the current partition. Pops up a partition change dialog and
  handles the settings afterwards.*/
bool DiskItem::edit( PartitionItem* current )
{
    partitionDialog->name = current->name();
    QString caption = current->name();
    if ( current->isExtended() )
			caption += tr(" [Extended]");
    else if (current->isLogical() )
			caption += tr(" [Logical]");
    else
			caption += tr(" [Primary]");
    partitionDialog->setCaption( caption );

    if ( current->isLogical() )
			fillFreeSpacesLogical( partitionDialog->freeSpaces, current->number );
    else
			fillFreeSpacesPrimary( partitionDialog->freeSpaces, current->number );
	
    if( current->id == 0 ) {
    	// the partition was empty => provide useful defaults
    	partitionDialog->isNewPartition = TRUE;
    	partitionDialog->hasSubpartitions = FALSE;
    	
    	partitionDialog->start = sectors; // space for the boot manager
    	partitionDialog->size = 1;

//  	partitionDialog->id = 0x83; // Linux native
    	partitionDialog->id = 0x19; // ReiserFS
    	partitionDialog->bootable = current->bootable;
    	
    	//validate free space
    	if ( partitionDialog->freeSpaces->count() == 0 ) {
 	    // no space left for that partition
   	    QMessageBox::critical( listView(), tr("No space"),
			  	tr("No space left for further partitions.") );
    	    return FALSE;
    	}
    } else {
    	partitionDialog->isNewPartition = FALSE;
    	partitionDialog->hasSubpartitions = current->firstChild() != 0;
    	partitionDialog->start = current->start;
    	partitionDialog->size = current->size;
    	partitionDialog->id = current->id;
    	partitionDialog->bootable = current->bootable;
    }

    partitionDialog->willBeFormated = current->willBeFormated;
    partitionDialog->isNew = current->isNew;
    partitionDialog->number = current->number;

    partitionDialog->doSettings( this );
		partitionDialog->exec();
	
    if ( partitionDialog->result() == QDialog::Accepted ) {
    	if ( current->isExtended() ){
    	    if ( !Fdisk::isExtendedType(partitionDialog->id) ||
    		 current->size != partitionDialog->size ||
    		 current->start != partitionDialog->start ) {
    		// a) the partition changes from extended to something else
    		// b) the partition changed size
    		//        => remove logical partitions
    				while (current->firstChild() )
    		  	  delete current->firstChild();
    	    }
    	}
	
    	bool didSomethingImportant
    	    = ( current->isNew != partitionDialog->isNew ||
    		current->id != partitionDialog->id ||
    		current->bootable != partitionDialog->bootable ||
    		current->start != partitionDialog->start ||
    		current->size != partitionDialog->size );
    	
    	current->start = partitionDialog->start;
    	current->size = partitionDialog->size;
    	current->id = partitionDialog->id;
    	current->bootable = partitionDialog->bootable;
    	current->willBeFormated = partitionDialog->willBeFormated;
    	current->isNew = partitionDialog->isNew;
    	
    	if ( current->isNew )
    	    Fdisk::markAsNew( current->name() );

    	Fdisk::setFormatting( current->name(), current->willBeFormated );

    	if ( current->isLogical() )
   	    renumberLogicalPartitions();
    	
			if( current->id != 0x82 )
	    	MountPoints::setMountPoint( current->name(), partitionDialog->mountPoint() );
			else
	    	MountPoints::setMountPoint( current->name(), "swap" );

    	// we need to update ALL items to unify the mount points
    	((PartitionView*)listView())->updateView();
    	current->updateView();
    		
    	if (didSomethingImportant) {
    	    changed = TRUE;
    	    original = FALSE;
    	}
    	
    	return didSomethingImportant;
    }
    return FALSE;
}


/* Fill the combox with free space ranges, assume partition "empty" is empty. */
void DiskItem::fillFreeSpacesPrimary( QComboBox* freeSpaces, int empty )
{
    freeSpaces->clear();
    freeSpacesTable.clear();

    PartitionItem *p = partition( empty );
    signed long beginCur = p->start;

Log::logMain( 35, "check\n" );
    p = firstNonEmptyPrimary( empty );
    if( !p || p->number == empty ){
    	insertFreeSpace( freeSpaces, sectors, heads*sectors*cylinders );
    	return;
    }

    if( p->start > sectors ){
Log::logMain( 35, "check2 %ld %ld\n", p->start );
    	splitFreeSpace( freeSpaces, sectors, p->start, beginCur );
    }

    signed long next = p->end() + 1;
    while( p ){
    	PartitionItem* p2 = nextNonEmptyPrimary( next, empty );
    	if( p2 ){
    	    if( p2->start > next ){
         		splitFreeSpace( freeSpaces, next, p2->start, beginCur );
Log::logMain( 35, "check3\n" );
          }
    	    next = p2->end() + 1;
    	}
    	p = p2;
    }

    if ( next < heads*sectors*cylinders ){
     	splitFreeSpace( freeSpaces, next, heads*sectors*cylinders, beginCur );
Log::logMain( 35, "check4\n" );
    }
}

/* verify the passed free space, assume partition "empty" is empty. */
bool DiskItem::verifyFreeSpacesPrimary( signed long start, signed long end, int empty )
{
    if( start >= end )
    	return FALSE;
    PartitionItem* p = firstNonEmptyPrimary( empty );
    if( !p || p->number == empty ){
    	return start >= sectors && end < heads*sectors*cylinders;
    }
    if( p->start > sectors ){
    	if( start >= sectors && end < p->start )
	      return TRUE;
    }

    signed long next = p->end()+1;
    while (p) {
    	PartitionItem* p2 = nextNonEmptyPrimary( next, empty );
    	if (p2) {
   	    if( p2->start > next ) {
      		if( start >= next && end < p2->start )
    		    return TRUE;
   	    }
   	    next = p2->end()+1;
    	}
    	p = p2;
    }

    if( next < heads*sectors*cylinders ) {
  	  if( start >= next && end < heads*sectors*cylinders )
	      return TRUE;
    }

    return FALSE;
}


/* verify the passed free space within the extended partition, assume partition "empty" is empty. */
bool DiskItem::verifyFreeSpacesLogical( signed long start, signed long end, int empty )
{
    if (start >= end )
    	return FALSE;

    PartitionItem* ep = extendedPartition();
    if (!ep)
    	return FALSE;

    PartitionItem* p = firstLogical( empty );
    if (!p || p->number == empty) {
    	return start >= ep->start + sectors && end < ep->start + ep->size;
    }

    if (p->start > ep->start + sectors) {
    	if (start >= ep->start + sectors && end < p->start)
	      return TRUE;
    }

    signed long next = p->end()+1;
    while( p ){
    	PartitionItem* p2 = nextLogical( next, empty );
    	if( p2 ){
	      if( p2->start > next + sectors ){
      		if( start >= next + sectors && end < p2->start )
		        return TRUE;
  	    }
	      next = p2->end()+1;
    	}
    	p = p2;
    }

    if( next + sectors < ep->start + ep->size ) {
    	if( start >= next + sectors && end < ep->start + ep->size )
	      return TRUE;
    }
    return FALSE;
}

bool DiskItem::splitFreeSpace( QComboBox* freeSpaces, signed long a_startZone, signed long a_endZone, signed long a_startExist ){
  bool result = false;
  if( a_startZone < a_startExist && a_startExist < a_endZone ){
//printf( "cluster: %ld\n", sectors*heads );
 		insertFreeSpace( freeSpaces,  a_startZone, a_startExist );
//printf( "check10: %ld\n", (a_startExist - a_startZone) );
  	insertFreeSpace( freeSpaces,  a_startExist, a_endZone );
//printf( "check11: %ld\n", (a_endZone - a_startExist) );
  	result = true;
  }else
  	insertFreeSpace( freeSpaces, a_startZone, a_endZone );
Log::logMain( 35, "split: %ld < %ld < %ld ---> %d\n", a_startZone, a_startExist, a_endZone, result );
  return result;
}

/* Fill the combox with free space ranges within the extended partition, assume partition "empty" is empty. */
void DiskItem::fillFreeSpacesLogical( QComboBox* freeSpaces, int empty )
{
    freeSpaces->clear();
    freeSpacesTable.clear();

    PartitionItem* ep = extendedPartition();
    if( !ep )
    	return;

    signed long beginCur = ep->start + sectors - 1;
    if( empty !=-1 ){
      PartitionItem *tp = partition( empty );
      if( tp )
        beginCur = tp->start;
    }
    	
    PartitionItem* p = firstLogical( empty );
    if( !p || p->number == empty ){ //absolutely empty
    	splitFreeSpace( freeSpaces, ep->start + sectors, ep->start + ep->size, beginCur );
    	return;
    }

    if( p->start > ep->start + sectors ){
  	  splitFreeSpace( freeSpaces,  ep->start + sectors, p->start, beginCur );
    }

    signed long next = p->end()+1;
    while (p) {
    	PartitionItem* p2 = nextLogical( next, empty );
    	if (p2) {
 	      if (p2->start > next + sectors){
      		splitFreeSpace( freeSpaces,  next + sectors,  p2->start, beginCur );
        }
	      next = p2->end()+1;
    	}
    	p = p2;
    }

    if( next + sectors < ep->start + ep->size ){
    	splitFreeSpace( freeSpaces, next + sectors, ep->start + ep->size, beginCur );
    }
}

/* Auxiliary function to insert a free space in the combobox. Ensures
  that the inserted free space ends at a cylinder boundary. */
void DiskItem::insertFreeSpace( QComboBox* freeSpaces, signed long start, signed long after )
{
    QString s;
//printf( "check12: %ld - %ld\n", cylinderOfSector( after ), cylinderOfSector( after - 1 ) );
    if( cylinderOfSector( after ) == cylinderOfSector( after - 1 ) )
    	after = sectorOfCylinder( cylinderOfSector( after ) ) - 1;
//printf( "check13: %ld\n", after );
    	
#ifdef ADVANCED_SIZE
    QTextOStream( &s ) << cylinderOfSector(start) << " - " << cylinderOfSector(after-1);
#else
    QTextOStream( &s ) << megaByteOfSectors( after-1 - start );
#endif
    // add in case of at least one cluster
    if( (after-1 - start) >= (sectors*heads-32) ){
      freeSpaces->insertItem( s );
      freeSpacesTable.append( new FreeSpace( start, after ) );
    }
}

void DiskItem::chooseCurrentItem( QComboBox *freeSpaces, signed long a_startRegion, signed long a_endRegion )
{
  QListIterator<FreeSpace> it( freeSpacesTable );
  FreeSpace *curItem = NULL;
  int i = 0;
  while( (curItem = it.current()) ){
//printf( "%ld, %ld, %ld, %ld\n", a_startRegion, curItem->start, a_endRegion, curItem->end );
    if( a_startRegion >= curItem->start && a_endRegion <= curItem->end ){
      if( freeSpaces->currentItem() != i )
        if( freeSpaces->count() )
          freeSpaces->setCurrentItem( i );
      return;
    }
    ++i;
    ++it;
  }
  if( freeSpaces->count() )
    freeSpaces->setCurrentItem(0);
}

void DiskItem::getCurrentFreeRegion( const QComboBox *freeSpaces, signed long *start, signed long *end )
{
  FreeSpace *cur = freeSpacesTable.at( freeSpaces->currentItem() );
  *start = cur->start;
  *end = cur->end;
}

/* return the first non empty primary partition, assume the passed
  partition is empty. */
PartitionItem* DiskItem::firstNonEmptyPrimary( int empty)
{
    PartitionItem* result = 0;
    for (int i = 1; i <= nrPrimary; i++) {
	PartitionItem* p = primaryPartition( i );
	if (p && p->isNonEmptyPrimary() && p->number != empty) {
	    if (!result)
		result = p;
	    else if (p->start < result->start)
		result = p;
	}
    }
    return result;
}


/*
  return the first non empty primary partition beginning with or after
  sector <start> , assume the passed partition is empty.
 */
PartitionItem* DiskItem::nextNonEmptyPrimary( signed long start, int empty )
{
    PartitionItem* result = 0;
    for (int i = 1; i <= nrPrimary; i++) {
    	PartitionItem* p = primaryPartition( i );
    	if( p && p->isNonEmptyPrimary() && p->start >= start && p->number != empty ) {
    	    if (!result)
        		result = p;
    	    else if (p->start < result->start)
        		result = p;
    	}
    }
    return result;
}

/*
  return the first logical partition, assume the passed partition is
  empty.
 */
PartitionItem* DiskItem::firstLogical( int empty)
{
    PartitionItem* result = 0;

    if ( !hasExtendedPartition() )
	return 0;

    PartitionItem* i = (PartitionItem*)extendedPartition()->firstChild();
    while (i) {
	if (i->number != empty) {
	    if (!result)
		result = i;
	    else if (i->start < result->start)
		result = i;
	}
	i = (PartitionItem*)i->nextSibling();
    }
    return result;
}


/*
  return the first non logical partition beginning with or after
  sector <start> , assume the passed partition is empty.
 */
PartitionItem* DiskItem::nextLogical(signed long start, int empty)
{
    PartitionItem* result = 0;
    PartitionItem* i = (PartitionItem*)extendedPartition()->firstChild();
    while (i) {
    	if (i->start >= start && i->number != empty) {
   	    if (!result)
      		result = i;
   	    else if (i->start < result->start)
      		result = i;
    	}
    	i = (PartitionItem*)i->nextSibling();
    }
    return result;
}

/*
  get the maximum possible free space for primary partition number = [1... nrPrimary]
 */
void DiskItem::getFreeSpace( int number, signed long& start, signed long& size) {
    // start

    int i = number - 1;
    while ( i > 0 && (primaryPartition( i ) && primaryPartition( i )->id == 0))
	i--;
    if ( i == 0)
	start = sectors;
    else
	start = primaryPartition( i )->start + primaryPartition(i)->size;

    // size
    i = number + 1;
    while ( i <= nrPrimary && primaryPartition( i )->id == 0)
	i++;
    if (i > nrPrimary) {
	size = (heads * sectors * cylinders) - start - 1;
    }
    else
	size = primaryPartition(i)->start - start -1;
}



/*
  returns the primary partition with number [1..nrPrimary]
 */
PartitionItem* DiskItem::primaryPartition( int number )
{
    if (number < 1 || number > nrPrimary)
			return 0; // should not happen
    QListViewItem* item = firstChild();
    if (!item)
			return 0;
    do {
			if ( ((PartitionItem*) item)->number == number)
	  	  return (PartitionItem*) item;
	
			item = item->nextSibling();
    } while (item);
    return 0;
}

/* returns the (primary or logical) partition with number [1..n] if
  available, otherwise 0. */
PartitionItem* DiskItem::partition( int number )
{
    if( number <= nrPrimary )
    	return primaryPartition( number );
    PartitionItem* f = hasExtendedPartition() ? (PartitionItem*)extendedPartition()->firstChild() : 0;
    PartitionItem* item = 0;
    if( f ){
    	item = f;
    	do{
	      if( ((PartitionItem*) item)->number == number )
      		return (PartitionItem*)item;
	      item = (PartitionItem*)item->nextSibling();
    	}while( item );
    }
    return item;
}


/*
  Tell the disk that its contents is dirty (i.e. requires write back)
 */
void DiskItem::invalidateDisk()
{
    changed = TRUE;
    original = FALSE;
}


/*
  Add a logical partition. If that is not possible, show an information message.
 */
bool DiskItem::addLogicalPartition()
{
    PartitionItem* ep = extendedPartition();
    if( !ep ){
    	QMessageBox::critical( listView(), tr("Logical partitions"),
    	tr("You need to create an extended partition first. Select \"Edit Partition\"\nand set the partition type to <Extended>") );
    	return FALSE;
    }

    partitionDialog->setCaption( tr("Add logical partition") );
    partitionDialog->name = "";

    fillFreeSpacesLogical( partitionDialog->freeSpaces, -1 );

    partitionDialog->isNewPartition = TRUE;
    partitionDialog->hasSubpartitions = FALSE;
	
    partitionDialog->start = ep->start + sectors;
    partitionDialog->size = 0;

//    partitionDialog->id = 0x83; // Linux native
    partitionDialog->id = 0x19; // ReiserFS
    partitionDialog->bootable = FALSE;
	
    //validate free space
    if( !partitionDialog->freeSpaces->count() ){
	// no space left for that partition
    	QMessageBox::critical( listView(), tr("No space"),
            	tr("No space left for further partitions.")
    			    /*tr("No space left inside the extended partition..")*/ );
    	return FALSE;
    }

    partitionDialog->number = nrPrimary+1; // something > nrPrimary to indicate a logical partition

    partitionDialog->doSettings( this );
		partitionDialog->exec();

    if ( partitionDialog->result() == QDialog::Accepted ) {
    	PartitionItem* pi = new PartitionItem( ep,
    						 this,
    						 TRUE,
    						 partitionDialog->number,
    						 partitionDialog->bootable,
    						 partitionDialog->start,
    						 partitionDialog->size,
    						 partitionDialog->id );
    	renumberLogicalPartitions();
			if( pi->id != 0x82 )
	    	MountPoints::setMountPoint( pi->name(), partitionDialog->mountPoint() );
			else
	    	MountPoints::setMountPoint( pi->name(), "swap" );

    	changed = TRUE;
    	original = FALSE;
    	pi->willBeFormated = partitionDialog->willBeFormated;

    	// we need to update ALL items to unify the mount points
    	((PartitionView*)listView())->updateView();	

    	return TRUE;
    }
    return FALSE;
}


/*
  Renumber all existing logical partitions according to the order on
  the disk.  In other words: emulate what the kernel does.
 */
void DiskItem::renumberLogicalPartitions() {
    PartitionItem* i = firstLogical(-1);
    int number = nrPrimary+1;
    while (i) {
    	i->number = number++;
    	i->updateView();
    	i = nextLogical(i->end()+1, -1);
    }
    enforceSortOrder();
}



/*
  Repartition the disk automatically to some reasonable settings
 */
void DiskItem::doAutoPartitioning()
{

    // very first: stop swapping on the disk
    Swapper::swapOff();

    // first: clear the disk

    // therefore clear the MBR first ... arrgl!
    { // remember ... even ugly code works sometimes
        int fd = ::open(disk.ascii(), O_WRONLY || O_SYNC);
        char *block = (char*)::calloc(1, 512);
        ::write(fd, block, 512);
        ::close(fd);
        ::free(block);
    }


    for (int i = 1; i <= nrPrimary; i++)
        if (primaryPartition( i ))
	    removeOrClear( primaryPartition( i ) );
    for (int i = 1; i <= nrPrimary; i++)
    	if (!primaryPartition( i ) )
	      (void) new PartitionItem( this, this, false/*TRUE:Maddy 20000130*/, i, FALSE, 0, 0, 0 );

    PartitionItem* proot = 0;
    PartitionItem* phome = 0;
    PartitionItem* pswap = 0;

    double capacit = capacity();

    if ( capacit < 1500) {
    	// less than 1.5G:   one root partition, 64M swap
    	PartitionItem* p = proot = partition( 1 );
    	p->start = sectors;
    	p->size = sectorOfCylinder(cylinderOfSector(heads * sectors * cylinders - 1 - 128000)) - sectors;
//    	p->id = 0x83;
    	p->id = 0x19;
    	p->bootable = TRUE;
    	p->willBeFormated = TRUE;
    	p->isNew = TRUE;
    	Fdisk::markAsNew( p->name() );
    	Fdisk::setFormatting( p->name(), p->willBeFormated);
    	MountPoints::setMountPoint( p->name(), "/" );

    	pswap = partition( 2 );
    	pswap->start = p->end() + 1;
    	pswap->size = heads * sectors * cylinders - pswap->start;
    	pswap->id = 0x82;
    	pswap->bootable = FALSE;
    	pswap->willBeFormated = TRUE;
    	pswap->isNew = TRUE;
    	Fdisk::markAsNew( pswap->name() );
    	Fdisk::setFormatting( pswap->name(), pswap->willBeFormated);
    } else if ( capacit < 4000 ) {
    	// less than 4G:   one root partition, 128M swap
    	PartitionItem* p = proot = partition( 1 );
    	p->start = sectors;
    	p->size = sectorOfCylinder(cylinderOfSector(heads * sectors * cylinders - 1 - 256000)) - sectors;
//    	p->id = 0x83;
    	p->id = 0x19;
    	p->bootable = TRUE;
    	p->willBeFormated = TRUE;
    	p->isNew = TRUE;
    	Fdisk::markAsNew( p->name() );
    	Fdisk::setFormatting( p->name(), p->willBeFormated);
    	MountPoints::setMountPoint( p->name(), "/" );

    	pswap = partition( 2 );
    	pswap->start = p->end() + 1;
    	pswap->size = heads * sectors * cylinders - pswap->start;
    	pswap->id = 0x82;
    	pswap->bootable = FALSE;
    	pswap->willBeFormated = TRUE;
    	pswap->isNew = TRUE;
    	Fdisk::markAsNew( pswap->name() );
    	Fdisk::setFormatting( pswap->name(), pswap->willBeFormated);
    } else {	// 4G or more:   root partition, home partition, 128M swap
    	PartitionItem* p = proot = partition( 1 );
    	p->start = sectors;
    	long sects = 0;
    	if( capacit >= 4000 && capacit < 8000 ){
      	sects = (long)((long)capacit * 1024); // half of the disk
      }else if( capacit >= 8000 ){
      	sects = 10240000; // 5GB
      }
    	p->size = sectorOfCylinder( cylinderOfSector(sects) ) - sectors;
//    	p->id = 0x83;
    	p->id = 0x19;
    	p->bootable = TRUE;
    	p->willBeFormated = TRUE;
    	p->isNew = TRUE;
    	Fdisk::markAsNew( p->name() );
    	Fdisk::setFormatting( p->name(), p->willBeFormated);
    	MountPoints::setMountPoint( p->name(), "/" );

    	pswap = partition( 2 );
    	pswap->start = p->end() + 1;
    	pswap->size = sectorOfCylinder(cylinderOfSector(pswap->start + 256000)) - pswap->start;
    	pswap->id = 0x82;
    	pswap->bootable = FALSE;
    	pswap->willBeFormated = TRUE;
    	pswap->isNew = TRUE;
    	Fdisk::markAsNew( pswap->name() );
    	Fdisk::setFormatting( pswap->name(), pswap->willBeFormated);

    	p = phome = partition( 3 );
    	p->start = pswap->end() + 1;
    	p->size = heads * sectors * cylinders - p->start;
    	p->id = 0x83;
    	p->bootable = FALSE;
    	p->willBeFormated = TRUE;
    	p->isNew = TRUE;
    	Fdisk::markAsNew( p->name() );
    	Fdisk::setFormatting( p->name(), p->willBeFormated);
    	MountPoints::setMountPoint( p->name(), "/home" );
    }

    // finally start praying and write the partition table
    QApplication::setOverrideCursor( waitCursor );
    writePartitionTable();
    QApplication::syncX();

    // create the filesystems, swap always first (since we will use the swap partition immediately)
    if ( pswap )
			((PartitionView*)listView())->formatPartition( pswap->name());
    if ( proot )
			((PartitionView*)listView())->formatPartition( proot->name());
    if ( phome )
			((PartitionView*)listView())->formatPartition( phome->name());

    QApplication::restoreOverrideCursor();

    // we need to update ALL items to unify the mount points
    ((PartitionView*)listView())->updateView();

    // invalidate the view for safety
    ((PartitionView*)listView())->invalidateView();
}



