/***************************************************************************
                                 diskitem.h
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef DISKITEM_H
#define DISKITEM_H

#include "config.h"
#include "diskorpartitionitem.h"
#include "log.h"

#include <qcombobox.h>
#include <qlist.h>

#include "partitionitem.h"
#include "changepartitiondialog.h"

// a region on the disk, which is not used
// this regoins are added to choice in the ChangePartitionDialog
class FreeSpace{
public:
    FreeSpace( signed long a_start, signed long a_end ){
      start = a_start;
      end = a_end;
    };
    signed long start;
    signed long end;
};

// item in the PartitionView which represents a whole hard drive
class DiskItem : public QObject, public DiskOrPartitionItem {
    Q_OBJECT
public:
    DiskItem( const QString& disk_, QListView* parent, ChangePartitionDialog* dialog, QListViewItem *after );
    ~DiskItem();

    bool isDisk() { return TRUE; }

    void insertPartition( const QString& line );

    QString disk;

    PartitionItem* findPartition( signed long sector );
    bool hasExtendedPartition();
    PartitionItem* extendedPartition();

    void renumberLogicalPartitions();

    signed long heads;
    signed long sectors;
    signed long cylinders;
    double capacity(); // MBs
    signed long cylinderOfSector( signed long );
    signed long sectorOfCylinder( signed long );

    int nrPrimary;

    double megaByteOfSectors( signed long );
    signed long sectorsOfMegaByte( double );

    bool verifyFreeSpacesPrimary( signed long start, signed long end, int empty );
    bool verifyFreeSpacesLogical( signed long start, signed long end, int empty );

    void chooseCurrentItem( QComboBox *freeSpaces, signed long start, signed long end );

    PartitionItem* partition( int number );

    void invalidateDisk();

    void getCurrentFreeRegion( const QComboBox*, signed long *start, signed long *end );

public slots:
    bool edit( PartitionItem* );
    bool addLogicalPartition();
    void removeOrClear( PartitionItem* );

    bool writePartitionTable();
    void defaultPartitionTable();

    void doAutoPartitioning();

private:
    void getFreeSpace( int number, signed long& start, signed long& size);
    PartitionItem* primaryPartition( int number );

    void fillFreeSpacesPrimary( QComboBox* freeSpaces, int empty );
    void insertFreeSpace( QComboBox* freeSpaces, signed long start, signed long after);
    PartitionItem* firstNonEmptyPrimary( int empty );
    PartitionItem* nextNonEmptyPrimary(signed long start, int empty);

    void fillFreeSpacesLogical( QComboBox* freeSpaces, int empty );
    PartitionItem* firstLogical( int empty );
    PartitionItem* nextLogical(signed long start, int empty);

    bool splitFreeSpace( QComboBox* freeSpaces, signed long a_startZone, signed long a_endZone, signed long a_startExist );

    ChangePartitionDialog* partitionDialog;

    QString partitions;
    void initPartitionTable();

    bool valid;
    bool changed;
    bool original;

    QList<FreeSpace> freeSpacesTable;
};

#endif
