/***************************************************************************
                                disksel.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "disksel.h"
#include "linlinehelp.h"
#include "partitionview.h"
#include "fdisk.h"
#include "partitionitem.h"
#include "partitioninfo.h"
#include "diskitem.h"
#include "liconloader.h"
#include "language.h"

#include <qstringlist.h>
#include <qapplication.h>


DiskSelector::DiskSelector(  PartitionView* pview_, QWidget *parent, const char *name )
    : GBLIPage( parent, name)
{
    pview = pview_;

    QHBoxLayout * lh = new QHBoxLayout( this, 0, 6);

    QVBoxLayout * l = new QVBoxLayout( 6 );
		lh->addStretch( 1 );
    lh->addLayout( l, 8 );
		lh->addStretch( 1 );

    view = new QListView( this );
		QFont font( QFont::defaultFont() );
//		font.setFamily( "times" );
		font.setPointSize( Language::iSmallFontSize );
    view->setFont( font );
    view->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
    connect( view, SIGNAL( selectionChanged(QListViewItem*) ),
	     this, SLOT( selectionChanged(QListViewItem*) ) );
    view->setColumnWidthMode( view->addColumn(tr("Device name")),
			      QListView::Maximum );
    int sizeCol = view->addColumn(tr("Size   :"));
    view->setColumnAlignment( sizeCol, AlignRight );
    view->setColumnWidthMode( sizeCol, QListView::Maximum );
    int usedFor = view->addColumn(tr("Currently used for"));
    view->setColumnAlignment( usedFor, AlignCenter );
    view->setColumnWidthMode( usedFor, QListView::Maximum );

    l->addWidget( view, 10 );

    formatButton = new QPushButton( tr("Prepare selected disk for Linux"), this);
    l->addSpacing( 8 );
    l->addWidget( formatButton, 0, AlignCenter );
    l->addSpacing( 8 );
    formatButton->setEnabled( FALSE );
    connect( formatButton, SIGNAL( clicked() ), this, SLOT( formatButtonClicked() ) );

    view->setFocus();
}

DiskSelector::~DiskSelector()
{
}


void DiskSelector::selectionChanged( QListViewItem *l )
{
    selectedDisk = l?l->text(0):QString::null;
    repaint();
    if (selectedDisk != formattedDisk ) {
			formatButton->setEnabled( TRUE );
#ifndef GBLI_DEVEL
			emit nextEnabled( this, FALSE );
#endif
    }
}


void DiskSelector::showEvent( QShowEvent * )
{
    init();
}


void DiskSelector::hideEvent( QHideEvent * )
{
}

void DiskSelector::partitionTableChanged()
{
    emit nextEnabled( this, FALSE );
    formattedDisk = QString::null;
    init();
}


void DiskSelector::init()
{
    PartitionInfo::unMounting();

    QString tmp = selectedDisk;
    view->clear();
    selectedDisk = tmp;
	
    double selcap = 0;
    bool selwasunused = FALSE;

    QStringList allDisks = Fdisk::availableDisks();
    QStringList::Iterator disk;
    for (disk = allDisks.begin(); disk != allDisks.end(); ++disk ) {       
    	bool unused = FALSE;
    	QString usage = tr("nothing");
    	if ( pview->hasLinuxPartitions( *disk ) ) {
   		  usage = tr("Linux");
  	    if ( pview->hasDosWindowsPartitions( *disk ) )
	    		usage += "/" + tr("Dos/Windows");
    	}
    	else if ( pview->hasDosWindowsPartitions( *disk ) )
  	    usage = tr("Dos/Windows");
    	else if ( pview->hasPartitions( *disk ) )
  	    usage = tr("Unknown Operating System");
    	else
  	    unused = TRUE;
	
    	QListViewItem* i = new QListViewItem( view,
    					      *disk,
    					      pview->capacityOfDiskAsString( *disk ),
    					      usage );
//    	i->setPixmap(0, LIconLoader::loadIcon("hds_full.gif"));
    	
    	// check if the install source is on this disk
    	DiskItem *di = pview->disk(*disk);
    	if (di){
  	    PartitionItem *pi;
  	    int nr=1;
  	    while( (pi = di->partition(nr++)) )
  	      if (Fdisk::isInstallSource(pi->name())){
	    		  // skip this disk
  	  		  delete i;
    			  i = 0;
    			  continue;
    			}
   	  }

    	if (selectedDisk.isEmpty() ) {
   	    double cap = pview->capacityOfDisk( *disk );
   	    if ( (cap > selcap && unused == selwasunused ) || ( unused && !selwasunused ) ) {
	    		selectedDisk = *disk;
  	  		selcap = cap;
    			selwasunused = unused;
    			view->setSelected( i, TRUE);
    	  }
    	}	else {
  	    // we had a selection already => preserve it.
  	    if ( *disk == selectedDisk ) {
  	  		view->setSelected( i, TRUE );
		    }
    	}
    }

    if ( !selectedDisk.isEmpty() ) {
	   	emit nextEnabled( this, selectedDisk == formattedDisk );
    }
}

void DiskSelector::formatButtonClicked()
{
    if ( selectedDisk.isEmpty() )
			return; // nothing to do

    QListViewItem* i = view->firstChild();
    while ( i->text(0) != selectedDisk )
			i = i->nextSibling();

    i->setText(2, tr("Preparing...") );
    view->viewport()->repaint();
    QApplication::syncX();

    pview->disk( selectedDisk )->doAutoPartitioning();

    i->setText(2, tr("Linux") );

    formattedDisk = selectedDisk;
    formatButton->setEnabled( FALSE );
    emit nextEnabled( this, TRUE );
}





