/***************************************************************************
  																GBLI
                             -------------------
    begin                : Fri Oct 29 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GBLI_H
#define GBLI_H

#include "config.h"
#include "liconloader.h"
#include "fdisk.h"
#include "diskpartitioner.h"
#include "etcfiles.h"
//#include "partitionsel.h"
#include "upgradepage.h"
#include "partitioninfo.h"
#include "lcreateusers.h"
#include "lmonitor.h"
#include "lvideocard.h"
#include "lvideomodeselector.h"
#include "target.h"
#include "mouse.h"
#include "networkpage.h"
#include "keyboard.h"
//#include "timezone.h"
#include "disksel.h"
#include "grub.h"
#ifdef ALSA_SOUND
	#include "SoundAlsa.h"
#else
	#include "lsound.h"
#endif
#include "finalpage.h"
#include "printercfg.h"
#include "steplabel.h"
#include "log.h"

#include "modem.h"
#include "isppage.h"

#include <qwizard.h>
#include <qlist.h>
#include <qtranslator.h>
#include <qfont.h>
#include <qprogressbar.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qtextstream.h>
#include <qtimer.h>
#include <qdatetime.h>
#include <qmessagebox.h>
#include <qaccel.h>
#include <qfile.h>
#include <qdir.h>
#include <qmime.h>
#include <qwidgetstack.h>

// for the kwm hack
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

// symlink(), sync(), reboot()
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/reboot.h>
#include <stdlib.h>
#include <iostream.h>

#include "gblipage.h"
#include "installselect.h"
#include "PackageSelect.h"
#include "addmodulespage.h"
#ifdef GBLI_SERVER
	#include "servicessetup.h"
#endif

class QProgressBar;
class QLabel;

// main class, which manages page calls: creating, showing and finishing
// Inherits hacked version of QWizard
class GBLI: public QWizard
{
    Q_OBJECT

public:
    GBLI();
    ~GBLI();

	 // this method is called when we want to know if to show the given page
   bool appropriate( QWidget * ) const;

public slots:
   void startUpdate();

protected slots:
   void help();
   void next();
   void back();
	

   void increaseDebug(){
     if( Log::iDebugLevel < Log::iMaxDebug ) ++Log::iDebugLevel;
     printf( "New debug level: %d\n", Log::iDebugLevel );
   }
   void decreaseDebug(){
     if( Log::iDebugLevel > 0 ) --Log::iDebugLevel;
     printf( "New debug level: %d\n", Log::iDebugLevel );
   }

   void showProgressRow();
   void hideProgressRow();
   void setProgress(){ setProgress( 0 ); };
   void setProgress( int a_i );

   void getWindowsSettings();

   // main method: calls finish functions of the pages and finishes the
   // installation
   virtual void accept();

   // if user choose to stop the installation
   virtual void reject(){
			sync();
  		setResult( 128 );
  		hide();
  		qApp->exit( 128 );
   }

protected:
    // lay outs button row
    void layOutButtonRow( QHBoxLayout * );
    // draws the title
    void layOutTitleRow( QHBoxLayout * layout, const QString & title );

		// QWizard hack (needs also QWizardPrivate to be protected in QWizard)
    void layOut();
    void layOutInit();
    // end of the QWizard hack

		void addPage( QWidget *page, const QString &name );
    void addPage( QWidget *page, const QString &name, const QString &icon_name );
		void layOutIcons();

    void keyPressEvent(QKeyEvent *e);
    void runXProgram(  const char *a_search, const char *a_prg, const char *a_args, signed int a_iNice );

private slots:
    void packageInstallationFinished();
    // show another page, called after icon click
		void changePage( QWidget *icon, const QString &icon_name );
    // continues to create pages started in init()
    void setupPages2();
    void reboot();
		// saves screenshot to temporary directory
    void screendump();
		// opens an xterm
    void xterm();
    // checks if all the pages were configured
		bool checkStates();

private:
		// creates the pages
    void init();
    // makes translation of messages to be correct
    void setupTranslator();
    // formats the time and displays it in the given label
		void setTime( QLabel *w, QTime &time );

		void ConfCleanups();

    QTranslator* translator;

    // pages
    Mouse * mouse;
    Modem * modem;
    ISPPage * ISP;
    LVideoCard * videoCard;
    LMonitor * monitor;
    LVideoModeSelector * videoMode;
    Target* target;
    DiskPartitioner* diskpartitioner;
    DiskSelector* disksel;
    PartitionSelector* partitionselRoot;
  	UpgradePage *updatePage;	
		PartitionInfo* partitionInfo;
		NetworkPage * net;
    Keyboard * keyboard;
//    TimeZone* timezone;
    LCreateUsers* createusers;
    GRUB *grub;
#ifdef ALSA_SOUND
    SoundAlsa *sound;
#else
    LSound *sound;
#endif
		InstallSelect *installselect;
		PackageSelect *packageselect;
		FinalPage *finalpage;
		PrinterCfg *prncfg;
		AddModulesPage *modulessetup;
#ifdef GBLI_SERVER
		ServicesSetup *services;
#endif

		// framework
    QLabel * titleLabel;
    QLabel * beginLabel;
    QLabel * endLabel;
    QLabel * penguinLabel;
    QProgressBar* progressBar;

		// showed to make correct layout when finish button is not shown
		QLabel *finishStub;

		// packages installation is finished
		bool installFinished;
    // packages installation is going
		bool installGoing;

		// both packages and pages are finished
		bool allFinished;

		// packages installation milestones
		int aiProgrCheck[32];
		int aiProgrTimes[32];
		int curCheckPoint;
		QTime timer;

		// for inter-connecting pages and icons
		QDict<StepLabel> lblSteps;
		QList<StepLabel> lblStepsList;

		// Windows settings already read from the system
		bool gotWindowsSettings;

};


#endif


