/***************************************************************************
                                 hash.cpp
                             -------------------                                         
    begin                : Wed Jan 12 2000                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include "hash.h"
#include "log.h"

#define CHUNK 1

struct filePath
{
    char * dir;	// path to filename
    char * base;	// filename
};

struct bucket
{
    struct filePath * data;
    int allocated;
    int firstFree; /* as in data[firstFree] */
};

struct hash_table
{
    int size;
    int entries;
    int overHead;
    struct bucket *bucket;
};

struct hash_table *htNewTable(int size)
{
	struct hash_table *res;
	int i = 0;

//	logMessage("in htNewTable 12");
	res = (struct hash_table *) malloc(sizeof(struct hash_table));
	res->bucket = (struct bucket *) malloc(sizeof(struct bucket) * size);
	res->size = size;
	res->entries = 0;
	res->overHead = sizeof(struct bucket) * size + CHUNK * sizeof(char *);

	while (i < size) {
   	 res->bucket[i].data = (struct filePath *) malloc(CHUNK * sizeof(*res->bucket[i].data));
   	 res->bucket[i].allocated = CHUNK;
   	 res->bucket[i].firstFree = 0;
   	 i++;
	}

	return res;
}

void htFreeHashTable(struct hash_table *ht)
{
	struct bucket * b;
	int item;

//	logMessage("in htFreeHashTable 22");
	b = ht->bucket;
	while (ht->size--) {
   	 for (item = 0; item < b->firstFree; item++) {
      	  free(b->data[item].dir);
      	  free(b->data[item].base);
   	 }
   	 free(b->data);
   	 b++;
	}
	free(ht->bucket);
	free(ht);
}

void htAddToTable(struct hash_table *t, const char * dir, const char * base)
{
	static int hash = 1;

	if (!dir || !base)
   	 return;

	hash = htHashStrings(dir, base) % t->size;
	if (in_table_aux(t, hash, dir, base) != -1)
   	 return;

	if (t->bucket[hash].firstFree == t->bucket[hash].allocated) {
   	 t->bucket[hash].allocated += CHUNK;
   	 t->bucket[hash].data = (struct filePath *) realloc(t->bucket[hash].data,
				               	 t->bucket[hash].allocated * sizeof(*(t->bucket->data)));
   	 /*printf("Bucket %d grew to %d\n", hash, t->bucket[hash].allocated);*/
   	 t->overHead += sizeof(char *) * CHUNK;
	}
	/*printf("In bucket %d, item %d\n", hash, t->bucket[hash].firstFree);*/
	t->bucket[hash].data[t->bucket[hash].firstFree].dir = strdup(dir);
	t->bucket[hash].data[t->bucket[hash].firstFree++].base = strdup(base);
	t->entries++;
}

unsigned int htHashStrings(const char *s, const char *t)
{
	unsigned int res = 0;

	while (*s)
   	 res = ((res<<1) + (int)(*(s++)));
	while (*t)
   	 res = ((res<<1) + (int)(*(t++)));

	return res;
}

/* returns bucket # containing item, or -1 */
int in_table_aux(struct hash_table *t, int hash, const char * dir, const char * base)
{
	int x;

	x = 0;
	while (x < t->bucket[hash].firstFree) {
   	 if (! strcmp(t->bucket[hash].data[x].dir, dir) &&
      	  ! strcmp(t->bucket[hash].data[x].base, base)) {
      	  return x;
   	 }
   	 x++;
	}

	return -1;
}

int htInTable(struct hash_table *t,  const char * dir, const char * base)
{
	int hash;

	hash = htHashStrings(dir, base) % t->size;

	if (in_table_aux(t, hash, dir, base) == -1)
   	 return 0;
	return 1;
}

void htRemoveFromTable(struct hash_table *t, const char * dir, const char * base)
{
	int hash;
	int item;
	int last;

	hash = htHashStrings(dir, base) % t->size;
	if ((item = in_table_aux(t, hash, dir, base)) == -1)
   	 return;

	free(t->bucket[hash].data[item].dir);
	free(t->bucket[hash].data[item].base);

	last = --t->bucket[hash].firstFree;
	t->bucket[hash].data[item] = t->bucket[hash].data[last];
}

void htHashStats(struct hash_table *t)
{
	int i = 0;
	int empty = 0;

	while (i < t->size) {
   	 if (t->bucket[i].firstFree != 0) {
      	  /*printf("Bucket %d used %d\n", i, t->bucket[i].firstFree);*/
   	 } else {
      	  empty++;
   	 }
   	 i++;
	}

	Log::logMain(32, "Total Buckets : %d\n", t->size);
	Log::logMain(32, "Empty Buckets : %d\n", empty);
	Log::logMain(32, "Total Entries : %d\n", t->entries);
	Log::logMain(32, "Total Overhead: %d\n", t->overHead);
	Log::logMain(32, "Avergage Depth: %f\n", (double)t->entries / (double)t->size);
}

unsigned int hash_val(const char *s)
{
unsigned int hashval;

	for(hashval = 0; *s != '\0'; s++)
  	hashval = *s + 31 * hashval;

	return hashval % HASHSIZE;
}

