/***************************************************************************
                               lkeyboard.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "keyboard.h"
#include "config.h"
#include "linlinehelp.h"
#include "configfile.h"
#include "sysconfigfile.h"
#include "language.h"
#include "log.h"

#include <qlistbox.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qapplication.h>

#include <qfile.h>
#include <qtextstream.h>
#include <unistd.h>
#include <sys/wait.h>

// cross table between the language and keyboard layout
struct keyboardsTable {
    char * name, * lang, * keyboard;
}defaultKeyboards[] = {
    { "Belgian (be, be-latin1)", "be", "be-latin1" },
    { "Belgian (be, be2-latin1)", "be", "be2-latin1" },
    { "Bulgarian (bg, bg)", "bg", "bg" },
    { "Canadian (ca, ca)", "ca", "ca" },
    { "Czechoslovakian (cs, cz-lat2)", "cs", "cz-lat2" },
    { "Czechoslovakian (cs, cz-lat2-prog)", "cs", "cz-lat2-prog" },
    { "Danish (dk, dk)", "dk", "dk" },
    { "Danish (dk, dk-latin1)", "dk", "dk-latin1" },
		{ "Estonian (ee, et)", "et", "et" },
    { "Finnish (fi, fi)", "fi", "fi" },
    { "Finnish (fi, fi-latin1)", "fi", "fi-latin1" },
    { "French (fr, fr)", "fr", "fr" },
    { "French (fr, fr-latin0)", "fr", "fr-latin0" },
    { "French (fr, fr-latin1)", "fr", "fr-latin1" },
    { "French (fr, fr-pc)", "fr", "fr-pc" },
    { "German (de, de)", "de", "de" },
    { "German (de, de-latin1)", "de", "de-latin1" },
    { "German (de, de-latin1-nodeadkeys)", "de", "de-latin1-nodeadkeys" },
    { "Hungarian (hu, hu)", "hu", "hu" },
    { "Hungarian (fr_CH, fr_CH)", "fr_CH", "fr_CH" },
    { "Hungarian (fr_CH, hu101)", "fr_CH", "hu101" },
    { "Italian (it, it)", "it", "it" },
    { "Italian (it, it-ibm)", "it", "it-ibm" },
    { "Italian (it, it2)", "it", "it2" },
    { "Japanese (jp, jp106)", "jp", "jp106" },
    { "Norwegian (no, no)", "no", "no" },
    { "Norwegian (no, no-latin1)", "no", "no-latin1" },
    { "Polish (pl, pl)", "pl", "pl" },
    { "Portugese (pt, pt-latin1)", "pt", "pt-latin1" },
		{ "Russian (ru, KOI8-R)", "ru", "ru" },
		{ "Russian (ru, ru1)", "ru", "ru1" },
		{ "Russian (ru, ru2)", "ru", "ru2" },
		{ "Russian (ru, ru-ms)", "ru", "ru-ms" },
		{ "Russian (ru, ru_win)", "ru", "ru_win" },
		{ "Russian (ru, ru-cp1251)", "ru", "ru-cp1251" },
    { "Spanish (es, es)", "es", "es" },
    { "Swedish (se, se-latin1)", "sv", "se-latin1" },
    { "Swedish (se, se)", "sv", "se" },
		{ "Swiss German (de_CH, de_CH)", "de_CH", "de_CH" },
		{ "Thai (th, th)", "th", "th" },
    { "Turkish (tr, trq)", "tr", "trq" },
    { "United Kingdom (gb, us)", "gb", "us" },
    { "U.S. English (us, us)", "us", "us" } };

// keyboard models
struct modelsTable {
    char * name, * type;
} keyboardModels[] = {
  { "Generic 101-key PC", "pc101" },
  { "Generic 102-key Intl PC", "pc102" },
  { "Generic 104-key PC", "pc104" },
	{ "Dell 101-key PC", "dell101" },
  { "Everex STEPnote", "STEPnote" },
	{ "Keytronic FlexPro", "FlexPro" },
  { "Microsoft Natural", "microsoft" },
  { "Northgate OmniKey 101", "omnikey101" },
  { "Winbook Model XP5", "XP5" },
	{ "Japanese 106-key", "jp106" } };

Keyboard::Keyboard( QWidget * parent, const char * name )
    : GBLIPage( parent, name )
{
		// these directories are needed to set the keyboard and test it
		if( !QDir( "/var/lib/xkb" ).exists() ) {
	    mkdir( "/var", 0755 );
  	  mkdir( "/var/lib", 0755 );
    	mkdir( "/var/lib/xkb", 01755 );
		}
		
    QHBoxLayout * lh = new QHBoxLayout( this, 0, 6 );

    QVBoxLayout * l = new QVBoxLayout( 6 );
		lh->addStretch( 1 );
    lh->addLayout( l, 8 );

    QLabel* label = new QLabel( tr("Select model:"), this );
		label->setAutoMask( true );
		l->addSpacing( 10 );
    l->addWidget( label );
    listModel = new QListBox( this );
    l->addWidget( listModel, 10 );

    label = new QLabel( tr("Select Layout:"), this );
		label->setAutoMask( true );
    l->addWidget( label );
    listLayout = new QListBox( this );
    l->addWidget( listLayout, 10 );

    label = new QLabel( tr("Test here:"), this );
		label->setAutoMask( true );
    l->addWidget( label );
    editTest = new QLineEdit( this );
    l->addWidget( editTest );
		l->addSpacing( 10 );

		lh->addStretch( 1 );

    setLayout = -1;
    setModel = -1;

    listModel->setFocus();

    initialized = FALSE;
}

void Keyboard::showEvent( QShowEvent * )
{
    init();
}

void Keyboard::hideEvent( QHideEvent * )
{
    set();
}

void Keyboard::init()
{
    if (initialized)
			return;
    initialized = TRUE;

		int iMax = sizeof( keyboardModels ) / sizeof( struct modelsTable );
		for( int i = 0; i < iMax; i++ ){
			listModel->insertItem( QString( keyboardModels[i].name ) );
		}

		iMax = sizeof( defaultKeyboards ) / sizeof( struct keyboardsTable );
		for( int i = 0; i < iMax; i++ ){
			listLayout->insertItem( QString( defaultKeyboards[i].name ) );
		}

    listModel->setCurrentItem( 1 );
    listLayout->setCurrentItem( 0 );

		// find a layout for current language
		char short_lang[5];
		strcpy( short_lang, Language::lang->key->ascii() );
		int k = 0;
    for( ; k < iMax; k++ )
			if( !strcasecmp( defaultKeyboards[k].lang, short_lang ) ){
		    listLayout->setCurrentItem( k );
				break;
			}
		if( k == iMax ){
	    listLayout->setCurrentItem( iMax-1 );
		}

    editTest->installEventFilter( this );
		set();
		emit statusChanged( GBLIPage::Good );
}

// setup selected layout when user enters the test field
bool Keyboard::eventFilter( QObject *o, QEvent * e)
{
    if ( o == editTest && e->type() == QEvent::FocusIn )
			set();
    return FALSE;
}

void Keyboard::set( char *a_layout )
{
    if ( listLayout->currentItem() == setLayout &&
				 listModel->currentItem() == setModel ){
Log::logMain(32, "keyboard: nothing to change\n" );
			return;
		}
    setModel = listModel->currentItem();
    setLayout = listLayout->currentItem();

		char *layout = a_layout; // override choosen
		if( !layout ){
			layout = new char[8];
			strcpy( layout, (char*)xkb().latin1() ); //defaultKeyboards[setLayout].keyboard;
		}
    QApplication::flushX();
    int pid = ::fork();
    if (pid == 0) {
			execlp("setxkbmap", "setxkbmap",
	      "-rules", "xfree86",
	      "-model", keyboardModels[setModel].type,
	      "-layout", layout,
#ifdef GBLI_DEVEL
	      "-display","127.0.0.1:0", 0 );
#else
	      "-display","127.0.0.1:1", 0 );
#endif
			exit(1);
    }
Log::logMain(0, "setxkbmap -rules xfree86 -model %s -layout %s -display 127.0.0.1:1\n", keyboardModels[setModel].type, layout );

//printf( "Keyboard::set pid: %d\n", pid );
    ::waitpid(pid, NULL, 0);
}

QString Keyboard::layout()
{
	if( setLayout < 0 )
    return QString( "us" );
	else
    return QString( defaultKeyboards[setLayout].keyboard );
}

QString Keyboard::xkb()
{
	if( setLayout < 0 )
    return QString( "us" );
	else{
		char buf[8];
		strcpy( buf, defaultKeyboards[setLayout].lang );
		// for Etonian and Swedish layout name doesn't match with langauge name
		if( !strcmp( defaultKeyboards[setLayout].lang, "et" ) )
			strcpy( buf, "ee" );
		else if( !strcmp( defaultKeyboards[setLayout].lang, "sv" ) )
			strcpy( buf, "se" );
    return QString( buf );
	}
}

QString Keyboard::model()
{
	if( setModel < 0 )
    return QString( "pc102" );
	else
    return QString( keyboardModels[setModel].type );
}

void Keyboard::finish()
{
	::mkdir( (NEWROOT +"/etc/sysconfig").ascii(), 0755 );
	system( ("touch "+ NEWROOT +"/etc/sysconfig/keyboard").ascii() );
	SysConfigFile config( NEWROOT.ascii(), "/etc/sysconfig/keyboard" );
	config.load();
	if (layout() == "fi")  //Added by dk
  	config.set( "KEYTABLE", "fi-latin9" ); /*dk*/
  else  /*dk*/
		config.set( "KEYTABLE", defaultKeyboards[setLayout].keyboard );
	config.save();
}
