/***************************************************************************
                               language.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "config.h"
#include "language.h"
#include "xserver.h"
#include "liconloader.h"
#include "configfile.h"
#include "sysconfigfile.h"
#include "log.h"

#include <unistd.h>
#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <libintl.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlist.h>
#include <qstring.h>
#include <qfile.h>
#include <qdir.h>
#include <qfont.h>
#include <qfontinfo.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qapplication.h>
#include <qtextcodec.h>
#include <qcombobox.h>
#include <qmessagebox.h>

QList<langInfo> Language::langs;
langInfo* Language::lang = NULL;

// defaults, changed in main.cpp
int Language::iBigFontSize = 24;
int Language::iNormalFontSize = 20;
int Language::iSmallFontSize = 17;

Language::Language( QWidget * parent, const char * name )
    : QWidget( parent, name ), blGMT( false )
{
	setFocusPolicy( QWidget::StrongFocus );

	setFont( QFont::defaultFont() );


#ifdef QT_THREAD_SUPPORT
	changecount = new QSemaphore( 1 );
#endif

	this->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

  QVBoxLayout * l  = new QVBoxLayout( 5 );
  QHBoxLayout *lrh = new QHBoxLayout( 5 );
  QHBoxLayout* h = new QHBoxLayout( this, 0, 6 );

  l->addStretch( 1 );
  l->addLayout( lrh, 8 );
  l->addStretch( 1 );

//	h->addSpacing( 10 );

  QVBoxLayout * l1  = new QVBoxLayout( 2 );
	l1->addStretch( 1 );
	lrh->addStretch( 1 );
	lrh->addLayout( l1, 1 );

  world = new World( this );
	l1->addStretch( 1 );
  l1->addWidget( world );

//	l1->addSpacing( 2 );
  comboZone = new QComboBox( false, this );
  l1->addWidget( comboZone );
	l1->addSpacing( 5 );
  comment = new QLabel( "", this );
  comment->setAlignment( AlignTop );
  QFont commentFont = comment->font();
  commentFont.setItalic( TRUE );
  comment->setFont( commentFont );
  l1->addWidget( comment, 2 );
	comment->setAutoMask( true );

#ifdef TIME_FIELD
 	QLabel *lab = new QLabel( tr( "Enter your current time:" ), this );
 	lab->setAutoMask( true );
 	curTime = new QLineEdit( "XX:XX", this );
 	curTime->setMinimumWidth( 50 );
 	curTime->setMaximumWidth( 50 );
 	curTime->setMaxLength( 5 );
 	updateTimer = new QTimer( this );
 	connect( updateTimer, SIGNAL( timeout() ), this, SLOT( updateTimeField() ) );
#else
	chkGMT = new QCheckBox( tr( "Hardware clock runs in Greenwich Mean Time (GMT)" ), this );
 	chkGMT->setAutoMask( true );
 	connect( chkGMT, SIGNAL( toggled(bool) ), this, SLOT( setGMT(bool) ) );
#endif	

	l1->addStretch( 1 );

 	QHBoxLayout * lhr = new QHBoxLayout( 6 );
 	l1->addLayout( lhr );
#ifdef TIME_FIELD
 	lhr->addWidget( lab, 1 );
 	lhr->addWidget( curTime, 1 );
#else
 	lhr->addWidget( chkGMT, 1 );
#endif
 	lhr->addStretch( 10 );

	l1->addStretch( 1 );

  connect( world, SIGNAL( selected(const QString&) ),
     this, SLOT( setTimezone(const QString&) ) );

  connect( comboZone, SIGNAL( activated(const QString&) ),
     world, SLOT( highlight(const QString&) ) );

  connect( comboZone, SIGNAL( activated(const QString&) ),
     this, SLOT( setTimezone(const QString&) ) );

  Language::loadLanguages();

  lboxLangs = new QListBox( this );
	lboxLangs->setAutoMask( true );
// For screenshots
  QString strBeta( " (todo)"/*"       "*/ );
  QListIterator<langInfo> it( langs );
  langInfo *curLang = NULL;
	QString str1;
	QString flag;
  while( (curLang = it.current()) ){
    str1 = (curLang->beta? (*curLang->lang + strBeta): *curLang->lang);
		if( !strcmp( curLang->key->ascii(), "sv" ) )
			flag = "/usr/share/locale/l10n/se/flag.png";
		else if ( !strcmp( curLang->key->ascii(), "uk" ) )//Added by dk	
			flag = "/usr/share/locale/l10n/ua/flag.png";
		else
			flag = "/usr/share/locale/l10n/"+ QString::fromLatin1( *curLang->key ) +"/flag.png";
		if( QFile( flag ).exists() )
			lboxLangs->insertItem( QPixmap( flag ), str1 );
    else
      lboxLangs->insertItem( str1 );
    ++it;
  }
  QFont f( "times" );
  f.setPointSize( Language::iNormalFontSize );
  f.setCharSet( QFont::ISO_8859_1 );
	lboxLangs->setFont( f /*QFont::defaultFont()*/ );

	lboxLangs->setFocusPolicy( QWidget::StrongFocus );

  lboxLangs->setMinimumHeight( 332/*neededHeight*/ );
  lboxLangs->setMinimumWidth( lboxLangs->sizeHint().width() + 30 );
  lboxLangs->setMaximumWidth( lboxLangs->sizeHint().width() + 60 );

  lrh->addStretch( 1 );
  lrh->addWidget( lboxLangs );
  lrh->addStretch( 1 );

  connect( lboxLangs, SIGNAL(highlighted(int)), this, SLOT(setSelected(int)) );

  h->addLayout( l );

  prepared = FALSE;
	prepare();

	setTimezone( timezone() );
}

// load zone-language translation table
void Language::prepare(){
    if ( prepared )
			return;
    prepared = TRUE;

    world->readInZoneInfo( comboZone );
    world->highlight( timezone() );

    QFile Tlangs( QString( GBLI_DATA_PATH ) +"/data/zone_language.tab" );
    if( Tlangs.open( IO_ReadOnly ) ){
      QTextStream TSlangs1( &Tlangs );
  		QString temp;
  		int pos;
  		while( (temp = TSlangs1.readLine()).length() ){
  			if( (pos = temp.find( "," )) > 0 && temp.left(1) != "#" ){
  				t2l.insert( temp.right( temp.length() - pos - 1 ), new QString( temp.left( pos ) ) );
//printf( "%s - %s\n", temp.left( pos ).ascii(), temp.right( temp.length() - pos - 1 ).ascii() );
  			}
  		}
  	}
}

// set appropriate zone for the selected city
void Language::setTimezone( const QString& z ){
    if (comboZone->currentText() != z ) {
    	for (int i = 0; i < comboZone->count(); i++) {
        if (z == comboZone->text(i) ) {
		   		comboZone->setCurrentItem( i );
        }
    	}
    }

    comment->setText( this->wrapString(world->getComment( z ), 48 ));

		QString *str1 = t2l.find( z );
		//Added by dk
		/*dk*/
		if ( str1 )
		{
			if ( *str1 == QString( "uk" ) )
			{
    	  *str1 = QString( "UA" );
			}
		}
		/*dk*/
//printf( "%s\n", (*str1).ascii() );
		int eng = -1;
		int iLang = -1;
		int iMax = Language::langs.count();
		for( int k = 0; k < iMax; k++ ){
			QString str = *Language::langs.at( k )->key;
//printf( "%d - %s\n", k, (str).ascii() );
			if( str1 && str == *str1 ){
		    lboxLangs->setCurrentItem( k );
				iLang = k;
				break;		
			}
			if( str == "en" )
				eng = k;
		}
		if( iLang < 0 && eng >= 0 )
	    lboxLangs->setCurrentItem( eng );

		finishTime();
#ifdef TIME_FIELD
		updateTimeField();
#endif
}

QString Language::timezone(){
    return comboZone->currentText();
}

// tried to use crazy technology of guessing time by user input
// but it is absolutely fuzzy, so made it simple: select zone and GMT
void Language::updateTimeField(){
#ifdef TIME_FIELD
Log::logMain(32, "update time field entered\n" );
//	if( timechange.locked() ) return;
#ifdef QT_THREAD_SUPPORT
	(*changecount)++;
#endif
Log::logMain(32, "update time field\n" );
	//	timechange.lock();

	QString curT = curTime->text();
	QTime tim = QTime::currentTime();
	
	printf( "%s\n", tim.toString().latin1() );
	system( "date" );	

	// lets calculate difference
	if( curT != "XX:XX" ){
		int min, hour;

		if( QFile( "/etc/localtime.real" ).exists() ){
			system( "cp -a /etc/localtime /etc/localtime.gmt" );
			system( "cp -a /etc/localtime.real /etc/localtime" );
			system( ( QString("hwclock --hctosys --%1").arg(blGMT?"utc":"localtime") ).latin1() );			
		}

//setenv( "TZ", "Europe/Tallinn -120", 1 );
//		tzset();

  	struct timeb timestr;
  	ftime( &timestr );
  	int curzone = timestr.timezone;
printf( "Timezone: %d\n", curzone );
//printf( "Timezone2: %d\n", timezone );

		if( QFile( "/etc/localtime.gmt" ).exists() ){
			system( "cp -a /etc/localtime /etc/localtime.real" );
			system( "cp -a /etc/localtime.gmt /etc/localtime" );
			system( ( QString("hwclock --hctosys --%1").arg(blGMT?"utc":"localtime") ).latin1() );
		}

    int fd[2];
    ::pipe(fd);
    int pid = ::fork();
    if( !pid ){
			::close( 0 );
    	dup2( fd[1], 2 );
    	dup2( fd[1], 1 );
    	::open( "/dev/null", O_RDWR );
    	::dup( 0 );
    	::dup( 0 );    	
			execlp( "hwclock", "hwclock", "--show", "--localtime", 0 );
    	exit(1);
    }
    ::close( fd[1] );

    QFile f;
    if ( f.open(IO_ReadOnly, fd[0]) ) {    // file opened successfully
     	QTextStream t( &f );	    // use a text stream
    	QString result = t.read();
			char buf[32];
			sscanf( result.ascii(), "%s %s %s %d:%d", buf, buf, buf, &hour, &min );
     	::close( fd[0] );
    }

    ::waitpid(pid, NULL, 0);

Log::logMain(32, "HW: %d:%d\n", hour, min );
		int HMin = hour * 60 + min;
//printf( "HW: %d\n", HMin );

		int curH = 0, curM = 0;
		sscanf( curT.latin1(), "%d:%d", &curH, &curM );
Log::logMain(32, "US: %d:%d\n", curH, curM );

		int UMin = curH * 60 + curM;
//printf( "USER: %d\n", UMin );

		int diff = UMin - HMin + (blGMT?curzone:0);
		bool oldGMT = blGMT;
Log::logMain(32, "diff: %d, GMT: %d\n", diff, blGMT );
		if( abs(diff) > 1 ){
			if( !blGMT && ( (diff>0 && curzone<0) || (diff<0 && curzone>0) ) ){
//printf( "case0" );
				if( abs(diff) > abs(curzone)/2 ){
//printf( " - 1" );
					blGMT = true;
					diff += curzone;
				}
//printf( "\n" );
			}else if( blGMT ){
//printf( "case1" );
// let's leave it in localtime
				if( abs(diff) > abs(curzone)/2 ){
//printf( " - 1" );
					blGMT = false;
					diff -= curzone;
				}
//printf( "\n" );
			}

			if( oldGMT != blGMT ) finishTime();

			if( abs(diff) > 1 ){
				int HDiff = diff / 60;
				int MDiff = diff % 60;
				hour += HDiff;
				min += MDiff;
				if( min < 0 ){ min += 60; hour--; }
				if( hour < 0 ) hour += 24;
Log::logMain(32, "DIFF: %d:%d\n", HDiff, MDiff );
				system( ( QString("hwclock --set --date=\"%1:%2\" --%3").arg(hour).arg(min).arg(blGMT?"utc":"localtime") ).latin1() );
				system( ( QString("hwclock --hctosys --%1").arg(blGMT?"utc":"localtime") ).latin1() );
			}
  	}
	}

	// show final time
	QTime tim_final = QTime::currentTime();
	int min_final = tim_final.minute();
	curTime->setText( QString::number( tim_final.hour() ) +":"+
		(min_final<10?"0":"")+ QString::number( min_final ) );

#ifdef QT_THREAD_SUPPORT
	(*changecount)--;
#endif
//	timechange.unlock();
printf( "update time field finished\n\n" );
#endif
}

// load languages table
void Language::loadLanguages()
{
  QFile Flangs( QString( GBLI_DATA_PATH ) +"/data/languages.tab" );
  if( Flangs.open( IO_ReadOnly ) ){
    QTextStream TSlangs( &Flangs );
		QString temp;
		int pos, posPrev;
		langInfo *tempLang = NULL;
	  QString envLang = QString::fromLatin1( getenv( "LANG" ) );
	  if( envLang.isEmpty() || envLang == "C" ) envLang = "en";
//printf( "default lang=%s\n", envLang.ascii() );
		while( (temp = TSlangs.readLine()).length() ){
      if( temp.simplifyWhiteSpace().left( 1 ) == "#" ) continue;
		  tempLang = new langInfo;
      pos = posPrev = 0;
      pos = temp.find( ',', pos );
      tempLang->lang = new QString( temp.mid( posPrev, pos-posPrev ).simplifyWhiteSpace() );
      posPrev = ++pos;
      pos = temp.find( ',', pos );
      tempLang->key = new QString( temp.mid( posPrev, pos-posPrev ).simplifyWhiteSpace() );
      posPrev = ++pos;
      pos = temp.find( ',', pos );
      tempLang->font = new QString( temp.mid( posPrev, pos-posPrev ).simplifyWhiteSpace() );
      posPrev = ++pos;
      pos = temp.find( ',', pos );
      tempLang->map = new QString( temp.mid( posPrev, pos-posPrev ).simplifyWhiteSpace() );
      posPrev = ++pos;
      pos = temp.find( ',', pos );
      tempLang->lc_all = new QString( temp.mid( posPrev, pos-posPrev ).simplifyWhiteSpace() );
      posPrev = ++pos;
      pos = temp.find( ',', pos );
      tempLang->charset = new QString( temp.mid( posPrev, pos-posPrev ).simplifyWhiteSpace() );
      posPrev = ++pos;
      pos = temp.find( ',', pos );
      tempLang->beta = temp.mid( posPrev, pos-posPrev ).simplifyWhiteSpace().toInt();
      posPrev = ++pos;

      pos = temp.find( ',', pos );
      tempLang->kimap = new QString( temp.mid( posPrev, pos-posPrev ).simplifyWhiteSpace() );
      posPrev = ++pos;

      tempLang->sKDEFont = new QString( temp.mid( posPrev, temp.length()-posPrev ).simplifyWhiteSpace() );

			

      if( *tempLang->key == envLang )
        Language::lang = tempLang;
      langs.append( tempLang );
    }
    Flangs.close();
  }else
		Log::logMain( 32, "couldn't find language definitions!\n" );
  if( !Language::lang )
    Language::lang = (langInfo*)Language::langs.at( 0 );
  else
    Language::langs.find( Language::lang );

}

void Language::showEvent( QShowEvent * )
{
    lboxLangs->setCurrentItem( Language::langs.at() );
  	lboxLangs->setFocus();
#ifdef TIME_FIELD
		updateTimeField();
		updateTimer->start( 20000 );
#endif
}

void Language::setSelected( int arg )
{
  Language::lang = (langInfo*)Language::langs.at( arg );
}

// set the font specific for every language
QFont::CharSet Language::setupFont( QFont *a_font )
{
  QFont::CharSet chars = QFont::ISO_8859_1;
  if( !Language::lang ) return chars;

  char buf[32];
  strcpy( buf, Language::lang->charset->ascii() );
  if( !strcmp( buf, "iso8859-1" ) ){
      chars = QFont::ISO_8859_1;
//			a_font->setFamily( "helvetica" );
//			a_font->setFamily( "fixed" );
			a_font->setFamily( "times" );
//			a_font->setPointSize( 20 );
  }else if( !strcmp( buf, "iso8859-2" ) ){
      chars = QFont::ISO_8859_2;
			a_font->setFamily( "times" );
//			a_font->setPointSize( 20 );
  }else if( !strcmp( buf, "koi8-r" ) ){
      chars = QFont::KOI8R;
			a_font->setFamily( "times" );
//			a_font->setPointSize( 20 );
      qApp->setDefaultCodec( QTextCodec::codecForName("KOI8-R"/*, 3*/) );
  }else if( !strcmp( buf, "iso8859-9" ) ){
      chars = QFont::ISO_8859_9;
			a_font->setFamily( "times" );
//			a_font->setPointSize( 20 );
  }else if( !strcmp( buf, "koi8-u" ) ){
      chars = QFont::KOI8U;
			a_font->setFamily( "times" );
//			a_font->setPointSize( 20 );
      qApp->setDefaultCodec( QTextCodec::codecForName("KOI8-U"/*, 3*/) );
	}
  a_font->setCharSet( chars );
	a_font->setPointSize( Language::iNormalFontSize );
//printf( "Set font: %s, %d!\n", a_font->family().ascii(), a_font->pointSize() );
//printf( "Set actual font: %s, %d, %d!\n", QFontInfo( *a_font ).family().ascii(), QFontInfo( *a_font ).pointSize(), (int)QFontInfo( *a_font ).charSet() );
	QFont::setDefaultFont( QFont(*a_font) );
  return chars;
}

// write time info
// writes it to the ramdisk, later this data is copied by real finish()
void Language::finishTime(){
//printf( "--------------\n" );
//printf( "finish time\n" );

  // write the entry in /etc/sysconfig/clock
	system( "touch /etc/sysconfig/clock" );
  SysConfigFile config2( "", "/etc/sysconfig/clock" );
  config2.load();
  config2.set( "UTC", blGMT?"true":"false", false );
  config2.set( "ARC", "false", false );
  config2.set( "ZONE", timezone().ascii() );
  config2.save();

	// copy zone info file
	QString zonefile = "/usr/share/zoneinfo/"+ timezone();

	if( timezone().length() && QFile( zonefile ).exists() ){
		system( "rm -rf /etc/localtime*" );
		if( !blGMT ){
			system( ("cp -a "+ zonefile +" /etc/localtime.real").latin1() );
			system( "cp -a /usr/share/zoneinfo/GMT /etc/localtime" );
		}else{
			system( ("cp -a "+ zonefile +" /etc/localtime").latin1() );
		}
		::sync();
		unsetenv( "TZ" );
		system( ( QString("hwclock --hctosys --%1").arg(blGMT?"utc":"localtime") ).latin1() );
		tzset();

  	struct timeb timestr;
  	ftime( &timestr );
//printf( "timezone: %d\n", timestr.timezone );

//tim = QTime::currentTime();
//printf( "curTime: %s\n", tim.toString().latin1() );

    int change_h = timestr.timezone / 60;
    int change_m = abs(timestr.timezone % 60);
    char buf[64];
    sprintf( buf, "GMT %d:%d", change_h, change_m );
//    printf( "string: %s\n", buf );
    setenv( "TZ", buf, 1 );
    tzset();

	}
	emit updateTime();
//printf( "end finish time\n" );

	// ispell check(GB or USA)
	QString	tz = timezone();
	QString	country = tz.left( tz.find('/') );

	if ( country == "America" )
		system( "ln -sf american.hash /tmp/english.hash" );
	else
		system( "ln -sf british.hash /tmp/english.hash" );
}

// write language locales, now also saves the time settings
void Language::finish( bool script ){
	char buf[16];
	int iPathLen = readlink(  "/tmp/default", buf, 16 );
	if( iPathLen != -1 ){
		if( !QDir( NEWROOT +"/usr/share/doc/HTML/" ).exists() )
			system( ("mkdir -p -m0775 "+ NEWROOT +"/usr/share/doc/HTML/").ascii() );
		system( ("cp -af /tmp/default "+ NEWROOT +"/usr/share/doc/HTML/").ascii() );
	}

#ifdef GBLI_DEVEL
  ::unlink( "/tmp/locale.sh" );
#endif

	unsigned int i18nSize = QFileInfo( NEWROOT + "/etc/sysconfig/i18n" ).size();

	ConfigFile config( script?"":NEWROOT.ascii(), script?"/tmp/locale.sh":"/etc/sysconfig/i18n" );
  FILE *fp = config.open( script?"w":"a" );

  char prefix[16];
  memset( prefix, 0, sizeof( prefix ) );

	//Added by dk
	/*dk*/
	char euro[16];
	memset ( euro, 0, sizeof( euro ) );
	if (*Language::lang->key == "fi")
		strcpy(euro,"@euro");
	/*dk*/

	if( fp ){
		if( i18nSize )// if not empty
			fprintf( fp, "\n" );
    if( script ){
			fprintf( fp, "#!/bin/sh\n" );
			strcpy( prefix, "export " );
	  }
    if( true /**Language::lang->key != "en"*/ ){// do we need to change default?	
			//old was 'key' for LANG & LINGUAS, let's try 'lc_all'
			if( script )
    		fprintf( fp, "%sLANG=%s\n", prefix, Language::lang->key->ascii() );
			else
    		fprintf( fp, "%sLANG=%s%s\n", prefix, Language::lang->lc_all->ascii(), euro );
    	fprintf( fp, "%sLINGUAS=%s%s\n", prefix, Language::lang->lc_all->ascii(), euro );
    	
    	fprintf( fp, "%sLANGUAGE=%s%s\n", prefix, Language::lang->lc_all->ascii(), euro );
    	fprintf( fp, "%sLOCALE=%s%s\n", prefix, Language::lang->lc_all->ascii(), euro );

		  if (*Language::lang->key == "fi") //Added by dk
      	fprintf( fp, "%sSYSFONT=lat0-16\n", prefix ); /*dk*/
		  else if( Language::lang->font->length() ) //modified by dk - added else
      	fprintf( fp, "%sSYSFONT=%s\n", prefix, Language::lang->font->ascii() );

		  if (*Language::lang->key == "fi") //Added by dk		
				fprintf( fp, "%sSYSFONTACM=iso16\n", prefix );	/*dk*/	
      else if( Language::lang->map->length() ) //modified by dk - added else
       	fprintf( fp, "%sSYSFONTACM=%s\n", prefix, Language::lang->map->ascii() );

  	  // ABSOLUTELY not sure about the following
    	fprintf( fp, "%sLC_ALL=%s%s\n", prefix, Language::lang->lc_all->ascii(), euro );

  	  // not sure about the following
    	fprintf( fp, "%sLC_COLLATE=%s%s\n", prefix, Language::lang->lc_all->ascii(), euro );
    	fprintf( fp, "%sLC_MONETARY=%s%s\n", prefix, Language::lang->lc_all->ascii(), euro );  	
    	fprintf( fp, "%sLC_TIME=%s%s\n", prefix, Language::lang->lc_all->ascii(), euro );

    	// strange hacks, heh ;o)    	
    	if( *Language::lang->key == "ru" || *Language::lang->key == "uk_UA" || *Language::lang->key == "uk" ){
        fprintf( fp, "%sLC_NUMERIC=POSIX\n", prefix );// someone says, it should be this way
        fprintf( fp, "%sLC_MESSAGES=en_GB\n", prefix );
      }else{
      	fprintf( fp, "%sLC_NUMERIC=%s\n", prefix, Language::lang->lc_all->ascii() );
      	fprintf( fp, "%sLC_MESSAGES=%s\n", prefix, Language::lang->lc_all->ascii() );
      }

    	//fprintf( fp, "%sLC_CTYPE=en_GB\n", prefix ); // lets test it this way
				fprintf( fp, "%sLC_CTYPE=%s%s\n", prefix ,Language::lang->lc_all->ascii(), euro); // lets test it this way
    }
  	fclose( fp );
  	
  	if( script )
  	  ::chmod( "/tmp/locale.sh", 0775 );
	}else
		Log::logMain(32, "Couldn't open locales configuration for writing!\n" );

	system( ("ln -sf "+ *Language::lang->key +" /tmp/default").ascii() );

	if ( !script )
	{
		if ( QDir( NEWROOT + "/usr/lib/ispell" ).exists() )
			system( ("cp -df /tmp/english.hash " + NEWROOT + "/usr/lib/ispell/").ascii() );
	}
}


QString Language::wrapString(QString s, int maxlen){
	if ((int)s.length() > maxlen)
	{
		int pos = 0;
		while ( (s.find(" ", pos) <= maxlen ) && (s.find(" ", pos) != -1) )
			pos = s.find(" ", pos + 1);
		if ( pos > 0 ) s.replace( pos, 1, "\n" );
	}
	
	return s;
}
