/****************************************************************************
**
** $Id: lispdb.cpp,v 1.5 2000/02/16 12:34:11 mhk Exp $
**
** Copyright (C) 1999 Calderasystems, Inc
** All rights reserved.
**
** This file is part of the Caldera OpenLinux Installer.
** Original Implementation: 1999 by Troll Tech AS
**
** This file may be distributed under the terms of the Q Public License
** as defined by Troll Tech AS of Norway and appearing in the file
** LICENSE included in the packaging of this file.
**
*****************************************************************************/


#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qlistview.h>


#include "lglobal.h"
#include "lispdb.h"
#include "llog.h"


void LISPItem::activate()
{
  setOpen( TRUE );
  QListViewItem * i = firstChild();
  QListView * lv = listView();
  if ( i && lv )
    lv->setSelected( i, TRUE );
}

                                                                                                                
LISPDB::LISPDB(QString filename)
{
  LISP *entry;

  db.setAutoDelete(TRUE);

  QFile file(filename);
  if (file.open(IO_ReadOnly))
    {
      QTextStream is(&file);

      QRegExp white("\\s+");
      int     pos, len;
      QString key, val;

      entry = new LISP;
      while (!is.eof())
	{
	  QString line = is.readLine().stripWhiteSpace();
	  if (line.left(1) == "#")
	    continue;

	  if (line.isEmpty() && !entry->name().isEmpty())
	    {
	      if (!entry->password().isEmpty() || !entry->user().isEmpty())
		entry->setCallByCall(true);
	      db.append(entry);
	      entry = new LISP;
	      continue;
	    }
	  if (line.isEmpty())
	    continue;

	  pos = white.match(line, 0, &len);
	  if (pos > 0)
	    {
	      key = line.left(pos).lower();
	      val = line.mid(pos+len);

	      if (key == "phone")
		entry->setPhone(val);
	      if (key == "login")
		entry->setUser(val);
	      if (key == "password")
		entry->setPassword(val);
	      if (key == "smtp")
		entry->setMail(val);
	      if (key == "proxy")
		entry->setProxy(val);
	      if (key == "name")
		{
		  QStringList cat;
		  while ((pos = val.find(",")) > 0)
		    {
		      cat.append(val.left(pos).stripWhiteSpace());
		      val = val.mid(pos+1);
		    }
		  entry->setCategory(cat);
		  entry->setName(val.stripWhiteSpace());
				 
		}
	      if (key == "DNS")
		{
		  pos = val.find(" ");
		  if (pos > 0)
		    {
		      entry->setDNS1(val.left(pos).stripWhiteSpace());
		      entry->setDNS2(val.mid(pos+1).stripWhiteSpace());
		    }
		  else
		    entry->setDNS1(val.stripWhiteSpace());
		}
	    }
	}      
    }  
}


QListViewItem *findItem(QListViewItem *item, QString name)
{
  while (item)
    {
      if (item->text(0) == name)
	return item;
      item = item->nextSibling();
    }

  return 0;
}



QListViewItem *getGroupItem(QListView *lv, const QStringList& groups)
{
  QListViewItem *parent, *item = 0;

  QStringList::ConstIterator it;
  for (it=groups.begin(); it != groups.end(); ++it)
    {
      parent = item;
      if (!item)
	item = lv->firstChild();
      else
	item = item->firstChild();
      
      item = findItem(item, *it);
      if (!item)
	{
	  // create new branch
	  if (parent)
	    item = new QListViewItem(parent);
	  else
	    item = new QListViewItem(lv);
	  item->setText(0, *it);
	}      
    }

  return item;
}


void LISPDB::fillListView(QListView *lv, LISPItem *u)
{
  user = u;
	
  QListIterator<LISP> it(db);
  for (; it.current(); ++it)
    {
      QListViewItem *item = getGroupItem(lv, it.current()->category());
      if (item)
	{
	  item = new LISPItem(item, it.current());
	  item->setText(0, it.current()->name());
	} 
    }

  lv->setSelected(user, true);
}


LISP *LISPDB::getEntry(QListViewItem *item)
{
  if (item == user)
    return ((LISPItem*)user)->lisp();

  if (item->childCount() != 0)
    return 0;

  return ((LISPItem*)item)->lisp();
}

void LISPDB::addEntry(LISP* pNewEntry)
{
	db.append(pNewEntry);
}
