/***************************************************************************
			                            log.cpp
                             -------------------                                         
    begin                : Sun Oct 31 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include <qapplication.h>

#include "log.h"
#include "config.h"
#include "upgradepage.h"

FILE *Log::mainlog = NULL;
int Log::iDebugLevel = 0; // less number - less messages (old 2)
int Log::iMaxDebug = 3;
char *Log::mainLogName = "install.log";

Log::Log( bool a_blUseLocal, char * a_szName )
{
	char name[128];;
	memset( name, 0, sizeof( name ) );
	logfile = 0;

  if( a_blUseLocal ){
		strcpy( name, a_szName?a_szName:"debug.log" );
  }else{
		strcpy( name, a_szName?a_szName:"/dev/tty3" );
  }
	logfile = fopen( name, "a+" );
	if( !logfile )
		::printf( "Error on log file opening (%s) - no log will be made!\n", name );
}

Log::~Log()
{
  Log::close();
}

void Log::log( int a_iDebugLevel, const char * s, ... )
{
	if( (a_iDebugLevel & 0x1F) > iDebugLevel ) return;
  va_list args;
  va_start( args, s );
  doLog( s, args );
	if( a_iDebugLevel & 0x20 )
		printfLog( s, args );
  va_end( args );
}

void Log::doLog( const char * s, va_list args )
{
  if( !logfile ) return;
	int len = strlen( s );
  fprintf( logfile, "* " );
  vfprintf( logfile, s, args );
	if( s[len-1] != '\n' )
	  fprintf( logfile, "\n" );
  fflush( logfile );
}

void Log::close()
{
  if( logfile ){
		fclose( logfile );
		logfile = NULL;
  }
}

void Log::printfLog( const char * s, va_list args )
{
	fprintf( stdout, "* " );
	vfprintf( stdout, s, args );
	int len = strlen( s );
	if( s[len-1] != '\n' )
		fprintf( stdout, "\n" );
	fflush( stdout );
}


void Log::logMain( int a_iDebugLevel, const char * s, ... )
{
	if( (a_iDebugLevel & 0x1F) > Log::iDebugLevel ) return;
  va_list args;
  va_start( args, s );
  Log::doLogMain( s, args );
	if( a_iDebugLevel & 0x20 )
		Log::printfLogMain( s, args );
  va_end( args );
}

void Log::doLogMain( const char * s, va_list args )
{
	if( !s ) return;
  if( !mainlog ){
//		::printf( "trying to open mainlog!\n" );
    sleep( 1 );
		char buf[100];
		sprintf( buf, "%s/tmp", NEWROOT.latin1() );
    ::mkdir( buf, 01777 );
		sprintf( buf, "%s/tmp/%s", NEWROOT.latin1(), mainLogName );

		mainlog = fopen( buf, "a+" );
//		CHECK_PTR( mainlog );
		if( !mainlog )
		{
		  if( iDebugLevel >= 2 )
		    ::printf( "failed open mainlog!!!\n" );
		  return;
		}
	}
  fprintf( mainlog, "* " );
  vfprintf( mainlog, s, args );
	int len = strlen( s );
	if( s[len-1] != '\n' )
	  fprintf( mainlog, "\n" );

  fflush( mainlog );
}

void Log::printl( int a_iDebugLevel, const char * s, ... )
{
	if( (a_iDebugLevel & 0x1F) > Log::iDebugLevel ) return;
  va_list args;
  va_start( args, s );
	Log::printfLogMain( s, args );
  va_end( args );
}

void Log::closeMain()
{
  if( iDebugLevel > 2 )
    ::printf( "closing mainlog...\n" );
 	if( mainlog ){
 	  ::fflush( mainlog );
		::sync();
		::sleep(1);
		::sync();
		if( !fclose( mainlog ) ){
  		mainlog = NULL;
  		if( iDebugLevel > 2 )
  		  ::printf( "closed successfully\n" );
    }else{
      ::printf( "failed close mainlog, errno: %d\n", errno );
  		::sync();  		
			char buf[100];
			sprintf(buf, "fuser -v %s/tmp/%s", NEWROOT.ascii(), mainLogName);
	  	::system(buf);
//	  	::system( ("fuser -v "+ NEWROOT +"/tmp/install.log").latin1() );
  	}
	}else if( iDebugLevel > 2 )
	  ::printf( "no log to close!\n" );
}

void Log::printfLogMain( const char * s, va_list args )
{
	if( !s ) return;
	fprintf( stdout, "* " );
	vfprintf( stdout, s, args );
	int len = strlen( s );
	if( s[len-1] != '\n' )
		fprintf( stdout, "\n" );
	fflush( stdout );
}
