
#include "config.h"

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>


#include <qdict.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>


#include "lutils.h"
#include "llog.h"


void splitString(QString q, char c, QStringList &list)
{
  int pos;
  QString item;
 
  while ( (pos = q.find(c)) >= 0)
    {
      item = q.left(pos);
      list.append(item);
      q.remove(0,pos+1);
    }
  list.append(q);
} 


bool        map_read = false;
QDict<char> grub_map;

void read_grub_map()
{
  if (map_read)
    return;

  // generate map
  ::mkdir("/boot", 0755);
  ::mkdir("/boot/grub", 0755);
  if (system(("chroot "+ NEWROOT +" /bin/sh -c \"echo quit | /usr/sbin/grub --batch --device-map=/boot/grub/device.map\"").latin1() ))
    {
      dbg() << "GRUB could not generate map file!" << endl;
      return;
    }

  // read in map
  QFile map( NEWROOT +"/boot/grub/device.map" );
  if (!map.exists())
    return;

  grub_map.clear();
  QRegExp white("\\s+");
  if (map.open(IO_ReadOnly))
    {
      QTextStream ts(&map);

      while (!ts.eof())
	{
	  QString line = ts.readLine();
	  if (line.left(1) == "#")
            continue;

	  dbg() << "grub: " << line << endl;
	  
	  QString device, name;
	  
	  int pos, len;
	  pos = white.match(line, 0, &len);
          if (pos > 0)
            {
              name = line.left(pos);
              device = line.mid(pos+len);

	      dbg() << "Device=" << device << " Name=" << name << endl;

	      grub_map.insert(device, strdup(name.ascii()));
	    }
	}

      map.close();
    }
  else
    return;

  map_read = true;
}


QString bsddevice(QString device)
{
  read_grub_map();
  
  QString disk = bsddisk(device);

  // get the partition number
  QString nr;
  int pos=0;
  while (device.at(device.length()-pos-1).isNumber())
    pos++;

  if (!pos)
    return disk; // no partition number

  nr = device.right(pos);
  disk.insert(disk.length()-1, ",");
  int number = nr.toInt();
  if (number<1)
    {
      dbg() << "illegal partition number" << number << endl;
      number=1;
    }
  nr.setNum(number-1);
  disk.insert(disk.length()-1, nr);

  dbg() << "Name of partition " << device << " = " << disk << endl;

  return disk;
}


QString bsddisk(QString device)
{
  QString disk;

  read_grub_map();

  // first, lookup up device directly
  disk = grub_map[device];

  if (disk.isEmpty())
    {
      // now without the partition number
      int len = 0;
      while (device.at(device.length()-len-1).isNumber())
	len++;
      device = device.left(device.length()-len);

      disk = grub_map[device];
    }

  if (disk.isEmpty())
    dbg() << "Grub device name for " << device << " not found!" << endl;
  else
    dbg() << "Grub device name for " << device << " = " << disk << endl;

  return disk;
}


QString AT(const QString &cmd)
{
  if (cmd.isEmpty())
    return cmd;
  if (cmd.left(2) == "AT")
    return cmd;
  return QString("AT")+cmd;
}


