/***************************************************************************
																	mouse.cpp
                             -------------------
    begin                : Fri Oct 29 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mouse.h"

#include <qlayout.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qapplication.h>
#include <qsocketnotifier.h>
#include <qcombobox.h>
#include <stdio.h>
#include <qfileinfo.h>
#include <qtextstream.h>

#include <qmultilineedit.h>

#include "linlinehelp.h"

// xf86misc.h needs this
#include <X11/Xlib.h>
// for the mouse shit
#include <X11/extensions/xf86misc.h>

// ioctl
#include <sys/ioctl.h>

// fcntl
#include <fcntl.h>
#include <unistd.h>

// kill
#include <signal.h>
#include <iostream.h>

#include <stdlib.h>

bool Mouse::runningUSB = false;

//////////////////////////////////////////////
// bloody hack!!! TODO! Because of XFree 4 ;((
#define MTYPE_USB             21
//////////////////////////////////////////////


/* XPM */
static const char *mouse_x[] = {
/* width height ncolors chars_per_pixel */
"16 16 3 1",
/* colors */
". c None",
"  c #000000",
"M c #FFFFFF",
/* pixels */
"MM..............",
"M M.............",
"M  M............",
"M   M...........",
"M    M..........",
"M     M.........",
"M      M........",
"M       M.......",
"M        M......",
"M     MMMMM...M.",
"M  M  M..M M.M M",
"M M.M  M..M M M.",
"MM..M  M...M M..",
".....M  M.M M M.",
".....M  MM M.M M",
"......MM..M...M."
};

MouseView::MouseView( QWidget *parent, const char *name)
    : QWidget( parent, name )
{
    state = NoButton;
    wheelOffset = 0;
    hasWheel = FALSE;
    setMaximumWidth( 200 );
}

void MouseView::paintState( bool onlyWheel)
{
    int bw = width()/3;
    int bh = height()*2/5;

    QRect left( 0, 0, bw, bh );
    QRect mid( bw, 0, bw, bh );
    QRect right( 2*bw, 0, bw, bh );
    QRect wheel( width()/2 - bw/4, bh/4, bw/2, bh/2 );
    QRect body( 0, bh, 3*bw, height()-bh-1 );

    QPainter p(this);

    if (!onlyWheel) {
			p.setBrush( state & LeftButton ? black : white );
			p.drawRect( left );

			p.setBrush( state & MidButton ? black : white );
			p.drawRect( mid );

			p.setBrush( state & RightButton ? black : white );
			p.drawRect( right);

			p.setBrush( white );
			p.drawRect( body );

			p.drawText( body, AlignCenter | WordBreak, tr("Test mouse here!") );
    }

    if (hasWheel) {
			p.drawRect( wheel.x()-1, wheel.y()-1, wheel.width()+2, wheel.height()+2);
			p.fillRect( wheel.x(), wheel.y(), wheel.width(), wheel.height(), white );
			p.setBrush( HorPattern );
			p.setBrushOrigin( 0, wheelOffset );
			p.drawRect( wheel );
    }
}

void MouseView::paintEvent( QPaintEvent * )
{
    paintState();
}

bool MouseView::eventFilter( QObject *, QEvent * e)
{

    if (e->type() == QEvent::MouseButtonPress || e->type() == QEvent::MouseButtonRelease) {
			state = ( (QMouseEvent*) e)->stateAfter();
			repaint (FALSE );
    }
    else if (e->type() == QEvent::Wheel ) {
			if ( ((QWheelEvent*) e)->delta() < 0 )
	  	  wheelOffset++;
			else
	  	  wheelOffset--;
			paintState ( TRUE );
    }
    return FALSE;
}

void MouseView::setWheel( bool b )
{
    hasWheel = b;
    resetState();
}


void MouseView::resetState()
{
    state = (ButtonState) 0 ;
    repaint( TRUE );
}

void MouseView::enterEvent( QEvent * )
{
    // do nothing on enter. Remember that we get an enter event when a
    // combobox (or any other popup) close. You mouseMoveEvent instead
    // (see below).
}

void MouseView::mouseMoveEvent( QMouseEvent * )
{
    setFocus();
}

MouseObserver::MouseObserver( QObject * parent, const char * name )
    : QObject( parent, name)
{
		rn = NULL;
		hasDetected = false;		
		char *pcPtr;
		QFileInfo mf( "/dev/mouse" );

		 mouse = PS2;
		 port = PSAux;
		 protocol = MTYPE_PS_2;				

		if( mf.exists() ){
			char mlink[128];
			memset( mlink, 0, sizeof( mlink ) );
			readlink( "/dev/mouse", mlink, 128 );
//printf( "Mouse device: %s\n", mlink );
			if( !strcmp( mlink, "/dev/input/mice" ) ){ // USB
			  mouse = USB;
			  protocol = MTYPE_PS_2;
			  port = HIDBP;
      }else if( !strstr( mlink, "psaux" ) ) { // serial
      	mouse = SERIAL;
				protocol = MTYPE_MICROSOFT;
        pcPtr = strstr( mlink, "tty" );
				
				if ( pcPtr )
				  port = getPortNum( pcPtr + 3 );
				else
					  port = S0;
			}	
		}
    emulate3rd = false;
		allowCheck = false;
		blProbe = false;
		blProbing = false;
//printf( "type finally: %d\n", mouse );
}

void MouseObserver::runProbing()
{
	if( blProbing ) return;
	blProbing = true;
	cout << "running mouseprobe" << endl;
#ifndef GBLI_DEVEL
//#ifdef abcdef
	    int fd[2];
	    ::pipe( fd );
  	  QApplication::flushX();
  	  pid = ::fork();
    	if (pid == 0) {
				::close( fd[0] );
	
			 	if (fd[1] != STDOUT_FILENO) {
 	  		  dup2( fd[1], STDOUT_FILENO);
 	    		::close( fd[1] );
			 	}
				execlp( "mdetect", "mdetect", 0 );
				exit(1);
	    }
//printf( "MouseObserver::runProbing pid: %d\n", pid );
  	  ::close( fd[1] );

    	int s;
	    if ( (s = fcntl( fd[0], F_GETFL, 0 ) ) >= 0 ) {
				s |= FNDELAY;
				fcntl( fd[0], F_SETFL, s );
	    }

  	  rn = new QSocketNotifier( fd[0], QSocketNotifier::Read, this);
			blProbe = true;
    	connect( rn, SIGNAL(activated(int)),
	    	this, SLOT(readMouseInfo(int)) );

	    qApp->setOverrideCursor( QCursor( QPixmap( mouse_x ) ) );
#endif
}

MouseObserver::~MouseObserver()
{
    if ( rn ) {
			qApp->restoreOverrideCursor();
			delete rn;
			rn = 0;
			if ( pid )
	  	  ::kill( pid, 15 );
			pid = 0;
    }
}

void MouseObserver::readMouseInfo( int fd )
{
//printf("readMouseInfo\n");

    int available = 0;

 if ( ::ioctl( fd, FIONREAD, (char*)&available ) >= 0 ) {
	if ( available ) {
	    // text came in
	    QByteArray rba( available );
	    int nread;
	    nread = ::read( fd, rba.data(), available );
	    for( int i=0; i < nread; i++ )
		mdetect += rba[i];
	} else {
//printf("gotall\n");
    // we got it all
    if( rn ){
			qApp->restoreOverrideCursor();
      delete rn;
   	  rn = 0;
     	::close( fd );
		}

		// oooh.  slow motion.
		if ( mdetect.find( "Detected psaux mouse" ) >= 0 ){
  	  mouse = PS2;
	    protocol = MTYPE_PS_2;
		}else if ( mdetect.find( "Detected logitech mouse" ) >= 0 ){
	    mouse = SERIAL;
			protocol = MTYPE_LOGITECH;
		}else if ( mdetect.find( "Detected microsoft mouse" ) >= 0 ){
	    mouse = SERIAL;
	    protocol = MTYPE_MICROSOFT;
		}else if ( mdetect.find( "Detected USB mouse") >= 0 ){
  	  mouse = USB;
	    protocol = MTYPE_PS_2;
		}

		if ( mdetect.find( "/dev/psaux" ) >= 0 )
	    port = PSAux;
		else if ( mdetect.find( "/dev/ttyS0" ) >= 0 )
	    port = S0;
		else if ( mdetect.find( "/dev/ttyS1" ) >= 0 )
  	  port = S1;
		else if ( mdetect.find( "/dev/ttyS2" ) >= 0 )
	    port = S2;
		else if ( mdetect.find( "/dev/ttyS3" ) >= 0 )
	    port = S3;
		else if ( mdetect.find( "/dev/input/mice" ) >= 0)
	    port = HIDBP;

		blProbing = false;
    hasDetected = TRUE;
    emit gotMouse();
	}
 }
}

void MouseObserver::setType( Type p )
{
//printf( "Type: %d\n", (int)p );
    mouse = (Type)p;
		switch( mouse ) {
			case PS2: 	port = PSAux;		break;
			case USB:  	port = HIDBP;	  break;
			default:    port = port;    break;
		}
}

void MouseObserver::tellXServer()
{
  if( !blProbe && !allowCheck ) return;
  QString newname = getPortName();

//Not a good idea to run such things on a live system though
Log::logMain(33, "tellXServer - type: %d, protocol: %d, port: %d\n", mouse, protocol, port );

#ifdef GBLI_DEVEL
//  QString newname = getPortName();
	printf( "Port name: %s\n", newname.ascii() );
#else
  char mlink[ 128 ];
  XF86MiscMouseSettings m;
  Display *pCurrDisp;

	pCurrDisp = qt_xdisplay();
//	pCurrDisp = XOpenDisplay( NULL );
//printf( "disp: %d\n", pCurrDisp );
	
//  int res =
	::XF86MiscGetMouseSettings( pCurrDisp, &m );
//printf( "Display*: %d, get mouse res: %d\n", pCurrDisp, res );

//  QString newname = getPortName();
//printf( "Port name: %s, type:%d, baud:%d, sample:%d, res:%d, buttons:%d\n", newname.ascii(), m.type, m.baudrate, m.samplerate, m.resolution, m.buttons );

	// TODO: somehow it failes to get MouseSettings on XFree4 ;(((
	m.baudrate = 1200;
	m.samplerate = 150;
//	m.resolution = 100;
	m.buttons = 3;

  memset( mlink, 0, sizeof( mlink ) );
	readlink( "/dev/mouse", mlink, 128 );
	if( strcmp( mlink, newname ) ) {
	  ::unlink( "/dev/mouse" );
		::symlink( newname, "/dev/mouse" );
	}
  m.type = protocol;
  m.emulate3buttons = emulate3rd;
  m.flags |= MF_REOPEN;

  // set the cursor position, NO MATTER when the fork()ing mouse
  // movements might arrive.
  qApp->setOverrideCursor( Qt::waitCursor );
  QApplication::flushX();
  /*int res =*/ (int)::XF86MiscSetMouseSettings( pCurrDisp, &m );
  qApp->restoreOverrideCursor();
/**///printf( "Set result: %d\n", res );
#endif
  QApplication::syncX();
  QApplication::flushX();
	blProbe = false;
}

const char * MouseObserver::getPortName()
{
  switch( port ) {
  	case PSAux:
		  return "/dev/psaux";
    case S0:
		  return "/dev/ttyS0";
    case S1:
		  return "/dev/ttyS1";
    case S2:
		  return "/dev/ttyS2";
    case S3:
		  return "/dev/ttyS3";
    case HIDBP:
      return "/dev/input/mice";
  }
  return "/dev/psaux";
}

MouseObserver::Port MouseObserver::getPortNum( char *a_szPortName )
{

  if ( !strcmp( a_szPortName, "psaux" ) )
    return PSAux;
  if ( !strcmp( a_szPortName, "S0" ) )
	  return S0;
  if ( !strcmp( a_szPortName, "S1" ) )
	  return S1;
  if ( !strcmp( a_szPortName, "S2" ) )
	  return S2;
  if ( !strcmp( a_szPortName, "S3" ) )
	  return S3;
  if ( !strcmp( a_szPortName, "input/mice" ) )
	  return HIDBP;
  if ( !strcmp( a_szPortName, "/dev/input/mice" ) )
	  return HIDBP;
  if ( !strcmp( a_szPortName, "mice" ) )
	  return HIDBP;
	
  return PSAux;
}


/* ################################################################## */

Mouse::Mouse( QWidget * parent, const char * name )
    : GBLIPage( parent, name )
{
//printf( "come to mouse!" );
//	m_pbttnProbe = new QPushButton( "  "+ tr( "Probe" ) +"  ", this );

//  m_pchkActivate = new QCheckBox( tr("&Apply settings instantly"), this );

	m_plblType = new QLabel( tr("Type:"), this );
	m_plblType->setAutoMask( true );
	m_plboxType = new QListBox( this );
	m_plboxType->insertItem( tr("PS/2 mouse"), MouseObserver::PS2 );
	m_plboxType->insertItem( tr("Serial mouse"), MouseObserver::SERIAL );
	m_plboxType->insertItem( tr("USB mouse"), MouseObserver::USB );
	m_plboxType->setMinimumHeight( m_plboxType->sizeHint().height()+ 15 );
	m_plboxType->setMaximumHeight( 70 );
	m_plboxType->setMinimumWidth( m_plboxType->sizeHint().width()+ 15 );

	m_plblProtocol = new QLabel( tr("Protocol:"), this );
	m_plblProtocol->setAutoMask( true );
  m_plboxProtocol = new QListBox( this );
	m_plboxProtocol->setMinimumHeight( 60 );
	int iMaxWidth = m_plboxProtocol->sizeHint().width()+ 80;
	m_plboxProtocol->setMaximumWidth( iMaxWidth );

	m_plboxType->setMaximumWidth( iMaxWidth );

	m_plblPort = new QLabel( tr("Port:"), this );
	m_plboxPort = new QComboBox( false, this );
	m_plblPort->setAutoMask( true );
	m_plboxPort->insertItem( "/dev/ttyS0 (COM1)" );
	m_plboxPort->insertItem( "/dev/ttyS1 (COM2)" );
	m_plboxPort->insertItem( "/dev/ttyS2 (COM3)" );
	m_plboxPort->insertItem( "/dev/ttyS3 (COM4)" );
	m_plboxPort->setMinimumHeight( m_plboxPort->sizeHint().height()+ 7 );
	m_plboxPort->setMaximumWidth( iMaxWidth );

  m_pchkEmulate = new QCheckBox( tr("&Emulate 3rd button"), this );
	m_pchkEmulate->setAutoMask( true );
m_pchkEmulate->setChecked( true );
m_pchkEmulate->hide();

  m_pchkActivate = new QPushButton( tr("&Apply settings"), this );

  m_testMouse = new MouseView( this );
  m_testMouse->installEventFilter( m_testMouse );
	m_testMouse->setFixedSize( 100, 140 );

  observer = new MouseObserver( this, "Observer" );
  gotMouse();

	/** Initialize layouts */
	QVBoxLayout *loFirst = new QVBoxLayout( 6 );
//  loFirst->addWidget( m_pbttnProbe );
	loFirst->addWidget( m_plblType );
	loFirst->addWidget( m_plboxType );
	loFirst->addSpacing( 5 );
	loFirst->addStretch( 1 );
	loFirst->addWidget( m_plblProtocol );
	loFirst->addWidget( m_plboxProtocol, 12 );
	loFirst->addSpacing( 5 );
	loFirst->addStretch( 1 );
	loFirst->addWidget( m_plblPort );
	loFirst->addWidget( m_plboxPort );
	loFirst->addStretch( 1 );

	QHBoxLayout *loHTest = new QHBoxLayout();
	loHTest->addStretch( 1 );
	loHTest->addWidget( m_testMouse );
	loHTest->addStretch( 1 );

	loHTest->addStretch( 1 );

	QVBoxLayout *loVTest = new QVBoxLayout();
	loVTest->addStretch( 1 );
	loVTest->addLayout( loHTest, 1 );
	loVTest->addStretch( 1 );

	QVBoxLayout *loSecond = new QVBoxLayout( 7 );
	loSecond->addStretch( 1 );
	loSecond->addSpacing( 10 );
	loSecond->addWidget( m_pchkEmulate );
	loSecond->addSpacing( 10 );
  loSecond->addWidget( m_pchkActivate );
	loSecond->addSpacing( 10 );
	loSecond->addStretch( 1 );
	loSecond->addLayout( loVTest );
	loSecond->addSpacing( 20 );

	QHBoxLayout *loHorz = new QHBoxLayout( this, 7 );

	loHorz->addStretch( 1 );
	loHorz->addLayout( loFirst, 7 );
	loHorz->addSpacing( 5 );
	loHorz->addLayout( loSecond, 3 );
	loHorz->addStretch( 1 );

//	connect( m_pbttnProbe, SIGNAL( clicked() ), observer, SLOT( runProbing () ) );
  connect( m_plboxType, SIGNAL(highlighted(int)), this, SLOT(setType(int)) );
  connect( m_plboxProtocol, SIGNAL(highlighted(int)), this, SLOT(setProtocol(int)) );
  connect( m_plboxPort, SIGNAL(highlighted(int)), this, SLOT(setPort(int)) );
  connect( m_pchkEmulate, SIGNAL(toggled(bool)), this, SLOT(emulateClicked(bool)) );
  connect( observer, SIGNAL( gotMouse() ), this, SLOT( gotMouse() ) );
	connect( m_pchkActivate, SIGNAL(clicked()/*toggled(bool)*/), observer, SLOT(checkValue()/*setAllowCheck(bool)*/) );

//	m_pbttnProbe->setFocus();
	m_plboxType->setFocus();
}

void Mouse::initMouse(){
#ifndef GBLI_DEVEL

  system("modprobe -qs isa-pnp >/dev/null 2>&1" );
  system("modprobe -qs serial >/dev/null 2>&1" );
	system( "touch -t 01010000 "+ QString( CONF_MODULES ) );

	int usb_ok = 0;
  if( system("modprobe -qs usb-uhci >/dev/null 2>&1" ) )
	  usb_ok = !system("modprobe -qs usb-ohci >/dev/null 2>&1" );
	else
		usb_ok = 1;
	if( usb_ok ){
		system( "mkdir /tmp/usb >/dev/null 2>&1" );
		usb_ok = !system( "mount -t usbdevfs none /tmp/usb" );
		if( usb_ok ){
			sync();
			sleep( 1 );
  		int mouse_ok = !system( "cat /tmp/usb/devices|grep -E \"^I.*Cls=03.*Prot=02\" >/dev/null 2>&1" );
  		int keyboard_ok = !system( "cat /tmp/usb/devices|grep -E \"^I.*Cls=03.*Prot=01\" >/dev/null 2>&1" );
//printf( "USB mouse: %d, keyboard: %d\n", mouse_ok, keyboard_ok );
  		int hid_ok = !system("modprobe -qs hid >/dev/null 2>&1" );
			usb_ok = ( hid_ok && (mouse_ok || keyboard_ok) );
  		if( usb_ok && mouse_ok )
  			Mouse::runningUSB =	true;
 	  	system("modprobe -qs mousedev >/dev/null 2>&1" );
 	  	system("modprobe -qs keybdev >/dev/null 2>&1" );
		}
	}

	if( !QFile( "/dev/input/mice" ).exists() ){
		system( "mkdir /dev/input" );
	  system( "mknod /dev/input/mice c 13 63" );// useful even if we didn't get usb
	}
	system( "touch -t 01010000 "+ QString( CONF_MODULES ) );

  ::unlink( "/dev/mouse" );

  if( !Mouse::runningUSB ){
	  system( "mouseconfig --kickstart" );
	    // it will make us a link and create config file
	}
  if( !QFileInfo( "/dev/mouse" ).exists() ){
  	if( Mouse::runningUSB ){// may be we have USB mouse?
      ::symlink( "/dev/input/mice", "/dev/mouse" );
      QFile	gpmFile( "/etc/sysconfig/mouse" );
      if( gpmFile.open(IO_WriteOnly) ){
        QTextStream	gpm(&gpmFile);
        gpm << "XMOUSETYPE=\"PS/2\"" << endl;
        gpm << "MOUSETYPE=\"ps/2\"" << endl;
    	  gpmFile.close();
    	}
    }
	}

#endif
}

void Mouse::showEvent( QShowEvent * )
{
	m_plboxType->setFocus();
}

void Mouse::hideEvent( QHideEvent * )
{
  observer->tellXServer();
}

Mouse::~Mouse()
{
	  delete m_plboxType;
	  delete m_plboxProtocol;
	  delete m_plblPort;
	  delete m_plboxPort;
	  delete m_pchkEmulate;
	  delete m_testMouse;
}

void Mouse::gotMouse()
{
  setType( observer->getType() );

	switch( observer->getPort() ){
		case MouseObserver::S0: m_plboxPort->setCurrentItem( 0 ); break;
		case MouseObserver::S1: m_plboxPort->setCurrentItem( 1 ); break;
		case MouseObserver::S2: m_plboxPort->setCurrentItem( 2 ); break;
		case MouseObserver::S3: m_plboxPort->setCurrentItem( 3 ); break;
		default:                m_plboxPort->setCurrentItem( 0 ); break;
	}
}

int Mouse::getBackProtocol( int i ){
	int res = 0;
	switch( i ){
		case MTYPE_PS_2: res = 0;  break;
		case MTYPE_MICROSOFT: res = 0;  break;// zero element in serial list
		case MTYPE_LOGITECH: res = 1;  break; // in serial list
			// IntelliMouse
 	  case MTYPE_IMPS2: m_plboxType->currentItem()?res=3:res=2; break;
		case MTYPE_MMANPLUSPS2: m_plboxType->currentItem()?res=5:res=4; break;
		default: res = 0;  break;
	}
	return res;
}

void Mouse::setPort( int a_i )
{
	m_plboxPort->setCurrentItem( a_i );
	observer->setPort( getPort() );
	observer->tellXServer();
}

MouseObserver::Port Mouse::getPort()
{
	switch( m_plboxPort->currentItem() ){
		case 0: return MouseObserver::S0;
		case 1: return MouseObserver::S1;
		case 2: return MouseObserver::S2;
		case 3: return MouseObserver::S3;		
		default: return MouseObserver::PSAux;
	}
}

void Mouse::setUpProtocolList()
{
	m_plboxProtocol->clear();
	m_plboxProtocol->setEnabled( true );
	switch( m_plboxType->currentItem() ){
		case MouseObserver::PS2:
		  m_plboxProtocol->insertItem( tr("Generic PS/2 mouse") );
		  m_plboxProtocol->insertItem( tr("Auto") );
		  m_plboxProtocol->insertItem( tr("Microsoft IntelliMouse") );
	  	m_plboxProtocol->insertItem( tr("ThinkingMouse") );
		  m_plboxProtocol->insertItem( tr("Logitech MouseManPlus") );
  		m_plboxProtocol->insertItem( tr("GlidePoint") );
		  m_plboxProtocol->insertItem( tr("NetMouse") );
  		m_plboxProtocol->insertItem( tr("NetScroll") );
			break;
		case MouseObserver::USB:
		  m_plboxProtocol->insertItem( tr("PS/2") );
  		m_plboxProtocol->insertItem( tr("USB") );
		  m_plboxProtocol->insertItem( tr("Auto") );
		  m_plboxProtocol->insertItem( tr("Microsoft IntelliMouse") );		
	  	m_plboxProtocol->insertItem( tr("ThinkingMouse") );
		  m_plboxProtocol->insertItem( tr("Logitech MouseManPlus") );
  		m_plboxProtocol->insertItem( tr("GlidePoint") );
		  m_plboxProtocol->insertItem( tr("NetMouse") );
  		m_plboxProtocol->insertItem( tr("NetScroll") );
			if( !system( "grep \"IMPS/2\" -i /etc/X11/XF86Config >/dev/null 2>&1" ) ){
				observer->setProtocol( MTYPE_IMPS2 );
			}else if( !system( "grep MouseManPlus -i /etc/X11/XF86Config >/dev/null 2>&1" ) ){
				observer->setProtocol( MTYPE_MMANPLUSPS2 );
			}
  		break;		
		case MouseObserver::SERIAL:
		  m_plboxProtocol->insertItem( tr("Microsoft compatible") );
		  m_plboxProtocol->insertItem( tr("Logitech protocol (old)") );
		  m_plboxProtocol->insertItem( tr("Logitech MM Series") );
		  m_plboxProtocol->insertItem( tr("Logitech MouseMan") );
		  m_plboxProtocol->insertItem( tr("Auto") );
		  m_plboxProtocol->insertItem( tr("Mouse Systems") );
		  m_plboxProtocol->insertItem( tr("MMHittab") );
		  m_plboxProtocol->insertItem( tr("GlidePoint serial") );
		  m_plboxProtocol->insertItem( tr("Microsoft IntelliMouse") );
		  m_plboxProtocol->insertItem( tr("Logitech MouseManPlus") );
		  m_plboxProtocol->insertItem( tr("ThinkingMouse") );
		  m_plboxProtocol->insertItem( tr("System Mouse") );
//		  m_plboxProtocol->insertItem( tr("Bus mouse") );
//		  m_plboxProtocol->insertItem( tr("Xqueue") );
//		  m_plboxProtocol->insertItem( tr("OSMouse") );
			break;
	}
  setProtocol( getBackProtocol( observer->getProtocol() ) );
}

int Mouse::getProtocol()
{
	switch( m_plboxType->currentItem() ) {
		case MouseObserver::PS2:
			switch( m_plboxProtocol->currentItem() ) {
		    case 0: return MTYPE_PS_2;
				case 1: return MTYPE_AUTOMOUSE;
	  	  case 2: return MTYPE_IMPS2;
				case 3: return MTYPE_THINKINGPS2;
				case 4: return MTYPE_MMANPLUSPS2;
				case 5: return MTYPE_GLIDEPOINTPS2;
				case 6: return MTYPE_NETPS2;
				case 7: return MTYPE_NETSCROLLPS2;
				default:return MTYPE_PS_2;
			}
		case MouseObserver::USB:
			switch( m_plboxProtocol->currentItem() ) {
		    case 0: return MTYPE_PS_2;			
		    case 1: return MTYPE_USB;
				case 2: return MTYPE_AUTOMOUSE;
	  	  case 3: return MTYPE_IMPS2;
				case 4: return MTYPE_THINKINGPS2;
				case 5: return MTYPE_MMANPLUSPS2;
				case 6: return MTYPE_GLIDEPOINTPS2;
				case 7: return MTYPE_NETPS2;
				case 8: return MTYPE_NETSCROLLPS2;
				default:return MTYPE_PS_2;	  	
			}
		case MouseObserver::SERIAL:
			switch( m_plboxProtocol->currentItem() ) {
		    case 0: return MTYPE_MICROSOFT;
				case 1: return MTYPE_LOGITECH;
				case 2: return MTYPE_MMSERIES;
				case 3: return MTYPE_LOGIMAN;
				case 4: return MTYPE_AUTOMOUSE;
				case 5: return MTYPE_MOUSESYS;
				case 6: return MTYPE_MMHIT;
				case 7: return MTYPE_GLIDEPOINT;
				case 8: return MTYPE_IMSERIAL;
				case 9: return MTYPE_IMSERIAL;
				case 10:return MTYPE_THINKING;
				case 11:return MTYPE_SYSMOUSE;
				case 12:return MTYPE_BUSMOUSE;
				case 13:return MTYPE_XQUEUE;
				case 14:return MTYPE_OSMOUSE;
				default:return MTYPE_MICROSOFT;
			}
		default:
			return MTYPE_MICROSOFT;
	}
}

void Mouse::setType( int p ){

  observer->setType( (MouseObserver::Type)p );
	m_plboxType->setCurrentItem( p );
  bool serial = (p == MouseObserver::SERIAL);
	m_plboxPort->setEnabled( serial );
//	observer->setPort( getPort() );

	setUpProtocolList();

}

void Mouse::setProtocol( int i ){

		m_plboxProtocol->setCurrentItem( i );
		observer->setProtocol( getProtocol() );
    m_testMouse->setWheel( hasWheel() );
		observer->setType( (MouseObserver::Type)m_plboxType->currentItem() );
		observer->tellXServer();

}

bool Mouse::hasWheel()
{
// BestLinux TODO: I think it's not very correct for our list of mouses
  return
		( (observer->getType() == MouseObserver::PS2) &&
		  (observer->getProtocol() != MTYPE_PS_2  &&
  		 observer->getProtocol() != MTYPE_GLIDEPOINTPS2) ) ||
    (observer->getType() == MouseObserver::USB);
}

void Mouse::emulateClicked( bool a_bl )
{
  observer->setEmulation( a_bl );
	observer->tellXServer();
}

const char * Mouse::getGPMName()
{
	switch( observer->getProtocol() ) {
    case MTYPE_MICROSOFT     : return "Microsoft";
		case MTYPE_MOUSESYS      : return "MouseSystems";
    case MTYPE_MMSERIES      : return "MMSeries";
    case MTYPE_LOGITECH      : return "Logitech";
    case MTYPE_BUSMOUSE      : return "Busmouse";
    case MTYPE_LOGIMAN       : return "MouseMan";
    case MTYPE_PS_2          : return "ps/2";
    case MTYPE_MMHIT         : return "MMHitTab";
    case MTYPE_GLIDEPOINT    : return "GlidePoint";
    case MTYPE_IMSERIAL			 : return "ms3";
    case MTYPE_THINKING      : return "ms3";
    case MTYPE_IMPS2         : return "imps2";
    case MTYPE_THINKINGPS2   : return "ps/2";
    case MTYPE_MMANPLUSPS2   : return "ps/2";
    case MTYPE_GLIDEPOINTPS2 : return "ps/2";
    case MTYPE_NETPS2        : return "netmouse";
    case MTYPE_NETSCROLLPS2  : return "netmouse";
    case MTYPE_SYSMOUSE      : return "SysMouse";
    case MTYPE_AUTOMOUSE     : return "pnp";
		case MTYPE_XQUEUE        : return "Xqueue";
		case MTYPE_OSMOUSE       : return "OSMouse";
		case MTYPE_USB           : return "usb";
		default                  : return "Microsoft";
	}
}

const char * Mouse::getMouseName()
{
	switch( observer->getProtocol() ) {
    case MTYPE_MICROSOFT     : return "Microsoft";
		case MTYPE_MOUSESYS      : return "MouseSystems";
    case MTYPE_MMSERIES      : return "MMSeries";
    case MTYPE_LOGITECH      : return "Logitech";
    case MTYPE_BUSMOUSE      : return "BusMouse";
    case MTYPE_LOGIMAN       : return "MouseMan";
    case MTYPE_PS_2          : return "PS/2";
    case MTYPE_MMHIT         : return "MMHitTab";
    case MTYPE_GLIDEPOINT    : return "GlidePoint";
    case MTYPE_IMSERIAL			 : return "IntelliMouse";
    case MTYPE_THINKING      : return "ThinkingMouse";
    case MTYPE_IMPS2         : return "IMPS/2";
    case MTYPE_THINKINGPS2   : return "ThinkingMousePS/2";
    case MTYPE_MMANPLUSPS2   : return "MouseManPlusPS/2";
    case MTYPE_GLIDEPOINTPS2 : return "GlidePointPS/2";
    case MTYPE_NETPS2        : return "NetMousePS/2";
    case MTYPE_NETSCROLLPS2  : return "NetScrollPS/2";
    case MTYPE_SYSMOUSE      : return "SysMouse";
    case MTYPE_AUTOMOUSE     : return "Auto";
		case MTYPE_XQUEUE        : return "Xqueue";
		case MTYPE_OSMOUSE       : return "OSMouse";
		case MTYPE_USB           : return "usb";
		default                  : return "Microsoft";
	}
}

void Mouse::finish()
{
  if( observer->getType() == MouseObserver::USB ){
		if( !QFile( NEWROOT +"/dev/input/mice" ).exists() ){
			system( ("mkdir "+ NEWROOT +"/dev/input").ascii() );
		  system( ("mknod "+ NEWROOT +"/dev/input/mice c 13 63").ascii() );
		}
/// Should be written by modulespage
//	  QFile rconf( NEWROOT +CONF_MODULES );
//	  if( rconf.open( IO_WriteOnly | IO_Append ) ){
//	    QTextStream t( &rconf );
//	    t << endl << "alias usb-controller ";
//			if( !system( "grep \"USB UHCI\" -i /tmp/usb/devices >/dev/null 2>&1" ) )
//		    t << "usb-uhci" << endl;
//			else
//		    t << "usb-ohci" << endl;
//	    rconf.close();
//		}

//	  QFile rconf( NEWROOT +"/etc/rc.d/rc.modules" );
//	  bool addBeginning = !rconf.exists();
//	  if( rconf.open( IO_WriteOnly | IO_Append ) ){
//	    QTextStream t( &rconf );
//	    if( addBeginning )
//	      t << "#!/bin/bash";
//	    t << endl << "modprobe -qs input >/dev/null 2>&1" << endl;
//	    t << "modprobe -qs keybdev >/dev/null 2>&1" << endl;
//	    t << "modprobe -qs mousedev >/dev/null 2>&1" << endl;
//			if( system( "grep \"USB UHCI\" -i /tmp/usb/devices >/dev/null 2>&1" ) )
//		    t << "modprobe -qs usb-ohci >/dev/null 2>&1" << endl;
//			else
//		    t << "modprobe -qs usb-uhci >/dev/null 2>&1" << endl;
//	    t << "modprobe -qs hid >/dev/null 2>&1" << endl;
//	    rconf.close();
//      system( ("chmod u+rwx "+ NEWROOT +"/etc/rc.d/rc.modules").ascii() );
//			char buf[32];
//			int iPathLen = readlink( (NEWROOT +"/etc/rc.modules").ascii(), buf, 32 );
//			if( iPathLen == -1 )
//				system( ("ln -s /etc/rc.d/rc.modules "+ NEWROOT +"/etc/rc.modules >/dev/null 2>&1").ascii() );
//	  }
	}
	system( "umount /tmp/usb >/dev/null 2>&1" );
#ifndef DEVFS
	
	bool bRemount=false;
	int res;
	if(!system(("mount |grep \"on " + NEWROOT + "/dev\" &>/dev/null").latin1()))
		{
			res = system(("umount "+ NEWROOT +"/dev").latin1());
			Log::logMain( 38, "Unmounting %s/dev: %d", NEWROOT.latin1(), res );
      bRemount = true;
    }

  ::unlink( (NEWROOT +"/dev/mouse").latin1() );
	QString NewMLink = getPortName();

//	printf ( "Linking mouse 2 to %s\n", NewMLink.ascii() );
  ::symlink(NewMLink.latin1(),  (NEWROOT +"/dev/mouse").latin1());

if (bRemount){
#ifdef DEVFS
		res = system( ("mount -t devfs none "+ NEWROOT +"/dev").latin1() );
		Log::logMain( 38, "Mounting  %s/dev: %d", NEWROOT.latin1(), res );
#else
		res = system( ("mount --bind /dev "+ NEWROOT +"/dev").latin1() );
		Log::logMain( 38, "Mounting  %s/dev: %d", NEWROOT.latin1(), res );		
#endif
}

//	Log::logMain( 32, "IFNDEF DEVFS: ::symlink( %s, %s);",getPortName(), (NEWROOT +"/dev/mouse").latin1());
#else
	// TODO: switch to enabling it in /etc/devfsd.conf ?
	QFile outf( NEWROOT +"/etc/rc.d/rc.local" );
	if( outf.open( IO_WriteOnly | IO_Append ) ){
    QTextStream outt( &outf );
		outt << endl << "if [ ! -f /dev/mouse ] ; then" << endl;
		outt << "\tln -s " << getPortName() << " /dev/mouse" << endl;
    outt << "fi" << endl;
		outf.close();
	}
#endif

  QFile	gpmFile( NEWROOT +"/etc/sysconfig/mouse" );
  if( !gpmFile.open(IO_WriteOnly) )
    return;

  QTextStream	gpm(&gpmFile);
  gpm << "MOUSETYPE=\"" << getGPMName() << "\"\n";
  gpm << "XMOUSETYPE=\"" << getMouseName() << "\"\n";
  gpm << "FULLNAME=\"" << getMouseName() << "\"\n";
  gpm << "XEMU3=\""     << (emulate3buttons() ? "yes" : "no") << "\"\n";
	gpmFile.close();

}

