/***************************************************************************
                          			network.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "config.h"
#include "network.h"
#include "configfile.h"
#include "sysconfigfile.h"
#include "liconloader.h"

#include <qlineedit.h>
#include <qvalidator.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsocket.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcombobox.h>
#include <qlistview.h>

#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qheader.h>
#include <qsizepolicy.h>
#include <qtabbar.h>
#include <qmessagebox.h>

#include "linlinehelp.h"
#include "lwriter.h"
#include "log.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#include <stdlib.h>

#define NET_DEBUG

#ifdef NET_DEBUG
//	# define D(args...)	printf(##args)
	# define D(args...)	Log::logMain( 33, ##args )
#else
	# define D(args...)	do { } while (0)
#endif

QString wrapString( QString s, int maxLength )
{
	if ((int)s.length() > maxLength)
	{
		int pos = 0;
		while ( (s.find(" ", pos) <= maxLength ) && (s.find(" ", pos) != -1) )
			pos = s.find(" ", pos + 1);
		if ( pos > 0 ) s.replace( pos, 1, "\n" );
	}
	
	return s;
}

AddressEdit::AddressEdit( Mode a_m, QWidget * parent, Protocol a_pr, const char * name )
    : QFrame( parent, name ), m( a_m ), pr( a_pr ), e( false )
{
    int sw = fontMetrics().width( "." );
    QHBoxLayout * l = new QHBoxLayout( this, 2, 0 );

#ifndef GBLI_IPv6
    b1 = new QLineEdit( this );
    b1->setValidator( new QIntValidator( 0, m == Address ? 223 : 255, b1 ) );
    b1->setFrame( FALSE );
    b1->setAlignment( AlignRight );
    b1->setMaxLength( 3 );
    b1->setText( "0" );

    b2 = new QLineEdit( this );
    b2->setValidator( new QIntValidator( 0, 255, b2 ) );
    b2->setFrame( FALSE );
    b2->setAlignment( AlignRight );
    b2->setMaxLength( 3 );
    b2->setText( "0" );

    b3 = new QLineEdit( this );
    b3->setValidator( new QIntValidator( 0, 255, b3 ) );
    b3->setFrame( FALSE );
    b3->setAlignment( AlignRight );
    b3->setMaxLength( 3 );
    b3->setText( "0" );

    b4 = new QLineEdit( this );
    b4->setValidator( new QIntValidator( 0, 255, b4 ) );
    b4->setFrame( FALSE );
    b4->setAlignment( AlignRight );
    b4->setMaxLength( 3 );
    b4->setText( "0" );

    l->addWidget( b1 );
    l->addSpacing( sw );
    l->addWidget( b2 );
    l->addSpacing( sw );
			//can be used for removing old data    l->addWidget( b3 );
    l->addSpacing( sw );
    l->addWidget( b4 );
    l->activate();

    setFrameStyle( QFrame::WinPanel + QFrame::Sunken );
    b1->installEventFilter( this );
    b2->installEventFilter( this );
    b3->installEventFilter( this );
    b4->installEventFilter( this );

#else

//printf( "AE:protocol: %d\n", pr );

    setFrameStyle( QFrame::WinPanel + QFrame::Sunken );
		int fields = 9;
		for( int i = 0; i < fields; i++ ){
      ab[i] = new QLineEdit( this );
      ab[i]->setFrame( false );
			if( i == 8 )
	      ab[i]->setAlignment( AlignLeft );
			else
	      ab[i]->setAlignment( AlignRight );
      ab[i]->setText( "0" );
	    l->addWidget( ab[i] );
			if( i < fields-1 ){
				labAB[i] = new QLabel( this, "" );
				labAB[i]->setAutoMask( true );
				labAB[i]->setFixedSize( sw, 1 );
				l->addWidget( labAB[i] );
//		    l->addSpacing( sw );
			}
	    ab[i]->installEventFilter( this );
		}

		setProtocol( pr );

    l->activate();
#endif

    setFixedSize(sizeHint());
}


AddressEdit::~AddressEdit(){
}

// used for validation of the address
bool AddressEdit::eventFilter( QObject * o, QEvent * e ){
    if ( !e || !o )
			return FALSE;

		// delete leading zeros
    if ( e->type() == QEvent::KeyPress && o->inherits( "QLineEdit" ) ) {
    	QLineEdit * l = (QLineEdit *)o;
    	// hack - delete leading zeroes quite often, to keep the length down
    	if ( l->text().length() >= 1 && l->cursorPosition() &&
  	     !l->hasMarkedText() && l->text()[0] == '0' ) {
  	    int i = l->cursorPosition();
  	    l->setText( l->text().mid( 1, 999 ) );
  	    l->setCursorPosition( i-1 );
    	}
    }

		// select all
    if ( e->type() == QEvent::FocusIn && o->inherits( "QLineEdit" ) ) {
    	QLineEdit * l = (QLineEdit *)o;
    	l->selectAll();
    }

#ifndef GBLI_IPv6
		// validate address
    if ( e->type() == QEvent::FocusOut && o->inherits( "QLineEdit" ) ) {
    	QLineEdit * l = (QLineEdit *)o;
    	const QValidator * v = l->validator();
    	int i = 0;
    	QString tmp = l->text();
    	if ( tmp.isEmpty() )
				l->setText( "0" );
    	else if ( v && v->validate( tmp, i ) != QValidator::Acceptable )
   	    l->setText( (tmp == l->text()) ? tmp : QString("0") ); // anything
    	i = l->text().toInt();
    	if ( m == Netmask ) {
    	    // Turn this into a valid netmask by smearing bits to the left.
  	    i |= (i << 1);
  	    i |= (i << 2);
  	    i |= (i << 4);
  	    i &= 255;

  	    if ( i > 0 ) {
      		if ( o == b2 ) {
    		    b1->setText( "255" );
  	   		} else if ( o == b3 ) {
    		    b1->setText( "255" );
    		    b2->setText( "255" );
      		} else if ( o == b4 ) {
    		    b1->setText( "255" );
    		    b2->setText( "255" );
    		    b3->setText( "255" );
      		}
	      }
  	    if ( i < 255 ) {
      		if ( o == b1 ) {
      		    b2->setText( "0" );
      		    b3->setText( "0" );
      		    b4->setText( "0" );
      		} else if ( o == b2 ) {
      		    b3->setText( "0" );
      		    b4->setText( "0" );
      		} else if ( o == b3 ) {
      		    b4->setText( "0" );
      		}
  	    }
    	}
    	if ( i != l->text().toInt() ) {
    	    tmp.sprintf( "%u", i );
    	    l->setText( tmp );
    	}
    	return FALSE;
    }

    if ( !e || e->type() != QEvent::KeyPress ||
  	 !( ((QKeyEvent*)e)->text() == "." ||
  	    ((QKeyEvent*)e)->key() == Key_Enter ||
  	    ((QKeyEvent*)e)->key() == Key_Return ||
  	    ((QKeyEvent*)e)->key() == Key_Right ||
  	    ((QKeyEvent*)e)->key() == Key_Left ||
  	    ((QKeyEvent*)e)->key() == Key_Tab ||
  	    ((QKeyEvent*)e)->key() == Key_BackTab ) )
  			return FALSE;

    if ( ((QKeyEvent*)e)->key() == Key_BackTab ||
		 ((QKeyEvent*)e)->key() == Key_Left ) {

    	if ( ((QKeyEvent*)e)->key() == Key_Left &&
    	     ((QLineEdit*)o)->cursorPosition() > 0 )
    	    return FALSE;

    	if ( o == b2 ) {
    	    b1->setFocus();
    	    b1->setCursorPosition( b1->text().length() );
    	} else if ( o == b3 ) {
    	    b2->setFocus();
    	    b2->setCursorPosition( b2->text().length() );
    	} else if ( o == b4 ) {
    	    b3->setFocus();
    	    b3->setCursorPosition( b3->text().length() );
    	} else {
    	    return FALSE;
    	}

    	return TRUE;
    }

    // at this point, we know we're moving right.

    if ( ((QKeyEvent*)e)->key() == Key_Right &&
		 ((QLineEdit*)o)->cursorPosition() < (int)((QLineEdit*)o)->text().length() )
			return FALSE;

    // at this point we know we're REALLY moving right.

    if ( o == b1 ) {
			b2->setFocus();
			b2->setCursorPosition( 0 );
    } else if ( o == b2 ) {
			b3->setFocus();
			b3->setCursorPosition( 0 );
    } else if ( o == b3 ) {
			b4->setFocus();
			b4->setCursorPosition( 0 );
    } else {
			if ( ((QKeyEvent*)e)->key() == Key_Right )
		    return FALSE;
			emit returnPressed(); // it's a lie but...
			return FALSE;
    }
#else
		// validator
    if ( e->type() == QEvent::FocusOut && o->inherits( "QLineEdit" ) ) {
    	QLineEdit * l = (QLineEdit *)o;
    	int i = 0;
    	QString tmp = l->text();
			if( pr == IPv4 ){
	    	const QValidator * v = l->validator();
      	if ( tmp.isEmpty() )
  				l->setText( "0" );
      	else if ( v && v->validate( tmp, i ) != QValidator::Acceptable ){
//printf( "not valid\n" );
     	    l->setText( (tmp == l->text()) ? tmp : QString("0") ); // anything
				}
      	i = l->text().toInt();
      	if ( m == Netmask ) {
      	    // Turn this into a valid netmask by smearing bits to the left.
    	    i |= (i << 1);
    	    i |= (i << 2);
    	    i |= (i << 4);
    	    i &= 255;

    	    if ( i > 0 ) {
        		if ( o == ab[1] ) {
      		    ab[0]->setText( "255" );
    	   		} else if ( o == ab[2] ) {
      		    ab[0]->setText( "255" );
      		    ab[1]->setText( "255" );
        		} else if ( o == ab[3] ) {
      		    ab[0]->setText( "255" );
      		    ab[1]->setText( "255" );
      		    ab[2]->setText( "255" );
        		}
  	      }
    	    if ( i < 255 ) {
        		if ( o == ab[0] ) {
        		    ab[1]->setText( "0" );
        		    ab[2]->setText( "0" );
        		    ab[3]->setText( "0" );
        		} else if ( o == ab[1] ) {
        		    ab[2]->setText( "0" );
        		    ab[3]->setText( "0" );
        		} else if ( o == ab[2] ) {
        		    ab[3]->setText( "0" );
        		}
    	    }
      	}
      	if ( i != l->text().toInt() ) {
      	    tmp.sprintf( "%u", i );
      	    l->setText( tmp );
      	}
			}else{
      	if ( tmp.isEmpty() )
  				l->setText( "0" );
				l->setText( l->text().lower() );
				// TODO: validating of IPv6 here, at least check for valid digits and letters
			}	
    	return FALSE;
    }

    if ( !e || e->type() != QEvent::KeyPress ||
			 !( ((QKeyEvent*)e)->text() == "." ||
	    ((QKeyEvent*)e)->key() == Key_Enter ||
	    ((QKeyEvent*)e)->key() == Key_Return ||
	    ((QKeyEvent*)e)->key() == Key_Right ||
	    ((QKeyEvent*)e)->key() == Key_Left ||
//	    ((QKeyEvent*)e)->key() == Key_Tab ||
	    ((QKeyEvent*)e)->key() == Key_BackTab ) ){
				return FALSE;
		}

		// move left
    if ( ((QKeyEvent*)e)->key() == Key_BackTab ||
		 ((QKeyEvent*)e)->key() == Key_Left ) {

    	if ( ((QKeyEvent*)e)->key() == Key_Left &&
    	     ((QLineEdit*)o)->cursorPosition() > 0 )
    	    return FALSE;

			int i = 0;
			bool found = false;
			while( i < 9 && ab[i] && ab[i]->isVisible() ){
				if( o == ab[i] ){
					if( i > 0 ){
						ab[i-1]->setFocus();
	    	    ab[i-1]->setCursorPosition( ab[i-1]->text().length() );
					}
					found = true;
					break;
				}
				i++;
			}
			if( !found ) return false;

    	return true;
    }

    // at this point, we know we're moving right.
    if ( ((QKeyEvent*)e)->key() == Key_Right &&
		 ((QLineEdit*)o)->cursorPosition() < (int)((QLineEdit*)o)->text().length() )
			return false;

    // at this point we know we're REALLY moving right.
		int i = 0;
		bool found = false;
		while( i < 9 && ab[i] && ab[i]->isVisible() ){
			if( o == ab[i] ){
				if( i < 8 && ab[i+1]->isVisible() ){
					ab[i+1]->setFocus();
    	    ab[i+1]->setCursorPosition( 0 );
				}
				found = true;
				break;
			}
			i++;
		}
		if( !found ){
			if ( ((QKeyEvent*)e)->key() == Key_Right )
		    return false;
			emit returnPressed(); // it's a lie but...
			return false;
    }
#endif

  return true;
}

void AddressEdit::setProtocol( int a_pr ){
	// protocols were added together with IPv6 support
#ifdef GBLI_IPv6
	pr = (AddressEdit::Protocol)a_pr;
	int fields = 4;
	int max_len = 3;
	QFont font = ab[0]->font();
	if( (Protocol)a_pr == IPv6 ){
//		font.setPointSize( font.pointSize() - 1 );
		fields = 9;
		max_len = 4;
	}
	
	for( int i = 0; i < 9; i++ ){
		if( i < fields ){
			ab[i]->show();
		}else{
			ab[i]->hide();
		}
		if( fields == 4 ){
			if( i < 3 )
				labAB[i]->show();
			else if( i < 8 )
				labAB[i]->hide();
		}else if( i < 8 )
			labAB[i]->show();
		ab[i]->setMaxLength( max_len );
		if( fields == 9 ){
			if( i == 8 ){
	      ab[i]->setValidator( new QIntValidator( 0, 999, ab[i] ) );
	   	  ab[i]->setAlignment( AlignLeft );
			}else{
		    ab[i]->setValidator( NULL );
				// TODO: make normal validator
	   	  ab[i]->setAlignment( AlignCenter );
			}
		}else{
 			if( i == 0 )
	      ab[i]->setValidator( new QIntValidator( 0, m == Address ? 223 : 255, ab[i] ) );
			else
	      ab[i]->setValidator( new QIntValidator( 0, 255, ab[i] ) );
      ab[i]->setAlignment( AlignRight );
		}
		ab[i]->setFont( font );
	}

  setFixedSize(sizeHint());
	QWidget *par = parentWidget();
	if( par )
		par->updateGeometry();

#endif
}

QString AddressEdit::text(){
#ifndef GBLI_IPv6
	    return b1->text() + "." + b2->text() + "." + b3->text() + "." + b4->text();
#else
		QString res = "";
		int iMax = (pr == IPv6 && ab[4] && ab[4]->isEnabled())?9:4;
		QString delim = (iMax == 4)?".":":";
		for( int i = 0; i < iMax; i++ ){
			QString tmp = ab[i]->text();
			while( tmp.length() > 1 && tmp[0] == '0' )
				tmp = tmp.right( tmp.length() - 1 );
			res += tmp;
			if( i < iMax-1 ){
				if( i == 7 )
					res += "/";
				else
					res += delim;
			}
		}
		return res;
#endif
}

struct in_addr AddressEdit::address(){
#ifndef GBLI_IPv6
    struct in_addr	addr;

    if (!inet_aton(text(), &addr))
			addr.s_addr = 0;
    return addr;
#else
	if( pr == IPv4 ){
    struct in_addr	addr;
    if (!inet_aton(text(), &addr))
			addr.s_addr = 0;
    return addr;
	}else{
		// TODO: don't know yet what to do with IPv6, it should be smth special
		// but it is not needed actually
    struct in_addr	addr;
		addr.s_addr = 0;
		return addr;
	}
#endif
}

QSize AddressEdit::sizeHint() const{
#ifndef GBLI_IPv6
  return QSize( fontMetrics().width( "255.255.255.255" ) + 20,
	  b1->sizeHint().height() + 4 );
#else
	if( pr == IPv4 ){
    return QSize( fontMetrics().width( "255.255.255.255" ) + 20,
		  ab[0]->sizeHint().height() + 4 );
	}else{
    return QSize( fontMetrics().width( "abcd1:abcd1:abcd1:abcd1:abcd1:abcd1:abcd1:abcd1/000" ) + 20,
		  ab[0]->sizeHint().height() + 4 );
	}
#endif
}


QSizePolicy AddressEdit::sizePolicy() const{
#ifndef GBLI_IPv6
    return b1->sizePolicy();
#else
		return ab[0]->sizePolicy();
#endif
}

void AddressEdit::drawContents( QPainter * p ){
#ifndef GBLI_IPv6
    p->setPen( NoPen );
    p->fillRect( contentsRect(), colorGroup().base() );
    p->setPen( colorGroup().text() );
    p->drawText( b1->x() + b1->width(), b1->y(),
		 b2->x() - b1->x() - b1->width(), b1->height(),
		 AlignCenter,
		 "." );
    p->drawText( b2->x() + b2->width(), b2->y(),
		 b3->x() - b2->x() - b2->width(), b2->height(),
		 AlignCenter,
		 "." );
    p->drawText( b3->x() + b3->width(), b1->y(),
		 b4->x() - b3->x() - b3->width(), b3->height(),
		 AlignCenter,
		 "." );
#else
    p->setPen( NoPen );
    p->fillRect( contentsRect(), colorGroup().base() );
    p->setPen( colorGroup().text() );

		int iMax = pr == IPv4?3:8;
		QString delim = pr == IPv4?".":":";
		for( int i = 0; i < iMax; i++ ){
			if( i == 7 )
				delim = "/";
      p->drawText( ab[i]->x() + ab[i]->width(), ab[i]->y(),
  		 ab[i+1]->x() - ab[i]->x() - ab[i]->width(), ab[i]->height(),
  		 AlignCenter, delim );
		}
#endif
}

bool AddressEdit::edited() const{
#ifndef GBLI_IPv6
    return b1->edited() || b2->edited() || b3->edited() || b4->edited();
#else
	int iMax = (pr == IPv6 && ab[4] && ab[4]->isVisible())?9:4;
	for( int i = 0; i < iMax; i++ )
		if( ab[i]->edited() ) return true;
	return false;
#endif
}

void AddressEdit::setAddress( struct in_addr addr ){
#ifndef GBLI_IPv6
    unsigned long haddr;
    char	octet[4][10];
    int		i;

    haddr = ntohl(addr.s_addr);
    for (i = 0; i < 4; i++, haddr >>= 8)
			sprintf(octet[i], "%u", (unsigned int) (haddr & 0xFF));

    b1->setText(octet[3]);
    b2->setText(octet[2]);
    b3->setText(octet[1]);
    b4->setText(octet[0]);
#else
    unsigned long haddr;
    char	octet[4][10];
    int		i;

    haddr = ntohl(addr.s_addr);
    for (i = 0; i < 4; i++, haddr >>= 8)
			sprintf(octet[i], "%u", (unsigned int) (haddr & 0xFF));

    ab[0]->setText(octet[3]);
    ab[1]->setText(octet[2]);
    ab[2]->setText(octet[1]);
    ab[3]->setText(octet[0]);
#endif
}

void AddressEdit::setAddress( const QString & a ){
#ifndef GBLI_IPv6
    if ( !a.length() || a == "0.0.0.0" )
			return;

    struct in_addr tmp;
    if ( ::inet_aton( a, &tmp ) == 0 )
			return;
    setAddress(tmp);
#else
    if ( !a.length() || a == "0:0:0:0:0:0:0:0" || a == "0.0.0.0" ||
					a == ":0" ){
				D( "setAddress: %s\n", a.ascii() );
				return;
		}

		if( pr == IPv4 ){
				D( "setAddress (IPv4): %s\n", a.ascii() );
      struct in_addr tmp;
      if ( ::inet_aton( a, &tmp ) == 0 )
  			return;
      setAddress(tmp);
		}else{
			D("setAddress (IPv6): %s\n", a.ascii() );
			int i = 0;
			int pos = a.length()-1, prev_pos = a.length()-1, len;
			QString str;
			while( i < 9 && pos >= 0 ){
				if( i == 0 ){
					pos = a.findRev( "/", pos );
					if( pos < 0 ){
						i++;
						pos = a.findRev( ":", pos );
					}
				}else{
					pos = a.findRev( ":", pos );
				}
				pos++;
//				if( pos == -1 ){
//					if( prev_pos >= 0 ){
//						pos = 0;
//						prev_pos++;
//					}else
//						break;
//				}
				len = prev_pos - pos + 1;
				str = a.mid( pos, len );
				str.truncate( 4 );
				str = str.lower();
				ab[8-i]->setText( str );
//printf( "i: %d - %s (%d,%d - %d)\n", i, str.ascii(), pos, prev_pos, len );
				pos -= 2;
				prev_pos = pos;
				i++;
			}
		}
#endif
}


void AddressEdit::setEnabled( bool enable ){
#ifndef GBLI_IPv6
    b1->setEnabled( enable );
    b2->setEnabled( enable );
    b3->setEnabled( enable );
    b4->setEnabled( enable );
    QWidget::setEnabled( enable );
#else
		for( int i = 0; i < 9; i++ )
	    ab[i]->setEnabled( enable );
    QWidget::setEnabled( enable );
#endif
}


Network::Network( Netinfo *net, QWidget * parent, QString a_adapter, const char * name )
    : QWidget( parent, name ), adapter( a_adapter ), info( net )
{
		this->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

		QVBoxLayout *loVert = new QVBoxLayout( this, 5 );
		QHBoxLayout *loHorz = new QHBoxLayout();
		loVert->addLayout( loHorz, 1 );

		QVBoxLayout *V1 = new QVBoxLayout();
		loHorz->addLayout( V1, 1 );

    bg = new QButtonGroup( 0 );
    QRadioButton *r, *d;

    r = new QRadioButton( tr( "No ethernet" ), this );
		r->setAutoMask(true);
		r->setFocus();

    bg->insert( r, Netinfo::NoConfig );
    V1->addWidget( r );
    d = new QRadioButton( wrapString( tr( "Ethernet configured using DHCP" ), 20 ), this );
//    d = new QRadioButton( tr( "Ethernet configured using DHCP" ), this );
		d->setAutoMask(true);
    bg->insert( d, Netinfo::UseDHCPConfig );
    V1->addWidget( d );
    r = new QRadioButton( wrapString( tr( "Ethernet configured statically" ), 20 ), this );
//    r = new QRadioButton( tr( "Ethernet configured statically" ), this );
		r->setAutoMask(true);
    if (net->hadDHCP())
		 {
      d->setChecked(true);
			d->setFocus();
		 }
    else
     {

			r->setChecked( true );
			//Add default selection
      r->setFocus();
			}
    bg->insert( r, Netinfo::UseStaticConfig );
    V1->addWidget( r );

    connect( bg, SIGNAL(clicked(int)),
	     this, SLOT(setMode(int)) );

		QHBoxLayout *H2 = new QHBoxLayout();
		loHorz->addLayout( H2, 3 );

		H2->addSpacing( 5 );
		QGridLayout *GL = new QGridLayout();
//		QVBoxLayout *V3 = new QVBoxLayout();
		H2->addStretch( 1 );
		H2->addLayout( GL, 1 );
		H2->addSpacing( 5 );
//		V3->addLayout( GL, 1 );
//		V3->addStretch( 1 );
		GL->setColStretch( 1, 1 );

#ifdef GBLI_IPv6
		QLabel *lab = new QLabel( tr( "Interface:"), this );
		lab->setAutoMask( true );
		GL->addWidget( lab, 0, 0, AlignLeft );
		intCombo = new QComboBox( false, this );
		intCombo->insertItem( tr("Intranet"), Netinfo::Intranet );
		intCombo->insertItem( tr("Internet"), Netinfo::Extranet );
		intCombo->insertItem( tr("Both nets"), Netinfo::Bothnet );
		GL->addWidget( intCombo, 0, 1, AlignLeft );
		changeInterface( Netinfo::Intranet );
		connect( intCombo, SIGNAL( activated(int) ), this, SLOT( changeInterface(int) ) );

		lab = new QLabel( tr( "Protocol:"), this );
		lab->setAutoMask( true );
		GL->addWidget( lab, 1, 0, AlignLeft );
		ipCombo = new QComboBox( false, this );
		ipCombo->insertItem( "IPv4", AddressEdit::IPv4 );
		ipCombo->insertItem( "IPv6", AddressEdit::IPv6 );
		ipCombo->insertItem( tr("IPv4 and IPv6"), AddressEdit::IPv46 );
		GL->addWidget( ipCombo, 1, 1, AlignLeft );
		connect( ipCombo, SIGNAL( activated(int) ), this, SLOT( changeProtocol(int) ) );
#endif

    ipEdit = new AddressEdit( AddressEdit::Address, this );
    netmaskEdit = new AddressEdit( AddressEdit::Netmask, this );
//		routerEdit = new AddressEdit( AddressEdit::Router, this );

#ifdef GBLI_IPv6
    ip6Edit = new AddressEdit( AddressEdit::Address, this, AddressEdit::IPv6 );
#endif

		l1 = new QLabel(tr("IP Address:"), this );
		l1->setAutoMask(true);
		GL->addWidget( l1, 2, 0, AlignLeft );
		GL->addWidget( ipEdit, 2, 1, AlignLeft );

		l2 = new QLabel(tr("Netmask:"), this );
		l2->setAutoMask(true);
		GL->addWidget( l2, 3, 0, AlignLeft );
		GL->addWidget( netmaskEdit, 3, 1, AlignLeft );

#ifdef GBLI_IPv6
		intCombo->setFixedWidth( ipEdit->size().width() );
		ipCombo->setFixedWidth( ipEdit->size().width() );
		QGridLayout *GL2 = new QGridLayout( 2, 4 );
		loVert->addStretch( 1 );
		loVert->addLayout( GL2, 1 );
		loVert->addStretch( 4 );
		l3 = new QLabel(tr("IPv6 Address:"), this );
		l3->setAutoMask(true);
		GL2->addWidget( l3, 0, 0, AlignLeft );
		GL2->addMultiCellWidget( ip6Edit, 0, 0, 1, 3, AlignLeft );
#endif

//    ipEdit->setFocus();

    addInputWidget( ipEdit );
    addInputWidget( netmaskEdit );
#ifdef GBLI_IPv6
    addInputWidget( ip6Edit );
#endif

    connect( info, SIGNAL(newConfig()),
	     this, SLOT(updateWidgets()) );

    if (net->hadDHCP())
      setMode(Netinfo::UseDHCPConfig);
    else
      setMode(Netinfo::UseStaticConfig);

		changeProtocol( AddressEdit::IPv4 );
#ifdef GBLI_IPv6
//printf( "check123 %s\n", adapter.ascii() );
		char buf[64];
		sprintf( buf, "192.168.1.%d", adapter.right( 1 ).toInt()+2 );
		net->setAddress( adapter, buf, Netinfo::Default );
		net->setNetmask( adapter, "255.255.255.0", Netinfo::Default );
//		sprintf( buf, "0:0:0:%d/0", adapter.right( 1 ).toInt()+1 );
//		net->setAddress6( adapter, buf, Netinfo::Default );
		net->setAddress6( adapter, "0:0:0:0:0:0:0:0/0", Netinfo::Default );
#endif

    // Update widget content with data from BOOTP/netprobe we have so far
    updateWidgets();
		setMode(0);
}

Network::~Network()
{
}

//////////////////////////////////////////////////////////////////
// Do return/focus handling
//////////////////////////////////////////////////////////////////
void Network::addInputWidget(QWidget *edit)
{
    connect( edit, SIGNAL(returnPressed()),
	     this, SLOT(updateInfo()) );
    edit->installEventFilter(this);
}

bool Network::eventFilter(QObject *, QEvent *e)
{
    if (e->type() == QEvent::FocusIn || e->type() == QEvent::FocusOut)
    	updateInfo();
    return false;
}

//////////////////////////////////////////////////////////////////
// Debug function
//////////////////////////////////////////////////////////////////
#ifdef NET_DEBUG
#define DUMP_WIDGET(w)	\
	printf("%-15s: \"%s\"%s\n", \
		#w, w->text().ascii(), w->edited()? " (edited)" : "")
#define DUMP_ALL_WIDGETS	\
	do {	\
		DUMP_WIDGET(ipEdit); \
		DUMP_WIDGET(netmaskEdit); \
		DUMP_WIDGET(routerEdit); \
		DUMP_WIDGET(ns1Edit); \
		DUMP_WIDGET(ns2Edit); \
		DUMP_WIDGET(nameEdit); \
		DUMP_WIDGET(nisEdit); \
	} while (0)
#endif

//////////////////////////////////////////////////////////////////
// Update displayed values from probe data
//////////////////////////////////////////////////////////////////
void Network::updateWidgets()
{
    D("updating widgets%s\n", ipEdit? "" : " (no ipEdit)");
    if (!ipEdit)
			return;
//    DUMP_ALL_WIDGETS;

#ifndef GBLI_IPv6
    if (!ipEdit->edited())
        ipEdit->setAddress(info->getAddress());

    if (!netmaskEdit->edited())
        netmaskEdit->setAddress(info->getNetmask());
#else
//printf( "update123 %s\n", adapter.ascii() );
    if (!ipEdit->edited())
        ipEdit->setAddress( info->getAddress( adapter ));

    if (!netmaskEdit->edited())
        netmaskEdit->setAddress( info->getNetmask( adapter ));

    if (!ip6Edit->edited())
        ip6Edit->setAddress( info->getAddress6( adapter ));

		ipCombo->setCurrentItem( info->getProtocol( adapter ));
#endif

//    if (!routerEdit->edited())
//        routerEdit->setAddress(info->getGateway());
}

void Network::changeProtocol( int a_pr ){
#ifdef GBLI_IPv6
	switch( a_pr ){
		case 2:
			ipEdit->show();
			netmaskEdit->show();
			l1->show(); l2->show();
			ip6Edit->show();
			l3->show();
			break;
		case 1:
			ipEdit->hide();
			netmaskEdit->hide();
			l1->hide(); l2->hide();
			ip6Edit->show();
			l3->show();
			break;
		case 0:
			ipEdit->show();
			netmaskEdit->show();
			l1->show(); l2->show();
			ip6Edit->hide();
			l3->hide();
			break;
		default: break;
	}
	ipCombo->setCurrentItem( a_pr );
	info->setProtocol( adapter, (AddressEdit::Protocol)a_pr );
#endif
}

void Network::changeInterface( int a_int ){
#ifdef GBLI_IPv6
	intCombo->setCurrentItem( a_int );
	info->setInterface( adapter, (Netinfo::Interface)a_int );
#endif
}

//////////////////////////////////////////////////////////////////
// Update network config from user input
//////////////////////////////////////////////////////////////////
void Network::updateInfo(){
    bool	changed = false;

    D("Network::updateInfo()\n");
//    DUMP_ALL_WIDGETS;

//    if (routerEdit->edited())
//	    info->setGateway(routerEdit->text(), Netinfo::User);

#ifndef GBLI_IPv6
    if (netmaskEdit->edited()
    	&& info->setNetmask(netmaskEdit->text(), Netinfo::User))
				changed = true;
    if (ipEdit->edited() && info->setAddress(ipEdit->text(), Netinfo::User))
			changed = true;
#else
    if (netmaskEdit->edited()
    	&& info->setNetmask( adapter, netmaskEdit->text(), Netinfo::User))
				changed = true;
    if (ipEdit->edited() && info->setAddress( adapter, ipEdit->text(), Netinfo::User))
			changed = true;
    if (ip6Edit->edited() && info->setAddress6( adapter, ip6Edit->text(), Netinfo::User))
			changed = true;
#endif

//    D("updateInfo(%schanged)\n", changed? "" : "un");
    if( changed )
			info->probe( true );

#ifndef GBLI_DEVEL
//		// lets don't mess up nfs mount
//		bool skip_eth0 = false;
//		if( adapter == "eth0" )
//			skip_eth0 = !system( "df|grep \"192.168.1.1:/data\"" );
//		if( !skip_eth0 ){
//			AddressEdit::Protocol prot = (AddressEdit::Protocol)ipCombo->currentItem();
//			if( prot == AddressEdit::IPv46 || prot == AddressEdit::IPv4 )
//				system( ("ifconfig "+ adapter +" "+ ipEdit->text()).ascii() );
//			if( prot == AddressEdit::IPv46 || prot == AddressEdit::IPv6 )
//				system( ("ifconfig "+ adapter +" inet6 "+ ip6Edit->text()).ascii() );
//		}
#endif
			
#ifdef NET_DEBUG_DUMP
    info->save("/tmp/netdebug");
#endif
}

void Network::setMode( int nm )
{
		bg->setButton( nm );

    //Adding selection redraw
		((QRadioButton*)bg->find(nm))->setFocus();
		
		//if ( !((QRadioButton*)bg->find(nm))->isChecked() )
			((QRadioButton*)bg->find(nm))->setChecked(true);

    bool e = ( nm == Netinfo::UseStaticConfig );
    ipEdit->setEnabled( e );
    netmaskEdit->setEnabled( e );
//    routerEdit->setEnabled( e );
#ifdef GBLI_IPv6
		ipCombo->setEnabled( e );
    ip6Edit->setEnabled( e );
		intCombo->setEnabled( nm != Netinfo::NoConfig );
    info->setMode( adapter, (Netinfo::Setup)nm );
#else
    info->setMode( (Netinfo::Setup)nm );
#endif
}


//////////////////////////////////////////////////////////////////
// SitInfo class
//////////////////////////////////////////////////////////////////
SitInfo::SitInfo()
{
	OnBoot = TRUE;
	Name = "";
	DestIP = "0.0.0.0";
	LocalIP = "0:0:0:0:0:0:0:0/0";
	RouteNetworks.clear();
	RouteNetworks.append( QString("2000::/3") );	
	RouteNetworks.append( QString("3ffe::/16") );	
}

SitInfo::~SitInfo()
{
}

//////////////////////////////////////////////////////////////////
// Netinfo item class
//////////////////////////////////////////////////////////////////
Netinfo::Item::Item(const char *n, const char *v)
: source(Default), name(n)
{
//printf( "new Item: %s\n", v );
	if( !v )
		value = "";
	else
		value = QString::fromLatin1(v);
}

Netinfo::Item::~Item()
{
    // NOP
}

bool Netinfo::Item::set(const char *v, Source s)
{
    if( s < source )
			return false;
//printf( "set: %s\n", v );
//printf( "set1: %d\n", value.length() );
//printf( "set2: %s\n", value.ascii() );
    if( !v || !strcmp(value, v) )
			return false;
#ifdef NET_DEBUG
    D("%s=\"%s\" ", name.ascii(), v);
    switch (s) {
      case Default:
  			D("(default)"); break;
      case Wildguess:
  			D("(guessed)"); break;
      case Probed:
  			D("(probed)"); break;
      case User:
  			D("(user)"); break;
      case Windows:
  			D("(windows)"); break;
			default: break;
    }
    D("\n");
#endif
    source = s;
    value = v;

		// Lets remove strange spaces, cause they r killing the system ;-)
		int pos = 0;
		while( (pos = value.find( " ", pos )) >= 0 )
			value.remove( pos, 1 );

    return true;
}


//////////////////////////////////////////////////////////////////
// Netinfo class
//////////////////////////////////////////////////////////////////
Netinfo::Netinfo(void)
:
  dhcp(FALSE),
#ifndef GBLI_IPv6
	mode(UseStaticConfig),
  address("Address", "192.168.1.2"),
  netmask("Netmask", "255.255.255.0"),
#endif
  gateway("Gateway", "0.0.0.0"),
  gatewayDevice("GatewayDevice", "eth0"),
  nameserver("Nameserver", "0.0.0.0"),
  nameserver2("Nameserver2", "0.0.0.0"),
  hostname("Hostname", "localhost.localdomain" /*"noname.nodomain.nowhere"*/ ),
  nisdomain("NISDomain", ""),
	proxy( "Proxy", "" ),
	proxyport( "ProxyPort", "3128" ),
	noproxyfor( "NoProxyFor", "" ),
	ADSLmode(UseStaticConfig),
	adsladdress("ADSLAddress", "0.0.0.0"),
  adslnetmask("ADSLNetmask", "255.255.255.0"),
	adslcard("ADSLCard", "eth0"),
	adsluser("ADSLUser",""),
	adslpass("ADSLPasswd",""),
	isdnuser("ISDNUser",""),
	isdnpass("ISDNPass",""),
	isdnoutphone("ISDNOutPhone",""),
	isdninphone("ISDNInPhone",""),
	isdntype("ISDNType","")
{
    netprobePath = getenv("NETPROBE");
    if (netprobePath == 0)
      netprobePath = "./extrabin/lnetprobe";

    // if running with dhpc information, use it.
    inputBOOTP();

		SITdevices.setAutoDelete( TRUE );

    probeSock = 0;
    probePid = 0;

    startProbe(Idle, false);
		ISDNboot=true;

}

Netinfo::~Netinfo()
{
    stopProbe();
}

//////////////////////////////////////////////////////////////////
// Parse FOO="bar" lines
//////////////////////////////////////////////////////////////////
static inline char *parseShellAssignment(char *name)
{
    char	*value, *ep;

    if ((value = strchr(name, '=')) == 0)
      return 0;
    *value++ = '\0';

    // Convert "bar" to bar
    if (*value == '"') {
      if ((ep = strchr(++value, '"')) == 0)
        return 0;
      *ep = '\0';
    } else {
      value = strtok(value, " \t\n\r");
    }

    // Skip empty string
    if (value == 0 || *value == '\0')
      return 0;

    return value;
}

//////////////////////////////////////////////////////////////////
// Start/stop netprobe
//////////////////////////////////////////////////////////////////
void Netinfo::probe(bool force)
{
    stopProbe();
#ifndef GBLI_IPv6
    if (netmask.source <= Probed) {
				// Run netmask probe first--the rest depends on it.
        startProbe(Netmask, force);
    } else {
				// Do all the other probes
				startProbe(GBLI, force);
    }
#else
		Item *item = netmask.find( "eth0" );
		if( item && item->source <= Probed ) {
				// Run netmask probe first--the rest depends on it.
        startProbe(Netmask, force);
    } else {
				// Do all the other probes
				startProbe(GBLI, force);
    }
#endif
}

void
Netinfo::startProbe(Probe probe, bool force)
{
// Maddy 1999 12 03 - No more probing
// while we don't have netwotk modules loaded
return;

#ifdef NETWORK_PROBING

#ifdef GBLI_DEVEL
	return;
#endif
    const char	*argv[16];
    int		i, fd[2], argc;

#if 0
    if (dhcp) {
			D("Skipping requested probe %d; already have BOOTP data\n", probe);
			return;
    }
#endif

    if (probePid) {
      D("Probe %d already running, %s\n",	probe, force? "terminating" : "skip");
			if (!force)
	  	  return;
      stopProbe();
    }

    // Don't start advanced probes unless we have a valid address.
    // Also skip probing for invalid IP addrs.
    if (probe != Idle) {
        struct in_addr	addr;
        unsigned long	haddr;

        if (address.source < User)
	  		  return;
        if (!inet_aton(address.value, &addr))
			    return;
        haddr = ntohl(addr.s_addr);
    	  if (haddr == 0 || (haddr >> 24) == 127 || (haddr >> 24) > 223)
	    		return;
    }

    D("starting probe %d...\n", probe);
    QApplication::flushX();
    ::pipe(fd);

    probePid = ::fork();
    if (probePid < 0) {
			probePid = 0;
			::close(fd[0]);
			::close(fd[1]);
			return;
    }
    if (probePid == 0) {
    	::close(fd[0]);

    	::dup2(::open("/dev/null", O_RDWR), 0);
    	::dup2(fd[1], 1);
    	::dup2(0, 2);

    	i = getdtablesize();
    	while (--i > 2)
    	    ::close( i );

    	argc = 0;
    	argv[argc++] = "netprobe";

    	argv[argc++] = "--flush";
    	argv[argc++] = "--ignore-ifconfig";
    	argv[argc++] = "--nicely";
    	if (probe == Idle) {
    	    argv[argc++] = "--idle";
    	} else {
    	    argv[argc++] = "--address";
    	    argv[argc++] = address.value;
    	    if (probe != Netmask) {
    	        argv[argc++] = "--netmask";
    	        argv[argc++] = netmask.value;
    	    }
    	    argv[argc++] = "--probe";
    	    switch (probe) {
    	    case Netmask:
    		argv[argc++] = "netmask";
    		break;
    	    case GBLI:
    		argv[argc++] = "gbli";
    		break;
    	    default:
    		exit(1);
    	    }
    	}
    	argv[argc] = 0;
    #ifdef NET_DEBUG
    	D("Invoking");
    	for (int z = 0; argv[z]; z++)
    		D(" %s", argv[z]);
    	D("\n");
    #endif
			::nice( 5 );
    	execv(netprobePath, (char **) argv);
    	D("OOPS: %m");
    	exit(1);
    }
		D("startProbe pid: %d\n", probePid );
    ::close(fd[1]);

    probeSock = new QSocket(fd[0], this);
    probeSock->setMode(QSocket::Ascii);
    connect(probeSock, SIGNAL(readyRead()),
	     this, SLOT(inputNetprobe()) );
    connect(probeSock, SIGNAL(closed()),
	     this, SLOT(inputNetprobe()) );
    currentProbe = probe;
#endif
}

void
Netinfo::stopProbe()
{
    if (probePid != 0) {
			D("Killing current probe %d, pid=%d\n", currentProbe, probePid);
			::kill(probePid, 15);
			::waitpid(probePid, NULL, 0 );
			probePid = 0;
    }
    if (probeSock != 0) {
			delete probeSock;
			probeSock = 0;
    }
}

//////////////////////////////////////////////////////////////////
// Read stuff from the bootpc file
//////////////////////////////////////////////////////////////////
void Netinfo::inputBOOTP()
{
    char	buffer[1024], *name, *value;
    int		count;
    FILE	*fp;

    if ((fp = fopen("/tmp/bootpc.out", "r")) == 0)
      return;

    D("Reading BOOTP information.\n");
    dhcp = TRUE;
    while (fgets(buffer, sizeof(buffer), fp) != NULL) {
	count = strlen(buffer);
	if (buffer[count-1] == '\n')
	  buffer[--count] = '\0';

	name = buffer;
	if ((value = parseShellAssignment(name)) == 0)
	  continue;

	if (!strcmp(name, "HOSTNAME")) {
	  hostname.set(value, User);
	} else if (!strcmp(name, "NISDOMAIN")) {
	  nisdomain.set(value, User);
	} else {
	  // Assume it's an address
	  struct in_addr	addr;

	  value[strspn(value, "0123456789.")] = '\0';
	  if (!inet_aton(value, &addr))
	    continue;
#ifndef GBLI_IPv6
	  if (!strcmp(name, "IPADDR"))
	    address.set( value, User );
	  else if (!strcmp(name, "NETMASK"))
	    netmask.set( value, User);
#else
	  if (!strcmp(name, "IPADDR"))
	    setAddress( "eth0", value, User );
	  else if (!strcmp(name, "NETMASK"))
	    setNetmask( "eth0", value, User);
#endif
	  else if (!strcmp(name, "GATEWAYS"))
	    gateway.set(value, User);
	  else if (!strcmp(name, "DNSSRVS"))
	    nameserver.set(value, User);
	}
    }
    fclose(fp);
}

//////////////////////////////////////////////////////////////////
// Read stuff from the netprobe pipe
//////////////////////////////////////////////////////////////////
void Netinfo::inputNetprobe()
{
    char	buffer[1024], *name, *value;
    bool	update = false;

    if (probeSock == 0)
	return;

    D("Input from netprobe\n");
    while (probeSock->canReadLine() > 0) {
	QString result = probeSock->readLine();

	// Annoying, but parsing with QString is even less readable IMHO.
	D("> %s", (const char *) result);
	if (strlen(result) >= sizeof(buffer))
	  continue;
	strcpy(buffer, result);

	name = buffer;
	if ((value = parseShellAssignment(name)) == 0)
	  continue;

	D("name=%s value=%s\n", name, value);
	if (!strcmp(name, "HOSTNAME")) {
	  hostname.set(value, Probed);
	} else if (!strcmp(name, "NISDOMAIN")) {
	  nisdomain.set(value, Probed);
	} else {
	  // Assume it's an address
	  struct in_addr addr;

	  value[strspn(value, "0123456789.")] = '\0';
	  if (!inet_aton(value, &addr))
	    continue;

#ifndef GBLI_IPv6
	  if (!strcmp(name, "ADDRESS")) {
	    address.set( value, Probed);
	  } else if (!strcmp(name, "NETWORK")) {
	    address.set( value, Wildguess);
	    addr.s_addr = htonl(ntohl(addr.s_addr) + 1);
	    gateway.set(inet_ntoa(addr), Wildguess);
	  } else if (!strcmp(name, "NETMASK")) {
	    netmask.set( value, Probed);
#else
	  if (!strcmp(name, "ADDRESS")) {
	    setAddress( "eth0", value, Probed);
	  } else if (!strcmp(name, "NETWORK")) {
	    setAddress( "eth0", value, Wildguess);
	    addr.s_addr = htonl(ntohl(addr.s_addr) + 1);
	    gateway.set(inet_ntoa(addr), Wildguess);
	  } else if (!strcmp(name, "NETMASK")) {
	    setNetmask( "eth0", value, Probed);
#endif
	  } else if (!strcmp(name, "GATEWAY")) {
	    gateway.set(value, Probed);
	  } else if (!strcmp(name, "DNSSERV0")
		  || !strcmp(name, "NAMESERVER")) {
	    nameserver.set(value, Probed);
	  } else {
	    continue;
	  }
	}
	update = true;
    }

    if (update)
        emit newConfig();

    if (probeSock->state() == QSocket::Idle) {
	stopProbe();
	if (currentProbe == Idle)
	  startProbe(Netmask, false);
	if (currentProbe == Netmask)
	  startProbe(GBLI, false);
    }
}

//////////////////////////////////////////////////////////////////
// Set various items from user dialog
//////////////////////////////////////////////////////////////////
#ifndef GBLI_IPv6
bool Netinfo::setMode( Setup m )
{
    if (mode == m)
			return false;
    mode = m;
    return true;
}
#else
bool Netinfo::setMode( QString adapter, Setup m )
{
		Setup *pmode = mode.find( adapter );
		if( !pmode ){
			pmode = new Setup;
			mode.insert( adapter, pmode );
		}

    if (*pmode == m)
			return false;
    *pmode = m;
    return true;
}
#endif

#ifdef GBLI_IPv6
bool Netinfo::setInterface( QString adapter, Netinfo::Interface a_int ){
//printf( "changeInterface: %s, %d\n", adapter.latin1(), a_int );
		Netinfo::Interface *p = interface.find( adapter );
		if( !p ){
			p = new Netinfo::Interface;
			interface.insert( adapter, p );
		}

    if( *p == a_int )
			return false;
    *p = a_int;
    return true;
}
#endif

bool Netinfo::setProtocol( QString adapter, AddressEdit::Protocol pr )
{
#ifdef GBLI_IPv6
		AddressEdit::Protocol *p = protocol.find( adapter );
		if( !p ){
			p = new AddressEdit::Protocol;
			protocol.insert( adapter, p );
		}

    if( *p == pr )
			return false;
    *p = pr;
    return true;
#endif
}


bool Netinfo::setADSLMode(Setup m)
{
    if (ADSLmode == m)
		return false;
    ADSLmode = m;
    return true;
}

bool Netinfo::setISDNMode(Setup m)
{
    if (ISDNmode == m)
		return false;
    ISDNmode = m;
    return true;
}

#ifndef GBLI_IPv6
bool Netinfo::setAddress( const char *value, Source src )
{
    return address.set(value, src);
}
#else
bool Netinfo::setAddress( QString adapter, const char *value, Source src)
{
//printf( "Netinfo::setAddress %s - %s\n", adapter.ascii(), value );
		Item *item = address.find( adapter );
		if( !item ){
			item = new Item( "Address", "" );
			address.insert( adapter, item );
		}
		return item->set(value, src);
}

bool Netinfo::setAddress6( QString adapter, const char *value, Source src)
{
//printf( "Netinfo::setAddress6 %s - %s\n", adapter.ascii(), value );
		Item *item = address6.find( adapter );
		if( !item ){
			item = new Item( "Address6", "" );
			address6.insert( adapter, item );
		}

		return item->set(value, src);
}
#endif

bool
Netinfo::setADSLAddress(const char *value, Source src)
{
    return adsladdress.set(value, src);
}

#ifndef GBLI_IPv6
bool Netinfo::setNetmask(const char *value, Source src)
{
    return netmask.set(value, src);
}
#else
bool Netinfo::setNetmask(QString adapter, const char *value, Source src)
{
		Item *item = netmask.find( adapter );
		if( !item ){
			item = new Item( "Netmask", "" );
			netmask.insert( adapter, item );
		}

		return item->set(value, src);
}
#endif

bool
Netinfo::setADSLNetmask(const char *value, Source src)
{
    return adslnetmask.set(value, src);
}

bool
Netinfo::setADSLCard(const char *value, Source src)
{
    return adslcard.set(value, src);
}

bool
Netinfo::setADSLUser(const char *value, Source src)
{
    return adsluser.set(value, src);
}

bool
Netinfo::setADSLPass(const char *value, Source src)
{
    return adslpass.set(value, src);
}

bool
Netinfo::setGateway(const char *value, Source src)
{
    return gateway.set(value, src);
}

bool
Netinfo::setGatewayDevice(const char *value, Source src)
{
    return gatewayDevice.set(value, src);
}

bool
Netinfo::setNameserver(const char *value, Source src)
{
    return nameserver.set(value, src);
}

bool
Netinfo::setNameserver2(const char *value, Source src)
{
    return nameserver2.set(value, src);
}

bool
Netinfo::setHostname(const char *name, Source src)
{
    return hostname.set(name, src);
}

bool
Netinfo::setNISDomain(const char *name, Source src)
{
    return nisdomain.set(name, src);
}

bool
Netinfo::setProxyName(const char *name, Source src)
{
    return proxy.set(name, src);
}

bool
Netinfo::setProxyPort(const char *name, Source src)
{
    return proxyport.set(name, src);
}

bool
Netinfo::setISDNPass(const char *name, Source src)
{
    return isdnpass.set(name, src);
}

bool
Netinfo::setISDNUser(const char *name, Source src)
{
    return isdnuser.set(name, src);
}

bool
Netinfo::setISDNInPhone(const char *name, Source src)
{
    return isdninphone.set(name, src);
}

bool
Netinfo::setISDNOutPhone(const char *name, Source src)
{
    return isdnoutphone.set(name, src);
}

bool
Netinfo::setISDNType(const char *name, Source src)
{
    return isdntype.set(name, src);
}

QString Netinfo::removePrefix( QString a_str ){
	QString tmp = a_str;
	int pos = tmp.find( "/" );
//printf( "pos (%s): %d\n", tmp.ascii(), pos );
	if( pos > 0 )
 		tmp = tmp.left( pos );
	tmp = replaceZeroSeq( tmp );
	return tmp;
}

QString Netinfo::insertZeroSeq( QString a_str )
{
	int i, pos;
	
	if (a_str.find("::") == -1) return a_str;

	if (a_str.left(1) == ":") a_str = "0" + a_str;
	for (i = 0, pos = 0; (pos = a_str.find(":", pos) + 1) != 0; i++ );

	i = 9 - i;
	QString tmp("");

	while (--i > 0) tmp += ":0";
	tmp += ":";

	a_str.replace(a_str.find("::"), 2, tmp);

	return a_str;
}

QString Netinfo::replaceZeroSeq( QString a_str ){
	QString tmp = a_str;
// 	int iMax = tmp.length();
// 	bool in_sect = false;
	int begin = -1, end = -1, cur = 0;
//	printf( "str: %s\n", tmp.ascii() );
	while( (cur=tmp.find( ":0:", cur )) >= 0 ){
		if( begin < 0 ){
			begin = cur;
			end = begin + 2;
		}else	if( cur == end ){
			end = cur + 2;
		}else if( cur > end ){
			break;
		}
		cur++;
	}
	if (tmp.mid(end,3) == ":0/")
		tmp = tmp.replace(end+1, 1, "");
	if( begin > 0 && end > 0 ){
		tmp = tmp.left( begin ) +"::"+ tmp.right( tmp.length() - end - 1 );
//printf( "str: %s\n", tmp.ascii() );
	}
	if( tmp.left( 3 ) == "0::" )
		tmp.replace( 0, 3, "::" );
	return tmp;
}

void Netinfo::save( const char *root, QString userName ){
//		D("Netinfo::save(%s)\n", root);

    if( getenv("NET_ROOTDIR") != 0 )
			root = getenv("NET_ROOTDIR");
    else if( root == 0 )
			root = "./root";

		system( ("mkdir -p "+ QString( root )).ascii() );

		saveTunnels( root );
    saveEther(root);
//		saveADSL(root);
		saveADSLppp(root, userName);
		saveISDN(root, userName);
    saveResolv(root);
#ifdef NO_USED
    saveNIS(root);
#endif
    saveSysNetwork(root);
		saveProxy(root);
		saveHosts( root );
#ifdef NO_USED
    saveTmpNetwork();
#endif
		saveXDMCP( root );
}

bool Netinfo::saveEther(const char *root)
{
//    struct in_addr	addr, mask, broadcast, network;
		char sysCom[128];

#ifndef GBLI_IPv6
    D("Save ether, mode=%d\n", mode);
    if( mode == NoConfig )
		{
			sprintf(sysCom, "rm -f %s/etc/sysconfig/network-scripts/ifcfg-eth0", root);
			system(sysCom);
			return false;
		}

		char fname[100];
		char dev[5];

//		if(ADSLmode!=0){
//		  strcpy(dev,adslcard.value);
//		  dev[strlen(dev)-1] = dev[strlen(dev)-1]==48?49:48; // NOT ADSL CARD
//    }
//		else
		strcpy(dev,"eth0");

		char buf[128];
		strcpy( buf, "/etc/sysconfig/network-scripts/ifcfg-" );
		
		strcat(buf,dev);
		strcpy(fname,buf);

//    SysConfigFile config( root, "/etc/sysconfig/network-scripts/ifcfg-eth0" );
		SysConfigFile config( root, fname );
//    inet_aton(address.value, &addr);
//    inet_aton(netmask.value, &mask);
//    broadcast.s_addr = addr.s_addr | ~(mask.s_addr);
//    network.s_addr = addr.s_addr & mask.s_addr;

/*  config.append("#!/bin/sh");
    config.append("#>>>Device type: ethernet");
    config.append("#>>>Variable declarations:");

    config.set("DEVICE",	"eth0" );
    config.set("IPADDR",	address.value);
    config.set("NETMASK",	netmask.value);
    config.set("NETWORK",	inet_ntoa(network));
    config.set("BROADCAST",	inet_ntoa(broadcast));
    //config.set("GATEWAY",	gateway.value);
    config.set("ONBOOT",	"yes");
    config.set("DYNAMIC",	(mode == UseStaticConfig)? "" : "dhcp");

    config.append("#>>>End variable declarations");*/
    config.set("DEVICE", dev, false );
    config.set("BOOTPROTO", (mode == UseStaticConfig) ? "none" : "dhcp", false );
		if( mode != UseDHCPConfig ){
      config.set("IPADDR", address.value, false );
      config.set("NETMASK", netmask.value, false );
//      config.set("NETWORK", inet_ntoa(network), false );
//      config.set("BROADCAST", inet_ntoa(broadcast), false );
		}
    config.set("ONBOOT", "yes", false );

		if (strcmp(getGateway(), "0.0.0.0"))
		{
			config.set("GATEWAY", getGateway(), false);
		{
		if (strcmp(getNameserver(), "0.0.0.0"))
		{
			config.set("DNS1", getNameserver(), false);
		{
		if (strcmp(getNameserver2(), "0.0.0.0"))
		{
			config.set("DNS2", getNameserver2(), false);
		{

    return config.save();
#else
		int iAdapter = 0;

		bool ipv4 = false;
		bool ipv6 = false;

   	while( iAdapter < ADAPTERS_MAX )
		{
			QString strAdapter = "eth" + QString::number( iAdapter );
  		Setup *pmode = mode.find( strAdapter );
			iAdapter++;

			sprintf(sysCom, "rm -f %s/etc/sysconfig/network-scripts/ifcfg-%s", root, strAdapter.ascii());
			system(sysCom);

			if ( !pmode ) continue;
			if ( *pmode == -1 ) continue;

//			printf("-->Adapter: %s %d\n", strAdapter.ascii(), *pmode);

  		char fname[100];
  		char dev[5];

			// ADSL is preferrable
  		//if( ADSLmode != Netinfo::NoConfig && adslcard.value == strAdapter ) continue;

  		strcpy( dev, strAdapter.ascii() );

  		char buf[128];
  		strcpy( buf, "/etc/sysconfig/network-scripts/ifcfg-" );
  		strcat(buf,dev);
  		strcpy(fname,buf);

  		SysConfigFile config( root, fname );

      if( *pmode == NoConfig )
			{
	      config.set("ONBOOT", "yes", false );
	      config.set("DEVICE", dev, false );
	      config.set("BOOTPROTO", "none", false );
				config.save();

  			continue;
			}

      config.set("ONBOOT", "yes", false );
      config.set("DEVICE", dev, false );
      config.set("BOOTPROTO", (*pmode == UseStaticConfig) ? "none" : "dhcp", false );
  		if( *pmode != UseDHCPConfig )
			{
				AddressEdit::Protocol *ppr = protocol.find( strAdapter );
				if( ppr )
				{
					if( *ppr == AddressEdit::IPv4 || *ppr == AddressEdit::IPv46 )
					{
    		  	Item *paddress = address.find( strAdapter );
						if( paddress && paddress->value != "0.0.0.0" )
						{
							ipv4 = true;
	            config.set("IPADDR", paddress->value, false );
	      			Item *pnetmask = netmask.find( strAdapter );
  						if( pnetmask ){
//                inet_aton(paddress->value, &addr);
//                inet_aton(pnetmask->value, &mask);
//                broadcast.s_addr = addr.s_addr | ~(mask.s_addr);
//                network.s_addr = addr.s_addr & mask.s_addr;
                config.set("NETMASK", pnetmask->value, false );
//                config.set("NETWORK", inet_ntoa(network), false );
//                config.set("BROADCAST", inet_ntoa(broadcast), false );
  						}
						}
					}
					if( *ppr == AddressEdit::IPv6 || *ppr == AddressEdit::IPv46 ){
						ipv6 = true;
   	        config.set("IPV6INIT", "yes", false );
    	  		Item *paddress6 = address6.find( strAdapter );
    				if( paddress6 && paddress6->value != "0:0:0:0:0:0:0:0/0" )
    	        config.set("IPV6ADDR", replaceZeroSeq( paddress6->value ), true );
//    				else
//    	        config.set("#IPV6ADDR", "", true );
//   	        config.set("#IPV6ADDR_SECONDARIES", "", true );
					}
					else
   	        config.set("IPV6INIT", "no", false );
    		}
			}
			if (!strcmp(getGatewayDevice(), dev))
			{
				if (strstr(getGateway(), ":"))
				{
					if (strcmp(getGateway(), "0:0:0:0:0:0:0:0/0"))
						config.set("GATEWAY", replaceZeroSeq( getGateway() ), false );
				}
				else
					if (strcmp(getGateway(), "0.0.0.0"))
						config.set("GATEWAY", getGateway(), false );

				if (strstr(getNameserver(), ":"))
				{
					if (strcmp(getNameserver(), "0:0:0:0:0:0:0:0/0"))
						config.set("DNS1", replaceZeroSeq( getNameserver() ), false );
				}
				else
					if (strcmp(getNameserver(), "0.0.0.0"))
						config.set("DNS1", getNameserver(), false );

				if (strstr(getNameserver2(), ":"))
				{
					if (strcmp(getNameserver2(), "0:0:0:0:0:0:0:0/0"))
						config.set("DNS2", replaceZeroSeq( getNameserver2() ), false );
				}
				else
					if (strcmp(getNameserver2(), "0.0.0.0"))
						config.set("DNS2", getNameserver2(), false );
			}

			config.save();

		}

		if( ipv4 ){
			system( ("touch "+ NEWROOT +"/etc/sysconfig/static-routes").ascii() );
		}

		if( ipv6 ){
	 		SysConfigFile config1( root, "/etc/sysconfig/network-scripts/ifcfg-sit0" );
  	  config1.set("DEVICE", "sit0", true );
  	  config1.set("BOOTPROTO", "none", true );
  	  config1.set("ONBOOT", "yes", true );
//      config1.set("#IPV6INIT", "\"no\" # Initialize IPv6 \"yes\" or \"no\"", false );
      config1.set("IPV6INIT", "yes", true );
			config1.save();

			system( ("touch "+ NEWROOT +"/etc/sysconfig/static-routes-ipv6").ascii() );
		}

    return true;
#endif
}

bool Netinfo::saveISDN(const char *root, QString userName)
{
	char sysCom[512], type[10];

#ifdef GBLI_DEVEL
	if ((ISDNmode==Netinfo::NoConfig))
#else
	if ((ISDNmode==Netinfo::NoConfig) || !strcmp(isdntype.value.ascii(),"unknown") || !strcmp(isdntype.value.ascii(),"not found"))
#endif
	{
		if (ISDNmode==NoConfig) {
	  	//some simple code to remove old configuration
			QFile qfOldConfig(QString(root)+ "/etc/sysconfig/network-scripts/ifcfg-ippp0");
			if (qfOldConfig.exists()) qfOldConfig.remove();

      QFile qfOldConfig1(QString(root)+ "/etc/sysconfig/isdncard");
			if (qfOldConfig1.exists()) qfOldConfig1.remove();
  		//logins and passwords are saved
			
		if (root!=NULL){
			sprintf(sysCom,"chroot %s chkconfig isdn --level 0123456 off",root);
#ifndef GBLI_DEVEL
			system(sysCom);
#else
			printf("%s\n",sysCom);
#endif
    }
	
	} //remove old config if not needed

		return true;
	}



	D("written ISDN config (mode=%i, type=%s)\n",ISDNmode,isdntype.value.ascii());
	
	strcpy(sysCom,root);
	strcat(sysCom,"/etc/ppp");
	makeDirs(sysCom);

	strcpy(sysCom,root);
	strcat(sysCom,"/etc/sysconfig/network-scripts/");
	makeDirs(sysCom);

//NB!!! check for ipppX numbers
//not a problem ,ippp is used only for sync. isdn
  SysConfigFile config(root, "/etc/sysconfig/network-scripts/ifcfg-ippp0");
  config.set("DEVICE", "ippp0", false);
  config.set("MSN", isdninphone.value, false);
  config.set("PHONE_OUT", isdnoutphone.value, false);
  config.set("DIALMODE", "auto", false);
  config.set("USER", getISDNUser(), false);
  if (ISDNboot)
	{
		config.set("ONBOOT", "yes");
	}
	else
	{
   	config.set("ONBOOT","no");
	}
  config.set("HUPTIMEOUT", "60");
  config.set("CONNECT", "yes");

	if(!config.save()){
			return false;
	}

  SysConfigFile config2(root, "/etc/sysconfig/isdncard");
  config2.set("MODULE", "hisax");
  sprintf(type, "type=%s", isdntype.value.ascii());
  config2.set("RESOURCES", type);

	if(!config2.save()) return false;

	if (strlen(getISDNUser()) && strlen(getISDNPass()))
	{
	 char *cpFile;
	 for (int iCount=0;iCount<2;iCount++){
	 if (iCount==0) cpFile="/etc/ppp/pap-secrets";
	 else						cpFile="/etc/ppp/chap-secrets";
			
	   SysConfigFile config3(root, cpFile);
		 QString sSearch(getISDNUser());
	 	 QString sReplace;
		 QString qsMode("600");
		 //warning sReplace should be empty string,if we going to
		 //delete previuos old record
		 if((!oldisdnuser.isEmpty()) && (sReplace.isEmpty()))
		 config3.replaceChapPapSecretsLine( oldisdnuser, sReplace);

		 sReplace.sprintf("%s\t*\t%s",getISDNUser(),getISDNPass());
		 config3.replaceChapPapSecretsLine( sSearch, sReplace);
		 config3.SetMode(qsMode);
	}

	}

/*Wrong source ip after reconnect workaround*/	
	if (root!=NULL){
			sprintf(sysCom,"chroot %s chkconfig isdn --level 0123456 off",root);
#ifndef GBLI_DEVEL
			system(sysCom);
#else
			printf("%s\n",sysCom);
#endif
			sprintf(sysCom,"chroot %s chkconfig isdn --level 235 on",root);
#ifndef GBLI_DEVEL
			system(sysCom);
#else
			printf("%s\n",sysCom);
#endif
    }

	SysConfigFile config4(root, "/etc/sysctl.conf");
	config4.load();
	config4.set("net.ipv4.ip_dynaddr", "1",false);
	config4.save();

#ifdef GBLI_STANDALONE
		if (root!=NULL){
			sprintf(sysCom,"chroot %s sysctl -p",root);
#ifndef GBLI_DEVEL
			system(sysCom);
#else
			printf("%s\n",sysCom);
#endif
    }
#endif

	return true;
}

bool Netinfo::saveADSL(const char *root)
{
    struct /*in_addr	addr, mask, broadcast,*/ network;
		char sysCom[100];


    D("Save ADSL, mode=%d\n", ADSLmode);
    if( ADSLmode == 0)
		{
 			return true;
		}

		sprintf(sysCom,"%s/etc/sysconfig/network-scripts/",root);
		makeDirs(sysCom);

		char fname[100];
		char dev[5];
		strcpy(dev,adslcard.value);

		strcpy(fname,"/etc/sysconfig/network-scripts/ifcfg-");		
		strcat(fname,dev);

		SysConfigFile config( root, fname );
		config.load();
    config.set("DEVICE", dev, false );
    config.set("BOOTPROTO", "none", false );
		if( ADSLmode == 2 )
			config.set("IPADDR", adsladdress.value, false );
		else
			config.set("IPADDR", "0.0.0.0", false );
    config.set("NETMASK", adslnetmask.value, false );
    config.set("ONBOOT", "yes", false );

		if (!strcmp(getGatewayDevice(), dev))
		{
			if (strstr(getGateway(), ":"))
			{
				if (strcmp(getGateway(), "0:0:0:0:0:0:0:0/0"))
					config.set("GATEWAY", replaceZeroSeq( getGateway() ), false );
			}
			else
				if (strcmp(getGateway(), "0.0.0.0"))
					config.set("GATEWAY", getGateway(), false );

			if (strstr(getNameserver(), ":"))
			{
				if (strcmp(getNameserver(), "0:0:0:0:0:0:0:0/0"))
					config.set("DNS1", replaceZeroSeq( getNameserver() ), false );
			}
			else
				if (strcmp(getNameserver(), "0.0.0.0"))
					config.set("DNS1", getNameserver(), false );

			if (strstr(getNameserver2(), ":"))
			{
				if (strcmp(getNameserver2(), "0:0:0:0:0:0:0:0/0"))
					config.set("DNS2", replaceZeroSeq( getNameserver2() ), false );
			}
			else
				if (strcmp(getNameserver2(), "0.0.0.0"))
					config.set("DNS2", getNameserver2(), false );
		}

    return config.save();
}
/*
saveADSLppp(const char *root, QString userName)
YP
rewritten with supporting ifcfg-pppX files
See also readADSL method
*/

bool Netinfo::saveADSLppp(const char *root, QString userName)
{
		char sysCom[100];
		QString qsFile,qsDevice; //qsFile and qsDevice: must be empty!

    //D("Save ADSLppp, mode=%d\n", ADSLmode);
    if (ADSLmode != NoConfig &&  (!adsluser.value || !adslpass.value)) return false;
		//save

		if (!adslpppfile.isEmpty())
		{ //previous opened configuration's file
			//initialized in readADSL
			qsFile = adslpppfile;
			int iInd = qsFile.findRev("ppp"); //from the end
			if (iInd !=-1)
			{
				     if (!QFile(QString(root) + qsFile).remove()) return false; //cleanup
						 qsDevice = qsFile.mid(iInd);
						/* some additional check may be needed*/
			}
			else{
					#ifdef GBLI_DEVEL
						perror("saveADSLppp: ppp part not found!");
						printf("file name %s\n",qsFile.latin1());
					#endif
						qsFile = (char *)NULL;
					}
			//correctness of pppX name is checked in readADSL
			//here can be weekness of code if someone corrupts adslpppfile var
			//hey,u .. >:|
		}
			
		if (ADSLmode == NoConfig) return true;
		/* Now old login and password are not removed
			 Because that is not specially needed
			 Backup is not done now
		*/


		/*If something is incorrect then try next code*/
		if ((adslpppfile.isEmpty()) || (qsDevice.isEmpty()))
		for (int iInd=0;iInd < MAXPPPX; iInd++)
		{ //now MAXPPX =32
			qsFile.sprintf("/etc/sysconfig/network-scripts/ifcfg-ppp%d",iInd);
		  if (QFile(QString(root) + qsFile).exists()){		
					qsFile = (char *)NULL; //make it empty
					continue;
			}
			else
			{		//find empty name for our configuration file
				  //qsFile is initialized upper
					#ifdef GBLI_DEVEL
						printf("Found empty pppX name %d\n",iInd);
					#endif
						qsDevice.sprintf("ppp%d",iInd);
					break;
			}
		}

		//final check
		if ((qsFile.isEmpty()) || (qsDevice.isEmpty())) return false;
		
		// TODO: check modem can be also set pppX
		// and not only modems
    //QFile(QString(root) + qsFile).remove(); //cleanup again,for guarantee
		SysConfigFile config( root, qsFile.latin1() );
    //config.load(); //code do nothing
    config.set("DEVICE", qsDevice.latin1(), false );
    config.set("ONBOOT", "yes", false );
    config.set("TYPE", "xDSL", false );
    config.set("BOOTPROTO", "none", false );
    config.set("ETH", adslcard.value, false );
    config.set("USER", adsluser.value, false );
    config.set("FIREWALL", "none", false );
    config.set("USEPEERDNS", "yes", false );

    bool ret=config.save();

		sprintf(sysCom,"%s/etc/ppp",root);
		makeDirs(sysCom);

		if( adsluser.value.length() && adslpass.value.length() )
		{
  		if(ret)
  		{ //Updated code
				char *cpFile;
				for(int iCount=0;iCount<2;iCount++){
					 if (iCount==0) cpFile = "/etc/ppp/pap-secrets";
					 else 					cpFile = "/etc/ppp/chap-secrets";
				   SysConfigFile config3(root, cpFile);
					 QString sSearch(getADSLUser());
				 	 QString sReplace;
					 QString qsMode("600");
					 //warning sReplace should be empty string,if we going to
					 //delete previuos old record
					 if((!oldadsluser.isEmpty()) && (sReplace.isEmpty()))
					 config3.replaceChapPapSecretsLine( oldadsluser, sReplace);

					 sReplace.sprintf("%s\t*\t%s",getADSLUser(),getADSLPass());
					 config3.replaceChapPapSecretsLine( sSearch, sReplace);
					 config3.SetMode(qsMode);
				}
				
  		} //if ret
		}

		return ret;
}

bool Netinfo::saveResolv(const char *root)
{
#ifndef GBLI_IPv6
		if( mode == UseDHCPConfig ){
			system( ("touch "+ NEWROOT +"/etc/resolv.conf").ascii() );
			return true;
		}

    FILE	*fp;
    int		i;

    ConfigFile config( root, "/etc/resolv.conf" );

    fp = config.open( "w" );
    if (fp == 0)
			return false;

    if ((i = hostname.value.find('.')) > 0) {
			QString	domain = hostname.value.mid(i+1, (unsigned int) -1);

			fprintf(fp, "domain %s\nsearch %s\n",
			  domain.ascii(), domain.ascii() );
    }

    if( mode == NoConfig ){
	    fclose(fp);
			return false;
		}

    if (nameserver.value != "0.0.0.0")
			fprintf(fp, "nameserver %s\n", nameserver.value.ascii());
    if (nameserver2.value != "0.0.0.0")
			fprintf(fp, "nameserver %s\n", nameserver2.value.ascii());

    fclose(fp);
#else
    FILE	*fp;
    int		i;

    ConfigFile config( root, "/etc/resolv.conf" );

    fp = config.open( "w" );
    if( fp == 0 ) return false;

    if ((i = hostname.value.find('.')) > 0) {
			QString	domain = hostname.value.mid(i+1, (unsigned int) -1);
			fprintf(fp, "domain %s\nsearch %s\n", domain.ascii(), domain.ascii() );
    }

    if( nameserver.value != "0.0.0.0" )
			fprintf(fp, "nameserver %s\n", Netinfo::removePrefix( nameserver.value ).ascii() );
    if( nameserver2.value != "0.0.0.0" )
			fprintf(fp, "nameserver %s\n", Netinfo::removePrefix( nameserver2.value ).ascii() );

    fclose(fp);
#endif
    return true;
}

#ifdef NOT_USED
bool Netinfo::saveNIS(const char *root)
{
    FILE	*fp;

    if (nisdomain.value.isEmpty())
			return true;

    ConfigFile config( root, "/etc/nis.conf" );
    fp = config.open( "a" );
    if (fp == 0)
			return false;
    fprintf( fp, "domain %s broadcast\n", nisdomain.value.ascii() );
    fclose( fp );
    return true;
}
#endif

bool Netinfo::saveProxy( const char *root )
{
#ifndef GBLI_IPv6
  if( mode == NoConfig ) return false;
#else
	// lets do it always, doesn't change anything
#endif

	if( !proxy.value.length() ) return false;

  // TODO: save appropriate proxy settings into some other progs ?!
  FILE *fp;

	int ind = 0;
	QString strProxy( proxy.value );
	while( (ind = strProxy.find( "," )) != -1 )
		strProxy = strProxy.replace( ind, 1, "." );
	while( (ind = strProxy.find( "\\" )) != -1 )
		strProxy = strProxy.replace( ind, 1, "/" );
	char delim[] = " \"\'()[]{}<>";
	for( int i = 0; delim[i]; i++ )
		while( (ind = strProxy.find( delim[i] )) != -1 )
			strProxy = strProxy.remove( ind, 1 );
	if( !proxy.value.length() ) return false;
	if( (ind = strProxy.find( "http://" )) == -1 ){
		strProxy = "http://"+ strProxy;
	}else if( ind > 0 )
		return false;
	if( proxyport.value.length() )
		strProxy += ":"+ proxyport.value;

	ConfigFile config( root, "/etc/profile" );
  fp = config.open( "a" );
  if( fp ){
	  fprintf(fp,"\n# Proxy settings\n\n");
  	fprintf(fp,"export http_proxy=\"%s\"\n", strProxy.ascii() );
	  fprintf(fp,"export https_proxy=\"%s\"\n", strProxy.ascii() );
  	fprintf(fp,"export ftp_proxy=\"%s\"\n", strProxy.ascii() );
	  fclose(fp);
	}

	config = ConfigFile( root, "/usr/share/config/kfmrc" );
  fp = config.open( "a" );
  if( fp ){
    fprintf(fp,"\n[Browser Settings/Proxy]\n");
    fprintf(fp,"HTTP-Proxy=%s\n", strProxy.ascii() );
		if( noproxyfor.value.length() )
	    fprintf(fp,"NoProxyFor=%s\n", noproxyfor.value.ascii() );
    fprintf(fp,"UseProxy=Yes\n" );
    fprintf(fp,"FTP-Proxy=%s\n", strProxy.ascii() );
    fclose(fp);
	}

	config = ConfigFile( root, "/etc/skel/.kde/share/config/kfmrc" );
  fp = config.open( "a" );
  if( fp ){
    fprintf(fp,"\n[Browser Settings/Proxy]\n");
    fprintf(fp,"HTTP-Proxy=%s\n", strProxy.ascii() );
		if( noproxyfor.value.length() )
	    fprintf(fp,"NoProxyFor=%s\n", noproxyfor.value.ascii() );
    fprintf(fp,"UseProxy=Yes\n");
    fprintf(fp,"FTP-Proxy=%s\n", strProxy.ascii() );

    fclose(fp);
	}
	return true;
}

bool Netinfo::saveSysNetwork(const char *root)
{
#ifndef GBLI_IPv6
    SysConfigFile config( root, "/etc/sysconfig/network" );
/* Old GBLI code
    config.load();
    config.set("NETWORKING",	"yes");
    config.set("IF_LIST",	"dynamic"); // as per raymund's request
    config.set("HOSTNAME",	hostname.value);
    config.append( "# IF_LIST set to dynamic by GBLI\n" );
*/
    // Red Hat seems like new. :)
    config.load();
 		bool bEnableNetworking = mode || ISDNmode || ADSLmode;
		
		config.set("NETWORKING", (!bEnableNetworking?"no":"yes"), false );
    //config.set("NETWORKING", (mode == NoConfig?"no":"yes"), false );
//    config.set("FORWARD_IPV4",	"false", false );

		QString cmd;
		cmd = "touch ";
		cmd += root;
		cmd += "/etc/sysctl.conf";
		system( cmd.ascii() );
		ConfigFile sysctlcfg( root, "/etc/sysctl.conf" );
		// Enable packet forwarding if server
		#ifdef GBLI_SERVER
		sysctlcfg.replaceLine(QString("net.ipv4.ip_forward"), QString("net.ipv4.ip_forward = 1"), true );
		#else
		sysctlcfg.replaceLine(QString("net.ipv4.ip_forward"), QString("net.ipv4.ip_forward = 0"), true );
		#endif

		QString host = hostname.value;
		if(host.length()==0)
		{
			host="localhost.localdomain";
		}
    int point = host.find( '.' );
//		printf("Hostname: %s, point: %i.\n",host.ascii(),point);

		if( point >= 0 ){
	    config.set("HOSTNAME", host.left(point), false );
	    config.set("DOMAINNAME", host.right( host.length() - point - 1 ), false );
		}else{
	    config.set("HOSTNAME", host, false );
//	    config.set("DOMAINNAME", nisdomain.value, false );
		}

//		char dev[5];
//
//		if(ADSLmode!=0){
//		  strcpy(dev,adslcard.value);
//		  dev[strlen(dev)-1] = dev[strlen(dev)-1]==48?49:48; // NOT ADSL CARD
//    }
//		else strcpy(dev,"eth0");


//    if( mode != NoConfig ){
//      if ( !strstr( gateway.value, "0.0.0.0" )  ) {
//  			config.set("GATEWAY", gateway.value, false );
//        config.set("GATEWAYDEV", dev, false );
//      }
//      if ( !strstr( nameserver.value, "0.0.0.0") ) {
//  			config.set("NS1", nameserver.value, false );
//      }
//      if( !strstr( nameserver2.value, "0.0.0.0" ) ) {
//  			config.set("NS2", nameserver2.value, false );
//      }
//		}

    return config.save();
#else

  	int iAdapter = 0;
		bool ipv4 = false, ipv6 = false;

  	while( iAdapter < ADAPTERS_MAX ){
  		QString strAdapter = "eth" + QString::number( iAdapter );
  		Setup *pmode = mode.find( strAdapter );
			AddressEdit::Protocol *ppr = protocol.find( strAdapter );
  		iAdapter++;
      if( !pmode || !ppr ) continue;
			if( *pmode != NoConfig ){
  			if( *ppr == AddressEdit::IPv46 || *ppr == AddressEdit::IPv4 ){
//					Item *tmp = address.find( strAdapter );
//					if( tmp && tmp->value != "0.0.0.0" )
	  				ipv4 = true;
				}
  			if( *ppr == AddressEdit::IPv46 || *ppr == AddressEdit::IPv6 ){
//					Item *tmp = address6.find( strAdapter );
//					if( tmp && tmp->value != "0:0:0:0:0:0:0:0/0" )
	  				ipv6 = true;
				}
				if( ipv4 && ipv6 ) break; // everything possible already found
			}
		}

		// Check if tunnel is set
		if ( !SITdevices.isEmpty() )
		{
			ipv6 = true;
		}

    SysConfigFile config( root, "/etc/sysconfig/network" );
    config.load();

		
	/* Networking should be enabled fot ADSL and ISDN
		 ISDN and ADSL daemons will not start */

	/*it looks that our current ifup-isdn script supports only ipv4*/
    bool bEnableNetworking;
		if (ipv4 /*|| ipv6*/)
				 bEnableNetworking = false;
		else
				 bEnableNetworking = ISDNmode || ADSLmode;
		
    config.set("NETWORKING", (ipv4 || bEnableNetworking?"yes":"no"), false );
    config.set("NETWORKING_IPV6", (ipv6?"yes":"no"), false );
//    config.set("FORWARD_IPV4",	"false", false );

		QString host = hostname.value;
		if( host.length()== 0 ){
			host="localhost.localdomain";
		}
    int point = host.find( '.' );
//		printf("Hostname: %s, point: %i.\n",host.ascii(),point);

		if( point >= 0 ){
	    config.set("HOSTNAME", host.left(point), false );
	    config.set("DOMAINNAME", host.right( host.length() - point - 1 ), false );
		}else{
	    config.set("HOSTNAME", host, false );
//	    config.set("DOMAINNAME", nisdomain.value, false );
		}

		QString cmd;
		cmd = "touch ";
		cmd += root;
		cmd += "/etc/sysctl.conf";
		system( cmd.ascii() );
		ConfigFile sysctlcfg( root, "/etc/sysctl.conf" );
		// Enable/Disable packet forwarding
#ifndef GBLI_SERVER
		sysctlcfg.replaceLine(QString("net.ipv4.ip_forward"), QString("net.ipv4.ip_forward = 0"), true );
		sysctlcfg.replaceLine(QString("net.ipv6.conf.default.forwarding"), QString("net.ipv6.conf.default.forwarding = 0"), true );
#else
		sysctlcfg.replaceLine(QString("net.ipv4.ip_forward"), ipv4?QString("net.ipv4.ip_forward = 1"):QString("net.ipv4.ip_forward = 0"), true );
		sysctlcfg.replaceLine(QString("net.ipv6.conf.default.forwarding"), ipv6?QString("net.ipv6.conf.default.forwarding = 1"):QString("net.ipv6.conf.default.forwarding = 0"), true );
#endif


//		char dev[5];
//
//		if(ADSLmode!=0){
//		  strcpy(dev,adslcard.value);
//		  dev[strlen(dev)-1] = dev[strlen(dev)-1]==48?49:48; // NOT ADSL CARD
//    }
//		else strcpy(dev,"eth0");
//
//    if( true /**pmode != NoConfig */){
//      if ( !strstr( gateway.value, "0.0.0.0" ) && !strstr( gateway.value, "0:0:0:0:0:0:0:0/0" ) ) {
//  			config.set("GATEWAY", Netinfo::removePrefix( gateway.value ), true );
//        config.set("GATEWAYDEV", dev, false );
//      }
//      if ( !strstr( nameserver.value, "0.0.0.0") && !strstr( nameserver.value, "0:0:0:0:0:0:0:0/0" ) ) {
//  			config.set("NS1", Netinfo::removePrefix( nameserver.value ), true );
//      }
//      if( !strstr( nameserver2.value, "0.0.0.0" ) && !strstr( nameserver2.value, "0:0:0:0:0:0:0:0/0" ) ) {
//  			config.set("NS2", Netinfo::removePrefix( nameserver2.value ), true );
//      }
//		}

    return config.save();
#endif
}

#ifdef NO_USED
bool Netinfo::saveTmpNetwork(void)
{
    SysConfigFile config( "", "/tmp/network" );

    if (mode != NoConfig)
		config.set("CONF_eth0_STAT",	"up");

    config.set("CONF_eth0_IP",		address.value );
    config.set("CONF_eth0_MASK",	netmask.value );
    config.set("CONF_eth0_DYNAMIC",	(mode == UseDHCPConfig)? "dhcp" : "");
    config.set("CONF_ROUTER1_IP",	gateway.value);
//    config.set("CONF_FQ_HOSTNAME",	hostname.value);
    config.set("CONF_DNS_SERVER1_IP",	nameserver.value);
    config.set("CONF_DNS_SERVER2_IP",	nameserver2.value);
    //config.set("CONF_DNS_SERVER3_IP",	nameserver3.value);
    config.set("CONF_NIS_DOMAIN",	nisdomain.value);
    config.set("CONF_NIS_SERVER1_IP",	"");
    config.set("CONF_NIS_SERVER2_IP",	"");
    config.set("CONF_NIS_SERVER3_IP",	"");
    config.set("CONF_PROXY_NAME",	proxy.value);
    config.set("CONF_PROXY_PORT",	proxyport.value);
    config.set("CONF_NO_PROXY",	noproxyfor.value );

    return config.save();
}
#endif

bool Netinfo::saveXDMCP(const char * root ){
#ifdef GBLI_IPv6

D("saveXDMCP\n" );

  QFile shadowFile( NEWROOT + "/etc/X11/gdm/gdm.conf" );
  QFile newShadowFile( NEWROOT + "/etc/X11/gdm/gdm.conf.new" );
	if( shadowFile.open( IO_ReadOnly ) && newShadowFile.open( IO_WriteOnly ) ){
  	QTextStream shadow( &shadowFile );
  	QTextStream newShadow( &newShadowFile );
		QString temp;
		bool in_sect = false;
		while( !shadow.atEnd() ){
			temp = shadow.readLine();
			if( temp.left( 7 ) == "[xdmcp]" ){
				newShadow << temp << endl;
				in_sect = true;
			}else if( in_sect && temp.left( 6 ) == "Enable" ){
				newShadow << "Enable=1" << endl;
				in_sect = false;
			}else
				newShadow << temp << endl;
		}
		shadowFile.close();
		newShadowFile.close();
		::sync();

		QString strPath = NEWROOT +"/etc/X11/gdm/gdm.conf";
    ::unlink( strPath.ascii() );
		::rename( (strPath +".new").ascii(), strPath.ascii() );
		::sync();
	}
  ::sync();

	if( system( ("grep \":1 local\" "+ NEWROOT +"/etc/X11/xdm/Xservers >/dev/null 2>&1").ascii() ) ){
    ConfigFile config( root, "/etc/X11/xdm/Xservers" );
    FILE *fp = config.open("a");
    if( fp ){
  		fprintf(fp, "#:1 local /usr/X11R6/bin/X\n" );
  		fprintf(fp, "#:2 local /usr/X11R6/bin/X\n" );
  		fprintf(fp, "#:3 local /usr/X11R6/bin/X\n" );
  		fprintf(fp, "#:4 local /usr/X11R6/bin/X\n" );
  		fprintf(fp, "#:5 local /usr/X11R6/bin/X\n" );		

  		fclose( fp );
  	}
	}

	return true;
#else
	return true;
#endif
}

bool Netinfo::saveTunnels(const char * root ){
	QString	tmp;

	QString path = "/etc/sysconfig/network-scripts/ifcfg-";

	system(QString("rm -f ") + root + path + "sit*");

	if ( QFile(QString(root) + "/etc/sysconfig/static-routes-ipv6").exists() )
	{
		// Remove old configuration
		system(QString("sed -e /sit/d ") + root + "/etc/sysconfig/static-routes-ipv6 > " + root + "/etc/sysconfig/static-routes-ipv6.new");
		system(QString("mv -f ") + root + "/etc/sysconfig/static-routes-ipv6.new " + root + "/etc/sysconfig/static-routes-ipv6");
	}

	if ( SITdevices.isEmpty() )
		return TRUE;

	for ( uint i = 0; i < SITdevices.count(); i++ )
	{
		SitInfo*	si = SITdevices.at( i );
		QString dev = QString("sit") + QString::number( i + 1 );
		QString	file = path + dev;

		system(QString("touch ") + root + file);
		SysConfigFile	config( root, file.ascii() );
		config.load();
		
		if ( !si->getName().isEmpty() )
			config.set( "NAME", si->getName(), TRUE );
		
		config.set( "DEVICE", dev, FALSE );
		config.set( "BOOTPROTO", "none", FALSE );
		config.set( "ONBOOT", ( si->getOnBoot() ) ? "yes" : "no", FALSE );
		config.set( "IPV6INIT", "yes", FALSE );
		config.set( "IPV6ADDR", si->getLocalIP(), FALSE );
		config.set( "IPV6TUNNELIPV4", si->getDestination(), FALSE );

		config.save();

		QStringList	list = si->getRouteList();
    for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
			system( QString("echo \"") + dev + "\t" + (*it) + "\" >> " + root + "/etc/sysconfig/static-routes-ipv6");
	}

	return TRUE;	
}

bool Netinfo::saveHosts(const char *root){
#ifndef GBLI_IPv6
  FILE *fp;

	ConfigFile config( root, "/etc/hosts" );
  fp = config.open( "w" /* "a" */ );
  if( fp ){
		int i = 0;
		char name[128];
		memset( name, 0, sizeof( name ) );
		strcpy( name, hostname.value.ascii() );
		while( name[i] && name[i] != '.' ){
			i++;
		}
		name[i] = 0;

		fprintf( fp, "127.0.0.1\tlocalhost\tlocalhost.localdomain\n" );
#ifndef GBLI_DEVEL
		system("hostname localhost");
#endif

		if( mode == NoConfig && strcmp( hostname.value.ascii(), "localhost.localdomain" ) )
		{
			fprintf( fp, "127.0.0.1\t%s\t%s\n", name, hostname.value.ascii() );
#ifndef GBLI_DEVEL
			system("hostname %s", name.latin1());
#endif
		}
//printf( "Hostname: %s\n", hostname.value.ascii() );
		if( mode != NoConfig && /*strcmp( address.value, "192.168.1.2" ) &&*/
		  strcmp( hostname.value.ascii(), "localhost.localdomain" /*"noname.nodomain.nowhere"*/ ) ){
			fprintf( fp, "%s\t%s\t%s\n", address.value.ascii(), name, hostname.value.ascii() );
			QString cmd = QString( "hostname " ) + QString( name );
#ifndef GBLI_DEVEL
			system( cmd.ascii() );
#endif
		}
   	fclose( fp );
		return true;
  }
#else
	int i = 0;
	char name[128];
	memset( name, 0, sizeof( name ) );
	strcpy( name, hostname.value.ascii() );
	while( name[i] && name[i] != '.' )
		i++;
	name[i] = 0;

	bool new_hostname = !!strcmp( hostname.value.ascii(), "localhost.localdomain" );
	if( new_hostname ){
		QString cmd = QString( "hostname " ) + QString( name );
#ifndef GBLI_DEVEL
		system( cmd.ascii() );
#endif
	}
#ifndef GBLI_DEVEL
	else
		system("hostname localhost");
#endif

//	bool ipv4_noconfig = new_hostname;
	bool ipv4 = false;
//	bool ipv6_noconfig = new_hostname;
	bool ipv6 = false;

	system("sed -e \"/^.*localhost.localdomain.*/d\" " + QString(root) + "/etc/hosts |\
					sed -e \"/^$/d\" | \
					sed -e \"/^.*" + prvHostName + ".*/d\" >" + QString(root) + "/etc/hosts.old" );
	
	ConfigFile config( root, "/etc/hosts" );
  FILE *fp;
//#ifdef GBLI_STANDALONE
//  fp = config.open( "a" );
//#else
  fp = config.open( "w" /* "a" */ );
//#endif
  if( fp ){
  	int iAdapter = 0;
		
  	while( iAdapter < ADAPTERS_MAX ){
  		QString strAdapter = "eth" + QString::number( iAdapter );
  		Setup *pmode = mode.find( strAdapter );
			AddressEdit::Protocol *ppr = protocol.find( strAdapter );
  		iAdapter++;
      if( !pmode || !ppr ) continue;

			if( *ppr == AddressEdit::IPv4 || *ppr == AddressEdit::IPv46 )
				ipv4 = true;
			if( *ppr == AddressEdit::IPv6 || *ppr == AddressEdit::IPv46 )
				ipv6 = true;
		}
		
		if( !ipv4 && !ipv6 )
			ipv4 = true; // lets enable old local loop address

		if( ipv4 )
			fprintf( fp, "127.0.0.1\tlocalhost.localdomain\tlocalhost\n" );
// 		if( ipv4 && ipv4_noconfig )
// 			fprintf( fp, "127.0.0.1\t%s\t%s\n", hostname.value.ascii(), name );
		if( ipv6 )
			fprintf( fp, "::1\t\tlocalhost.localdomain\tlocalhost\n" );
// 		if( ipv6 && ipv6_noconfig )
// 			fprintf( fp, "::1\t\t%s\t%s\n", hostname.value.ascii(), name );
 			
		fprintf( fp, "\n" );

		iAdapter = 0;  	
  	while( iAdapter < ADAPTERS_MAX ){
  		QString strAdapter = "eth" + QString::number( iAdapter );
  		Setup *pmode = mode.find( strAdapter );
			AddressEdit::Protocol *ppr = protocol.find( strAdapter );
  		iAdapter++;
      if( !pmode || !ppr ) continue;

//			if( *ppr == AddressEdit::IPv4 || *ppr == AddressEdit::IPv46 )
//				ipv4 = true;
//			if( *ppr == AddressEdit::IPv6 || *ppr == AddressEdit::IPv46 )
//				ipv6 = true;

  		if( *pmode != NoConfig && new_hostname )
  		{
				if( *ppr == AddressEdit::IPv4 || *ppr == AddressEdit::IPv46 ){
      		Item *paddress = address.find( strAdapter );
      		if( paddress && paddress->value != "0.0.0.0" ){
//		  			ipv4_noconfig = false;
	    			fprintf( fp, "%s\t%s\t%s\n", paddress->value.ascii(), hostname.value.ascii(), name );
					}
				}
				if( *ppr == AddressEdit::IPv6 || *ppr == AddressEdit::IPv46 ){
      		Item *paddress = address6.find( strAdapter );
      		if( paddress && paddress->value != "0:0:0:0:0:0:0:0/0" ){
//		  			ipv6_noconfig = false;
      			fprintf( fp, "%s\t%s\t%s\n", Netinfo::removePrefix( paddress->value ).ascii(), hostname.value.ascii(), name );
					}
				}
  		}
		}
		
		fprintf( fp, "\n" );

   	fclose( fp );
		
		system("cat " + QString(root) + "/etc/hosts.old >>" + QString(root) + "/etc/hosts");
		system("rm -f " + QString(root) + "/etc/hosts.old &>/dev/null");
		return true;
	}
#endif	
	return false;
}

bool Netinfo::readEther(const char *root)
{
		char value[256];

#ifndef GBLI_IPv6
		char fname[100];
		char dev[5];
		bool withQuotation;

		strcpy(dev,"eth0");

		strcpy(fname, "/etc/sysconfig/network-scripts/ifcfg-" );
		strcat(fname,dev);

		SysConfigFile config( root, fname );
		config.load();

		if (!config.get("BOOTPROTO", value, sizeof(value))) return false;
		if (!strcmp(value, "none"))
		{
			setMode(dev, UseStaticConfig);
      if (config.get("IPADDR", value, sizeof(value)))
				setAddress(dev, value, Probed);
      if (config.get("NETMASK", value, sizeof(value)))
				setNetmask(dev, value, Probed);
      if (config.get("GATEWAY", value, sizeof(value)))
				setGateway(value, Probed);
      if (config.get("DNS1", value, sizeof(value)))
				setNameserver(value, Probed);
      if (config.get("DNS2", value, sizeof(value)))
				setNameserver2(value, Probed);
		}
		else
			if (!strcmp(value, "dhcp")) setMode(dev, UseDHCPConfig);

    return true;
#else
		int iAdapter = 0;

		bool ipv4 = false;
		bool ipv6 = false;

		while( iAdapter < ADAPTERS_MAX )
		{
			QString strAdapter = "eth" + QString::number( iAdapter );
			ipv4 = false;
			ipv6 = false;

			iAdapter++;

			if ( getMode( strAdapter ) == -1 ) continue;

  		if (!QFile(QString(root) + "/etc/sysconfig/network-scripts/ifcfg-" + strAdapter).exists())
			{
				setMode(strAdapter, NoConfig);
				continue;
			}

//			Setup *pmode = mode.find( strAdapter );
			// ADSL is preferrable
//  		if( ADSLmode != Netinfo::NoConfig && adslcard.value == strAdapter ) continue;
//  		strcpy( dev, strAdapter.ascii() );

  		char fname[100];

  		strcpy( fname, ("/etc/sysconfig/network-scripts/ifcfg-" + strAdapter).ascii() );

#ifdef GBLI_DEVEL
			printf("Opening config file %s ...\n", fname);
#endif

  		SysConfigFile config( root, fname );
			config.load();

			if (config.get("IPV6INIT", value, sizeof(value)))
			{
				if (!strcmp(value, "yes"))
					ipv6 = true;
			}

			if (config.get("BOOTPROTO", value, sizeof(value)))
			{
				if (!strcmp(value, "dhcp"))
				{
					setMode(strAdapter, UseDHCPConfig);
				}
				else
					if (!strcmp(value, "none"))
					{
						setMode(strAdapter, UseStaticConfig);

						/// IPv4 configuration						
						if (config.get("IPADDR", value, sizeof(value)))
						{
							ipv4 = true;
							setAddress(strAdapter, value, Probed);
						}
						if (config.get("NETMASK", value, sizeof(value)))
						{
							ipv4 = true;
							setNetmask(strAdapter, value, Probed);
						}

						/// IPv6 configuration

						if (config.get("IPV6ADDR", value, sizeof(value)))
						{
							ipv6 = true;
							QString sValue = insertZeroSeq(QString(value));
							setAddress6(strAdapter, sValue.ascii(), Probed);
						}

						AddressEdit::Protocol	pr;

			      if (config.get("GATEWAY", value, sizeof(value)))
						{
							QString sValue(value);
							pr = AddressEdit::IPv4;
							if (strstr(value, ":"))
							{
								pr = AddressEdit::IPv6;
								sValue = insertZeroSeq(sValue);	
							}
              setRouterProtocol(pr);
							setGateway(sValue.ascii(), Probed);
							setGatewayDevice(strAdapter.ascii(), Probed);
						}
			      if (config.get("DNS1", value, sizeof(value)))
						{
							QString sValue(value);
							pr = AddressEdit::IPv4;
							if (strstr(value, ":"))
							{
								pr = AddressEdit::IPv6;
								sValue = insertZeroSeq(sValue);	
							}

							setNs1Protocol(pr);
							setNameserver(sValue.ascii(), Probed);
						}
			      if (config.get("DNS2", value, sizeof(value)))
						{
							QString sValue(value);
							pr = AddressEdit::IPv4;
							if (strstr(value, ":"))
							{
								pr = AddressEdit::IPv6;
								sValue = insertZeroSeq(sValue);	
							}

							setNs2Protocol(pr);
							setNameserver2(sValue.ascii(), Probed);
						}

					}
			}
//			else
//				printf("Error reading BOOTPROTO\n");

			if (ipv4 && ipv6) setProtocol( strAdapter, AddressEdit::IPv46 );
			else
				if (ipv4) setProtocol( strAdapter, AddressEdit::IPv4 );
				else
					if (ipv6) setProtocol( strAdapter, AddressEdit::IPv6 );
					else
						setMode(strAdapter, NoConfig);

		}
    return true;
#endif
}

bool Netinfo::readADSL(const char *root)
{
	char value[128];
	QString qsFile;
	
	setADSLMode(NoConfig);

  // Updated code, detecting proper pppX file
	for (int iInd=0;iInd < MAXPPPX; iInd++)
	{ //now MAXPPX =32
			qsFile.sprintf("/etc/sysconfig/network-scripts/ifcfg-ppp%d",iInd);
		  if (QFile(QString(root) + qsFile).exists()){
			//check: is it our config file or not
							SysConfigFile config( root, qsFile.latin1());
							config.load();
							config.get("TYPE", value, sizeof(value));
							if (strcmp(value, "xDSL")!=0) continue;   // Not ADSL
							else{
									adslpppfile = qsFile;
             			D("Reading ADSL parameters\n");
								  setADSLMode(UseDHCPConfig);  // if fact it's PPPoe

									if (config.get("ETH", value, sizeof(value))) //it looks that code do nothing
									setADSLCard(value, Probed);

									if (config.get("USER", value, sizeof(value) ))
									{
										setADSLUser(value, Probed);
										if (QFile(QString(root) + "/etc/ppp/pap-secrets").exists())
										{
											ConfigFile config2( root, "/etc/ppp/pap-secrets" );
											QString 	 sLogin(getADSLUser()),sPasswd;
											if (config2.getChapPapSecretsPasswd(sLogin,sPasswd))
											{
														oldadsluser = sLogin;
														setADSLPass(sPasswd.latin1(), Probed);				
														#ifdef GBLI_DEVEL
								 						 printf("ADSL value:%s\n",sPasswd.latin1());
								 						 printf("ADSL olduser:%s\n",oldadsluser.latin1());
														#endif
											}
										}
									} //if USER
										return true;
									} // end of else xDSL
			}
	}
	return false;
}

/*
bool Netinfo::readISDN(const char *root)
rewritten by YP, because previous version was not working at all
*/
bool Netinfo::readISDN(const char *root)
{
	char value[128];

	D("Reading ISDN parameters\n");
  /*	Set zero init settings,will be overwritten later */
	setISDNType("unknown", Probed);
	setISDNMode(NoConfig);

	if (!QFile(QString(root) + "/etc/sysconfig/isdncard").exists())
	{
		#ifdef GBLI_DEVEL
			perror("/etc/sysconfig/isdncard not found!");
		#endif
		return false;
	}

	SysConfigFile config( root,  "/etc/sysconfig/isdncard");
	config.load();
  //check this line
	//I would like to put 1 here,DHCP setttings
	setISDNMode((Netinfo::Setup) 2);

 	if (config.get("RESOURCES", value, sizeof(value) ))
	{	  char newValue[128];
			//small parsing,if we have know syntax
			if (sscanf(value,"type=%122s",newValue)==1)			
			setISDNType(newValue, Probed);	
	}

	/*read config from ippp0 */
	SysConfigFile config1( root, "/etc/sysconfig/network-scripts/ifcfg-ippp0");
	config1.load();
 	if (config1.get("MSN", value, sizeof(value) ))
	{
		setISDNInPhone(value, Probed);
	}
	if (config1.get("PHONE_OUT", value, sizeof(value) ))
	{
		setISDNOutPhone(value, Probed);
	}
	if (config1.get("ONBOOT", value, sizeof(value) ))
	{	
		QString qsValue(value);
		if (qsValue.find("no")!=-1)
					ISDNboot=false;
		else	ISDNboot=true;
	}

	if (config1.get("USER", value, sizeof(value) ))
	{
		setISDNUser(value, Probed);
		if (QFile(QString(root) + "/etc/ppp/pap-secrets").exists())
			{
				ConfigFile config2( root, "/etc/ppp/pap-secrets" );
				QString 	 sLogin(getISDNUser()),sPasswd;
				if (config2.getChapPapSecretsPasswd(sLogin,sPasswd))
				{
					oldisdnuser = sLogin;
					setISDNPass(sPasswd.latin1(), Probed);				
					#ifdef GBLI_DEVEL
 						 printf("ISDN value:%s\n",sPasswd.latin1());
 						 printf("ISDN olduser:%s\n",oldisdnuser.latin1());
					#endif
				}
			}
	} //if USER

	return true;
}


/*
bool Netinfo::readISDN_OLD(const char *root)
{
	char value[128];

	D("Reading ISDN parameters\n");

	if (!QFile(QString(root) + "/etc/sysconfig/isdn.conf").exists())
	{
		setISDNType("unknown", Probed);
		setISDNMode(NoConfig);
		#ifdef GBLI_DEVEL
					perror((QString(root) + "/etc/sysconfig/isdn.conf").latin1());
					perror("isdn.conf not present!");
		#endif
		return true;
	}

	SysConfigFile config( root,  "/etc/sysconfig/isdn.conf");
	config.load();

	setISDNMode((Netinfo::Setup) 2);
 	if (config.get("LOCAL_NUMBER", value, sizeof(value) ))
	{
		setISDNInPhone(value, Probed);
	}
	if (config.get("REMOTE_NUMBER", value, sizeof(value) ))
	{
		setISDNOutPhone(value, Probed);
	}
 	if (config.get("TYPE", value, sizeof(value) ))
	{
		setISDNType(value, Probed);
	}
	else
		setISDNType("unknown", Probed);

	if (QFile(QString(root) + "/etc/ppp/options.ipppd").exists())
	{
		SysConfigFile	config2(root, "/etc/ppp/options.ipppd");
		
		config2.load();
		if (config2.get("user", value, sizeof(value), ' '))
		{
			setISDNUser(value, Probed);

			if (QFile(QString(root) + "/etc/ppp/pap-secrets").exists())
			{
				#ifdef GBLI_DEVEL
					perror("Welcome to pap-secrets!");
				#endif
				SysConfigFile config3( root, "/etc/ppp/pap-secrets" );
				config3.load();

				if (config3.get(getISDNUser(), value, sizeof(value), '*'))
					setISDNPass(&value[1], Probed);
			}
		}

	}
#ifdef GBLI_DEVEL
	perror("It is time to go back from readISDN!");
#endif
	return true;
}
*/

bool Netinfo::readSysNetwork(const char *root)
{
	char value[128];
	
	if (!QFile(QString(root) + "/etc/sysconfig/network").exists()) return false;
	SysConfigFile config(root, "/etc/sysconfig/network");
	config.load();

#ifndef GBLI_IPv6
	config.get("NETWORKING", value, sizeof(value));
	if (!strcmp(value, "no"))
	{
		setMode(NoConfig);
		return true;
	}

	if (config.get("HOSTNAME", value, sizeof(value)))
	{
/*		setHostname(value, Probed);
		if (!strstr(value, "."))
			if (config.get("DOMAINNAME", value, sizeof(value)))
				setNISDomain(value, Probed);*/
		QString tmp = value;
		if(config.get("DOMAINNAME", value, sizeof(value))
		{
			tmp += ".";
			tmp += value;
		}
		setHostname(tmp.ascii(), Probed);
		// a gift by Maddy ;)
		// Thanks from Serg_ :)
	}

#else

	if (config.get("HOSTNAME", value, sizeof(value)))
	{
		QString tmp = value;
		if (config.get("DOMAINNAME", value, sizeof(value)))
		{
			tmp += ".";
			tmp += value;
		}
		setHostname(tmp.ascii(), Probed);
		prvHostName = tmp;		
	}

	AddressEdit::Protocol	pr;

	if (config.get("GATEWAY", value, sizeof(value)))
	{
		QString sValue(value);
		pr = AddressEdit::IPv4;
		if (strstr(value, ":"))
		{
			pr = AddressEdit::IPv6;
			sValue = insertZeroSeq(sValue);	
		}
    setRouterProtocol(pr);
		setGateway(sValue.ascii(), Probed);
	}

	if (config.get("GATEWAYDEV", value, sizeof(value)))
	{
		QString sValue(value);
		setGatewayDevice(sValue.ascii(), Probed);
	}

  if (config.get("NS1", value, sizeof(value)))
	{
		QString sValue(value);
		pr = AddressEdit::IPv4;
		if (strstr(value, ":"))
		{
			pr = AddressEdit::IPv6;
			sValue = insertZeroSeq(sValue);	
		}
  	setNs1Protocol(pr);
		setNameserver(sValue.ascii(), Probed);
	}
  if (config.get("NS2", value, sizeof(value)))
	{
		QString sValue(value);
		pr = AddressEdit::IPv4;
		if (strstr(value, ":"))
		{
			pr = AddressEdit::IPv6;
			sValue = insertZeroSeq(sValue);	
		}
		setNs2Protocol(pr);
		setNameserver2(sValue.ascii(), Probed);
	}

#endif		

	return true;

}

bool Netinfo::readTunnels(const char *root)
{
	char value[128];
	int i = 0;

	while (QFile(QString(root) + "/etc/sysconfig/network-scripts/ifcfg-sit" + QString::number(i + 1)).exists())
	{
#ifdef GBLI_DEVEL
		printf(" Found sit%d config\n", i+1);
#endif
		SitInfo	*si = new SitInfo();
		addSitInfo( si );

		SysConfigFile	config(root, "/etc/sysconfig/network-scripts/ifcfg-sit" + QString::number(i + 1));
		config.load();

		if (config.get("ONBOOT", value, sizeof(value)))
		{
			if (QString(value).lower() == "yes") si->setOnBoot(TRUE);
			else si->setOnBoot(FALSE);
		}
		if (config.get("NAME", value, sizeof(value)))
		{
			si->setName( QString(value) );
		}
		if (config.get("IPV6ADDR", value, sizeof(value)))
		{
			si->setLocalIP( QString(value) );
		}
		if (config.get("IPV6TUNNELIPV4", value, sizeof(value)))
		{
			si->setDestination( QString(value) );
		}
		
		QFile	routecfg(QString(root) + "/etc/sysconfig/static-routes-ipv6");
		if (routecfg.open( IO_ReadOnly ))
		{
			QTextStream	stream( &routecfg );
			QString line;
			QStringList	list;
			
			list.clear();

			while ( !stream.atEnd() )
			{
				line = stream.readLine();
				if ( line.left(3) == "sit" )
					if ( line.mid(3, line.simplifyWhiteSpace().find(" ") - 3).toInt() == i + 1 )
					{
//						printf("   %s\n", line.right( line.length() - line.simplifyWhiteSpace().find(" ") ).ascii() );
						list.append( insertZeroSeq( line.right( line.length() - line.simplifyWhiteSpace().find(" ") ) ) );
					}
			}
			routecfg.close();
			si->setRouteList( list );
    }
		
		i++;
	}
	return true; // dummy value ;(
}

SitInfo* Netinfo::getSitInfo( uint i )
{
	return SITdevices.at( i );
}

uint Netinfo::addSitInfo( SitInfo* newsit )
{
	SITdevices.append( newsit );

	return ( SITdevices.count() - 1 );
}

void Netinfo::removeSitInfo( uint i )
{
	SITdevices.remove( i );	
}

uint Netinfo::countSitInfo()
{
	return SITdevices.count();
}



NetworkCommon::NetworkCommon( Netinfo *net, QWidget * parent, const char * name )
    : QWidget( parent, name ), info( net )
{

		this->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

		QVBoxLayout *V1 = new QVBoxLayout( this, 5 );
		QHBoxLayout *ML = new QHBoxLayout();

		V1->addLayout( ML, 5 );
		
		ns1IPv6 = new QCheckBox( "IPv6", this );
		ns1IPv6->setAutoMask( true );
		connect( ns1IPv6, SIGNAL( toggled( bool ) ), this, SLOT( setNs1Protocol( bool ) ) );
		ns1Edit = new AddressEdit( AddressEdit::Other, this );
		ns2IPv6 = new QCheckBox( "IPv6", this );
		ns2IPv6->setAutoMask( true );
		connect( ns2IPv6, SIGNAL( toggled( bool ) ), this, SLOT( setNs2Protocol( bool ) ) );
    ns2Edit = new AddressEdit( AddressEdit::Other, this );
		routerIPv6 = new QCheckBox( "IPv6", this );
		routerIPv6->setAutoMask( true );
		connect( routerIPv6, SIGNAL( toggled( bool ) ), this, SLOT( setRouterProtocol( bool ) ) );
    routerEdit = new AddressEdit( AddressEdit::Router, this );
//    nisEdit = new QLineEdit( this );

		QGridLayout *grid = new QGridLayout();
		ML->addLayout( grid, 1 );
//		ML->addStretch( 1 );

		QLabel *l1 = new QLabel(tr("Name server:"),this);
    l1->setAutoMask(true);
		l1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)1, l1->sizePolicy().hasHeightForWidth() ) );
		grid->addWidget(ns1IPv6,0,1);
		grid->addWidget(l1,0,0);
		grid->addMultiCellWidget(ns1Edit,1,1,0,3,AlignLeft);
		QLabel *l2 = new QLabel(tr("Name server2:"),this);
    l2->setAutoMask(true);
		l2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)1, l2->sizePolicy().hasHeightForWidth() ) );
		grid->addWidget(ns2IPv6,2,1);
		grid->addWidget(l2,2,0);
		grid->addMultiCellWidget(ns2Edit,3,3,0,3,AlignLeft);
		QLabel *l3 = new QLabel(tr("Gateway:"),this);
    l3->setAutoMask(true);
		l3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)1, l3->sizePolicy().hasHeightForWidth() ) );
		grid->addWidget(routerIPv6,4,1);
		grid->addWidget(l3,4,0);
		grid->addMultiCellWidget(routerEdit,5,5,0,3,AlignLeft);
		
#ifdef GBLI_IPv6
		devCombo = new QComboBox(0,this);
		for (int iAdapter = 0; iAdapter < ADAPTERS_MAX; iAdapter++)
		{
			QString strAdapter = "eth" + QString::number(iAdapter);
			if (info->getMode( strAdapter ) != -1)
				devCombo->insertItem(strAdapter.ascii());
		}
		if (devCombo->count() == 0)
			devCombo->insertItem("eth0");

		QLabel *l4 = new QLabel( tr("Gateway device:"), this );
    l4->setAutoMask(true);
		grid->addWidget(l4,4,2,AlignRight);
		grid->addWidget(devCombo,4,3,AlignLeft);

		if (devCombo->count() == 1)
		{
			l4->hide();
			devCombo->hide();		
		}
#endif

//		QLabel *l4 = new QLabel(tr("NIS domain:"),this);
//    l4->setAutoMask(true);
//		grid->addWidget(l4,6,0);
//		grid->addMultiCellWidget(nisEdit,6,6,1,4,AlignLeft);

		updateWidgets();
}

NetworkCommon::~NetworkCommon(){
}

void NetworkCommon::setNs1Protocol( bool a_bl ){
	AddressEdit::Protocol pr = a_bl?AddressEdit::IPv6:AddressEdit::IPv4;
	ns1Edit->setProtocol( pr );
#ifdef GBLI_IPv6
	info->setNs1Protocol( pr );
#endif

	QWidget *par = parentWidget();
	if( par )
		par->updateGeometry();
}

void NetworkCommon::setNs2Protocol( bool a_bl ){
	AddressEdit::Protocol pr = a_bl?AddressEdit::IPv6:AddressEdit::IPv4;
	ns2Edit->setProtocol( pr );
#ifdef GBLI_IPv6
	info->setNs2Protocol( pr );
#endif
	QWidget *par = parentWidget();
	if( par )
		par->updateGeometry();
}

void NetworkCommon::setRouterProtocol( bool a_bl ){
	AddressEdit::Protocol pr = a_bl?AddressEdit::IPv6:AddressEdit::IPv4;
	routerEdit->setProtocol( pr );
#ifdef GBLI_IPv6
	info->setRouterProtocol( pr );
#endif
	QWidget *par = parentWidget();
	if( par )
		par->updateGeometry();
}

void NetworkCommon::addInputWidget(QWidget *edit){
    connect( edit, SIGNAL(returnPressed()), this, SLOT(updateInfo()) );
    edit->installEventFilter(this);
}

bool NetworkCommon::eventFilter(QObject *, QEvent *e){
    if (e->type() == QEvent::FocusIn || e->type() == QEvent::FocusOut)
    	updateInfo();
    return false;
}

void NetworkCommon::updateWidgets(){
//	printf( "NetworkCommon::updating widgets\n" );
    if (!ns1Edit)
			return;

//		setNs1Protocol(info->getNs1Protocol() == AddressEdit::IPv6);
		ns1IPv6->setChecked(info->getNs1Protocol() == AddressEdit::IPv6);
//		setNs2Protocol(info->getNs2Protocol() == AddressEdit::IPv6);
		ns2IPv6->setChecked(info->getNs2Protocol() == AddressEdit::IPv6);
//		setRouterProtocol(info->getRouterProtocol() == AddressEdit::IPv6);
		routerIPv6->setChecked(info->getRouterProtocol() == AddressEdit::IPv6);


    if (!ns1Edit->edited())
        ns1Edit->setAddress(info->getNameserver());
    if (!ns2Edit->edited())
        ns2Edit->setAddress(info->getNameserver2());
    if (!routerEdit->edited())
        routerEdit->setAddress(info->getGateway());

		if (info->getGatewayDevice())
			for (int i = 0; i < devCombo->count(); i++)
				if (devCombo->text(i) == info->getGatewayDevice()) devCombo->setCurrentItem(i);
		
//    if (!nisEdit->edited())
//        nisEdit->setText(info->getNISDomain());
}

void NetworkCommon::updateInfo(){
    bool	changed = false;

    if( ns1Edit->edited()
    	&& info->setNameserver(ns1Edit->text(), Netinfo::User))
				changed = true;
    if (ns2Edit->edited() && info->setNameserver2(ns2Edit->text(), Netinfo::User))
			changed = true;
    if (routerEdit->edited() && info->setGateway(routerEdit->text(), Netinfo::User))
			changed = true;
#ifdef GBLI_IPv6
		if (info->setGatewayDevice(devCombo->currentText().ascii(), Netinfo::User))
			changed = true;
#endif
//    if (info->setNISDomain(nisEdit->text(), Netinfo::User))
//			changed = true;

    if( changed )
			info->probe( true );
}

NetworkTunnel::NetworkTunnel( Netinfo *net, QWidget * parent, const char * name )
    : QWidget( parent, name ), info( net )
{

		this->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

		QVBoxLayout	*V1 = new QVBoxLayout( this, 5 );

		sitListView = new QListView( this, "Sit Devices List" );

		QFontMetrics	fm( QFont::defaultFont() );
		
		// 55, 280, 120
    sitListView->addColumn( tr( "Device" ), fm.width( tr( "Device" ) ) + 8 );
    sitListView->addColumn( tr( "Name" ), 455 - fm.width( tr( "Device" ) ) - fm.width( tr( "Tunnel endpoint" ) ) - 16 );
    sitListView->addColumn( tr( "Tunnel endpoint" ), fm.width( tr( "Tunnel endpoint" ) ) + 8 );

    sitListView->setVScrollBarMode( QListView::AlwaysOn );
    sitListView->setRootIsDecorated( FALSE );
 		sitListView->header()->setMovingEnabled( FALSE );
		sitListView->setAllColumnsShowFocus( TRUE );
		sitListView->setSorting( -1 );
		sitListView->setColumnAlignment( 0, AlignHCenter );
		sitListView->setColumnAlignment( 1, AlignLeft );
		sitListView->setColumnAlignment( 2, AlignLeft );

		for (int i = 0; i < sitListView->header()->count(); i++ )
		{
//			sitListView->header()->setResizeEnabled( FALSE, i );
			sitListView->header()->setClickEnabled( FALSE, i );
		}

		V1->addWidget( sitListView );

//		V1->activate();

		QHBoxLayout *H1 = new QHBoxLayout();
		V1->addLayout( H1 );

		QPushButton *AddButton = new QPushButton( tr("Add"), this);
		H1->addWidget(AddButton);

		ConfButton = new QPushButton( tr("Configure"), this);
		H1->addWidget(ConfButton);

		QPushButton *RemoveButton = new QPushButton( tr("Remove"), this);
		H1->addWidget(RemoveButton);

		connect( AddButton, SIGNAL( clicked() ), sitListView, SLOT( setFocus() ) );
		connect( AddButton, SIGNAL( clicked() ), this, SLOT( AddSitDevice() ) );
		connect( RemoveButton, SIGNAL( clicked() ), this, SLOT( RemoveSitDevice() ) );
		connect( ConfButton, SIGNAL( clicked() ), this, SLOT( ConfigureSitDevice() ) );
		connect( sitListView, SIGNAL( currentChanged(QListViewItem*) ), SLOT( CurrentDeviceChanged(QListViewItem*) ) );
		connect( sitListView, SIGNAL( doubleClicked(QListViewItem*) ), SLOT( ConfigureSitDevice() ) );

		tunnelSelected = -1;

		updateWidgets();
}

NetworkTunnel::~NetworkTunnel()
{
}

void NetworkTunnel::updateWidgets()
{
	QListViewItem*	selectedItem = NULL;
	sitListView->clear();

	if ((tunnelSelected == -1) && (info->countSitInfo() > 0))
		tunnelSelected = 0;
	for (int i = info->countSitInfo() - 1; i >= 0; i--)
	{
		SitInfo*	si = info->getSitInfo( i );
		QListViewItem*	Item1 = new QListViewItem( sitListView );
		Item1->setText( 0, QString("sit") + QString::number( i + 1 ) );
		Item1->setText( 1, si->getName() );
		Item1->setText( 2, si->getDestination() );
		if ( i == tunnelSelected )
			selectedItem = Item1;
	}
	if ( selectedItem )
	{
		sitListView->setCurrentItem( selectedItem );
		sitListView->setSelected( selectedItem, TRUE );
		ConfButton->setEnabled( true );
	}
  else
		ConfButton->setEnabled( false );
}

void NetworkTunnel::updateInfo()
{
}

void NetworkTunnel::AddSitDevice()
{
	
	if ( info->countSitInfo() == 25 )
		return;

	SitInfo*	si = new SitInfo();

//	si->setDestination( "0.0.0.0" );
//	si->setName( tr("Unnamed") );	

//	updateWidgets();

	if (si)
	{
		TunnelCfgDialog* CfgD = new TunnelCfgDialog( this, si );

		if ( CfgD->exec() == QDialog::Accepted )
		{
			tunnelSelected = info->addSitInfo( si );

			CfgD->updateInfo();
			
			updateWidgets();
		}
		else
			delete si;
	}
}

void NetworkTunnel::RemoveSitDevice()
{
	if ( tunnelSelected == -1 )
		return;

	info->removeSitInfo( tunnelSelected );

	if ( tunnelSelected >= (int)info->countSitInfo() )
		tunnelSelected--;

	updateWidgets();
}

void NetworkTunnel::ConfigureSitDevice()
{
	if ( tunnelSelected < 0 )
		return;

	SitInfo*	si = info->getSitInfo( tunnelSelected );
	if (si)
	{
		TunnelCfgDialog* CfgD = new TunnelCfgDialog( this, si );

		if ( CfgD->exec() == QDialog::Accepted )
		{
			CfgD->updateInfo();
			
			updateWidgets();
		}
	}
}

void NetworkTunnel::CurrentDeviceChanged( QListViewItem* currentItem )
{
	QListViewItem*	Item = sitListView->firstChild();
	int i = 0;
	tunnelSelected = -1;

	while ( Item )
	{
		if ( Item == currentItem )
		{
//			printf("Tunnel selected: %d\n", i);
			tunnelSelected = i;
			break;
		}
		Item = Item->nextSibling();
		i++;
	}
}

TunnelCfgDialog::TunnelCfgDialog( QWidget * parent, SitInfo * sitdev, const char * name = 0 )
	: QDialog( parent, name, TRUE )
{
	si = sitdev;
	setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );
	setCaption( tr("Tunnel configuration") );
	setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, sizePolicy().hasHeightForWidth() ) );

	QVBoxLayout*	MainLayout = new QVBoxLayout( this, 5 );

	TabForm = new QTabWidget( this );
	TabForm->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

	CfgPage = new QWidget( TabForm );
	CfgPage->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

	QVBoxLayout*	CfgPageLayout = new QVBoxLayout( CfgPage, 10 );

	OnBoot = new QCheckBox( tr("Enable on boot"), CfgPage );
	OnBoot->setAutoMask( true );
	CfgPageLayout->addWidget( OnBoot );
//	CfgPageLayout->addSpacing( 10 );

	QHBoxLayout*	NameDestLayout = new QHBoxLayout( 10 );
	QVBoxLayout*	NameLayout = new QVBoxLayout();

	QLabel*	NameLabel = new QLabel( tr("Tunnel name") + ":", CfgPage );
	NameLabel->setAutoMask( true );
  NameLayout->addWidget( NameLabel );

	NameEdit = new QLineEdit( CfgPage );
	NameEdit->setMaxLength( 25 );	
	NameLayout->addWidget( NameEdit );

	NameDestLayout->addLayout( NameLayout );

	QVBoxLayout*	DestLayout = new QVBoxLayout();
	
	QLabel*	EndPointLabel = new QLabel( tr("Tunnel endpoint") + ":", CfgPage );
	EndPointLabel->setAutoMask( true );
  DestLayout->addWidget( EndPointLabel );

	EndPointIP = new AddressEdit( AddressEdit::Address, CfgPage, AddressEdit::IPv4 );
  DestLayout->addWidget( EndPointIP );

	NameDestLayout->addLayout( DestLayout );

	CfgPageLayout->addLayout( NameDestLayout );
//	CfgPageLayout->addSpacing( 10 );
	
	QLabel*	LocalIPLabel = new QLabel( tr("IP Address:"), CfgPage );
	LocalIPLabel->setAutoMask( true );
	CfgPageLayout->addWidget( LocalIPLabel );

	LocalIP = new AddressEdit( AddressEdit::Address, CfgPage, AddressEdit::IPv6 );
  CfgPageLayout->addWidget( LocalIP );

	CfgPageLayout->addStretch( 1 );

	TabForm->addTab( CfgPage, "Options" );

	RoutesPage = new QWidget( TabForm );
	RoutesPage->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

	QVBoxLayout*	RoutesPageLayout = new QVBoxLayout( RoutesPage, 10 );

	QLabel*	RoutesLabel = new QLabel( tr("New route") + ":", RoutesPage );
	RoutesLabel->setAutoMask( true );
	RoutesPageLayout->addWidget( RoutesLabel );

	RouteIP = new AddressEdit( AddressEdit::Address, RoutesPage, AddressEdit::IPv6 );
  RoutesPageLayout->addWidget( RouteIP );
//	RoutesPageLayout->addStretch( 1 );

	QHBoxLayout*	AddRemoveLayout = new QHBoxLayout();

	QPushButton*	AddRouteButton = new QPushButton( tr("Add route"), RoutesPage );
	AddRemoveLayout->addWidget( AddRouteButton );

	QPushButton*	DelRouteButton = new QPushButton( tr("Remove route"), RoutesPage );
	AddRemoveLayout->addWidget( DelRouteButton );

	RoutesPageLayout->addLayout( AddRemoveLayout );

	RouteList = new QListBox( RoutesPage );
	RouteList->setMinimumSize( QSize( 0, 100 ) );
	RoutesPageLayout->addWidget( RouteList );

//	RoutesPage->addLayout( RoutesPageLaout );
	TabForm->addTab( RoutesPage, "Static routes" );

	MainLayout->addWidget( TabForm );

	QHBoxLayout*	OkCancelLayout = new QHBoxLayout();
	OkCancelLayout->setSpacing( 50 );
	
	QPushButton*	OKButton = new QPushButton( tr("Ok"), this );
	OkCancelLayout->addWidget( OKButton );

	QPushButton*	CANCELButton = new QPushButton( tr("Cancel"), this );
	OkCancelLayout->addWidget( CANCELButton );

	OkCancelLayout->addSpacing( 10 );
	MainLayout->addLayout( OkCancelLayout );

//	QSize size = this->minimumSize();
//	printf("Size: %dx%d\n", size.width(), size.height());
//	this->setFixedSize( this->minimumSize() );

	connect( OKButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( CANCELButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( AddRouteButton, SIGNAL( clicked() ), this, SLOT( AddRouteNetwork() ) );
	connect( DelRouteButton, SIGNAL( clicked() ), this, SLOT( RemoveRouteNetwork() ) );

	updateWidgets();
}

void TunnelCfgDialog::AddRouteNetwork()
{
	RouteList->insertItem( Netinfo::replaceZeroSeq( RouteIP->text() ) );	
}

void TunnelCfgDialog::RemoveRouteNetwork()
{
	if ( RouteList->currentItem() > -1 )
	{
		RouteList->removeItem( RouteList->currentItem() );
		if ( RouteList->count() > 0 )
			RouteList->setCurrentItem( 0 );
	}
}

void TunnelCfgDialog::updateWidgets()
{
	EndPointIP->setAddress( si->getDestination() );
	NameEdit->setText( si->getName() );
	OnBoot->setChecked( si->getOnBoot() );
	RouteList->insertStringList( si->getRouteList() );
	LocalIP->setAddress( Netinfo::insertZeroSeq( si->getLocalIP() ) );
	//	RouteIP->setAddress( "0:0:0:0:0:0:0:0/0" );
}

void TunnelCfgDialog::updateInfo()
{
	si->setDestination( EndPointIP->text() );
	si->setLocalIP( Netinfo::replaceZeroSeq( LocalIP->text() ) );
	si->setName( NameEdit->text() );
	si->setOnBoot( OnBoot->isChecked() );
	QStringList	tmpList;
	for (uint i = 0; i < RouteList->count(); i++)
		tmpList.append( RouteList->text( i ) );
	si->setRouteList( tmpList );
}

/**  */
void Netinfo::setISDNboot(bool isdnBoot){
	ISDNboot=isdnBoot;
}

