/***************************************************************************
                             packagelistitem.cpp
                             -------------------                                         
    begin                : Sun Dec 26 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "packagelistitem.h"
#include <qstring.h>
#include <qwidget.h>

PackageListItem::PackageListItem( QListView * parent, pkgSet *a_packages ):
	QListViewItem( parent, tr("All Packages") ), package( NULL ), packages( a_packages ), type( Root ),
								 iTotalSize( 0 ), iSelectedSize( 0 ), iNoUpdateSize( 0 ), selected( false )
{
	for( int i = 0; i < packages->numPackages; i++ )
//		if(packages->packages[i]->updatable)
			iTotalSize += packages->packages[i]->size;
}

PackageListItem::PackageListItem( PackageListItem * parent, QString a_label ):
	QListViewItem( parent, a_label ), package( NULL ), packages( NULL ), type( Group ),
								 iTotalSize( 0 ), iSelectedSize( 0 ), iNoUpdateSize( 0 ), selected( false )
{
}

PackageListItem::PackageListItem( PackageListItem * parent, QString a_label, packageInfo* a_package ):
	QListViewItem( parent, a_label ), package( a_package ), packages( NULL ), type( Package ),
								 iTotalSize( a_package->size ), iSelectedSize( 0 ), iNoUpdateSize( 0 ), selected( package->selected )
{
	package->item = this;
	if( !iTotalSize )
		iTotalSize = 1;
}

int PackageListItem::setPackageSelected()
{
	return setPackageSelected( !selected );
}

int PackageListItem::setPackageSelected( bool a_bl, bool a_blBacktrack )
{
//printf( "check111 - %s: %d -> %d\n", text(0).latin1(), selected, a_bl  );fflush( stdout );

//	printf("a_bl1 - %d, selected1 - %d\n", a_bl, selected);
	if( selected != a_bl ){
  	int oldSelectedSize = iSelectedSize;
  	if( type == Package ){ //Single Package
			if( !package->mustBe && package->updatable )
			{
    		package->selected = a_bl;
    		int change = a_bl?iTotalSize:0 - oldSelectedSize;
  //  printf( "sel: %d, change: %d\n", a_bl, change );
    		changeSize( change, a_blBacktrack );
			}
  	}
		else	if( !selected || (text(0) != "Base" || (!QListViewItem::parent() || QListViewItem::parent()->text(0) != "System")) ) // Group or Root
		{
  		PackageListItem *tmpItem = (PackageListItem *)firstChild();
//  printf( "iSel: %d\n", iSelectedSize );
  		int newSize = 0;
//			printf("a_bl2 - %d, selected2 - %d\n", a_bl, selected);
  		while( tmpItem ){
  			newSize += tmpItem->setPackageSelected( a_bl, false );
				tmpItem = (PackageListItem *)tmpItem->nextSibling();
			}
//  printf( "iSel: %d, iNew: %d\n", iSelectedSize, newSize );
  		changeSize( newSize - oldSelectedSize, a_blBacktrack );
  	}
	}
	return iSelectedSize;
}

void PackageListItem::changeSize( const signed int a_size, bool a_blBacktrack )
{
//printf( "change size2222\n" );fflush( stdout );
	if( type != Root ){
  	iSelectedSize += a_size;
  	setText( 1, sizeToString( iSelectedSize ) );
   	selected = (iSelectedSize >= (iTotalSize - iNoUpdateSize));
//printf( "sel: %d, total: %d\n", iSelectedSize, iTotalSize );
		if( a_blBacktrack/* && QListViewItem::parent()*/ )
		{
			((PackageListItem*)QListViewItem::parent())->changeSize( a_size );
		}
	}else{
		iSelectedSize = 0;
		bool select = true;
		for( int i = 0; i < packages->numPackages; i++ ){
			if( packages->packages[i]->selected ){
				iSelectedSize += packages->packages[i]->size;
			}else if( select && (packages->packages[i]->inmenu && packages->packages[i]->updatable)){
//				printf("name - %s, updatable - %d\n", packages->packages[i]->name, packages->packages[i]->updatable);
				select = false;
			}
		}
  	setText( 1, sizeToString( iSelectedSize ) );
		selected = select;
//printf( "set selected: %d\n", selected );
		emit totalChanged( iSelectedSize );
	}
}

void PackageListItem::initSize( int *a_iSelected, int *a_iTotal, int *a_iNoUpdate )
{
//printf( "init size: %s - %d\n", package->name, type );
	PackageListItem *tmpItem = NULL;
	int a1, a2, a3;
	switch( type ){
		case Package:
  		selected = package->selected;
  		iSelectedSize = selected?iTotalSize:0;
			iNoUpdateSize	= package->updatable ? 0 : iTotalSize;
			break;
		case Group:
//printf( "init size - group: %s\n", text( 0 ).latin1() );
    	iSelectedSize = iTotalSize = iNoUpdateSize = 0;
    	tmpItem = (PackageListItem *)firstChild();
    	while( tmpItem ){
    		tmpItem->initSize( &a1, &a2, &a3 );
 	  		iSelectedSize += a1;
 	  		iTotalSize += a2;
				iNoUpdateSize += a3;
    		tmpItem = (PackageListItem *)tmpItem->nextSibling();
    	}
 			selected = (iSelectedSize >= (iTotalSize - iNoUpdateSize));
			break;
		case Root:
//printf( "init size - root\n" );
    	tmpItem = (PackageListItem *)firstChild();
    	while( tmpItem ){
    		tmpItem->initSize( &a1, &a2, &a3 );
    		tmpItem = (PackageListItem *)tmpItem->nextSibling();
    	}
 			changeSize( 0, false );// root counts itself by it's own method
   		emit totalChanged( iSelectedSize );
			break;
	}
  setText( 1, sizeToString( iSelectedSize ) );
  setText( 2, sizeToString( iTotalSize ) );

	*a_iSelected = iSelectedSize;
	*a_iTotal = iTotalSize;
	*a_iNoUpdate = iNoUpdateSize;
//	*a_iTotal = tmpInfo->package->updatable ? iTotalSize : 0;
}

QString PackageListItem::sizeToString( const int a_size )
{
	if( a_size <= 0 )	return QString( "" );
	int normsize = a_size;
	int level = 0;
  while( normsize > 1024 && level < 2 ){
		normsize >>= 10;
		level++;
	}
	switch( level ){
		case 1: return QString::number( normsize ) += tr("kB");
		case 2: return QString::number( normsize ) += tr("MB");
		case 3: return QString::number( normsize ) += tr("GB");
		case 4: return QString::number( normsize ) += tr("TB");// ;-)
		default:; return QString::number( normsize );
	}
}
