/***************************************************************************
                              partitioninfo.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "config.h"
#include "partitioninfo.h"
#include "partitionview.h"
#include "partitionitem.h"
#include "mountpoint.h"
#include "fdisk.h"
#include "linlinehelp.h"
#include "liconloader.h"
#include "log.h" // try not to log on non-existent drive
#include "language.h"

#include <qapplication.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qdir.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <linux/hdreg.h>	/* HDIO_GETGEO */
#include <linux/unistd.h>	/* _syscall */
#include <fcntl.h>		/* O_RDWR */
#include <sys/ioctl.h>
#include <stdlib.h> /* system() */
#include <qmessagebox.h>
#include <qregexp.h>
#include <sys/mount.h>
#include <sys/stat.h>

PartitionInfoItem::PartitionInfoItem( QListView* parent, const QString& descr, const QString& mp,
					const QString& dev, const QString& size,
					PartitionInfoItem::FormatState s)
    : QCheckListItem( parent, descr, QCheckListItem::CheckBox)
		
{
    setText( 1, mp);
    setText( 2, dev + "  " );
    setText( 3, size);
    device = dev;
    setFormatState( s );

    isoptional = isSwapfile() || mp == "swap"; // swap partitions or a swapfile are optional

// tr( "swap" );

    setSelectable( FALSE );

//printf( "optional: %d\n", isOptional() );
    if ( !isOptional() ){
	    setOn( true );
			setPixmap(0, LIconLoader::loadIcon("hds_part.gif"));
		}else{
			setOn( s == PartitionInfoItem::FormatYes );
	    setFormatState( PartitionInfoItem::FormatNothing );
		}
}


void PartitionInfoItem::setFormatState(FormatState s)
{
    formatstate = s;
    switch (s) {
    case FormatYes:
			setText(4, PartitionInfo::tr("Yes"));
			break;
    case FormatNo:
			setText(4,  PartitionInfo::tr("No"));
			break;
    case FormatDone:
			setText(4,  PartitionInfo::tr("Done"));
			break;
    case FormatFormatting:
			setText(4, " "+ PartitionInfo::tr("Formatting") +" " );
			break;
    case FormatNothing:
    default:
			setText(4,  "" );
			break;
    }
}


void PartitionInfoItem::paintCell( QPainter * p,  const QColorGroup & cg,
				    int column, int width, int alignment )
{
    if ( formatState() == FormatFormatting ) {
    	QFont font = p->font();
    	QFont boldFont = font;
    	boldFont.setWeight( QFont::Bold );
    	p->setFont( boldFont );
    	QListViewItem::paintCell( p, cg, column, width, alignment );
    	p->setFont( font );
    }
    else {
    	if ( isOptional() )
    	    QCheckListItem::paintCell( p, cg, column, width, alignment );
    	else
    	    QListViewItem::paintCell( p, cg, column, width, alignment );
    }
}


PartitionInfo::PartitionInfo( PartitionView* pview_, QWidget *parent, const char *name )
    : GBLIPage( parent, name), done( false )
{
    pview = pview_;

    QHBoxLayout * lh = new QHBoxLayout( this, 0, 6);

    QVBoxLayout * l = new QVBoxLayout( 6 );
		lh->addStretch( 1 );
    lh->addLayout( l, 30 );
		lh->addStretch( 1 );

    l->addSpacing( 10 );

    view = new QListView( this );
		QFont font( QFont::defaultFont() );
//		font.setFamily( "times" );
		font.setPointSize( Language::iSmallFontSize );
    view->setFont( font );
    view->setSorting( -1 );
    view->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
    l->addWidget( view, 10 );

    view->addColumn(tr("Purpose"));
    int mountPoint = view->addColumn(tr("Mount Point"));
    view->setColumnAlignment( mountPoint, AlignCenter );
    view->setColumnWidthMode( mountPoint, QListView::Maximum );

    view->addColumn(tr("Device"));
    int sizeCol = view->addColumn(" "+ tr("Size") +" ");
    view->setColumnAlignment( sizeCol, AlignRight );
    view->setColumnWidthMode( sizeCol, QListView::Maximum );

    int formatCol = view->addColumn("   "+ tr("Formatting") +"   ");
    view->setColumnAlignment( formatCol, AlignCenter );
    view->setColumnWidthMode( formatCol, QListView::Maximum );

    formatButton = new QPushButton( tr("Format chosen partitions"), this );
    connect( formatButton, SIGNAL( clicked() ), this, SLOT( doFormatting() ) );

    l->addSpacing( 10 );
    l->addWidget( formatButton, 0, AlignCenter );
    l->addSpacing( 12 );

/*    LInlineHelp* inlineHelp = new LInlineHelp( this );
    inlineHelp->setDocument( "partitionsel_info_overview.qml" );
    lh->addWidget( inlineHelp );*/
}

PartitionInfo::~PartitionInfo()
{
}


void PartitionInfo::showEvent( QShowEvent * )
{
		formatButton->setEnabled( !done );

		if( !done )
	    init();
		else
			done = false;

    if ( formatButton->isEnabled() ){
#ifndef GBLI_DEVEL
      	emit nextEnabled( this, FALSE );
#endif
		}
		if( formatButton->isEnabled() )
			formatButton->setFocus();
		else
			formatButton->clearFocus();
}


void PartitionInfo::hideEvent( QHideEvent * )
{
//		done = false;
//    formatButton->setEnabled( TRUE ); //#####
}


void PartitionInfo::init()
{
    unMounting();

    bool valid = TRUE;
    view->clear();

    QString rootp = MountPoints::mountPartition( "/");
    QString userp = MountPoints::mountPartition( "/usr");
    QStringList swap = pview->allSwapPartitions();

    QStringList all = pview->allPartitions();
    QStringList::Iterator a;
    for (a = all.begin(); a != all.end(); a++) {
    	if (*a != rootp && *a != userp && !swap.contains(*a) ) {
    	    QString mp = MountPoints::mountPoint( *a );
    	    if ( !mp.isEmpty() || Fdisk::willBeFormated(*a) ) {
//    	    if ( !mp.isNull() ) {
		    		new PartitionInfoItem( view, tr("Extra Partition"), mp, *a, pview->sizeOfPartitionAsString(*a),
    					Fdisk::willBeFormated(*a)?
    					PartitionInfoItem::FormatYes:PartitionInfoItem::FormatNo);
    	    }
    	}
    }

    if ( swap.isEmpty() ) {
	//### support for swap files disabled
// 	new PartitionInfoItem( view, tr("Virtual Memory"), "swap", tr("swapfile"), "96 MB",
// 				PartitionInfoItem::FormatNothing);
Log::logMain( 33, "Just check: swap IS EMPTY!?\n" );
			QMessageBox::information( this, tr("No Swap Partition"),
				tr("You must assign a swap partition for the install to proceed."),
				QMessageBox::Ok );
    } else {
      QStringList::Iterator s;
			bool first = true;
    	for (s = swap.begin(); s != swap.end(); ++s){
    	  PartitionItem *pi = pview->partition(*s);
				if( pi->checkForSwapExceed() ){
					QString mess( tr("You have created a swap partition which is too large.\n"
						"The recommended size of a swap partition is %1 Megabytes.").arg( 128 ) );
//					char buf[128];
//					sprintf( buf, " ( %s ). ", pview->sizeOfPartitionAsString(*s).ascii() );
//					mess.replace( QRegExp( "\\. " ), buf );
					QMessageBox::information( this, tr("Swap Size Error"),
						mess,	QMessageBox::Ok );
				}
  	    new PartitionInfoItem( view , tr("Virtual Memory"), "swap", *s, pview->sizeOfPartitionAsString(*s),
			    (first?(PartitionInfoItem::FormatYes):(PartitionInfoItem::FormatNothing)) );
				first = false;
    	}
    }

    if ( !userp.isEmpty() ) {
			new PartitionInfoItem( view, tr("User Partition"), "/usr", userp, pview->sizeOfPartitionAsString(userp),
				PartitionInfoItem::FormatYes);
    }

    if ( rootp.isEmpty() ) {
			new PartitionInfoItem( view, tr("Root Partition"), "/", tr("None"), "0" );
			valid = FALSE;
    }else {
			new PartitionInfoItem( view, tr("Root Partition"), "/", rootp, pview->sizeOfPartitionAsString(rootp),
				PartitionInfoItem::FormatYes);
    }
}


/*!
  Formats all marked partitions with the appropriate file format (either ext2 or msdos)
 */
void PartitionInfo::doFormatting()
{
#ifdef GBLI_DEVEL
  return;
#endif

	// hack to enhance speed, test it later
//	system( "ln -sf /usr/sbin/hdparm /sbin/hdparm >/dev/null" );
//	system( "/etc/rc.d/init.d/hdparm start >/dev/null" );

  // test, if the kernel will be installed at a position below the
  // 1024th cylinder. (Note that this can only happen in expert mode and perhaps 
  // should be handled right there.)

  QString boot = MountPoints::mountPartition( "/boot" );
  if( boot.isEmpty() )
    boot = MountPoints::mountPartition("/");
 /*Rem by  aa
if( !pview->partitionBelow1024Cylinders(boot) )
 QMessageBox::critical(this, tr("Warning"), tr("Root partition not completely within first 1024 cylinders.\nYour system may not boot."));
 */
    formatButton->setEnabled( FALSE );
    QApplication::setOverrideCursor( waitCursor );
		view->clearFocus();

    // first do the swap partitions (if any). We want to swap on them
    // before creating the file systems since that eats up so much
    // memory.

    PartitionInfoItem* i = (PartitionInfoItem*)view->firstChild();
    while (i) {
//printf( "SWAP: %s, isSwap %d, isOptional %d, isOn %d\n", i->device.ascii(), i->isSwapPartition(), i->isOptional(), i->isOn() );
    	if ( i->isSwapPartition() &&
    	     ( !i->isOptional() || i->isOn() ) ) {

					Log::logMain( 33, "Formatting swap: %s!\n", i->device.ascii() );
					// HACK ?
					MountPoints::setMountPoint( i->device.ascii(), "swap" );

   		    i->setFormatState( PartitionInfoItem::FormatFormatting );
		    	view->viewport()->repaint();
  		  	QApplication::syncX();
    	    pview->formatPartition( i->device );
   		    i->setFormatState( PartitionInfoItem::FormatDone );
		    	view->viewport()->repaint();
  		  	QApplication::syncX();
    	}
    	i = (PartitionInfoItem*)i->nextSibling();
    }

    // now format all other partitions, while swap is (very likely)
    // available
	  int maxSlash = 1;
    i = (PartitionInfoItem*)view->firstChild();

    while( i ){
			// lets make it sure at least here
    	if ( i->formatState() != PartitionInfoItem::FormatYes ){
	    	i = (PartitionInfoItem*)i->nextSibling();
				continue;
			}

    	if ( i->formatState() == PartitionInfoItem::FormatYes )
    	    i->setFormatState( PartitionInfoItem::FormatFormatting );
    	view->viewport()->repaint();
    	QApplication::syncX();

    	if ( i->isPartition() )
    	    maxSlash = QMAX( maxSlash, MountPoints::mountPoint(i->device).contains('/') );
    	
    	if ( i->isPartition() &&
    	     i->formatState() == PartitionInfoItem::FormatFormatting
    	     && !i->isSwapPartition() ){
    	    pview->formatPartition( i->device );
      }
    	
    	if ( i->formatState() == PartitionInfoItem::FormatFormatting )
    	    i->setFormatState( PartitionInfoItem::FormatDone );
    	view->viewport()->repaint();
    	QApplication::syncX();
    	i = (PartitionInfoItem*)i->nextSibling();
    }

    QApplication::restoreOverrideCursor();
    emit nextEnabled(this, TRUE);
}

int insertMountPoints(QFile *file, const char *expression, QStringList *points)
{
  if (file->open(IO_ReadOnly))
	{
		QRegExp regexp(expression);
	  QTextStream inf(file);
  	while(!inf.eof())
		{
  		QString line = inf.readLine();
    	if (regexp.match(line) >= 0)
			{
				line = line.mid(line.find(" on "+ NEWROOT) + 4);
				line.truncate(line.find(" type"));
				QStringList::Iterator it = points->begin();
				while ( it != points->end() && (*it).length() > line.length() ) it++;
				if(!points->contains(line))
					points->insert(it, line);
			}
		}
		file->close();
		return 1;
	}
	return 0;
}


void PartitionInfo::unMounting()
{
	int insertOk = 0;
	Log::closeMain();
	QString logName(Log::mainLogName);
	system( ("cp -af "+ NEWROOT +"/tmp/" + logName + " /mnt >/dev/null 2>&1").latin1() );

  system( "mount > /tmp/mount.log" );
  QFile *file = new QFile( "/tmp/mount.log" );

  QStringList points;
		
	insertOk = insertMountPoints(file, ("/dev.* on " + NEWROOT).ascii(), &points);

  for (QStringList::ConstIterator it = points.begin(); it != points.end(); it++)
		insertOk &= insertMountPoints(file, (*it).ascii(), &points);

	points.sort();
	int count = points.count();

	if(insertOk)
	{
    for (QStringList::ConstIterator it = points.end(); count > 0; it--, count--)
		{
			if(it == points.end())
				it--;

      debug( "unmounting %s", (*it).ascii() );
      QString umount = "umount " + *it;
//printf( "umount cmd: %s\n", umount.ascii() );
      if( system( umount.ascii() ) ){
//				perror( "unmounting" );
		    ::sync();
				::sleep( 2 );
				// stop using the device, especially on debugging ;-)
				FILE *f;
				f = fopen( "/tmp/umount.log", "a" );
				fprintf( f, "\nFailed to unmount %s\n\n", (*it).ascii() );
				fclose( f );
				
				char buf[128];
				sprintf( buf, "/usr/sbin/fuser -v %s >> /tmp/umount.log", (*it).ascii() );
				system( buf );
				system( "lsof >> /tmp/umount.log" );
//				if( system( umount.ascii() ) ) // once more
//  				perror( "agressive unmounting" );
			}
    }
  }
	system( ("mv -f /mnt/" + logName + " " + NEWROOT +"/tmp/ >/dev/null 2>&1").latin1() );
}

/* Mounts the partitions to the chosen mount points below NEWROOT */
void PartitionInfo::doMounting(){
//Log::logMain( 32, "doMounting()\n" );
		qApp->setOverrideCursor( Qt::waitCursor );
    if ( !isVisible() )
			init(); // doMounting can be used from outside
#ifdef GBLI_DEVEL
	qApp->restoreOverrideCursor();	
  return;
#endif

    PartitionInfoItem* i = (PartitionInfoItem*)view->firstChild();
    int maxSlash = 1;

    while (i) {
    	if ( i->isPartition() )
   	    maxSlash = QMAX( maxSlash, MountPoints::mountPoint(i->device).contains('/') );
    	i = (PartitionInfoItem*)i->nextSibling();
    }

    // now mount the new linux partitions for the installation

		Log::closeMain();
		system( ("mv -f "+ NEWROOT +"/tmp/install.log /mnt >/dev/null 2>&1").latin1() );

    // first the root partition
    QString rootp = MountPoints::mountPartition( "/" );
    QString com;
    QTextOStream(&com) << "mount " << rootp << " " << NEWROOT;
    QApplication::flushX();
    system( com.ascii() );

		// remove old Windows mount points (only root is mounted)
		system( ("rm -f "+ NEWROOT +"/mnt/? >/dev/null 2>&1").ascii() );

		::mkdir( (NEWROOT +"/tmp").latin1(), 01777 );
		system( ("mv -f /mnt/install.log "+ NEWROOT +"/tmp/ >/dev/null 2>&1").latin1() );

    QDir dir;

    // then the others, sorted by the tree depth (i.e. number of slashes)
    for (int n = 1; n <= maxSlash; n++) {
    	i = (PartitionInfoItem*)view->firstChild();
    	while (i) {
   	    if ( i->isPartition() ) {
      		int id = pview->idOfPartition( i->device );
					if( Fdisk::isSwapType( id ) ){
		   	    i = (PartitionInfoItem*)i->nextSibling();
						continue;
					}
      		QString mp = MountPoints::mountPoint( i->device );
      		if ( i->device != rootp && mp.contains('/')==n ) {
      		  if( !mp.isEmpty() ){
      		    QString d( NEWROOT + mp );
							int pos = 10;
							while( pos < (signed)d.length() && (pos = d.find( "/", pos )) >= 0 ){
	      		    dir.mkdir(  d.left( pos ).ascii() );
                pos++;
							}
      		    dir.mkdir(  d.ascii() );
       		    com.truncate(0);
							QString type( "auto" );
							if( id == 0x83 ){
								type = "ext2";
							}else if( id == 0x19 ){
								type = "reiserfs";
							}else if( id == 0x6 ){
								type = "msdos";
							}else if( Fdisk::isVFatType( id ) ){
								type = "vfat";
							}

							if( type.length() ){
        		    QTextOStream(&com) << "mount -t" << type << " " << i->device << " " << d;
	       		    QApplication::flushX();
  	     		    system( com.ascii() );
							}

//Log::logMain( 33, "%s\n", com.ascii() );fflush( stdout );
      		  }
      		}
   	    }
   	    i = (PartitionInfoItem*)i->nextSibling();
    	}
    }
    // ### ### does not check return value of mount ### ###
    writeFstab();

    publishMountPoints();
		qApp->restoreOverrideCursor();
//Log::logMain( 32, "finished doMounting()\n" );

		int res;
		if( !QFile( NEWROOT +"/proc" ).exists() ){
	    ::mkdir( (NEWROOT +"/proc").latin1(), 0755 );
			res = system( ("mount -t proc none "+ NEWROOT +"/proc").latin1() );
			Log::logMain( 1, "Mounting %s/proc: %d", NEWROOT.latin1(), res );
		}
#ifdef DEVFS
		if( !QFile( NEWROOT +"/dev" ).exists() ){
	    ::mkdir( (NEWROOT +"/dev").latin1(), 0755 );
			res = system( ("mount -t devfs none "+ NEWROOT +"/dev").latin1() );
			Log::logMain( 1, "Mounting %s/dev: %d", NEWROOT.latin1(), res );
		}
#else
		 if( !QFile( NEWROOT +"/dev" ).exists() ){
	    ::mkdir( (NEWROOT +"/dev").latin1(), 0755 );
			res = system( ("mount --bind /dev "+ NEWROOT +"/dev").latin1() );
			Log::logMain( 1, "Mounting %s/dev: %d", NEWROOT.latin1(), res );
		}
#endif
}


/*!
  inform the package installer about the mount points and available sizes
 */
void PartitionInfo::publishMountPoints()
{
    MountPoints::clearDirectories();
  
    PartitionInfoItem* i = (PartitionInfoItem*)view->firstChild();
    int maxSlash = 1;

    while (i) {
    	if ( i->isPartition() )
    	    maxSlash = QMAX( maxSlash, MountPoints::mountPoint(i->device).contains('/') );
    	i = (PartitionInfoItem*)i->nextSibling();
    }

    QString rootp = MountPoints::mountPartition( "/" );
    MountPoints::newDirectory( "/", pview->sizeOfPartition( rootp ) * 1024 );

    // then the others, sorted by the tree depth (i.e. number of slashes)
    for (int n = 1; n <= maxSlash; n++) {
    	i = (PartitionInfoItem*)view->firstChild();
    	while ( i ) {
   	    if ( i->isPartition() ) {
      		int id = pview->idOfPartition( i->device );
					if( Fdisk::isSwapType( id ) ){
		   	    i = (PartitionInfoItem*)i->nextSibling();
						continue;
					}
      		QString mp = MountPoints::mountPoint( i->device );
      		if ( i->device != rootp && mp.contains('/') == n ) {
     		    if( !mp.isEmpty() ){
//							if( id == 0x83 || id == 0x6 || id == 0x19 || id == 0x0b )
	     		      MountPoints::newDirectory( mp, pview->sizeOfPartition( i->device ) * 1024 );
     		    }
  	   		}
	 	    }
   	    i = (PartitionInfoItem*)i->nextSibling();
    	}
    }
}

void PartitionInfo::writeFstab()
{
Log::logMain( 33, "write fstab\n" );

    ::mkdir( (NEWROOT + "/etc").ascii(), 0755 );
    QFile f( (NEWROOT + "/etc/fstab").ascii() );
    if ( f.open(IO_WriteOnly) ) {
    	QTextStream t( &f );
    	PartitionInfoItem* i = (PartitionInfoItem*)view->firstChild();

    	t << "/dev/fd0\t/mnt/floppy\tauto\tnoauto,user,sync 0 0 " << endl;
    	t << "proc\t\t/proc\t\tproc\tdefaults\t0 0 " << endl;
#ifndef DEVFS
    	t << "devpts\t\t/dev/pts\tdevpts\tgid=5,mode=620\t0 0 " << endl;
#else
			// Maddy: Not sure about it:
//    	t << "devfs\t\t/dev\tdevfs\tdefaults\t0 0 " << endl;
#endif
    	t << "none\t\t/dev/shm\tshm\tdefaults\t0 0 " << endl;

    	QString line;
    	QString mp;
    	while( i ){
Log::logMain( 33, "mp: %s, id: %d\n", MountPoints::mountPoint( i->device ).ascii(), pview->idOfPartition( i->device ) );
  	    if ( i->isPartition() && ( !i->isOptional() || i->isOn() ) ) {
	    		line = i->device;
  	  		mp = MountPoints::mountPoint( i->device );
    			int id = pview->idOfPartition( i->device );  	  		
    			if( (mp.isEmpty() || mp == tr("Unused")) &&
    			  !Fdisk::isSwapType( id )/*hack-bug*/ ){
    		    i = (PartitionInfoItem*)i->nextSibling();
    		    continue;
	   		  }
  	  		line += "\t";

					// Mount Points
					if( Fdisk::isSwapType( id ) ){
      		    line += "swap\t\t";
// Strange bug: docs say that swap should be mounted to /dev/shm, but then
// the system works unreliable - either bug or feature ;(
//      		    line += "/dev/shm\t";
					}else{
     		    line += mp + "\t";
						if( mp.length() <= 7 )
							line += "\t";
					}

					// Types
      		if ( id == 0x83 ){
						line += "ext2";
      		}else if ( id == 0x19 ){
						line += "reiserfs";
      		}else if (Fdisk::isSwapType( id )){
     		    line += "swap";
      		}else if ( id == 0x6 ){
						line += "msdos";
      		}else if ( Fdisk::isWindowsType( id ) && id != 0x07 /*Fdisk::isVFatType( id )*/ ){
      		  line += "vfat";
					}else{
    		    line += "auto";
					}
					if( id == 0x19 )
        		line += " ";
					else
        		line += "\t";

					// Options
      		if( Fdisk::isSwapType( id ) ){
      		    line += "defaults,pri=1";
					}else if( Fdisk::isWindowsType( id ) && (!Fdisk::isHPFS_NTFS( id )) /*&& mp.left(5) == "/mnt/" && mp.length() == 6*/ ){
        		line += "user,exec,check=strict,umask=0007,gid=100";
						if( Fdisk::isVFatType( id ) )
	        		line += ",uni_xlate,posix,nonumtail";
						if( mp.left(5) == "/mnt/" && mp.length() == 6 ){
  						QString kde_link = NEWROOT +"/root/Desktop/"+ mp.right(1).upper() +":";
  						system( ("mkdir -p -m0750 "+ NEWROOT +"/root/Desktop").ascii() );
  				    QFile f_link( kde_link );
  				    if ( f_link.open(IO_WriteOnly) ) {
  				    	QTextStream t_link( &f_link );
                t_link << "[Desktop Entry]" << endl << "Dev=" << i->device << endl;
  							t_link << "FSType=" << ((id==0x6)?"msdos":"vfat");
  							t_link << ",user,exec,check=strict,umask=0007,gid=100";
  							if( Fdisk::isVFatType( id ) )
  		        		t_link << ",uni_xlate,posix,nonumtail";
  							t_link << endl;
  							t_link << "MountPoint=" << mp << endl;
  							t_link << "ReadOnly=false" << endl << "Type=FSDevice" << endl;
  							t_link << "Icon=hdd_mount" << endl;
  							t_link << "UnmountIcon=hdd_unmount" << endl;
  							f_link.close();
  						}
						}
      		}else{
							if (Fdisk::isHPFS_NTFS( id ))
		        		line += "defaults,ro"; //NTFS driver is buggy,so mount partition in ro mode
							else
		        		line += "defaults";
					}

					// Checking
					line += "\t";
					if( id == 0x6 ){ // exclusion for msdos: we have fsck.msdos
     		    line += "0 2 ";
      		}else if( Fdisk::isWindowsType( id ) || Fdisk::isSwapType( id ) ){
     		    line += "0 0 ";
      		}else{
     		    line += "1 ";
     		    if ( mp == "/" )
	      			line += "1 ";
     		    else
  	    			line += "2 ";
      		}
                		
      		t << line << '\n';
        }
        i = (PartitionInfoItem*)i->nextSibling();
    	}

			QDir d( "/proc/ide/" );
			if( d.exists() && d.isReadable() ){
  			d.setFilter( QDir::Dirs );
  			d.setNameFilter( "hd*" );
  			const QFileInfoList *list = d.entryInfoList();
  			QFileInfoListIterator it( *list );      // create list iterator
  			QFileInfo *fi;
				int iCDNum = 0;
				char cddev[64];
  			while( (fi = it.current()) ){
//  printf( "Drive: %s\n", QString( fi->absFilePath() + "/media" ).ascii() );
  				QFile fileMedia( fi->absFilePath() + "/media" );
  				if( fileMedia.exists() && fileMedia.open( IO_ReadOnly ) ){
  					QString str;
  					if( fileMedia.readLine( str, 6 ) != -1 && str == "cdrom" ){
  						if( !iCDNum ){
								::mkdir( (NEWROOT +"/dev").latin1(), 0755 );
								sprintf( cddev, "/dev/%s", fi->fileName().ascii() );
#ifndef DEVFS
								::symlink( cddev, (NEWROOT +"/dev/cdrom").latin1() );
#else
              	QFile outf( NEWROOT +"/tmp/devfsd.conf" );
              	if( outf.open( IO_WriteOnly | IO_Append ) ){
                  QTextStream outt( &outf );
              		outt << endl << "LOOKUP\t\t^cdrom$\t\tCFUNCTION GLOBAL symlink " << fi->fileName() << " $devpath" << endl;
              		outt << "REGISTER\t^" << fi->fileName().ascii() << "$\tCFUNCTION GLOBAL symlink " << fi->fileName().ascii() << " cdrom" << endl;
              		outt << "UNREGISTER\t^" << fi->fileName().ascii() << "$\tCFUNCTION GLOBAL unlink cdrom" << endl;
              		outf.close();
              	}
#endif
								::mkdir( (NEWROOT +"/mnt").latin1(), 0755 );
								::mkdir( (NEWROOT +"/mnt/cdrom").latin1(), 0755 );
								::mkdir( (NEWROOT +"/mnt/floppy").latin1(), 0755 );	                                                                	
  				    	t << "/dev/" << fi->fileName() << "\t/mnt/cdrom\tiso9660\tnoauto,ro,user,exec\t0 0 " << endl;
  						}else{
								sprintf( cddev, "%s/mnt/cdrom%d", NEWROOT.latin1(), iCDNum );
								::mkdir( cddev, 0755 );
  				    	t << "/dev/" << fi->fileName() << "\t/mnt/cdrom" << iCDNum << "\tiso9660\tnoauto,ro,user,exec\t0 0 " << endl;
  						}
							iCDNum++;
						}
//  printf( "Type: %s\n", str.ascii() );
  					fileMedia.close();
  				}
  				++it;							
  			}
			}else{
				Log::logMain( 32, "Couldn't read /proc/ide directory!\n" );
			}		

    	f.close();
    }

    QStringList all = pview->allPartitions();
    QStringList::Iterator a;
    for (a = all.begin(); a != all.end(); a++) {
      QString str = *a;
      copyWindowsFonts( str );
    }

		system( ("/bin/cp -af "+ NEWROOT +"/etc/fstab "+ NEWROOT +"/etc/fstab.orig").ascii() );
}

void PartitionInfo::copyWindowsFonts( QString &srcDevName ){
  PartitionItem *it = pview->partition( srcDevName );
  if( it->isWindows() )
Log::logMain( 34, "win fonts check: %s - %d", srcDevName.ascii(), it->winFontsPath.length() );
  if( it->isWindows() && it->winFontsPath.length() ){
Log::logMain( 34, "start copying win fonts" );
		QString mpt = MountPoints::mountPoint( it->name() );
		if( mpt == QString::null || !mpt.length() ){
			mpt = "/mnt/tmp";
	    system( ("mount "+ srcDevName +" "+ mpt +" -t msdos").ascii() );
		}
    system( ("mkdir -p -m0755 "+ NEWROOT +"/usr/share/fonts/winfonts >/dev/null 2>&1").ascii() );
    system( ("cp -af " + mpt +"/"+ it->winFontsPath +"/*.ttf "+ NEWROOT +"/usr/share/fonts/winfonts").ascii() );
		if( mpt == "/mnt/tmp" )
	    system( "umount /mnt/tmp" );
    system( ("/usr/sbin/convert_fonts.sh "+ NEWROOT +"/usr/share/fonts/winfonts &").ascii() );
Log::logMain( 34, "finished copying win fonts" );
  }
}
