/***************************************************************************
                              partitionitem.cpp
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "partitionitem.h"
#include "diskitem.h"
#include "diskorpartitionitem.h"
#include "mountpoint.h"
#include "fdisk.h"
#include "liconloader.h"
#include "partitioninfo.h"

#include <sys/mount.h>
#include <stdlib.h>

#include <qdir.h>
#include <qtextstream.h>
#include <qapplication.h>

PartitionItem::PartitionItem( QListViewItem* parent, DiskItem* diskItem_, bool isNew_,
				int number_, bool bootable_, signed long start_, signed long size_, int id_, QListViewItem* after = NULL )
    : DiskOrPartitionItem( parent, after )
{
    diskItem = diskItem_;
    willBeFormated = false;
    init( isNew_, number_, bootable_, start_, size_, id_);
    setOpen( TRUE );
}

void PartitionItem::init( bool isNew_, int number_, bool bootable_, signed long start_, signed long size_, int id_)
{
    number = number_;
    bootable = bootable_;
    start = start_;
    size = size_;
    id = id_;
    isNew = isNew_;
}

bool PartitionItem::checkForSwapExceed()
{
  signed long temp = diskItem->cylinderOfSector( size-63 );// bad magic here - need to be checked ;(
  signed long temp2 = diskItem->cylinderOfSector( diskItem->sectorsOfMegaByte( 128 ) );
Log::logMain( 33, "Swap comparing: cur %ld - max %ld cylinders (size %ld)\n", temp, temp2, size );
  return temp > (temp2+2); // once more bad magic
}

void PartitionItem::checkForFilesystem()
{
Log::logMain( 33, "Check: %s, id: %d, new: %d\n", name().ascii(), id, isNew );

    if ( isNew ) {
			Fdisk::markAsNew( name() );
    }else if ( Fdisk::startup && (id == 0x83/* || id == 0x19*/) ){
    	if ( ::mount( name(), "/mnt/tmp", "ext2",
    		      MS_MGC_VAL + MS_RDONLY, NULL ) ) {// trying to mount
Log::logMain(33, "ext2 failed\n" );
    			if ( ::mount( name(), "/mnt/tmp", "reiserfs",
        		      MS_MGC_VAL + MS_RDONLY, NULL ) ) {// trying to mount
Log::logMain(33, "reiser failed\n" );
        	    Fdisk::markAsNew( name() );
        	    isNew = TRUE;
        	} else {
Log::logMain(33, "reiser\n" );
							id = 0x19;
        	    ::umount( "/mnt/tmp" );
				      willBeFormated = FALSE;
        	}
    	}else{
Log::logMain(33, "ext2\n" );
    	    ::umount( "/mnt/tmp" );
		      willBeFormated = FALSE;
    	}
    }else if ( Fdisk::isWindowsType( id ) ){
Log::logMain(33, "win\n" );
    	if ( !::mount( name(), "/mnt/tmp", "msdos",
 	      MS_MGC_VAL + MS_RDONLY, NULL ) ) {
//printf( "check\n" );
				if( !Fdisk::winDrive.length() && !system( "cp -af /mnt/tmp/BL/linux.cfg /tmp >/dev/null 2>&1" ) )
					Fdisk::winDrive = QString( name() );
 	      winFontsPath = seekForWinFonts();
				if( !Fdisk::winDrive.length() && winFontsPath.length() )
					Fdisk::winDrive = QString( name() );
  	    ::umount( "/mnt/tmp" );
	      willBeFormated = FALSE;
    	}else{
// Could be causing that bug with formatting Win partition...
//  	    Fdisk::markAsNew( name() ); // is it really required?
//  	    isNew = TRUE;
			}
		}

    bool mustBeFormated = ( Fdisk::isSwapType(id) |
			    MountPoints::mountPoint(name())== "/" |
			    MountPoints::mountPoint(name()) == "/usr" |
          MountPoints::mountPoint(name()) == "/boot"	|
			    Fdisk::willBeFormated( name() ) );

    willBeFormated = mustBeFormated | isNew;
//printf( "checkForFileSystem %s : willBeFormated - %d\n", name().ascii(), willBeFormated );

    // do not format an extended partition
    if( Fdisk::isExtendedType(id) )
      willBeFormated = FALSE;

    // do not format your install source
    if( Fdisk::isInstallSource(name()) )
			willBeFormated = FALSE;

    Fdisk::setFormatting( name(), willBeFormated );

    updateView();
}

QString PartitionItem::name()
{
    QString result;
    //do it
    if (disk().left(8) == "/dev/rd/" || disk().left(9) == "/dev/ida/" || disk().left(11) == "/dev/cciss/")
      QTextOStream( &result ) << disk() << "p" << number;
    else
      QTextOStream( &result ) << disk() << number;
    return result;
}

signed long PartitionItem::cylinderOfSector( signed long s )
{
    return diskItem->cylinderOfSector( s );
}

double PartitionItem::megaByteOfSectors( signed long s )
{
    return diskItem->megaByteOfSectors( s );
}

QString PartitionItem::disk()
{
    return diskItem->disk;
}

signed long PartitionItem::heads()
{
    return diskItem->heads;
}

signed long PartitionItem::sectors()
{
    return diskItem->sectors;
}

signed long PartitionItem::cylinders()
{
    return diskItem->cylinders;
}


/* The item has changed. Inform the listview about the new values. */
void PartitionItem::updateView(){
//printf( "Item::updateView: %s\n", name().ascii() );

    int col = 0;
    setText( col, name());
//printf( "mount point: \"%s\"\n", MountPoints::mountPoint(name()).ascii() );
//		if( MountPoints::mountPoint( name() ) != "swap" )
//	    setPixmap( col, LIconLoader::loadIcon("hds_part.gif"));
//		else
//	    setPixmap( col, NULL );

    QString c;
    ++col;

    setText(col++, QString(" ")+Fdisk::sysname( id ) );

//printf( "mount point: \"%s\"\n", MountPoints::mountPoint(name()).ascii() );
    if (MountPoints::mountPoint(name()) == QObject::tr("Unused"))
      setText(col++, "");
    else
      setText(col++, MountPoints::mountPoint( name() ) );

    if (id != 0) {
#ifdef ADVANCED_SIZE_VIEW
	  	c.setNum( cylinderOfSector(start) );
			setText(col++, c);
		 	c.setNum( cylinderOfSector(end()) );
			setText(col++, c);
#endif
    } else {
			// empty primary partition
#ifdef ADVANCED_SIZE_VIEW
			setText(col++, "");
			setText(col++, "");
#endif
    }

    if (id != 0) {
			c.setNum( sizeInMB() );
			setText(col++, c +" "+ qApp->translate("PartitionItem", "MB" ) );
		}else{
			setText(col++, "");
		}


    setText(col++, bootable?"*": " ");

    if ( willBeFormated )
			setText(col++, QObject::tr("Yes") );
    else
			setText(col++, QObject::tr("No") );

}

/* Dump this partition in the sfdisk dump format */
QString PartitionItem::dump()
{   //  /dev/sda1 : start=       63, size=   96327, Id=83, bootable

    QString result;
    QTextOStream os( &result );
    os << name() << " : start= " << (signed long)start << ", size= " << (signed long)size << ", ";
      os.flags( QTextStream::hex );
    os << "Id =" << id;
    if (bootable)
			os << ", bootable";
    os << "\n";
//printf( "**dump: %s", result.ascii() );
    return result;
}

/* Is the partition an extended partition? */
bool PartitionItem::isExtended()
{
    return Fdisk::isExtendedType( id );
}

/* Is the partition a swap partition? */
bool PartitionItem::isSwap()
{
    return Fdisk::isSwapType( id );
}

/* Is the partition contains Microsoft Windows? */
bool PartitionItem::isWindows()
{
    return Fdisk::isWindowsType( id );
}

QString PartitionItem::seekForWinFonts(){
  QString result( "" );

Log::logMain( 33, "Seeking for win_fonts!\n" );

  if( !result.length() ){
    char *paths[] = { "windows", "winnt", "Windows", "WINDOWS", "WINNT", "WinNT", NULL };
    char *font_path[] = { "fonts", "FONTS", "Fonts", NULL };
    QString prefix = QString( "/mnt/tmp/" );


    for( int i = 0; paths[i]!=NULL; i++ ){
      QString path = prefix + QString::fromLatin1( paths[i] );
      for( int k = 0; font_path[k]!=NULL; k++ ){
        QString path1 = path +"/"+ QString::fromLatin1( font_path[k] );
Log::logMain( 33, "Trying: %s\n", path1.ascii() );
        if( QDir( path1 ).exists() ){
          result = QString::fromLatin1( paths[i] ) +"/"+ QString::fromLatin1( font_path[k] );
Log::logMain( 33, "Found Windows Fonts in: %s!\n", result.ascii() );
          break;
        }
      }
      if( result.length() )
        break;
    }
  }
  return result;
}