/***************************************************************************
                          printercfg.h  -  description                              
                             -------------------                                         
    begin                : Fri Feb 11 2000                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		 					 : Vadim Khakhulin, Natalja Drozzina
    author email         : vh@bestlinux.net, nd@bestlinux.net
    url			 						 : http://www.bestlinux.net
    email		 						 : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/
#ifndef PRINTERCFG_H
#define PRINTERCFG_H

#include "config.h"

#include <qwidget.h>
#include <qstring.h>
#include <qgroupbox.h>
#include <qlist.h>
#include <qstringlist.h>
#include <qlistbox.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qapplication.h>
#include <qsplitter.h>
#include <qxml.h>

#include "network.h"
#include "networkpage.h"
#include "gblipage.h"
#include "splitstring.h"
#include "ini_file.h"

#include "windowsconfig.h"

#include <qdialog.h>

#define ARRAY_SIZE 4
#define PROC "/proc"
#define SPOOL_BY_DEFAULT "/var/spool/lpd/lp"

class QLabel;
class QListBox;
class PrnEntry;

// class for parsing xml file printerdb.foo
class StructureParser: public QXmlDefaultHandler
{
public:
	StructureParser(QList<PrnEntry> *list_)
	{
		list = list_;
		xmlFields.SetSectionsString("id,make,model,driver");
		startDrv = false;
		curCount = 0;
	}
	QString getXmlFields() { return xmlFields.returnSections(); }
private:
	bool startDocument() { return true; }
	bool startElement(const QString&, const QString&, const QString&, const QXmlAttributes&);
	bool endElement(const QString&, const QString&, const QString&);
	bool characters(const QString&);

	QList<PrnEntry> *list;
	QString curName;
	bool startDrv;
	int curCount;
	SplitString xmlFields;
};

// One printer chain: make->model->drivers
class PrinterList: public QObject
{
public:
    PrinterList(PrinterList *parent, const QString &name, QString id_ = "");
    ~PrinterList() { }

    QString printerListName() { return pName; }
    QString printerId() { return id; }

protected:
    QString pName;
		QString id;
};

// creates list view using data from PrinterLists
class PrinterListItem : public QListViewItem
{
public:
    PrinterListItem( QListView *parent, PrinterList *f );
    PrinterListItem( PrinterListItem *parent, PrinterList *f );

    void insertSubFolders(const QObjectList *lst);
    QString id;
};

// data for one printer
class PrnEntry
{
public:
	PrnEntry() { }
	PrnEntry(QString *id_, QString *make_, QString *model_, QString *drivers_)
	{
		id = *id_;
//  	fields.append(id_);
  	fields.append(make_);
  	fields.append(model_);
  	fields.append(drivers_);
	}
	~PrnEntry() {	}
	void putValue(QString str)
	{
		fields.append(new QString(str));
	}
	QList<QString> fields;
	QString id;
	QString drivers;
};

/*********************************************/

class PrinterCfg : public GBLIPage
{

   Q_OBJECT

public:
	PrinterCfg(QWidget *parent = 0, const char *name = 0, NetworkPage *net_ = 0);
	~PrinterCfg()
	{
		delete printersView;
	}
	bool wasHere;
	SplitString infoData, canChangeData, checkBoxesData;
	SplitString Info, canChange, checkBoxes;

	enum { Local=0, Remote, SMB, Netware };

	void finish();
	void Sound() { QApplication::beep(); }
	
  QString* ReturnStr(int indx)
  {	
	  QString *end = new QString();
		switch(indx)
		{
			case 1:	return &sQueueResult;	break;
			case 2:	return &sSpoolResult;	break;			
			default: break;	
		}
		return end;
  }

	void showEvent(QShowEvent *);
	void setInfoFromWindows(int printerType, QString device, SplitString netValues, SplitString advValues, QListViewItem *model);
	bool lookForPrinter(QString winPrinter_, QListViewItem **retModel);
private slots:
  void ShowAdditional( int index );
	void selectionChanged(QListViewItem *listItem);
  void TestPage();
	void Advanced();
private:
	void FillEntry();				
  void NullValue();

  void setEnablLocal( bool bValue );
  void setEnablRemote( bool bValue );
  void setEnablSmb( bool bValue );
  void setEnablWare( bool bValue );

  QGroupBox *FrameDraw();

  QWidget *LocalLayout( QGroupBox *gFrame );
  QWidget *RemoteLayout( QGroupBox *gFrame );
  QWidget *SmbLayout( QGroupBox *gFrame );
  QWidget *NetWareLayout( QGroupBox *gFrame );

  void MainLayout();
	
  int getCurIndex();

  QString ConvertType(int);
  bool ResultValue();
  bool CreateSpoolDir(QString sSpoolP, QString sRootSpool);

  bool ConfigureQueue(QString sRoot);
  void CreatePrintcap(QString sRoot);
	void DefaultSettings();

	void SetFalseTrue(bool);
	void CheckNetwork();
	
	int generateFooData(QString);
	void setupPrinters();
	void createMfCfg(QString);
	void createOverride();
	void createPerlMassive(QString);
	void createScriptCfg(QString, QString);
	void createAlchemistScript();

	void getAllPrinters();
	void getWinSettings();

	QStringList printerMake;
	QStringList *printerModel;
	QStringList *printerId;

  QWidget *groupAdd[ARRAY_SIZE];
  QList<PrnEntry> list;

  QComboBox *cmbType;
  QComboBox *cmbDevice;

  QLabel *lblType;
  QLabel *lblDevice;

  QPushButton *butTest;
  QPushButton *butConf; // test button

  AddressEdit *editRemoteHost;
  QLineEdit *editQueueRemote;

  QLineEdit *editHost;
  AddressEdit *editIP;
  QLineEdit *editName;
  QLineEdit *editUser;
  QLineEdit *editPass;
  QLineEdit *editWork;

  AddressEdit *editNcpServer;
  QLineEdit *editNcpQueue;
  QLineEdit *editNcpUser;
  QLineEdit *editNcpPass;

  QString sTypeResult;
  QString sDevice;
  QString sDeviceHelp;

  QString sQueueResult;
  QString sSpoolResult;
	
  QString sRemoteHost;
  QString sRemoteQueue;

  QString sSmbHost;
  QString sSmbIP;
  QString sSmbUser;
  QString sSmbName;
  QString sSmbPass;
  QString sSmbWorkgr;

  QString sNcpServer;
  QString sNcpQueue;
  QString sNcpUser;
  QString sNcpPass;

  QGroupBox *grBot;
  QLabel *lblHost;
  QLabel *lblQueueRemote;

  QLabel *lblSmbHost;
  QLabel *lblSmbIP;
  QLabel *lblSmbName;
  QLabel *lblSmbUser;
  QLabel *lblSmbPass;
  QLabel *lblWork;

  QLabel *lblServer;
  QLabel *lblNcpQueue;
  QLabel *lblNcpUser;
  QLabel *lblNcpPass;

  bool bPort;

	QGroupBox *groupTop;
  QPushButton *butAdvanced;

	SplitString xmlFields;
	QListView *printersView;
	QList<PrinterList> lstPrinters;
//	QList<PrinterListItem> lstItem;

	QString defaultChkBoxesVal;
	QString perlScript;
	QString postscriptWrapper, wrapper, smbprint, ncpprint, overFile;
	QString driver, id;
	QString dataFile, defaultPageSize;
	IniFile	Ini;

  QStringList pageSizes;

	NetworkPage *net;
};

class AdvancedD: public QDialog
{
public:
	AdvancedD(QWidget *parent, const char *name, PrinterCfg *PCfg, QString dataFile_);
	~AdvancedD()
	{
		delete[] combos;
		delete[] chkBoxes;
		delete[] comboValMas;
		delete[] comboComMas;
	}
		
  void ResultFromSecWin(PrinterCfg *PCfg);
	void Sound() { QApplication::beep(); }
private:
	QGroupBox *groupTop2;
  QLabel *lblPixel;
  QComboBox *cmbPixel;
  QLabel *lblSize;
  QComboBox *cmbSize;
  QLabel *lblRes;
  QComboBox *cmbRes;
  QLabel *lblQueue;
  QLineEdit *editQueueMain;
  QLabel *lblSpool;
  QLineEdit *editSpool;

	QCheckBox **chkBoxes;
	QComboBox **combos;
	SplitString **comboValMas;
	SplitString **comboComMas;

	IniFile	Ini;
	SplitString comboVal, comboCom;
	int amountCombo, amountCheck;

  int EntryNum;
};

#endif
