/***************************************************************************
                          steplabel.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Nov 30 2000                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		 : Anton Safonov (Maddy)                                         
    author email         : as@bestlinux.net                                          
    url			 : http://www.bestlinux.net                          
    email		 : linux@bestlinux.net                              

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "steplabel.h"
#include "liconloader.h"

#include <qfile.h>

StepLabel::StepLabel( QWidget * parent, QWidget * a_page, const char * name, GBLIPage::Status a_iStatus ):
	QLabel( parent, name ), blCurrent( false ), page( NULL ), blVisited( false ), backColor( 128, 128, 128 ), backLabel( NULL ){
		page = a_page;
		setMouseTracking( true );
		setStatus( a_iStatus );
		backLabel = new QLabel( parent, "" );
		backLabel->stackUnder( this );
		backLabel->clear();
		backLabel->setBackgroundColor( backColor );
		backLabel->setFixedSize( 48, 48 );
}

void StepLabel::mousePressEvent ( QMouseEvent * e ){
	emit pressed( this, name() );
}

void StepLabel::setStatus( int /*GBLIPage::Status*/ a_iStatus ){
	iStatus = (GBLIPage::Status)a_iStatus;
//printf( "%s, c %d, v %d\n", name(), blCurrent, blVisited );
	if( !blCurrent && !blVisited ){
		backColor = QColor( 190, 190, 190 ); // like disabled
	}else{
  	switch( iStatus ){
  		case GBLIPage::Good:
  			backColor = QColor( 80, 200, 80 );
  			break;
  		case GBLIPage::Bad:
  			backColor = QColor( 203, 0, 0 ); //255,197,8
  			break;
  		case GBLIPage::Disabled:
  			backColor = QColor( 190, 190, 190 );
  			break;
  		default: break;
		}
	}
	updateIcon();
}

void StepLabel::updateIcon(){
	QString tmp = name();
	if( !blVisited && available( tmp +"_grey" ) ) tmp += "_grey";
	tmp += ".gif";
	setPixmap( LIconLoader::loadStepIcon( tmp ) );
	adjustSize();
}

bool StepLabel::available( QString name ){
	return QFile( LIconLoader::getStepsPath() +"/"+ name +".gif" ).exists();
}

void StepLabel::move( int x, int y ){
	if( backLabel ){
//		if( blCurrent )
//			backLabel->setBackgroundColor( QColor( 245, 245, 0 ) );
//		else
			backLabel->setBackgroundColor( backColor );
		backLabel->move( x - 4, y - 4 );
	}
	QLabel::move( x, y );
}

void StepLabel::setCurrent( bool a_bl ){
	if( blCurrent != a_bl ){
		blCurrent = a_bl;
		setStatus( iStatus );
	}
	updateIcon();
}

void StepLabel::setVisited( bool a_bl ){
	if( a_bl && blVisited != a_bl ){
		blVisited = true;
		setStatus( iStatus );
	}
	updateIcon();
}
