/***************************************************************************
                                 target.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "target.h"
#include "fdisk.h"
#include "partitionview.h"
#include "partitioninfo.h"
#include "linlinehelp.h"
#include "liconloader.h"
#include "installselect.h"
#include "log.h"

#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qprogressbar.h>
#include <qapplication.h>
#include <qmessagebox.h>

Target::Target( PartitionView* view, QWidget * parent, const char * name )
    : GBLIPage( parent, name )
{
    pview = view;
    gb = new QButtonGroup(tr("Install"), 0);

    QHBoxLayout * lh = new QHBoxLayout( this, 0, 6);

    QVBoxLayout * l = new QVBoxLayout( 6 );
		lh->addStretch( 6 );
    lh->addLayout( l, 3 );
		lh->addStretch( 6 );

    QRadioButton * r;

    QHBoxLayout* lrh = new QHBoxLayout;
    l->addStretch();
    l->addSpacing( 55 );
    l->addLayout( lrh );
    l->addStretch();

    QVBoxLayout* lr = new QVBoxLayout;
    lrh->addStretch( 3 );
    lrh->addLayout( lr, 5 );
    lrh->addStretch( 3 );

    entireHarddiskButton = r = new QRadioButton( tr("Whole disk"), this );
		r->setAutoMask( true );
    lr->addWidget( r, 3 );
    gb->insert( r, (int)Disk );
    r->setFocus();

//		lr->addSpacing( 10 );
//    r = new QRadioButton( tr( "Use free space" ), this );
//		r->setAutoMask( true );
//    lr->addWidget( r, 3 );
//    gb->insert( r, (int)Partitions );
//r->setEnabled( false );
////    preparedPartitionsButton = r;

		lr->addSpacing( 10 );
    r = new QRadioButton( tr( "Prepared partition(s)" ), this );
		r->setAutoMask( true );
    lr->addWidget( r, 3 );
    gb->insert( r, (int)Partitions );
    preparedPartitionsButton = r;

		lr->addSpacing( 10 );
    r = new QRadioButton( tr( "Custom" ), this );
		r->setAutoMask( true );
    lr->addWidget( r, 2 );
    gb->insert( r, (int)Custom );
    customButton = r;

		lr->addSpacing(20);
    r = new QRadioButton(tr( "Update existing Linux OS" ), this);
		r->setAutoMask(true);
    lr->addWidget(r, 3);
    gb->insert(r, (int)Update);
		updateButton = r;

    connect( gb, SIGNAL(clicked(int)),
	     this, SLOT(setSelected(int)) );

		QWidget *messHolder = new QWidget( this );
 		QHBoxLayout *mess_lo = new QHBoxLayout( messHolder );
 		QVBoxLayout *mess_lov = new QVBoxLayout( mess_lo );
 		
    messHolder->setFixedSize( 300, 50 );
//		messHolder->setAutoMask( true );
		messHolder->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );
 				
    diskCheckLabel = new QLabel( tr("Checking disks, please wait...."), messHolder );
		diskCheckLabel->setAutoMask( true );
    diskCheckLabel->setAlignment( QLabel::AlignCenter );
    diskCheckProgressBar = new QProgressBar( messHolder );
		diskCheckProgressBar->setAutoMask( true );

    mess_lov->addWidget( diskCheckLabel );
    mess_lov->addWidget( diskCheckProgressBar );
    l->addSpacing( 5 );
    l->addWidget( messHolder );
    l->addSpacing( 35 );

    connect( view, SIGNAL( diskCheckMaximum(int)),
	     diskCheckProgressBar, SLOT( setTotalSteps(int) ) );
    connect( view, SIGNAL( diskCheckProgress(int)),
	     diskCheckProgressBar, SLOT( setProgress(int) ) );

/*    inlineHelp = new LInlineHelp( this );
    lh->addWidget( inlineHelp );*/

    customButton->setChecked( TRUE );
    setSelected( Custom );
    firstShow = TRUE;

    entireHarddiskButton->setEnabled( FALSE );
    preparedPartitionsButton->setEnabled( FALSE );
    customButton->setEnabled( FALSE );
    updateButton->setEnabled(FALSE);
}


Target::~Target()
{
    delete gb;
    gb = 0;
}


void Target::setSelected( int s2 )
{
		if( s2 == Disk ){
			int res = QMessageBox::warning( this, tr( "Warning" ), tr(
       "All of the partitions on your hard drive(s) will be erased.\n"
       "\n"
       "This means that all of the data on your system will be destroyed.\n"
       "\n"
       "If you do not want to lose all of your partitions, select \"Cancel\" now, "
       "and perform a \"Custom\" install." ),
				tr("Ok"), tr("Cancel"), NULL, 1 );
			if( res ){
		    ((QRadioButton*)gb->find( s ))->setChecked( true );
				return;
			}
		}
		
		//Selection redrawing
		((QRadioButton*)gb->find( s2 ))->setFocus();

    s = (Set)s2;
}

Target::Set Target::selected() const
{
    return s;
}

void Target::showEvent( QShowEvent * )
{
    if( firstShow )
		{
			QTimer::singleShot( 0, this, SLOT( checkDisks() ) );
      firstShow = FALSE;
    }else{
      PartitionInfo::unMounting();

			if(InstallSelect::blUpdateNP) // if rebuild of old rpm database failed or there are no partitions with Linux
			{
	  	  updateButton->setDisabled(true);
	    	preparedPartitionsButton->setChecked(true);
			}
		}
}


void Target::checkDisks()
{
    QApplication::syncX();
    qApp->processEvents();
    diskCheckProgressBar->reset();
    pview->checkState();
    if ( pview->hasLinuxPartitions() ) {
			preparedPartitionsButton->setEnabled( TRUE );
			preparedPartitionsButton->setChecked( TRUE );
			setSelected ( Partitions );
			preparedPartitionsButton->setFocus();

			InstallSelect::blUpdateNP = false; // if there are no partitions with Linux OS
  	  updateButton->setDisabled(true);

			QStringList parts = pview->allPartitions();
			/*Added by dk, 25.02.2002, to inform about partition number overflow on disk*/
			/*dk*/
			if (pview->partitionOverflow)
			{
				preparedPartitionsButton->setEnabled( FALSE );
				updateButton->setEnabled(FALSE);
				customButton->setFocus();
			}
			/*dk*/
  	  QStringList::Iterator s;
    	for (s = parts.begin(); s != parts.end(); ++s)
			{
	  		if (pview->idOfPartition( *s ) == 0x83 || pview->idOfPartition( *s ) == 0x19)
				{
//					Log::logMain(33, "checking - %s\n",(*s).ascii());
					InstallSelect::blUpdateNP = notAvailForUpdate((*s).ascii());
//					Log::logMain(33, "blUpdateNP: %d\n", (int)InstallSelect::blUpdateNP);

					if(!InstallSelect::blUpdateNP)
					{
						updateButton->setEnabled(true);
						break;
					}
				}
			}		
    }
    else {
			preparedPartitionsButton->setEnabled( FALSE );
			updateButton->setEnabled(FALSE);
    }
    QTimer::singleShot( 1500, diskCheckLabel, SLOT(hide()) );
    QTimer::singleShot( 1500, diskCheckProgressBar, SLOT(hide()) );

    entireHarddiskButton->setEnabled( TRUE );
    customButton->setEnabled( TRUE );
}

int Target::notAvailForUpdate(const char *part)
{
	unsigned int i;
	int failed;
	char *fs[] = { "ext2", "reiserfs" };
	char *exist[] = { "/mnt/tmp/etc", "/mnt/tmp/bin", "/mnt/tmp/sbin",
										"/mnt/tmp/var", "/mnt/tmp/lib" };

	failed = 1;

	for(i = 0; i < sizeof(fs) / sizeof(fs[0]); i++)
 	{
  	if(!::mount(part, "/mnt/tmp", fs[i], 0, NULL))
		{
			Log::logMain(32, "notAvailForUpdate: mounted - %s\n", part);
			failed = 0;
			break;
		}
	}
   					
	if(!failed)
	{
		for(i = 0; i < sizeof(exist) / sizeof(exist[0]); i++)
		{
			if(!QFile::exists(QString(exist[i])))
			{
				Log::logMain(32, "notAvailForUpdate: %s doesn't exist for %s\n", exist[i], part);
				::umount("/mnt/tmp");
				return 1;
			}
		}
		::umount("/mnt/tmp");
	}
	else
		return 1;

	return 0;
}

