/***************************************************************************
                             testvideomode.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qapplication.h>
#include <qdrawutil.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qpoint.h>

#include <qwindowsstyle.h>

#include "xserver.h"
#include "config.h"

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <signal.h>

#include <stdlib.h>

#include <signal.h>


class FakeKDE: public QWidget
{
public:
    FakeKDE( const QString & message );

    void show();

protected:
    void mouseReleaseEvent( QMouseEvent * );
    void keyReleaseEvent( QKeyEvent * );

private:
    QString m;
};


FakeKDE::FakeKDE( const QString & message )
  : QWidget( 0, 0 ), m( message )
{
    move( 0, 0 );
    resize( QApplication::desktop()->size() );
}


void FakeKDE::show()
{
  setBackgroundMode(QWidget::NoBackground);

    QWidget::show();
    int w = QApplication::desktop()->width( );
    int h = QApplication::desktop()->height( );

    // get a painter
    QPainter p( this );
    QColorGroup cg( QApplication::palette( ).normal( ) );

    // draw the background
    QPixmap tile( QString( GBLI_DATA_PATH ) +"/graphics/kdescreen/tile.xpm" );
    p.drawTiledPixmap( 0, 0, w, h, tile );

    // draw the taskbar
    qDrawShadePanel( &p, 0, 0, w, 24, cg, false, 1,
		     new QBrush( cg.background( ) ) );
    QPixmap task( QString( GBLI_DATA_PATH ) +"/graphics/kdescreen/taskbar.xpm" );
    p.drawPixmap( 0, 0, task );

    // draw the icons
    QPixmap icons( QString( GBLI_DATA_PATH ) +"/graphics/kdescreen/icons.xpm" );
    p.drawPixmap( 0, 32, icons );

    // draw the panel
    qDrawShadePanel( &p, 0, h-47, w, 47, cg, false, 2,
		     new QBrush( cg.background( ) ) );
    QPixmap left( QString( GBLI_DATA_PATH ) +"/graphics/kdescreen/panel_left.xpm" ),
			right( QString( GBLI_DATA_PATH ) +"/graphics/kdescreen/panel_right.xpm" ),
			middle( QString( GBLI_DATA_PATH ) +"/graphics/kdescreen/panel_middle.xpm" );
    p.drawPixmap( 0, h-left.height()-2, left );
    p.drawPixmap( w-right.width(), h-right.height( )-2, right );
    p.drawPixmap( ( w-middle.width() )/2, h-middle.height()-2, middle );

    // draw the testpicture
    int tx = w/10;
    int ty = h/8;
    int tw = 2*w/3;
    int th = 2*h/3;

    for ( int ix=0; ix<16; ix++ )
			for ( int iy=0; iy<16; iy++ )
	  	  p.fillRect( tx+ix*tw/16, ty+iy*th/16, tw/16+1, th/16+1,
					QBrush( QColor( 16*ix, iy*16, ix*iy ) ) );

    QPixmap title_left( QString( GBLI_DATA_PATH ) +"/graphics/kdescreen/title_left.xpm" ),
			title_right( QString( GBLI_DATA_PATH ) +"/graphics/kdescreen/title_right.xpm" ),
			title_middle( QString( GBLI_DATA_PATH ) +"/graphics/kdescreen/title_middle.xpm" );
    p.drawTiledPixmap( tx, ty-22, tw, 22, title_middle );
    p.drawPixmap( tx, ty-22, title_left );
    p.drawPixmap( tx+tw-title_right.width( ), ty-22, title_right );
    qDrawShadePanel( &p, tx, ty, tw, th, cg, false, 2 );

    // draw the message box
    p.setFont( QFont("helvetica", 24 ) );
    QRect r = p.boundingRect( 0, 0, w, h, AlignCenter, m );
    r.moveCenter( QPoint( w/2, h/2 ) );
    r = QRect( r.left()-16, r.top()-16, r.width()+32, r.height()+32 );
    QBrush tmp( colorGroup().background() );
    qDrawWinPanel( &p, r, colorGroup(), FALSE, &tmp );
    p.setPen( colorGroup().foreground() );
    p.drawText( r, AlignCenter, m );

		QCursor::setPos( QPoint( 10, 10 ) );
}


void FakeKDE::mouseReleaseEvent( QMouseEvent * )
{
    qApp->quit();
}


void FakeKDE::keyReleaseEvent( QKeyEvent * )
{
    qApp->quit();
}


int main( int argc, char ** argv )
{
    setenv( "DISPLAY", "127.0.0.1:2", 1 );

    int pid = fork( );
    if ( pid == 0 ) {
			int i = getdtablesize( );
			while( i-- >= 0 )
	  	  ::close( i );
			open( "/dev/null", O_RDWR );
			dup( 0 );
			dup( 0 );

			execlp( "/tmp/xlink", "xlink", ":2", "vt09",
	      "-bestRefresh", "-xf86config", "/tmp/X11/XF86Config", 0 );
			exit( 0 );
    } 
//printf( "ltestvideomode pid: %d\n", pid );

    bool r = FALSE;
    while( !r ) {
    	// make a nice socket for the X server probe
    	int datafd = socket( AF_INET, SOCK_STREAM, 0 );
    	if ( datafd < 0 )
    	    return 42;
    	int on = 1;
    	if (setsockopt( datafd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on))<0)
    	    return 69;
    	struct sockaddr_in probe;
    	probe.sin_family = AF_INET;
    	probe.sin_addr.s_addr = htonl( 0x7f000001 );
    	probe.sin_port = htons( 6001 );

    	// then try to connect until an attempt succeeds
    	r = connect( datafd, (struct sockaddr *)&probe, sizeof(probe) ) >= 0;
    	if ( !r )
    	    sleep( 1 );
    }

    QApplication::setColorSpec( QApplication::ManyColor );
    // don't let X change our color settings
    //    QApplication::useXResourceManager( FALSE );
    QApplication::setStyle( new QWindowsStyle );

    QApplication a( argc, argv );

    FakeKDE f( argv[1] );
    f.show();

    QApplication::desktop()->setBackgroundColor( Qt::black );

    QTimer::singleShot( 10000, qApp, SLOT( quit( ) ) );
    a.exec( );

    ::kill( pid, 15 );
		::sleep( 1 );
    ::kill( pid, 9 );
    ::waitpid( pid, NULL, 0 );
}


