/***************************************************************************
                               upgradepage.cpp
                             -------------------                                         
    begin                : Tue Jan 4 2000                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Vadim Khakhulin, Anton Safonov (Maddy)
    author email         : vh@bestlinux.net, as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "config.h"
#include "upgradepage.h"
#include "log.h"
#include "installselect.h"
#include "packagelistitem.h"
#include "windows.h"
#include "mountpoint.h"

#include <qlayout.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#include <mntent.h>
#include <sys/swap.h>

//#include <rpmdb.h>
#include <db.h>

#ifdef GBLI_RPM3
	#define RPMTAG_OLDFILENAMES RPMTAG_FILENAMES
#endif

extern char *commercialList[];

UpgradePage::UpgradePage(PartitionView* pview_, PartitionSelector::Mode m, QWidget * parent, const char * name, Target *target_)
    : PartitionSelector(pview_, m, parent, name, target_, this)
{
	newMnt = 0;
}

UpgradePage::~UpgradePage() {
}

/*
"A complete log of your upgrade will be in /tmp/upgrade.log when the upgrade "
"is finished. After rebooting, please read it to ensure configuration files "
"are properly updated."

"Rebuilding"

"Rebuilding RPM database..."

"Rebuild of RPM database failed. You may be out of disk space?"

"Installation Path"

"Install"
"Upgrade"

""
"Would you like to install a new system or upgrade a system which already "
"contains Best Linux 2.0 or later?"

"Converting RPM database..."

"Finding packages to upgrade..."

"You don't have any Linux partitions. You can't upgrade this system!"

"Upgrade Packages"

"The packages you have installed, and any other packages which are needed to "
"satisfy their dependencies, have been selected for installation. Would you "
"like to customize the set of packages that will be upgraded?"

*/


/* Adds all files in the second file list which are not in the first
   file list to the hash table. */
void UpgradePage::compareFileList(int availFileCount, char **availBaseNames,
                            char ** availDirNames, int * availDirIndexes,
                            int instFileCount, char **instBaseNames,
                            char ** instDirNames, int * instDirIndexes,
                            struct hash_table *ht)
{
	int installedX, availX, rc;
	char * availDir, * availBase;
	char * instDir, * instBase;

	availX = 0;
	installedX = 0;
	while (installedX < instFileCount)
	{
   	instBase = instBaseNames[installedX];
   	instDir = instDirNames[instDirIndexes[installedX]];

   	if (availX == availFileCount)
		{
      	 /* All the rest have moved */
			if (strncmp(instDir, "/etc/rc.d/", 10))
	    	htAddToTable(ht, instDir, instBase);
    	installedX++;
   	}
		else
		{
     	availBase = availBaseNames[availX];
     	availDir = availDirNames[availDirIndexes[availX]];

     	rc = strcmp(availDir, instDir);
     	if (!rc)
        	 rc = strcmp(availBase, instBase);

     	if (rc > 0)
			{
       	/* Avail > Installed -- file has moved */
       	if (strncmp(instDir, "/etc/rc.d/", 10))
					htAddToTable(ht, instDir, instBase);
       	installedX++;
     	}
			else if (rc < 0)
			{
       	/* Avail < Installed -- avail has some new files */
       	availX++;
     	}
			else
			{
       	/* Files are equal -- file not moved */
       	availX++;
       	installedX++;
     	}
   	}
	}
}

static int pkgCompare( const void * first, const void * second )
{
    packageInfo ** a = (packageInfo **)first;
    packageInfo ** b = (packageInfo **)second;

    // put packages w/o names at the end
    if (!(*a)->name) return 1;
    if (!(*b)->name) return -1;

    return strcasecmp( (*a)->name, (*b)->name );
}

void UpgradePage::addLostFiles()
{
	char *name;
	struct packageInfo **pack;
	struct packageInfo key;
	struct packageInfo *keyaddr = &key;
	char **installedFiles;
	char **installedDirs;
	int_32 * installedDirIndexes;
	int installedFileCount;
	Header h = NULL;
	rpmdbMatchIterator mi;
	
	mi = rpmdbInitIterator(db, RPMDBI_PACKAGES, NULL, 0);
	while ((h = rpmdbNextIterator(mi)) != NULL)
	{
		headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);
		if (name && !strcmp(name, "metroess"))
		{
//      	metro was removed from 5.1, but leave it if it's already installed
      	continue;
   	}
   	key.name = name;

   	pack = (struct packageInfo **) bsearch( &keyaddr, psp->packages, psp->numPackages,
              						    	sizeof(*psp->packages), pkgCompare );

   	if (!pack)
		{
     	if (headerGetEntryMinMemory(h, RPMTAG_BASENAMES, NULL,
                  	  (const void **) &installedFiles, &installedFileCount))
			{
        	headerGetEntryMinMemory(h, RPMTAG_DIRINDEXES, NULL,
                  	 (const void **) &installedDirIndexes, NULL);
         	headerGetEntryMinMemory(h, RPMTAG_DIRNAMES, NULL,
                  	 (const void **) &installedDirs, NULL);

         	compareFileList(0, NULL, NULL, NULL, installedFileCount,
                        	 installedFiles, installedDirs,
                        	 installedDirIndexes, ht);

         	free(installedFiles);
         	free(installedDirs);
      	}
   	}
	}
	rpmdbFreeIterator(mi);
}

int UpgradePage::findPackagesWithObsoletes()
{
	int count, obsoletesCount;
	struct packageInfo **pip;
	char **obsoletes;

	count = psp->numPackages;
	pip = psp->packages;
	while (count--)
	{
  	if ((*pip)->selected)
		{
      	pip++;
      	continue;
   	}

   	if (headerGetEntryMinMemory((*pip)->h, RPMTAG_OBSOLETENAME, NULL,
                  	(const void **) &obsoletes, &obsoletesCount))
		{
    	while (obsoletesCount--)
			{
      	if (rpmdbCountPackages(db, obsoletes[obsoletesCount]) > 0)
				{
					(*pip)->selected = 1; // XXX
         	break;
       	}
    	}
			free(obsoletes);
   	}
   	pip++;
	}

	return 0;
}

static void errorFunction(void)
{
}

int UpgradePage::findUpgradePackages()
{
	int skipThis;
	Header h, installedHeader;
	char *name, *version;
	int count;
	char **installedFiles, **availFiles;
	char **installedDirs, ** availDirs;
	int_32 * installedDirIndexes, * availDirIndexes;
	int installedFileCount, availFileCount;
	struct packageInfo **pip;
	int *epoch, rc;

	count = psp->numPackages;
	pip = psp->packages;

	while (count--)
	{
#ifdef GBLI_STANDALONE
		if( !(*pip)->selected )
		{
			continue;
		}
#endif
		h = (*pip)->h;
		name = NULL;
		headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);
		if (!name)
		{
      	  /* bum header */
			return INST_ERROR;
		}

		headerGetEntry(h, RPMTAG_VERSION, NULL, (void **) &version, NULL);
		rc = headerGetEntry(h, RPMTAG_EPOCH, NULL, (void **) &epoch, NULL);

		rpmdbMatchIterator mi;

		mi = rpmdbInitIterator(db, RPMTAG_NAME, name, 0);
		skipThis = (mi ? 0 : 1); // esli v sisteme ne installirovan paket s imenem, kotoroe vzjato iz hdlist, to propustit' ego

		rpmErrorSetCallback(errorFunction);
		while((installedHeader = rpmdbNextIterator(mi)) != NULL)
		{
			headerGetEntry(installedHeader, RPMTAG_VERSION, NULL, (void **) &version, NULL);
			rc = headerGetEntry(installedHeader, RPMTAG_EPOCH, NULL, (void **) &epoch, NULL);

			if(rpmVersionCompare(installedHeader, h) >= 0)
			{
				if(!strcmp(name, "perl") && !strcmp(version, "5.00503"))
					break;
					
     	  /* already have a newer version installed */
				skipThis = 1;
				(*pip)->updatable = false;
				break;
			}
		}
		rpmdbFreeIterator(mi);
		rpmErrorSetCallback(NULL);
//		if(!skipThis)
//				Log::logMain( 34, ("No newer version installed") );

		if(skipThis)
		{
			Log::logMain( 34, ("DO NOT INSTALL") );
		}
		else
		{
//		Log::logMain( 34, ("UPGRADE") );
//			printf("selected name: %s\n", (*pip)->name);

			(*pip)->selected = 1; // XXX

			if(!headerGetEntryMinMemory(h, RPMTAG_BASENAMES, NULL,
                     	 (const void **) &availFiles, &availFileCount))
			{
				availFiles = NULL;
				availFileCount = 0;
			}
			else
			{
				headerGetEntryMinMemory(h, RPMTAG_DIRNAMES, NULL,
																(const void **) &availDirs, NULL);
				headerGetEntryMinMemory(h, RPMTAG_DIRINDEXES, NULL,
																(const void **) &availDirIndexes, NULL);
			}

			rpmdbMatchIterator mi;
			mi = rpmdbInitIterator(db, RPMTAG_NAME, name, 0);
			while((installedHeader = rpmdbNextIterator(mi)) != NULL)
			{
				if(headerGetEntryMinMemory(installedHeader, RPMTAG_BASENAMES,
							NULL, (const void **) &installedFiles, &installedFileCount))
				{
					headerGetEntryMinMemory(installedHeader, RPMTAG_DIRNAMES,
													NULL, (const void **) &installedDirs, NULL);
					headerGetEntryMinMemory(installedHeader, RPMTAG_DIRINDEXES,
													NULL, (const void **) &installedDirIndexes, NULL);

					compareFileList(availFileCount, availFiles,
                             availDirs, availDirIndexes,
                             installedFileCount, installedFiles,
                             installedDirs, installedDirIndexes,
                             ht);

					free(installedFiles);
					free(installedDirs);
				}
			}
			rpmdbFreeIterator(mi);

			if(availFiles)
			{
				free(availFiles);
				free(availDirs);
			}
		}
		pip++;
	}
	return 0;
}

int UpgradePage::removeMovedFilesAlreadyHandled()
{
	char *name;
	int i, count;
	Header h;
	char **availFiles, ** availDirs;
	int_32 * availDirIndexes;
	int availFileCount;
	struct packageInfo **pip;

	count = psp->numPackages;
	pip = psp->packages;
	while (count--)
	{
   	h = (*pip)->h;
   	if ((*pip)->selected)
		{
       name = NULL;
       headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);
       if (headerGetEntryMinMemory(h, RPMTAG_BASENAMES, NULL,
                  	  (const void **) &availFiles, &availFileCount))
			 {
         	headerGetEntryMinMemory(h, RPMTAG_DIRNAMES, NULL, (const void **) &availDirs, NULL);
         	headerGetEntryMinMemory(h, RPMTAG_DIRINDEXES, NULL, (const void **) &availDirIndexes, NULL);

         	for (i = 0; i < availFileCount; i++)
          {
            if (htInTable(ht, availDirs[availDirIndexes[i]], availFiles[i]))
					  {
              htRemoveFromTable(ht, availDirs[availDirIndexes[i]], availFiles[i]);
              break;
            }
         	}

         	free(availFiles);
         	free(availDirs);
       }
   	}

   	pip++;
	}
	return 0;
}

int UpgradePage::findPackagesWithRelocatedFiles()
{
	char *name;
	int i, count;
	Header h;
	char **availFiles, **availDirs;
	int_32 * availDirIndexes;
	int availFileCount;
	struct packageInfo **pip;
	int_16 * availFileModes;

	count = psp->numPackages;
	pip = psp->packages;
	while (count--)
	{
    h = (*pip)->h;
    if (! (*pip)->selected)
	  {
	   	name = NULL;
    	headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);
//			printf("relocated - %s\n", name);
    	if (headerGetEntry(h, RPMTAG_BASENAMES, NULL,
                  	 (void **) &availFiles, &availFileCount))
			{
         	headerGetEntryMinMemory(h, RPMTAG_DIRNAMES, NULL,
                     	(const void **) &availDirs, NULL);
         	headerGetEntryMinMemory(h, RPMTAG_DIRINDEXES, NULL,
                     	(const void **) &availDirIndexes, NULL);
         	headerGetEntryMinMemory(h, RPMTAG_FILEMODES, NULL,
                     	(const void **) &availFileModes, NULL);

         	for (i = 0; i < availFileCount; i++)
					{
      	   	if (S_ISDIR(availFileModes[i])) continue;

           	if(htInTable(ht, availDirs[availDirIndexes[i]], availFiles[i]))
						{
	        	 	htRemoveFromTable(ht, availDirs[availDirIndexes[i]], availFiles[i]);
             	(*pip)->selected = 1; // XXX
           	}
         	}
         	free(availFiles);
         	free(availDirs);
      }
   	}
   	pip++;
	}

	return 0;
}

int UpgradePage::unmarkPackagesAlreadyInstalled()
{
	Header h, installedHeader;
	char *name, *version;
	struct packageInfo **pip;
	int count;

	count = psp->numPackages;
	pip = psp->packages;
	while (count--)
	{
   	if ((*pip)->selected)
		{
      	h = (*pip)->h;
      	/* If this package is already installed, don't bother */
      	name = NULL;
      	headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);

      	if (!name) {
         	 /* bum header */
         	 return INST_ERROR;
      	}
   	    rpmdbMatchIterator mi;

      	mi = rpmdbInitIterator(db, RPMTAG_NAME, name, 0);
      	rpmErrorSetCallback(errorFunction);
      	while((installedHeader = rpmdbNextIterator(mi)) != NULL)
  			{
         	if (rpmVersionCompare(installedHeader, h) >= 0)
	  			{
						if(!strcmp(name, "perl"))
						{
							headerGetEntry(installedHeader, RPMTAG_VERSION, NULL, (void **) &version, NULL);
							if(!strcmp(version, "5.00503"))
								break;
    				}

           	/* already have a newer version installed */
           	(*pip)->selected = 0;
           	break;
         	}
      	}
      	rpmdbFreeIterator(mi);
      	rpmErrorSetCallback(NULL);
   	}

   	pip++;
	}
	return 0;
}

static void emptyErrorCallback(void) {
}

int UpgradePage::findUpdates(InstallSelect *a_update, PackageSelect *packSelect = 0)
{
#ifndef GBLI_STANDALONE
		int res = mountFileSystem();
		res = checkLinuxesVersions();
		printf("result after checkLinuxesVersions: %d\n", res);
#else
		if( !a_update->cs.numComponents )// we don't have CD -> nothing to update
			return INST_NOP;
		int res = checkLinuxesVersions();
#endif
		if( res != INST_OK )
		{
      umountFS();
			return res;
    }

		res = a_update->loadData();
		if( res != INST_OK )
		{
      umountFS();
			return res;
		}

//    Log::logMain(34, "headers read - %d\n", (a_update->ps).numPackages);
		psp = &(a_update->ps);
    rpmErrorCallBackType old;

    Log::logMain( 34, "findUpdates() ..." );

    rpmReadConfigFiles( NULL, NULL );

    rpmSetVerbosity( RPMMESS_FATALERROR );
    old = rpmErrorSetCallback( emptyErrorCallback );

		if(!QFile::exists(NEWROOT + "/var/lib/rpm/Packages") &&
				QFile::exists(NEWROOT + "/var/lib/rpm/packages.rpm"))
		{
	    Log::logMain( 34, ("Rebuild of rpm database") );
  		res = QMessageBox::warning(this, tr("Rebuild of rpm database"),
																 tr("Your old rpm database will be rebuilt according \n"
																		"to database structure of rpm version 4.\n"
																		"Update is not possible without this."),
																	QMessageBox::Ok, QMessageBox::Cancel);

			if(res == QMessageBox::Cancel)
			{
				umountFS();
				return INST_REBUILD_CANCEL;
      }

			QApplication::setOverrideCursor(waitCursor);
			if(rpmdbRebuild(NEWROOT.latin1()))
			{
		    Log::logMain(34, ("Rebuild of rpm database failed"));
				QApplication::restoreOverrideCursor();
	  		res = QMessageBox::critical( this, tr("Rebuild failed"),
	  	 		tr("Failed to rebuild your old rpm database.\n"
						 "It is not possible to continue with update."),
   				QString("Ok"));
				umountFS();
				return INST_REBUILD_FAILED;
			}
	    Log::logMain(34, ("Rebuild of rpm database succeeded"));
		}

		if(QFile::exists(NEWROOT + "/var/lib/rpm/Packages") &&
				QFile::exists(NEWROOT + "/var/lib/rpm/packages.rpm")) // %pre script of rpm-4 exit(1) if these two files are present
	      	::system(("mv " + NEWROOT + "/var/lib/rpm/packages.rpm " + NEWROOT + "/var/lib/rpm/_packages.rpm").latin1());

    if(!QApplication::overrideCursor())
			QApplication::setOverrideCursor(waitCursor);

		if(rpmdbOpen(NEWROOT.latin1(), &db, O_RDONLY, 0644))
		{
   		Log::logMain( 32, "failed opening %s/var/lib/rpm/Packages", NEWROOT.latin1() );
			umountFS();
 	  	return INST_ERROR;
   	}

    rpmErrorSetCallback( old );
    rpmSetVerbosity( RPMMESS_NORMAL );

    ht = htNewTable( 1103 );

    /* For all packages that are installed, if there is no package       */
    /* available by that name, add the package's files to the hash table */
    addLostFiles();
    Log::logMain( 34, ("added lost files") );
printCount();

    /* Find packages that are new, and mark them in installThisPackage,  */
    /* updating availPkgs with the count.  Also add files to the hash   */
    /* table that do not exist in the new package - they may have moved */
    if( findUpgradePackages() ){
			rpmdbClose( db );
			return INST_ERROR;
    }

    Log::logMain( 34, ("found basic packages to upgrade") );
printCount();

    /* Remove any files that were added to the hash table that are in */
    /* some other package marked for upgrade.                         */
    removeMovedFilesAlreadyHandled();
    Log::logMain( 34, ("removed extra files which have moved") );
printCount();

//	esli paket ne vybran, no v nem est' fajly, kotorye soderzats'ja v hash table, to pometit' etot paket
		findPackagesWithRelocatedFiles();
    Log::logMain( 34, ("found packages with relocated files") );
printCount();

//	esli paket ne pomechen, pometit' ego, esli u nego v obsolete propisan paket, kotoryj stoit v sisteme
    findPackagesWithObsoletes();
    Log::logMain( 34, ("found packages that obsolete installed packages") );
printCount();

//	esli paket, vybrannyj v findPackagesWithRelocatedFiles() i findPackagesWithObsoletes(),
//	uze installirovan, to selected=0
    unmarkPackagesAlreadyInstalled();
    Log::logMain( 34, ("unmarked packages already installed") );
printCount();

    htFreeHashTable( ht );

    rpmdbClose(db);

		QApplication::restoreOverrideCursor();

#ifdef GBLI_STANDALONE
	QStringList strComList;
	QList<packageInfo> comPacks;
	int i = -1;

	while(commercialList[++i])
		strComList.append(commercialList[i]);
		
	for(int i = 0; i < psp->numPackages; i++)
		if((psp->packages)[i]->selected)
			if(strComList.find((psp->packages)[i]->name) != strComList.end())
				comPacks.append((psp->packages)[i]);
	
	if(!comPacks.isEmpty())
	{
	  KeyForBLC *keyBLC = new KeyForBLC(packSelect, "BLED Key");
	  int chk = keyBLC->exec();
		if(chk == QDialog::Rejected)
		{
			packageInfo *pInfo;
			for(pInfo = comPacks.first(); pInfo != 0; pInfo = comPacks.next())
			{
				printf("unselected: %s\n", pInfo->name);
				pInfo->selected = false;				
			}
		}
	}
#endif

    return 0;
}

void UpgradePage::umountFS()
{
#ifndef GBLI_STANDALONE
	QString com;

	Log::closeMain();
	::system(("cp -af " + NEWROOT + "/tmp/update.log /mnt >/dev/null 2>&1").latin1());

	if(newMnt)
	{
		int indx = 0;
		while(newMnt[indx])
		{
			::umount(newMnt[indx]);
			free(newMnt[indx]);
			indx++;
		}
	}

	QTextOStream(&com) << "umount " << NEWROOT;
	QApplication::flushX();
 	system(com.latin1());

	::system(("mv -f /mnt/update.log " + NEWROOT + "/tmp/ >/dev/null 2>&1").latin1());
#endif
}

int UpgradePage::mountFileSystem()
{
	FILE *ft;
	char buf[128], mntPath[128];
	struct mntent *entry;
	int indx = 0;
	unsigned int i;
	QString rootPart;
	newMnt = 0;

	rootPart = MountPoints::mountPartition("/");
	Log::logMain(32, "in mountFileSystem; mounting %s\n", rootPart.latin1());
	Log::closeMain();

	::system(("mv -f " + NEWROOT + "/tmp/update.log /mnt >/dev/null 2>&1").latin1());

	sprintf(buf, "%s/etc/fstab", NEWROOT.latin1());

	QString com;
	QTextOStream(&com) << "mount " << rootPart << " " << NEWROOT;
  QApplication::flushX();
 	if(system(com.latin1())) // couldn't mount root
	{
		::system(("mv -f /mnt/update.log " + NEWROOT + "/tmp/ >/dev/null 2>&1").latin1());
		Log::logMain(32, "Couldn't mount %s\n", rootPart.latin1());
		return INST_ERROR;
	}

	ft = setmntent((const char *)buf, "r");
  if(!ft)
	{
		::umount(NEWROOT.latin1());
		::system(("mv -f /mnt/update.log " + NEWROOT + "/tmp/ >/dev/null 2>&1").latin1());
		Log::logMain(32, "Cannot read %s/etc/fstab", NEWROOT.latin1());
		return INST_ERROR;
	}

	int rec = 0;
	while(getmntent(ft))
		rec++;
	endmntent(ft);

	newMnt = (char **)malloc(sizeof(char*) * rec);

	ft = setmntent((const char *)buf, "r");
	while((entry = getmntent(ft)))
	{
		if(strncmp(entry->mnt_fsname, "/dev/", 5)) continue;
    if(strstr(entry->mnt_opts, "noauto")) continue;

    if( strncmp( entry->mnt_fsname, "/dev/hd", 7 ) &&
	      strncmp( entry->mnt_fsname, "/dev/sd", 7 ) &&
  	    strncmp( entry->mnt_fsname, "/dev/rd/", 8 ) &&
  	    strncmp( entry->mnt_fsname, "/dev/cciss/", 11 ) &&
  	    strncmp( entry->mnt_fsname, "/dev/dpti", 9 ) &&
	  		strncmp( entry->mnt_fsname, "/dev/ida/", 9 ) )
	  	continue;

		if(!strcmp(entry->mnt_dir, "/"))
			continue;

		if(!strcmp(entry->mnt_type, "ext2") || !strcmp(entry->mnt_type, "msdos") ||
			 !strcmp(entry->mnt_type, "vfat") || !strcmp(entry->mnt_type, "reiserfs") )
		{
			sprintf(mntPath, "%s%s", NEWROOT.latin1(), entry->mnt_dir);
			printf("Mounting additional partition: %s onto %s\n", entry->mnt_fsname, mntPath);

			if(::mount(entry->mnt_fsname, mntPath, entry->mnt_type, 0, NULL))
      	continue;
			else
			{
				newMnt[indx] = (char *)malloc(strlen(entry->mnt_fsname) + 1);
				for(i = 0; i < strlen(entry->mnt_fsname); i++)
					newMnt[indx][i] = (entry->mnt_fsname)[i];
				newMnt[indx][i] = '\0';
				indx++;
			}
		}
	}
	newMnt[indx] = 0;
	endmntent(ft);

	::system(("mkdir " + NEWROOT + "/tmp &>/dev/null; mv -f /mnt/update.log " + NEWROOT + "/tmp/ >/dev/null 2>&1").latin1());

  return INST_OK;
}

int UpgradePage::checkLinuxesVersions()
{
	int res = 0, i;
	char *mas;
	char *release[] = { "/etc/lsb-release", "/etc/sot-release" };
	bool compare_versions = false;
	QFile sotrel;

	for(i = 0; i < 2; i++)
	{
		if(QFile(NEWROOT + release[i]).exists())
		{
			compare_versions = true;    	 	
			break;
		}
	}

	if(compare_versions)
	{
		QFile sotrel(NEWROOT + release[i]);
 		if(sotrel.open(IO_ReadOnly))
		{
			mas	= new char[sotrel.size()];
			sotrel.readBlock(mas, sotrel.size());
			sotrel.close();
			if(strstr(mas, "Best Linux") || strstr(mas, "SOT Linux"))
				return INST_OK;
		}
	}

 	res = QMessageBox::critical( this, tr("Wrong Best/SOT Linux version"),
			tr("The OS you are trying to update is not a Best/SOT Linux OS.\n"
				 "Attempting to update a non-Best/SOT Linux OS \n"
				 "is hazardous to your system.\n"
         "Do you wish to continue?"),
  		 		QMessageBox::Yes, QMessageBox::No );

	if( res != QMessageBox::Yes )
		return INST_WRONG_VERSION;

	return INST_OK;
}

void UpgradePage::notMustBe(InstallSelect *a_update)
{
	a_update->withX = false;
	for(int i = 0; i < psp->numPackages; i++)
	{
		if(psp->packages[i]->selected == false)
			psp->packages[i]->mustBe = 0;
		else
		{
    	if(strstr(psp->packages[i]->name, "XFree86"))
				a_update->withX = true;
		}
	}
}

void UpgradePage::printCount()
{
#ifndef GBLI_DEVEL
		return;
#endif
    int i, upgradeCount;
    struct packageInfo **pip;

    upgradeCount = 0;
    pip = psp->packages;
    i = psp->numPackages;
    while( i-- ){
     	if ((*pip)->selected) {
   	    upgradeCount++;
     	}
     	pip++;
    }
    Log::logMain( 33, "marked %d packages for upgrade", upgradeCount );
}

int UpgradePage::eraseUnselectedRPMS( InstallSelect *a_update )
{
  rpmdb db;
  int mode = O_RDWR | O_EXCL;
  int count = 0;
  int numFailed = 0;
  rpmTransactionSet rpmdep;
  struct rpmDependencyConflict * conflicts;
  int numConflicts;
  int stopUninstall = 0;
  int numPackages = 0;
  rpmProblemSet probs;
	QList<packageInfo> remove_items;
	
  if( rpmdbOpen( NEWROOT.latin1(), &db, mode, 0644) ) {
    Log::logMain( 32, "Failed to open rpmdb with root as %s to uninstall packages!\n", NEWROOT.latin1() );
		return INST_ERROR;
  }

	char *name;
	struct packageInfo **pack;
	struct packageInfo key;
	struct packageInfo *keyaddr = &key;
	Header h = NULL;
	rpmdbMatchIterator mi_sys;

  rpmdep = rpmtransCreateSet( db, NEWROOT.latin1() );

	mi_sys = rpmdbInitIterator( db, RPMDBI_PACKAGES, NULL, 0 );
	while( (h = rpmdbNextIterator(mi_sys)) != NULL ){
		headerGetEntry( h, RPMTAG_NAME, NULL, (void **)&name, NULL );
   	key.name = name;
   	pack = (struct packageInfo **) bsearch( &keyaddr, a_update->ps.packages, a_update->ps.numPackages,
              						    	sizeof(*(a_update->ps.packages)), pkgCompare );
//printf( "name: %s - %s\n", name, pack?(*pack)->name:"" );
		if( pack && !(*pack)->selected ){
      rpmdbMatchIterator mi;
      mi = rpmdbInitIterator( db, RPMDBI_LABEL, name, 0 );
      count = rpmdbGetIteratorCount(mi);
      if( count <= 0 ){
        Log::logMain( 32, "package %s is not installed\n", name );
        numFailed++;
  //    }else if( !(count == 1 || (interfaceFlags & UNINSTALL_ALLMATCHES)) ){
  //      Log::logMain( 34, "\"%s\" specifies multiple packages\n"), *arg );
  //      numFailed++;
      }else{
        Header h;	/* XXX iterator owns the reference */
        while( (h = rpmdbNextIterator(mi)) != NULL ){
  	    	unsigned int recOffset = rpmdbGetIteratorOffset(mi);
    	  	if( recOffset ){
      	    rpmtransRemovePackage( rpmdep, recOffset );
						if( remove_items.findRef( *pack ) )
							remove_items.append( *pack );
      	    numPackages++;
      		}
        }
      }
      rpmdbFreeIterator(mi);
    }
	}
  rpmdbFreeIterator(mi_sys);

	// not --nodeps
  if( true /*!(interfaceFlags & UNINSTALL_NODEPS)*/ ){ // check for deps
Log::logMain(32, "numPackages: %d\n", numPackages );
Log::logMain(32, "numFailed: %d\n", numFailed );
    if( rpmdepCheck(rpmdep, &conflicts, &numConflicts) ){
        numFailed = numPackages;
        stopUninstall = 1;
    }
Log::logMain(32, "numConflicts: %d\n", numConflicts );

    if( !stopUninstall && conflicts ){
				QString startStr = tr( "Removing these packages would break dependencies:" );
				QString fullMessStr = startStr + "\n\n";
        Log::logMain( 32, startStr );

        int i, iRealConflicts = 0;

        for (i = 0; i < numConflicts; i++) {
	        int j;

  	      /* Filter already displayed problems. */
    	    for (j = 0; j < i; j++) {
      		  if( !sameProblem(conflicts + i, conflicts + j) )
			        break;
      	  }
        	if (j < i)
		        continue;

	        if (conflicts[i].sense == rpmDependencyConflict::RPMDEP_SENSE_REQUIRES){
						QString messStr = tr( "%1 is needed by %2-%3-%4" );
						messStr = messStr.arg( QString::fromLatin1(conflicts[i].needsName) ).
							arg( QString::fromLatin1(conflicts[i].byName) ).
							arg( QString::fromLatin1(conflicts[i].byVersion) ).
							arg( QString::fromLatin1(conflicts[i].byRelease) );
  		      Log::logMain( 32, messStr.latin1() );
						fullMessStr += "    "+ messStr +"\n";
						++iRealConflicts;
					}
        }

        rpmdepFreeConflicts( conflicts, numConflicts );
        numFailed += numPackages;

				if( iRealConflicts <= 20  ){ // check for number of failed
					fullMessStr += "\n"+ tr("Do you still want to continue with update?"); // tr
					stopUninstall = QMessageBox::critical( this, tr( "Dependency problems" ),
						fullMessStr, QMessageBox::Yes, QMessageBox::No )
						!= QMessageBox::Yes;
				}else{
					fullMessStr = tr( "The set of packages you have unselected\n"
														"creates %1 dependency problems in your system!\n"
														"It is not possible to continue with update!" ).
												arg( iRealConflicts );
					QMessageBox::critical( this, tr( "Dependency problems" ),
						fullMessStr, tr( "Ok" ) );
					stopUninstall = 1;
				}
    }
Log::logMain(32,  "numPackages: %d\n", numPackages );
Log::logMain(32,  "numFailed: %d\n", numFailed );
  }

  if( !stopUninstall && numPackages ){
 		int res = QMessageBox::Yes;
		if( !a_update->cs.numComponents ){
  		res = QMessageBox::warning( this, tr("Packages uninstall"),
      	tr("Yor are going to uninstall %1 package(s)!\n"
  			"It will be not possible to restore them without\n"
  			"original \"SOT Linux Installation CD\"!\n"
  			"Do you want to continue with uninstall?" ).
  			arg( numPackages ),
       	QMessageBox::Yes, QMessageBox::No );
		}
	 	if( res == QMessageBox::Yes ){
			numFailed += rpmRunTransactions( rpmdep, NULL, NULL, NULL, &probs,
				(rpmtransFlags)0, (rpmprobFilterFlags)0 );

//printf( "new probs: %p\n", probs );

			packageInfo *pi;
			int count = 0;
			for( pi = remove_items.first(); pi != 0; pi = remove_items.next() ){
/*printf( "name: %s\n", pi->item->className().latin1() );*/
				QListViewItem *lvi = pi->item;
				if( !lvi ) continue;
				QListViewItem *par = lvi->parent();
				if( par ){
					((QListViewItem*)par)->takeItem( lvi );
					delete pi->item;
					pi->item = NULL;
					if( !par->childCount() ){
						QListViewItem *tmppar = par->parent();
						while( tmppar ){
							tmppar->takeItem( par );
							par = tmppar;
							tmppar = par->parent();
							if( tmppar && tmppar->childCount() > 1 ) break;
						}
					}
				}else
					continue;
				++count;		
			}
			Log::logMain( 32, "removed %d from the list\n", count );
		}
  }
Log::logMain(32, "numFailed: %d\n", numFailed );

  rpmtransFree( rpmdep );
  rpmdbClose( db );

  return numFailed;
}

int UpgradePage::sameProblem(struct rpmDependencyConflict * ap,
														 struct rpmDependencyConflict * bp)
{

  if (ap->sense != bp->sense)
	  return 1;

  if (ap->byName && bp->byName && strcmp(ap->byName, bp->byName))
  	return 1;
  if (ap->byVersion && bp->byVersion && strcmp(ap->byVersion, bp->byVersion))
	  return 1;
  if (ap->byRelease && bp->byRelease && strcmp(ap->byRelease, bp->byRelease))
  	return 1;

  if (ap->needsName && bp->needsName && strcmp(ap->needsName, bp->needsName))
	  return 1;
  if (ap->needsVersion && bp->needsVersion && strcmp(ap->needsVersion, bp->needsVersion))
	  return 1;
  if (ap->needsFlags && bp->needsFlags && ap->needsFlags != bp->needsFlags)
	  return 1;

  return 0;
}

